class CherenkovCalibration {
private:
  double Voltage[40];
  double Gain[40];
  double Delay[40];
  double myMIP;
  double HVcorrectionGADC;
  double HVcorrectionMPIF;
  ProcessingStatus* ProcStatus;
public:
  CherenkovCalibration () { 
    printf("[CherenkovCalibration]->Constructor Croaked: attenion, this call is obsolete!\n create the CherenkovCalibration with pointer to ProcessingStatus object!\n");
    exit(-1);
  }
  
  CherenkovCalibration (ProcessingStatus* ProcStat) {
    ProcStatus=ProcStat; // pointer to the ProcStatus object 
    SetGain2008();
    HVcorrectionGADC=-1.0;
    HVcorrectionMPIF=-1.0;
  }

  void SetGain2008() {
/* data based on "Cherenkov Counter Calibration" by S.Denisov, A.Kozelov 2009.02 */
    Voltage[0]= 0.9; Gain[0] =2899.; // protons/ADC count
    Voltage[1]= 1.0; Gain[1] =962.0;
    Voltage[2]= 1.1; Gain[2] =356.2;
    Voltage[3]= 1.2; Gain[3] =144.4;
    Voltage[4]= 1.3; Gain[4] =63.19;
    Voltage[5]= 1.4; Gain[5] =29.52;
    Voltage[6]= 1.5; Gain[6] =14.58;
    Voltage[7]= 1.6; Gain[7] =7.569;
    Voltage[8]= 1.7; Gain[8] =4.102;
    Voltage[9]= 1.8; Gain[9] =2.310;
    Voltage[10]=1.9; Gain[10]=1.346;
    Voltage[11]=2.0; Gain[11]=0.8092;
    Voltage[12]=2.1; Gain[12]=0.5002;
    Voltage[13]=2.2; Gain[13]=0.3171;
    Voltage[14]=2.3; Gain[14]=0.2057;
    Voltage[15]=2.4; Gain[15]=0.1363;
    Voltage[16]=2.5; Gain[16]=0.09212;
    Voltage[17]=2.6; Gain[17]=-1.;
    Voltage[18]=2.7; Gain[18]=-1.;
    Voltage[19]=3.4; Gain[19]=-1.;
    myMIP=1.0/12.28; // MIP/ADC @ 2.5kV from MIP study by A.S. 

    for (int i=0; i<17; i++) Gain[i]*=myMIP/Gain[16];

//    for (int i=0; i<17; i++) {
//      printf("[CherenkovCalibration] %2d HV=%3.1f GainIHEP=%7f(MIPs/ADC) GainAZ=%7f IHEP/AZ=%f\n",
//             i, Voltage[i], Gain[i], Correction2008(Voltage[i]), Gain[i]/Correction2008(Voltage[i]));
//    }

  }
  
  double GetCorrectionOld (double HV) {
    for (int i=0; i<20; i++) {
      if (Voltage[i]==HV) {
        printf("[CherenkovCalibration->GetCorrectionOld] HV=%f index=%d\n", HV, i);
        return Gain[i];
      }
    }
    printf ("[CherenkovCalibration->GetCorrectionOld] Correction not found for HV=%f\n", HV);
    exit(-1);
    return -1.0;
  }

  double GetCorrection (double HV) {
    for (int i=0; i<20; i++) {
      if (Voltage[i]<=HV && Voltage[i+1]>HV) {
        printf("[CherenkovCalibration->GetCorrection1] HV=%f index=%d\n", HV, i);
        return Gain[i];
      }
    }
    printf ("[CherenkovCalibration->GetCorrection1] Correction not found for HV=%f\n", HV);
    exit(-1);
    return -1.0;
  }

  double GetCorrectionGADC() {
    if (HVcorrectionGADC<0.0) SetCorrection();
    return HVcorrectionGADC;
  }
  double GetCorrectionMPIF() {
    if (HVcorrectionMPIF<0.0) SetCorrection();
    return HVcorrectionMPIF;
  }

  void SetCorrection() {
    double HV=ProcStatus->GetRunHeader()->HVbeam[3];
    if (HV>=0.8 && HV<=2.8) {
      if (ProcStatus->GetRun() < 463 && ProcStatus->GetRun() > 309) { //use base-2008-11 parametrization
        HVcorrectionGADC=Correction2008(2.5)/Correction2008(HV)/30.*2.63;    //in tracks/ADC!
        printf("[SetCorrectionGADC()] check before using it!\n");
        exit(-1);
      } else if (ProcStatus->GetRun() < 645) {                      //use base-2009-11 parametrization
        HVcorrectionGADC=Correction2009(2.5)/Correction2009(HV)/54.4*2.63;   //in tracks/ADC!!
        printf("[SetCorrectionGADC()] check before using it!\n");
        exit(-1);
      } else if (ProcStatus->GetRun() < 855) {                    //use base-2009-11 parametrization
        HVcorrectionGADC=Correction2009(2.55)/Correction2009(HV)/95.2*2.63;    //in tracks/ADC!!
        printf("[SetCorrectionGADC()] check before using it!\n");
        exit(-1);
      } else if (ProcStatus->GetRun() < 1070) {                    //use 2013 parametrization, 2012 p.e./p
        HVcorrectionGADC=Correction2013(2.5)/Correction2013(HV)/(41.7*0.218);    //in tracks/GADC!!
        HVcorrectionMPIF=HVcorrectionGADC/33.;                        
      } else if (ProcStatus->GetRun() < 1195) {                    //use 2013 parametrization, 2013 p.e./p
        HVcorrectionGADC=Correction2013(2.5)/Correction2013(HV)/(41.7*0.74);    //in tracks/GADC!!
        HVcorrectionMPIF=HVcorrectionGADC/33.;        
      } else {
        printf("[CherenkovCalibration->SetCorrection] no correction for Run#%d\n", ProcStatus->GetRun());
        exit(-1);
      }
      printf("HV correction is set for %.2fkV\n", HV);
      //exit(-1);
    } else {
      printf("[CherenkovCalibration->SetCorrection] no correction for HV=%f\n", HV);
      HVcorrectionGADC=0.0;
      HVcorrectionMPIF=0.0;
      //exit(-1);
    }
    printf("[CherenkovCalibration->SetCorrection] -> correction=%e tracks/GADC\n", HVcorrectionGADC);
    return;
  }
  
  double Correction2008(double HV) {
    //double result=10.68-0.232*HV;
    double result=10.966589+(-0.102387-0.066478*HV)*HV;
    return exp(result*log(HV));
  }
  double Correction2009(double HV) {
    double result=9.926711+(0.351145-0.131120*HV)*HV; // is 1 at 1kV. absolute calibration needed!!!!
    result=exp(result*log(HV));
    return result;
  }
  double Correction2013(double HV) {
    double result=10.09+(0.016-0.065*HV)*HV; // is 1 at 1kV. absolute calibration needed!!!!
    result=exp(result*log(HV));
    return result;
  }
};



