// ---- 2012 online with RF event display ----//
int DisplayAllPulses2012(TModule* Setup, ProcessingStatus* ProcStatus, CherenkovBuffer* CherBuff) {

  bool Dbg=false;
  char TitleC[200];
  //char titleC[200];
  char NamecC[200];
  int Rep=0;
  int nSlice=0;

  EventHeader *EH = ProcStatus->GetEventHeader();
  RunHeader   *RH = ProcStatus->GetRunHeader();
  BurstHeader *BH = ProcStatus->GetBurstHeader();
  int dWidth  = ProcStatus->GetDisplayWidth();
  //int dHeight = ProcStatus->GetDisplayHeight();
  int Nbunches = ProcStatus->GetNbunches();

  TLine  * displayLine  = new TLine(); 
  TLatex * displayLatex = new TLatex();


  if (dWidth>1800) dWidth=1800;
  Dbg=false;
  //Dbg=true;


  sprintf(TitleC, "Run #%d, Burst#%d, Event#%d(%d), T(10MHz)=%.1fms Phase=%.1fns RFoffset=%d iMax=%d Type=%02d SignalFactor=%.2f", 
	  EH->run, EH->burst, EH->event, EH->burstEvent, EH->scaler10mhz/10000., EH->GetTimeGlobal(),
          (EH->scaler6mhz)%30, EH->GetFirstBunch(), EH->pattern[0], EH->SignalFactor); 
  sprintf(NamecC, "Run%d_Burst%d_Event%d_%d", EH->run, EH->burst, EH->event, EH->burstEvent); 

  TH1D    *dummy = new TH1D("dummy_display_new", "", 2500, -150., 150.+25.*EH->nSlice);
  dummy->GetXaxis()->SetTitle("Time, ns");

  EventCanvasNew->SetTitle(TitleC);
  EventCanvasNew->SetName(NamecC);
  EventCanvasNew->Clear();
  EventCanvasNew->Divide(1, 6, 0.002, 0.002);
  EventCanvasNew->Update();
  EventCanvasNew->Show();

  
  //if (EH->isClone) EventCanvas->SetFillColor(2);

  dummy->SetTitleSize(0.04);
  dummy->SetStats(0);
  dummy->GetYaxis()->SetTitleSize(.1);
  dummy->GetYaxis()->SetTitleOffset(.4);
  dummy->GetYaxis()->SetLabelSize(.08);
  dummy->GetXaxis()->SetTitleSize(.1);
  dummy->GetXaxis()->SetTitleOffset(.75);
  dummy->GetXaxis()->SetLabelSize(.08);
  dummy->SetLineStyle(2);

  int iModX=RH->AntennaModule;
  TModule *ModuleX = &Setup[iModX];
  printf("[DisplayAllPulses2012] In beam:%d, Coherent Source = %s\n", RH->FCalInBeam, ModuleX->GetName()); 

  int iZone=1;
  for (int iMod=0;iMod<5;iMod++) {
    if (iMod==iModX) continue;
    TModule *Module = &Setup[iMod];
    EventCanvasNew->cd(iZone++);
    displayLine->SetLineWidth(1); 
    dummy->GetYaxis()->SetTitle("High Gain ADC");
    gPad->SetBottomMargin(0.2);
    gPad->SetRightMargin(0.01);
    gPad->SetTopMargin(0.02);
    gPad->SetGridy(1);

    nSlice=Module->GetNslice();
    int myGain=1;
    int fGain=1.0;
    
    if (iMod<4) {// display modules sums //
      double amin=Module->GetSumMin();
      double amax=Module->GetSumMax();

      
      myGain=1;
      if (Module->CheckHGoverflow()) {
        myGain=0;
        fGain=10.5;
      }
      TSignal* SumSignal=Module->GetChannel(4)->GetSignalLong(myGain);
      double finalMax=fGain*SumSignal->GetSignalFinalMax();
      double finalMin=fGain*SumSignal->GetSignalFinalMin();
      double tScale=amax/finalMax;
      printf("[DisplayAllPusles2012] amax=%f finalMax=%f Scale ratio = %.3fmV/uA\n",
              amax, finalMax, tScale); 
      if (finalMax<0.125) finalMax=.125; // mA
      
//      if (amax<finalMax) amax=finalMax;
//      if (amin>finalMin) amin=finalMin;
      if (amin>-0.15*amax) amin=-0.15*amax;
      if (1.1*amax<5.0)  amax=5.0;
      if (1.1*amin>-5.0) amin=-5.0;
      double dd=0.05*(amax-amin);

      //dummy->SetTitle(Module->GetName());
      dummy->GetYaxis()->SetRangeUser(amin-dd, amax+dd);
      dummy->DrawCopy();
      int nPedWindows=ProcStatus->GetPedWindows();

/*
      displayLine->SetLineWidth(16); /// preliminary pedestal windows from processing status
      displayLine->SetLineStyle(1);
      displayLine->SetLineColor(kYellow);
      for (int iw=0; iw<nPedWindows; iw++) {
        displayLine->DrawLine(ProcStatus->GetPedWindow0(iw), 0.0,
                              ProcStatus->GetPedWindow1(iw), 0.0);
      }
*/
      displayLine->SetLineWidth(32); /// pedestal windows from modules 
      displayLine->SetLineStyle(1);
      displayLine->SetLineColor(kYellow);
      nPedWindows=Setup[iMod].GetNPedWindows();
      for (int iw=0; iw<nPedWindows; iw++) {
        //if(!Setup[iMod].GetPedWinX(iw)) continue;
        displayLine->DrawLine(Setup[iMod].GetPedWin0(iw), 0.0,
                              Setup[iMod].GetPedWin1(iw), 0.0);
      }

      displayLine->SetLineWidth(1); /// RF cycles (bunch start)
      displayLine->SetLineStyle(2);
      displayLine->SetLineColor(kGray);
      int iRF=0;
      for (double tc=EH->GetTimeGlobal(); tc<dummy->GetXaxis()->GetXmax(); tc+=EH->GetRFperiod()) {
        displayLine->DrawLine(tc, amin,
                              tc, amax);
        if (iRF>0 && EH->GetBunchAction(iRF)==1 
                  && tc>Setup[iMod].GetPedWin0(0)+5. 
                  && tc<Setup[iMod].GetPedWin1(nPedWindows-1)-5.) {
          sprintf(TitleC, "%d", EH->GetTrackCount(iRF));
          displayLatex->DrawLatex(tc+25., 1.01*amin, TitleC);
        }
        iRF++;
      }


      displayLine->SetLineWidth(2); // redraw the "0" //
      displayLine->SetLineColor(kBlack);
      displayLine->SetLineStyle(2);
      displayLine->DrawLine(dummy->GetXaxis()->GetXmin(), 0., dummy->GetXaxis()->GetXmax(), 0.); 

      displayLine->SetLineWidth(2);
      displayLine->SetLineStyle(1);

/*
      displayLine->SetLineWidth(2); // pure high gain sum //
      displayLine->SetLineColor(kRed);
      for (int is=1; is<nSlice; is++) {
        displayLine->DrawLine(Module->GetTime(is-1), Module->GetSumSampleH(is-1), 
                              Module->GetTime(is),   Module->GetSumSampleH(is));
      }
*/     
      
      
      if (Module->CheckLGoverflow()) {
        displayLine->SetLineColor(kRed);
      } else {
        displayLine->SetLineColor(kGray);
      }
      if (Module->GetCold()) displayLine->SetLineColor(kBlue);

      //printf("[DisplayAllPulses2012] plotting %s, LGoverflow=%d\n", 
      //        SumSignal->GetName(), Module->CheckLGoverflow());
 
      displayLine->SetLineWidth(2);
      for (int is=1; is<nSlice; is++) {
        displayLine->DrawLine(Module->GetTime(is-1), SumSignal->GetAmpI(is-1), // show with noise suppressed
                              Module->GetTime(is),   SumSignal->GetAmpI(is));  // and interlaced between straight and delayed
      }
   
      //continue;

      int is0=SumSignal->GetFirstSample(0);
      int is1=SumSignal->GetLastSample(nPedWindows-1);
      if (is0==-1 || is1==-1) continue; // do not show FFT correction if it is not there

      //if (iMod!=iModX && EH->notEmpty) {      
      if (iMod!=iModX) {      

/*
        displayLine->SetLineWidth(1); // baseline fits at Pedestal windows //
        displayLine->SetLineColor(kGreen);
        for (int iw=0; iw<nPedWindows; iw++) {
          if(!Module->GetPedWinX(iw)) continue;
          is0=SumSignal->GetFirstSample(iw);
          for (int i=1; is0+i<nSlice; i++) {
            displayLine->DrawLine(Module->GetTime(is0+i-1), SumSignal->GetTailAmplitude(iw, i-1), 
                                  Module->GetTime(is0+i),   SumSignal->GetTailAmplitude(iw, i));
          }
        }

        displayLine->SetLineWidth(1); // 'tailed' signal //
        displayLine->SetLineColor(kBlue);
        is0=SumSignal->GetFirstSample(0);
        is1=SumSignal->GetLastSample(nPedWindows-1);
        for (int i=1; is0+i<nSlice; i++) {
          displayLine->DrawLine(Module->GetTime(is0+i-1), SumSignal->GetTailedSignal(i-1), 
                                Module->GetTime(is0+i),   SumSignal->GetTailedSignal(i));
        }

        displayLine->SetLineWidth(2); // 'no baseline correction' signal //
        displayLine->SetLineColor(kRed);
        for (int i=1; is0+i<is1; i++) {
          displayLine->DrawLine(Module->GetTime(is0+i-1), SumSignal->GetFinalSignal(i-1)+SumSignal->GetPolyCorrection(i-1), 
                                Module->GetTime(is0+i),   SumSignal->GetFinalSignal(i)+SumSignal->GetPolyCorrection(i));
        }

        displayLine->SetLineWidth(1); // 'baseline' correction //
        displayLine->SetLineColor(kBlue);
        for (int i=1; is0+i<is1; i++) {
          displayLine->DrawLine(Module->GetTime(is0+i-1), SumSignal->GetPolyCorrection(i-1), 
                                Module->GetTime(is0+i),   SumSignal->GetPolyCorrection(i));
        }
*/
        displayLine->SetLineWidth(2); // 'final' signal //
        displayLine->SetLineColor(kBlack);
        for (int i=1; is0+i<is1; i++) {
          displayLine->DrawLine(Module->GetTime(is0+i-1), tScale*SumSignal->GetFinalSignal(i-1), 
                                Module->GetTime(is0+i),   tScale*SumSignal->GetFinalSignal(i));
          //if (iMod>1) printf("[DisplayAllPulses2012] i=%4d t=%5.0f Q=%.1f\n", i,   
        }
      }

/*      
      displayLine->SetLineWidth(2); // 'raw' signal //
      displayLine->SetLineColor(kGreen);
      for (int i=1; i<nSlice; i++) {
        double a0=0.;
        double a1=0.;
        for (int iCh=0; iCh<4; iCh++) {
          a0+=Module->GetChannel(iCh)->GetSignalLong(myGain)->GetAmp2(i-1)*fGain;
          a1+=Module->GetChannel(iCh)->GetSignalLong(myGain)->GetAmp2(i)*fGain;
        }
        displayLine->DrawLine(Module->GetTime(i-1), a0, 
                              Module->GetTime(i),   a1);
      }
*/

      displayLatex -> SetTextColor(kBlack);
      displayLatex -> SetTextSize(0.08);
      displayLatex -> SetTextAlign(13);

      double LF_Frac=Module->GetChannel(4)->GetSignalLong(1)->GetLowFrequencyFFT(5);
      if (iMod==0) { 
        sprintf(NamecC, "%s Gain#%d Run#%d LogI=%.1e Attenuator=%.0f LFfraction=%f", 
                Module->GetName(), myGain, EH->run, RH->BeamIntensityCher, RH->AttnSet, LF_Frac);
      } else {
        sprintf(NamecC, "%s Gain#%d LFfraction=%f", Module->GetName(), myGain, LF_Frac);
      }
/*
      if (ProcStatus->GetOnline()) {
        sprintf(NamecC, "Sum %8s", Module->GetName());
      } else {
        sprintf(NamecC, "Sum %8s, Prehistory = %5.2e p/bunch", Module->GetName(), EH->GetGatedIntegral(iMod));
      }
*/
      displayLatex -> DrawLatex(0., amax, NamecC); 


      if (EH->notEmpty) {


/*
        displayLine->SetLineWidth(2);
        displayLine->SetLineColor(kRed);
 
        for (int bunch=0; bunch<Nbunches; bunch++) {
          int i0=Module->GetSampleX0(bunch);
          int i1=Module->GetSampleX1(bunch);
          for (int i=i0; i<=i1; i++) {
            displayLine->DrawLine(Module->GetTime(i-1), Module->GetSumSampleX(i-1), 
                                  Module->GetTime(i),   Module->GetSumSampleX(i));
          }
        }
*/
        for (int bunch=0; bunch<Nbunches; bunch++) {
          double Tbunch=ProcStatus->GetBunchTime(bunch);
          double RateADC = Setup[4].GetChannel(3)->GetResponseADC(bunch);
          double Rate =   RateADC*CherBuff->GetGainCorrection();
          double Sum =    Setup[iMod].GetMySumRateCal(bunch);
          double SumRAW = Setup[iMod].GetMySumRate(bunch);
          printf("[DisplayEvent 2011] %10s bunch#%d Rate=%.2ep rawADC=%7.1f SumOld=%7.1f SumNew=%7.1f SumNewCal=%7.1f\n", 
               Setup[iMod].GetName(),       bunch, Rate, SumRAW, Sum, 
               Setup[iMod].GetAnear(bunch), Setup[iMod].GetAnearCal(bunch));
/* 
          displayLatex -> SetTextColor(kBlack);
          displayLatex -> SetTextSize(0.08);
          displayLatex -> SetTextAlign(13);
          if (RateADC>15.) {
            sprintf(NamecC, "%.3f ADC/p", Sum/Rate);
            displayLatex -> DrawLatex(Tbunch+100., 0.8*amax, NamecC); 
          }
*/
          displayLine->SetLineWidth(2); 
          displayLine->SetLineColor(kBlack);
          displayLine->SetLineStyle(2);
          displayLine->DrawLine(Tbunch, amax, Tbunch, amin);
/*
          displayLine->DrawLine(Tbunch-50.,      SumRAW, Tbunch+50.,      SumRAW);
          displayLine->DrawLine(Tbunch-100., 0.5*SumRAW, Tbunch+100., 0.5*SumRAW);
          //sprintf(NamecC, "%.1f ns", Setup[iMod].GetThalf(bunch));
          //displayLatex -> SetTextSize(0.08);
          //displayLatex -> SetTextAlign(11);
          //displayLatex -> DrawLatex(Tbunch+100., 0.5*SumRAW, NamecC);
*/
        }    
      }

    } else if (iMod==4) { // cherenkov only

      TChannel *Channel = Module->GetChannel(3);
      TSignal *Signal;
      int iGain=0;
      int iColor = kBlack;
      if (Channel->HighGainOverflowDetected()) {
        Signal=Channel->GetSignalLong(0);
        dummy->GetYaxis()->SetTitle("Low Gain, MPI ADC");
      } else {
        Signal=Channel->GetSignalLong(1);
        dummy->GetYaxis()->SetTitle("High Gain, MPI ADC");
        iGain=1;
      }
      if (Channel->LowGainOverflowDetected()) {
        displayLine->SetLineColor(kRed);
        iColor = kRed;
      }
      double amax0=Signal->GetMax0();
      double amin0=Signal->GetMin0();
      double dd=amax0-amin0;    
      dummy->GetYaxis()->SetRangeUser(amin0-0.05*dd, amax0+0.05*dd);
      dummy->DrawCopy();
      
      displayLine->SetLineWidth(32); /// pedestal windows from modules 
      displayLine->SetLineStyle(1);
      displayLine->SetLineColor(kYellow);
      int nPedWindows=Setup[iMod].GetNPedWindows();
      for (int iw=0; iw<nPedWindows; iw++) {
        if(!Setup[iMod].GetPedWinX(iw)) continue;
        displayLine->DrawLine(Setup[iMod].GetPedWin0(iw), 0.0,
                              Setup[iMod].GetPedWin1(iw), 0.0);
      }

      displayLine->SetLineColor(kRed);
      displayLine->SetLineWidth(2);
      displayLine->SetLineStyle(1);
      for (int is=1; is<Signal->GetNS2013(); is++) {
        if (Signal->GetTime2013(is-1) > Channel->GetT0pmtRec()) {
          displayLine->DrawLine(Signal->GetTime2013(is-1), Signal->GetAmp2013rec(is-1)/2., 
                                Signal->GetTime2013(is),   Signal->GetAmp2013rec(is)/2.);
        } 
        if (Signal->GetTime2013(is) >  Channel->GetT1pmtRec()) {
          break;
        }
      }
      

      displayLine->SetLineWidth(1); /// RF cycles (bunch start)
      displayLine->SetLineStyle(2);
      displayLine->SetLineColor(kGray);
      for (double tc=EH->GetTimeGlobal(); tc<dummy->GetXaxis()->GetXmax(); tc+=EH->GetRFperiod()) {
        displayLine->DrawLine(tc, amin0, tc, amax0);
      }

      displayLine->SetLineWidth(2); // pure high gain sum //
      displayLine->SetLineStyle(1);

      displayLatex -> SetTextColor(kBlack);
      displayLatex -> SetTextSize(0.08);
      displayLatex -> SetTextAlign(13);
      sprintf(NamecC, "Cherenkov");
      displayLatex -> DrawLatex(0., amax0, NamecC); 

      displayLine->SetLineWidth(2);
      displayLine->SetLineStyle(1);
      displayLine->SetLineColor(kGray);
      displayLatex->SetTextColor(kGray);
      double tRF1=0.;
      double yOld=0.;
      int iOff=ProcStatus->GetEventHeader()->GetFirstBunch();
      //printf("[DisplayAllPulses2012] bunch offset=%d --> \n", iOff);

      for (int ir=0; tRF1<6200. && ir<100; ir++) {
        tRF1=ProcStatus->GetEventHeader()->GetTimeGlobal()+
          ir*ProcStatus->GetEventHeader()->GetRFperiod();
        int ib=ir+iOff;
        while (ib<0)  ib+=30;
        while (ib>29) ib-=30;
        int ib1=ib%30;
        double y=ProcStatus->GetBunchPattern(ib);
        //printf("      ir=%2d ib=%2d y=%.3f\n", ir, ib, y);
        y*=amax0;
        //sprintf(NamecC, "%d %d", ib, ib1);
        //displayLatex->DrawLatex(tRF1+0.5*EH->GetRFperiod(), 1.05*y, NamecC);
        displayLine->DrawLine(tRF1, yOld,
                              tRF1, y);
        displayLine->DrawLine(tRF1, y,
                              tRF1+EH->GetRFperiod(), y);
        yOld=y;        
        
      }


      displayLine->SetLineColor(kBlack);
      displayLine->SetLineWidth(2);
      for (int is=1; is<nSlice; is++) {
        displayLine->DrawLine(Signal->GetTime(is-1), Signal->GetAmp1(is-1), 
                              Signal->GetTime(is),   Signal->GetAmp1(is));
      }
/*
      displayLine->SetLineWidth(2); // 'raw' signal //
      displayLine->SetLineColor(kGray);
      for (int is=1; is<nSlice; is++) {
        displayLine->DrawLine(Signal->GetTime(is-1), Signal->GetAmp2(is-1), 
                              Signal->GetTime(is),   Signal->GetAmp2(is));
      }
*/
/*
      double dT=5.;
      for (int iDelay=0; iDelay<2; iDelay++) {
        displayLine->SetLineColor(kBlack+iDelay+1);
        displayLine->SetLineWidth(1);
        TSignal *SignalShort;
        SignalShort = Channel->GetSignalShort(2*iGain+iDelay);
        for (double t=SignalShort->GetTime(0)-5.; t<SignalShort->GetTime(nSlice/2-1)+5.; t+=dT) {
          displayLine->DrawLine(t-dT,  SignalShort->getSinWave(t-dT),
                                t,     SignalShort->getSinWave(t));
        }
        for (int i=0; i<nSlice/2; i++) {
          double a=SignalShort->GetAmp1(i)-SignalShort->getSinWave(SignalShort->GetTime(i));
          SignalShort->SetAmp1(i, a);
          Signal->SetAmp1(i*2+iDelay, a);
        }
      }

      displayLine->SetLineColor(kBlack);
      displayLine->SetLineWidth(2);
      for (int is=1; is<nSlice; is++) {
        displayLine->DrawLine(Signal->GetTime(is-1), Signal->GetAmp1(is-1), 
                              Signal->GetTime(is),   Signal->GetAmp1(is));
      }
*/

      printf("[main->DisplayAllPulses2012] GoodCherenkov=%d\n", BH->goodGatedCherenkov);
      if (BH->goodGatedCherenkov) {
        double cherScale=1.;
        if (iGain==0) cherScale=10.;
        displayLine->SetLineWidth(2);
        displayLatex->SetTextColor(kRed);
        double tRF=EH->GetRFperiod();
        double t00=EH->GetTimeGlobal();
        double s0=0.;
        double s1=0.;
        double s2=0.;
        for (int i=0; i<39; i++) {
          double t0=t00+i*tRF;
          double t1=t00+i*tRF+tRF;
          int iGC=CherBuff->GetSampleIndex(i);
          int iAction=ProcStatus->GetBunchAction(iGC%30);
          sprintf(NamecC, "%d", iGC%30);
/*
          if (iAction==1) {
            displayLine->SetLineColor(kRed);
          } else if (iAction==2) {
            displayLine->SetLineColor(6);
            double a=CherBuff->GetSampleAdjustD(i)/cherScale;
            s0+=1.0;
            s1+=a;
            s2+=a*a;
          } else {
            displayLine->SetLineColor(kBlue);
          }
*/
          displayLine->SetLineColor(kBlue);
//          displayLatex->DrawLatex(0.5*(t0+t1), 1.05*CherBuff->GetSampleAdjustD(i)/cherScale, NamecC);
          displayLine->DrawLine(t0, CherBuff->GetSampleAdjustD(i)/cherScale, 
                                t1, CherBuff->GetSampleAdjustD(i)/cherScale);
          if (i<38) displayLine->DrawLine(t1, CherBuff->GetSampleAdjustD(i)/cherScale, 
                                          t1, CherBuff->GetSampleAdjustD(i+1)/cherScale); 
          if (i>0 && EH->GetBunchAction(i)==1 
                  && t1<Module->GetPedWin1(Module->GetNPedWindows()-1) 
                  && t1>Module->GetPedWin0(0)) {
            myLatex->SetTextSize(0.08);
            myLatex->SetTextAlign(12);
            sprintf(NamecC, "%.1f", CherBuff->GetSampleAdjustDHV(i));
            myLatex->SetTextColor(kBlue);
            myLatex->DrawLatex(t0, amax0, NamecC);


            if(Setup[4].GetChannel(3)->GetClusterPMTbunchSET(i)) {
              sprintf(NamecC, "%.1f", Setup[4].GetChannel(3)->GetClusterPMTbunchAmpTracks(i));
              myLatex->SetTextColor(kRed);
              myLatex->DrawLatex(t0, amin0, NamecC);
            }            
          }
        }

        s1/=s0;
        s2/=s0;
        s2-=s1*s1;
        if (s2<0.0) s2=0.0;
        s2=sqrt(s2);
        //sprintf(NamecC, "Gated noise=%.1f (ADC)", s2);
        //displayLatex -> DrawLatex(0., 0.8*amax0, NamecC); 
        //sprintf(NamecC, "Gated noise/noise0=%.2f", s2/CherBuff->GetSpillNoise());
        //displayLatex -> DrawLatex(0., 0.6*amax0, NamecC); 
        

        displayLine->SetLineStyle(2);
        displayLine->SetLineColor(6);
        displayLine->SetLineWidth(1);
        displayLine->DrawLine(Signal->GetTime(0),         CherBuff->GetSpillNoise(),
                              Signal->GetTime(nSlice-1),  CherBuff->GetSpillNoise());
        displayLine->DrawLine(Signal->GetTime(0),        -CherBuff->GetSpillNoise(),
                              Signal->GetTime(nSlice-1), -CherBuff->GetSpillNoise());
      
      } // Gated Spill data is OK
      

      for (int bunch=0; bunch<Nbunches; bunch++) {
        double Tbunch=ProcStatus->GetBunchTime(bunch);
        double RateADC=Setup[4].GetChannel(3)->GetResponseADC(bunch);
        if (RateADC>15.0) {
          double Rate=RateADC*CherBuff->GetGainCorrection();
          displayLatex -> SetTextColor(kBlack);
          displayLatex -> SetTextSize(0.08);
          displayLatex -> SetTextAlign(13);
          sprintf(NamecC, "%5.2ep", Rate);
          //displayLatex -> DrawLatex(Tbunch+100., 0.8*amax0, NamecC); 
          displayLine->SetLineWidth(1); 
          displayLine->SetLineColor(kBlack);
          displayLine->SetLineStyle(2);
          //displayLine->DrawLine(Tbunch, amax0, Tbunch, amin0);
        }
      }    
    }
    EventCanvasNew->Update();
  }

  EventCanvasNew->Update();

  EventCanvasNew->cd(5);
  displayLine->SetLineWidth(2); 
  displayLine->SetLineStyle(1); 
  gPad->SetBottomMargin(0.2);
  gPad->SetRightMargin(0.01);
  gPad->SetTopMargin(0.1);
  gPad->SetGridy(1);
  
// show scitillatorios in next division

  double scintMax=-1.0e10;
  double scintMin=1.0e10;
  int iGain=1;
  dummy->GetYaxis()->SetTitle("High Gain ADC");

  for (int iMod1=4; iMod1<5; iMod1++) {
    int iCh0=0;
    int iCh1=3;
    if (iMod1==5) {
      iCh1=2; // shall be 3, but in test data is an extra fake 6MHz signal
      if (EH->run==852) iCh1=1;
    }

    for (int iCh=iCh0; iCh<iCh1; iCh++) {
      TChannel *Channel = Setup[iMod1].GetChannel(iCh);
      if (Channel->HighGainOverflowDetected()) {
        printf("[DisplayAllPulses2012] HG overflow in %s\n", Channel->GetName());
        dummy->GetYaxis()->SetTitle("Low Gain ADC");
        iGain=0;
      }
    }
  }

  for (int iMod1=4; iMod1<5; iMod1++) {
    int iCh0=0;
    int iCh1=3;
    if (iMod1==5) {
      iCh1=2; // shall be 3, but in test data is an extra fake 6MHz signal
      if (EH->run==852) iCh1=1;
    }

    for (int iCh=iCh0; iCh<iCh1; iCh++) {
      TChannel *Channel = Setup[iMod1].GetChannel(iCh);
      TSignal  *Signal  = Channel->GetSignalLong(iGain);
      if (Signal->GetMax0()>scintMax) scintMax=Signal->GetMax0();
      if (Signal->GetMin0()<scintMin) scintMin=Signal->GetMin0();  
    }
  }

  double tx0=0.;
  double tx1=0.; 
  for (int iMod1=4; iMod1<5; iMod1++) {
    int iCh0=0;
    int iCh1=3;
    if (iMod1==5) {
      iCh1=2; // shall be 2, but in test data is an extra fake 6MHz signal
      if (EH->run==852) iCh1=1;
    }

    for (int iCh=iCh0; iCh<iCh1; iCh++) {
      TChannel *Channel = Setup[iMod1].GetChannel(iCh);
      TSignal  *Signal  = Channel->GetSignalLong(iGain);
      tx0=Signal->GetTime(0);
      tx1=Signal->GetTime(nSlice-1);
      if (Signal->GetMax0()>scintMax) scintMax=Signal->GetMax0();
      if (Signal->GetMin0()<scintMin) scintMin=Signal->GetMin0();  
    }
  }

  double dd=scintMax-scintMin;    
  displayLine->SetLineWidth(2);
  displayLine->SetLineStyle(1);
  dummy->GetYaxis()->SetRangeUser(scintMin-0.05*dd, scintMax+0.05*dd);
  if (iGain==0) dummy->GetYaxis()->SetTitle("Low Gain ADC");
  if (iGain==1) dummy->GetYaxis()->SetTitle("High Gain ADC");
  dummy->DrawCopy();

  displayLine->SetLineWidth(32); /// pedestal windows from modules 
  displayLine->SetLineStyle(1);
  displayLine->SetLineColor(kYellow);
  int nPedWindows=Setup[4].GetNPedWindows();
  for (int iw=0; iw<nPedWindows; iw++) {
    if(!Setup[4].GetPedWinX(iw)) continue;
    displayLine->DrawLine(Setup[4].GetPedWin0(iw), 0.0,
                          Setup[4].GetPedWin1(iw), 0.0);
  }

  displayLine->SetLineWidth(1); /// RF cycles (bunch start)
  displayLine->SetLineStyle(2);
  displayLine->SetLineColor(kGray);
  for (double tc=EH->GetTimeGlobal(); tc<dummy->GetXaxis()->GetXmax(); tc+=EH->GetRFperiod()) {
    displayLine->DrawLine(tc, scintMin, tc, scintMax);
  }

/*
  displayLine->SetLineWidth(3);
  displayLine->SetLineStyle(1);
  displayLine->SetLineColor(kGreen);
  for (int bunch=0; bunch<Nbunches; bunch++) {
    double Tbunch=ProcStatus->GetBunchTime(bunch);
    displayLine->DrawLine(Tbunch, scintMin, Tbunch, scintMax);
  }
*/


  int iColor=1;
  int iDet=0;


  for (int iMod=4; iMod<5; iMod++) {
    nSlice=Setup[iMod].GetNslice();
    int iCh0=0;
    int iCh1=3;
    if (iMod==5) {
      iCh1=2; // shall be 2, but in test data is an extra fake 6MHz signal
      if (EH->run==852) iCh1=1;
    }

    for (int iCh=iCh0; iCh<iCh1; iCh++) {
      TChannel *Channel = Setup[iMod].GetChannel(iCh);
      TSignal   *Signal = Channel->GetSignalLong(iGain);
      displayLine->SetLineColor(iColor);
      displayLine->SetLineStyle(1);
      displayLine->SetLineWidth(2);
      displayLatex -> SetTextColor(iColor);
      displayLatex -> SetTextSize(0.08);
      displayLatex -> SetTextAlign(13);
      sprintf(NamecC, "%s gain=%d", Channel->GetName(), iGain);
      displayLatex -> DrawLatex(0., scintMax*(1.0 - 0.15*iDet), NamecC); 

      for (int is=1; is<nSlice; is++) {
        displayLine->DrawLine(Signal->GetTime(is-1), Signal->GetAmp1(is-1), 
                              Signal->GetTime(is),   Signal->GetAmp1(is));
      }
      iColor++;
      iDet++;
      if (iColor==5) iColor++;
    }
  }





  EventCanvasNew->cd(6);
  displayLine->SetLineWidth(1); 
  gPad->SetBottomMargin(0.2);
  gPad->SetRightMargin(0.01);
  gPad->SetTopMargin(0.1);
  gPad->SetGridy(1);
  if (isOnline) {
// show RF
    TModule *Module = &Setup[5]; // ex-monitor
    TChannel *Channel = Module->GetChannel(2); // ex-M3, now RF
    TSignal *Signal;
    nSlice=Module->GetNslice();
    Signal=Channel->GetSignalLong(0);
    dummy->GetYaxis()->SetTitle("Low Gain ADC");

    double amax0=Signal->GetMax0();
    double amin0=Signal->GetMin0();
    double dd1=amax0-amin0;    

    dummy->GetYaxis()->SetRangeUser(amin0-0.05*dd1, amax0+0.05*dd1);
    dummy->DrawCopy();
    myLatex->SetTextAlign(22);
    displayLine->SetLineStyle(1);
    displayLine->SetLineColor(kRed);
    displayLine->SetLineWidth(2);
    displayLine->DrawLine(EH->GetTimeGlobal(), amin0,
                          EH->GetTimeGlobal(), amax0);
// show sin wave
    displayLine->SetLineColor(kRed);
    displayLine->SetLineWidth(1);
    double dt=3.;
    for (double t0=Signal->GetTime(0); t0<Signal->GetTime(nSlice-1); t0+=dt) {
      double t1=t0+dt;
      displayLine->DrawLine(t0, EH->GetSinWave(t0), 
                            t1, EH->GetSinWave(t1));      
    }

// show residuals sin wave

    displayLine->SetLineColor(kGray);
    displayLine->SetLineWidth(1);
    for (int is=1; is<nSlice; is++) {
      double t1=Signal->GetTime(is);
      double t0=Signal->GetTime(is-1);      
      displayLine->DrawLine(t0, 10.*(Signal->GetAmp1(is-1)-EH->GetSinWave(t0)), 
                            t1, 10.*(Signal->GetAmp1(is)-EH->GetSinWave(t1)));      
    }

// show samples
    displayLine->SetLineWidth(2);
    displayLine->SetLineColor(kBlack);
    for (int is=0; is<nSlice; is++) {
      displayLine->DrawLine(Signal->GetTime(is), Signal->GetAmp1(is)-0.03*dd1, 
                            Signal->GetTime(is), Signal->GetAmp1(is)+0.03*dd1);
      displayLine->DrawLine(Signal->GetTime(is)-12., Signal->GetAmp1(is), 
                            Signal->GetTime(is)+12., Signal->GetAmp1(is));
    }
 
  } else { // show prehistory
    double Sum30max=-1.0e10;
    double Sum30min=1.0e10;
    double EventTime=EH->scaler10mhz*100; // ns!
    double Thist=0.5*ProcStatus->GetTdrift(1); // 50ms of history
    double T0=EventTime-Thist;
    double T1=EventTime+Thist/10.; 
    dummy->SetBins(1000, T0*1.0e-6, T1*1.0e-6);

    int nLaps=CherBuff->GetNlaps();
    int iLap0=-1;
    int iLap1=0;
    for (int iLap=1; iLap<nLaps; iLap++) {
      double t=CherBuff->GetLapTime(iLap);
      if (t>T0) {
        if (iLap0==-1) iLap0=iLap;
        if (CherBuff->GetLapHV(iLap)>Sum30max)     Sum30max=CherBuff->GetLapHV(iLap);
        if (CherBuff->GetLapHV(iLap)<Sum30min)     Sum30min=CherBuff->GetLapHV(iLap);
        if (CherBuff->GetMaxHV(iLap)>Sum30max)     Sum30max=CherBuff->GetMaxHV(iLap);
      }
      if (t>T1) break;
      iLap1=iLap;
    }
    
    if (Sum30max<50.) Sum30max=50.;
    gPad->SetGridy(1);
    dummy->GetXaxis()->SetTitle("Time, ms");
    dummy->GetYaxis()->SetTitle("Protons/Lap");
    dummy->GetYaxis()->SetRangeUser(1.05*Sum30min, 1.05*Sum30max);
    dummy->SetStats(0);
    dummy->DrawCopy();
    //int i3spike=0;

    //double dt=0.;
    //double wdt=0.;

    displayLine->SetLineColor(kRed);
    displayLine->SetLineStyle(1);
    displayLine->SetLineWidth(4);
    int currentGatedPointer=CherBuff->GetEventPointer(EH->burstEvent);
    for (int jEvt=EH->burstEvent; jEvt>-1; jEvt--) {
      int evtGatedPointer=CherBuff->GetEventPointer(jEvt);
      if (evtGatedPointer<0) break;
      double Tevt = EH->scaler10mhz*100. + EH->GetRFperiod() * (evtGatedPointer-currentGatedPointer); // ns!
      printf("[DispalyEvent2012] history jEvt=%d Tevt=%.1f#mus T0=%.1f#mus\n", 
             jEvt, Tevt*1.0e-6, T0*1.0e-6);
      if (Tevt<T0) break;
      displayLine->DrawLine(Tevt*1.0e-6, 1.05*Sum30min, Tevt*1.0e-6, 1.05*Sum30max);
      displayLine->SetLineWidth(2);
    }
    
    EventCanvasNew->Update();
  
  

    displayLine->SetLineStyle(1);
    displayLine->SetLineColor(kGray);
    displayLine->SetLineWidth(2);

    for (int iLap=iLap0+1; iLap<iLap1; iLap++) {
      displayLine->DrawLine(CherBuff->GetLapTime(iLap-1)*1.0e-6, CherBuff->GetMaxHV(iLap-1),
                            CherBuff->GetLapTime(iLap)*1.0e-6,   CherBuff->GetMaxHV(iLap));
    }
    EventCanvasNew->Update();

    displayLine->SetLineStyle(1);
    displayLine->SetLineColor(kBlack);
    displayLine->SetLineWidth(3);

    double Rate=0.;

    for (int iLap=iLap0+1; iLap<iLap1; iLap++) {
      displayLine->DrawLine(CherBuff->GetLapTime(iLap-1)*1.0e-6, CherBuff->GetLapHV(iLap-1),
                            CherBuff->GetLapTime(iLap)*1.0e-6,   CherBuff->GetLapHV(iLap));
      Rate+=CherBuff->GetLapHV(iLap);
    }
    Rate/=(iLap1-iLap0);
    sprintf(myText, "Rate=%.1fp/lap (%.0ep/sec)", Rate, 1.e9*Rate/4950.);
    myLatex->SetTextSize(0.1);
    myLatex->SetTextAlign(11);
    myLatex->SetTextColor(kBlack);
    myLatex->DrawLatex((T0+0.02*(T1-T0))*1.0e-6, 1.05*Sum30max, myText);

/* noise
    displayLine->SetLineStyle(1);
    displayLine->SetLineColor(6);
    displayLine->SetLineWidth(2);

    for (int iLap=iLap0+1; iLap<iLap1; iLap++) {
      displayLine->DrawLine(CherBuff->GetLapTime(iLap-1)*1.0e-6, CherBuff->GetNoiseHV(iLap-1),
                            CherBuff->GetLapTime(iLap)*1.0e-6,   CherBuff->GetNoiseHV(iLap));
    }
    displayLine->SetLineStyle(2);
    displayLine->SetLineColor(6);
    displayLine->SetLineWidth(2);
    displayLine->DrawLine(CherBuff->GetLapTime(iLap0+1)*1.0e-6, CherBuff->GetSpillNoiseHV(),
                          CherBuff->GetLapTime(iLap1-1)*1.0e-6, CherBuff->GetSpillNoiseHV());

*/
    for (int iMod=1; iMod<4; iMod++) {
      if (iMod==iModX) continue;
      double Tdrift=ProcStatus->GetTdrift(iMod);
      displayLine->SetLineColor(iMod+1);
      displayLine->SetLineStyle(1);
      displayLine->SetLineWidth(1);
      for (int iLap=iLap0+1; iLap<iLap1; iLap+=20) {
        if (iLap<20) continue;
        double T00=CherBuff->GetLapTime(iLap-20);
        double T10=CherBuff->GetLapTime(iLap);
        displayLine->DrawLine(T00*1.0e-6, CherBuff->GetIntegral(T00-EventTime, Tdrift, 2),
                              T10*1.0e-6, CherBuff->GetIntegral(T10-EventTime, Tdrift, 2));
      }    
      EventCanvasNew->Update();
    }
  } 
 
  EventCanvasNew->Update();
  EventCanvasNew->Show();
  EventCanvasNew->Update();
  

  Rep=1;
  ProcStatus->SetReportMode(Rep);

  dummy->Delete();
  displayLine->Delete();
  displayLatex->Delete();

  return Rep; 
}


