

int DisplayAllPulsesDetails(TModule* Setup, ProcessingStatus* ProcStatus, CherenkovBuffer* CherBuff) {
// shows all gains and channels, not just sums
  bool Dbg=false;
  //Dbg=true;
  EventHeader *EH = ProcStatus->GetEventHeader();
  RunHeader   *RH = ProcStatus->GetRunHeader();
  BurstHeader *BH = ProcStatus->GetBurstHeader();
  int dWidth  = ProcStatus->GetDisplayWidth();
  //int dHeight = ProcStatus->GetDisplayHeight();
  TLine  * displayLine  = new TLine(); 
  TLatex * displayLatex = new TLatex();

  if (dWidth>1800) dWidth=1800;
  
  char TitleC[200];
  char NamecC[200];
  double integ0=Setup[6].GetChannel(0)->GetSignalShort(0)->GetIntegratorLevel();
  double integ1=Setup[6].GetChannel(1)->GetSignalShort(0)->GetIntegratorLevel();
  double integ2=Setup[6].GetChannel(2)->GetSignalShort(0)->GetIntegratorLevel();
  double integ3=Setup[6].GetChannel(3)->GetSignalShort(0)->GetIntegratorLevel();

  double CherAV=0.0;
  int Nbunches=ProcStatus->GetNbunches();
  int Ngb=0;
  for (int bunch=0; bunch<Nbunches; bunch++) {
    CherAV+=Setup[4].GetChannel(3)->GetResponseADC(bunch);
    Ngb++;
  }
  if (Ngb!=0) {
    CherAV/=Nbunches;
  } else {
    CherAV=0.;
  }

  double maxFuzz=0.0;
  for (int iMod=0; iMod<6; iMod++) {
    TModule *Module = &Setup[iMod];
    for (int iChan=0; iChan<4; iChan++) {
      for (int iGain=0; iGain<2; iGain++) {
        TChannel *Channel = Module->GetChannel(iChan);
        TSignal *Signal = Channel->GetSignalLong(iGain);
        if (Signal->GetStatus()) {
          //if (Signal->GetFuzz()>maxFuzz) maxFuzz=Signal->GetFuzz();  
          maxFuzz+=Signal->GetFuzz();  
          //printf("[DisplayAllChannels] %16s signalFuzz=%f \t maxFuzz=%f\n", 
                 //Signal->GetName(), Signal->GetFuzz(), maxFuzz);
        }
      }
    }
  }


  sprintf(TitleC, "Run #%d, Burst#%d, Event#%d(%d), 10MHz=%d Type=%02d <Ch(HG)>=%.1f Integrator=(%.1f, %.1f, %.1f, %.1f) SignalFactor=%.1f", 
	  EH->run, EH->burst, EH->event, EH->burstEvent, EH->scaler10mhz, EH->pattern[0], 
          CherAV, integ0, integ1, integ2, integ3, EH->SignalFactor); 
  sprintf(NamecC, "Run%d_Burst%d_Event%d_%d", EH->run, EH->burst, EH->event, EH->burstEvent); 

  //TCanvas * SpikeCanvas = new TCanvas("Spike", "Spike", int(dWidth/1.2)+1, 0, int(dWidth/2.), int (0.5*dHeight-10.));
  //SpikeCanvas->Divide(1,2);
 
  EventCanvas -> Connect("ProcessedEvent(Int_t,Int_t,Int_t,TObject*)", 0, 0, 
                        "MyExecuteEvent(Int_t,Int_t,Int_t,TObject*)");
  
  TH1D    *dummy = new TH1D("dummy", "dummy", 2500, -150., 150.+25.*EH->nSlice);

  EventCanvas->SetTitle(TitleC);
  EventCanvas->SetName(NamecC);
  EventCanvas->Clear();
  EventCanvas->Divide(2, 7, 0.002, 0.002);
  
  if (EH->isClone) EventCanvas->SetFillColor(2);


  for (int iMod=0;iMod<6;iMod++) {
    TModule *Module = &Setup[iMod];
    dummy->GetXaxis()->SetTitle("Time, ns");
    dummy->GetYaxis()->SetTitle("ADC");
    double AmpMax[2]={5., 5.}, AmpMin[2]={-5., -5.};

    for (int iGain=0;iGain<2;iGain++) {
      char Title[60];
      if (iGain) sprintf(Title, "%s, High Gain", Module->GetName());
      else  sprintf(Title, "%s, Low Gain", Module->GetName());
      dummy->SetTitle(Title);
      EventCanvas->cd(1+iMod*2+iGain);
      gStyle->SetOptStat(0);
      //gPad->SetFillStyle(2);
      //gPad->SetFillColor(kRed);
      dummy->SetTitleSize(0.04);
      gPad->SetBottomMargin(0.2);
      gPad->SetRightMargin(0.01);
      gPad->SetTopMargin(0.01);

      int iCh0=0;
      int iCh1=4;
      if (iMod==5 && EH->run>850) { // only two monitors, iCh#3 is RF for April 2012
        iCh1=2;
      }

      for (int iChan=iCh0;iChan<iCh1;iChan++) {
        TChannel *Channel = Module->GetChannel(iChan);
        TSignal *Signal = Channel->GetSignalLong(iGain);
        if (Signal->GetStatus()) {
          double ama=Signal->GetMax();
          double ami=Signal->GetMin();
          if (Dbg) printf("[DisplayAllPulses]      %s min=%.2f max=%.2f\n",Signal->GetName(), ami, ama);
          if (ama>AmpMax[iGain]) AmpMax[iGain]=ama; 
          if (ami<AmpMin[iGain]) AmpMin[iGain]=ami; 
          //if (iMod==1) printf("[DisplayAllPulses] Long range: %.1f...%.1f\n", ami, ama);
	} else { // Check for the response from straight or delayed 
          for (int iDelay=0; iDelay<2; iDelay++) { 
            Signal = Channel->GetSignalShort(2*iGain+iDelay);
            if (Signal->GetStatus()) {
              double ama=Signal->GetMax();
              double ami=Signal->GetMin();
              if (Dbg) printf("[DisplayAllPulses]      %s min=%.2f max=%.2f\n",Signal->GetName(), ami, ama);
              if (ama>AmpMax[iGain]) AmpMax[iGain]=ama; 
              if (ami<AmpMin[iGain]) AmpMin[iGain]=ami;
              //if (iMod==1) printf("[DisplayAllPulses] Short range: %.1f...%.1f\n", ami, ama);
	    }
	  }
	}
      }

      
      if (iMod==4 && BH->goodGatedCherenkov) {
        double ChMax=CherBuff->GetCherenkovMax(); 
        double ChMin=CherBuff->GetCherenkovMin();
        ChMax=0.;
        ChMin=0.;
        //printf("[DisplayAllPulses] ChMax=%f ChMin=%f\n", ChMax,  ChMin);
        if (iGain==0) {
          if (ChMax/10.>AmpMax[iGain]) AmpMax[iGain]=ChMax/10.;
          if (ChMin/10.<AmpMin[iGain]) AmpMin[iGain]=ChMin/10.;
        } else {
          if (ChMax>AmpMax[iGain]) AmpMax[iGain]=ChMax;
          if (ChMin<AmpMin[iGain]) AmpMin[iGain]=ChMin;
        }
      }


      if (Dbg) printf("[DisplayAllPulses] %s Gain=%d min=%.2f max=%.2f\n", Module->GetName(), iGain, AmpMin[iGain], AmpMax[iGain]);
      dummy->GetYaxis()->SetRangeUser(1.05*AmpMin[iGain], 1.05*AmpMax[iGain]);
      dummy->GetYaxis()->SetTitleSize(.1);
      dummy->GetYaxis()->SetTitleOffset(.5);
      dummy->GetYaxis()->SetLabelSize(.08);
      dummy->GetXaxis()->SetTitleSize(.1);
      dummy->GetXaxis()->SetTitleOffset(.75);
      dummy->GetXaxis()->SetLabelSize(.08);      
      dummy->DrawCopy();


      for (int iChan=iCh0;iChan<iCh1;iChan++) {
        TChannel *Channel = Module->GetChannel(iChan);
        TSignal *Signal = Channel->GetSignalLong(iGain);
        if (!Signal->GetStatus()) { // Check for the response from straight or delayed 
          for (int iDelay=0; iDelay<2; iDelay++) { 
            Signal = Channel->GetSignalShort(2*iGain+iDelay);
            if (Signal->GetStatus()) {
	      Signal->SetGraph();
              break;
	    }
	  }
	}
        if (Signal->GetStatus()) { // Check for the response from straight or delayed 
          TGraphErrors *Graph = Signal->GetGraph(); 
	  if (Graph->GetN()) {
            int iColor=iChan+1;
            if (Dbg && ProcStatus->GetReportMode()==10006) {
              printf("[DisplayAllPulses]  Drawing %s %s Nsample=%d Gain=%d\n", 
		     Module->GetName(), Channel->GetName(), Graph->GetN(), iGain);
	      for (int i=0;i<Graph->GetN();i++) {
                double x, y;
                Graph->GetPoint(i, x, y);
		printf("[DisplayAllPulses] %3d %6.1f %6.1f\n", 
		         i, x, y);
	      }
	    }
	    Graph->SetMarkerColor(iColor);
	    Graph->SetMarkerStyle(21);
	    Graph->SetMarkerSize(0.15);
	    Graph->SetLineColor(iColor);

            //Graph->Draw("LPZz same");
            //Graph->Draw("same");

	    displayLine->SetLineColor(iColor);
	    displayLine->SetLineStyle(1);
            for (int i=1; i<Graph->GetN();i++) {
              displayLine->DrawLine(Signal->GetTime(i-1), Signal->GetAmp1(i-1),
                                    Signal->GetTime(i),   Signal->GetAmp1(i));
            }


            for (int bunch=0; bunch<Nbunches; bunch++) {
              double Tbunch=ProcStatus->GetBunchTime(bunch);
              char BunchVal[100];
              displayLatex -> SetTextSize(0.08);
              displayLatex -> SetTextAlign(12);
              displayLatex -> SetTextColor(iColor);
              sprintf(BunchVal, "%6.1f", Signal->GetResponseADC(bunch));
              displayLatex->DrawLatex(Tbunch+100., AmpMax[iGain]*(0.9-0.15*iChan), BunchVal);
   	    }
            
            //printf("[main->DisplayAllPulses] GoodCherenkov=%d\n", BH->goodGatedCherenkov);

            if (iMod==4 && iChan==3 && BH->goodGatedCherenkov) {
              double cherScale=1.;
              if (iGain==0) cherScale=10.;
              for (int i=0; i<39; i++) {
                //printf("[DisplayAllChannels] #%2d t=%f6.1 A=%f6.1\n", i, CherBuff->GetCherenkovTime(i), CherBuff->GetSampleAdjust(i));
                displayLine->SetLineWidth(2);
                displayLine->SetLineColor(kBlue);
                displayLine->DrawLine(CherBuff->GetCherenkovTime(i)-82.481,  
                                        CherBuff->GetSampleAdjustD(i)/cherScale, 
                                        CherBuff->GetCherenkovTime(i)+82.481, 
                                        CherBuff->GetSampleAdjustD(i)/cherScale);
                if (i<38) displayLine->DrawLine(CherBuff->GetCherenkovTime(i)+82.481, 
                                        CherBuff->GetSampleAdjustD(i)/cherScale, 
                                        CherBuff->GetCherenkovTime(i)+82.481, 
                                        CherBuff->GetSampleAdjustD(i+1)/cherScale); 
              } // gated sample 
            } // cherenkov 
	  } // Graph found 
	} // good signal
      } // channel loop

      if (iMod<4 && iMod != RH->AntennaModule) {
        displayLine->SetLineColor(kGray);
        displayLine->SetLineStyle(1);
        double a0=0.0;
        double a1=0.0;
        TSignal* mySignal=Module->GetChannel(4)->GetSignalLong(iGain);
        for (int i=0; i<Module->GetNslice();i++) {
          a1=0.0;
          for (int iChan=0; iChan<4; iChan++) {
            a1+=Module->GetChannel(iChan)->GetSignalLong(iGain)->GetFFTcorrection(i);
          }
          if (i>0) {
            displayLine->DrawLine(mySignal->GetTime(i-1), a0,
                                  mySignal->GetTime(i),   a1);
          }
          a0=a1;
        }
      } //iMod<4
    } // gain loop
    
    EventCanvas->Update(); 
  } // module loop 

  int iChan=2;
  int iMod=5;
  TModule *Module = &Setup[iMod];
  TChannel *Channel = Module->GetChannel(iChan);
  int nSlice=Module->GetNslice();

  int iGain=0;
  EventCanvas->cd(13);
  gPad->SetBottomMargin(0.2);
  gPad->SetRightMargin(0.01);
  gPad->SetTopMargin(0.01);
  TSignal *Signal = Channel->GetSignalLong(iGain);
  double ama=Signal->GetMax();
  double ami=Signal->GetMin();
  double dd=ama-ami;
  dummy->GetYaxis()->SetRangeUser(ami-0.075*dd, ama+0.075*dd);
  dummy->GetYaxis()->SetTitleSize(.1);
  dummy->GetYaxis()->SetTitleOffset(.5);
  dummy->GetYaxis()->SetLabelSize(.08);
  dummy->GetXaxis()->SetTitleSize(.1);
  dummy->GetXaxis()->SetTitleOffset(.75);
  dummy->GetXaxis()->SetLabelSize(.08);
  dummy->GetXaxis()->SetTitle("Time, ns");
  dummy->SetTitle("RF");
  dummy->GetYaxis()->SetTitle("ADC");
  dummy->GetYaxis()->SetTitle("ADC");
  dummy->DrawCopy();
// show samples
  displayLine->SetLineWidth(1);
  displayLine->SetLineStyle(1);
  displayLine->SetLineColor(kBlack);
  for (int is=1; is<nSlice; is++) {
    displayLine->DrawLine(Signal->GetTime(is-1), Signal->GetAmp1(is-1), 
                          Signal->GetTime(is),   Signal->GetAmp1(is));
  }
 
  EventCanvas->cd(14);
  gPad->SetBottomMargin(0.2);
  gPad->SetRightMargin(0.01);
  gPad->SetTopMargin(0.01);
  dummy->SetTitle("RF zoomed");
  dummy->GetXaxis()->SetRangeUser(-50., 1050.);
  dummy->DrawCopy();
  displayLine->SetLineWidth(1);
  double tOff=0.;
  for (int is=0; is<nSlice; is++) {
    if (Signal->GetTime(is)-tOff>1000.) {
      while (Signal->GetTime(is)-tOff>Signal->GetTime(0)+165.) tOff+=165.;
    }
    displayLine->DrawLine(Signal->GetTime(is)-tOff,    Signal->GetAmp1(is)+0.03*dd, 
                          Signal->GetTime(is)-tOff,    Signal->GetAmp1(is)-0.03*dd);
    displayLine->DrawLine(Signal->GetTime(is)-tOff-5., Signal->GetAmp1(is), 
                          Signal->GetTime(is)-tOff+5., Signal->GetAmp1(is));
  }
   
 
  EventCanvas->Update(); 
  EventCanvas->Show(); 
  EventCanvas->Update(); 


 
  if (showMode) DisplaySpike(ProcStatus, CherBuff);


  Dbg=false;
  int Rep=ProcStatus->GetReportMode();
  //Rep=ProcStatus->WhatToDo("[DisplayAllPusles]", -100);
  //ProcStatus->SetReportMode(Rep);
  //printf("[DisplayAllPulses] #0 Rep=%d\n", Rep);
  
  if (Rep>0) {
    //EventCanvas->Close();
    //SpikeCanvas->Close();
  }
  dummy->Delete();
  displayLine->Delete();
  displayLatex->Delete();
  //printf("[DisplayAllPulses] #4 Rep=%d\n", Rep);
  EventCanvas->Update(); 
  return Rep; 
}



