void DisplayModules(TModule* Setup, ProcessingStatus* ProcStatus, TCanvas* DMcanvas) {
char TitleC[200];
double yMax=0., yMin=0., dy=0.;
int iCol=1;
int nSlice=0;
int iRF=0;


TLine  * displayLine  = new TLine(); 
TLatex * displayLatex = new TLatex();
displayLatex->SetTextSize(0.075);

EventHeader *EH = ProcStatus->GetEventHeader();
RunHeader   *RH = ProcStatus->GetRunHeader();
//BurstHeader *BH = ProcStatus->GetBurstHeader();
 
//int Nbunches    = ProcStatus->GetNbunches();
int nPedWindows = 0;
int iModX       = RH->AntennaModule;

sprintf(TitleC, "Run #%d, Burst#%d, Event#%d(%d), T(10MHz)=%.1fms Phase=%.1fns RFoffset=%d iMax=%d Type=%02d SignalFactor=%.2f", 
	  EH->run, EH->burst, EH->event, EH->burstEvent, EH->scaler10mhz/10000., EH->GetTimeGlobal(),
          (EH->scaler6mhz)%30, EH->GetFirstBunch(), EH->pattern[0], EH->SignalFactor); 
TH1D    *dummy = new TH1D("dummy_display_new", "", 2500, -150., 150.+25.*EH->nSlice);
dummy->GetXaxis()->SetTitle("Time, ns");

DMcanvas->SetTitle(TitleC);

DMcanvas->Clear();
DMcanvas->Divide(1, 6, 0.002, 0.002);
DMcanvas->Update();
DMcanvas->Show();


dummy->SetTitleSize(0.04);
dummy->SetStats(0);
dummy->GetYaxis()->SetTitleSize(.1);
dummy->GetYaxis()->SetTitleOffset(.4);
dummy->GetYaxis()->SetLabelSize(.08);
dummy->GetXaxis()->SetTitleSize(.1);
dummy->GetXaxis()->SetTitleOffset(.75);
dummy->GetXaxis()->SetLabelSize(.08);
dummy->SetLineStyle(2);

int iZone=1;
for (int iMod=0; iMod<4; iMod++) { // module loop
  if (iMod==iModX) continue;

  TModule *Module = &Setup[iMod];
  nPedWindows=Module->GetNPedWindows();
  nSlice     =Module->GetNslice();
  int iG=1;
  if (Module->CheckHGoverflow()) iG=0;

  sprintf(TitleC, "%s gain#%d", Module->GetName(), iG);
  //dummy->SetTitle(TitleC);
  DMcanvas->cd(iZone++);
  gPad->SetBottomMargin(0.2);
  gPad->SetRightMargin(0.01);
  gPad->SetTopMargin(0.02);
  gPad->SetGridy(1);
  yMax= 5.;
  yMin=-5.;
  for (int iChan=0; iChan<4; iChan++) { // find boundaries for amp1 singals
    TSignal* mySignal=Module->GetChannel(iChan)->GetSignalLong(iG);
    if (yMax<mySignal->GetMax()) yMax=mySignal->GetMax();
    if (yMin>mySignal->GetMin()) yMin=mySignal->GetMin();
  }
  dy=0.05*(yMax-yMin);
  dummy->GetYaxis()->SetRangeUser(yMin-dy, yMax+dy);
  dummy->GetYaxis()->SetTitle("MPI ADC, mV");
  dummy->DrawCopy();
  displayLatex->DrawLatex(0., yMax-dy, TitleC);

  displayLine->SetLineWidth(32); /// pedestal windows from modules 
  displayLine->SetLineStyle(1);
  displayLine->SetLineColor(kYellow);
  nPedWindows=Module->GetNPedWindows();
  double tFirst=Module->GetPedWin0(0);
  double tLast=0.;
  for (int iw=0; iw<nPedWindows; iw++) {
    displayLine->DrawLine(Setup[iMod].GetPedWin0(iw), 0.0,
                          Setup[iMod].GetPedWin1(iw), 0.0);
    tLast=Module->GetPedWin1(iw);
  }
  displayLine->SetLineWidth(1); /// RF cycles (bunch start)
  displayLine->SetLineStyle(2);
  displayLine->SetLineColor(kGray);
  iRF=0;
  for (double tc=EH->GetTimeGlobal(); tc<dummy->GetXaxis()->GetXmax(); tc+=EH->GetRFperiod()) {
    displayLine->DrawLine(tc, yMin-dy, tc, yMax+dy);
    if (iRF>0 && EH->GetBunchAction(iRF)==1 && tc>tFirst+5. && tc<tLast-5.) {
      sprintf(TitleC, "%d", EH->GetTrackCount(iRF));
      displayLatex->DrawLatex(tc+25., yMin+dy, TitleC);
    }
    iRF++;
  }

  displayLine->SetLineStyle(1);
  iCol=1;
  for (int iChan=0; iChan<4; iChan++) { // find boundaries for amp1 singals
    TSignal* mySignal=Module->GetChannel(iChan)->GetSignalLong(iG);
    displayLine->SetLineColor(iCol);
    displayLine->SetLineWidth(2);
    for (int is=1; is<nSlice; is++) {
      displayLine->DrawLine(Module->GetTime(is-1), mySignal->GetAmpI(is-1), // show with noise suppressed
                            Module->GetTime(is),   mySignal->GetAmpI(is));  // and interlaced between straight and delayed
    } 
/*   
    displayLine->SetLineWidth(1);
    int is0=mySignal->GetFirstSample(0);
    int is1=mySignal->GetLastSample(nPedWindows-1);
    if (is0==-1 || is1==-1) continue; // do not show FFT correction if it is not there
    for (int is=1; is0+is<is1; is++) {
      displayLine->DrawLine(Module->GetTime(is0+is-1), mySignal->GetTailedSignal(is-1), //show tailed
                            Module->GetTime(is0+is),   mySignal->GetTailedSignal(is));  //
    }    
*/
    iCol++;
  }
//...........................................................................
  DMcanvas->cd(iZone++);
  gPad->SetBottomMargin(0.2);
  gPad->SetRightMargin(0.01);
  gPad->SetTopMargin(0.02);
  gPad->SetGridy(1);
  yMax= 1.0/(2.-iG);
  yMin=-0.5/(2.-iG);
  for (int iChan=0; iChan<4; iChan++) { // find boundaries for FFT singals
    TSignal* mySignal=Module->GetChannel(iChan)->GetSignalLong(iG);
    if (yMax<mySignal->GetSignalFinalMax()) yMax=mySignal->GetSignalFinalMax();
    if (yMin>mySignal->GetSignalFinalMin()) yMin=mySignal->GetSignalFinalMin();
  }
  dy=0.05*(yMax-yMin);
  dummy->GetYaxis()->SetRangeUser(yMin-dy, yMax+dy);
  sprintf(TitleC, "%s gain#%d", Module->GetName(), iG);
  dummy->GetYaxis()->SetTitle("Current, #muA");
  dummy->DrawCopy();

  displayLatex->DrawLatex(0., yMax-dy, TitleC);
  
  displayLine->SetLineStyle(1);
  displayLine->SetLineWidth(32); /// pedestal windows from modules 
  displayLine->SetLineStyle(1);
  displayLine->SetLineColor(kYellow);
  nPedWindows=Setup[iMod].GetNPedWindows();
  for (int iw=0; iw<nPedWindows; iw++) {
    displayLine->DrawLine(Setup[iMod].GetPedWin0(iw), 0.0,
                          Setup[iMod].GetPedWin1(iw), 0.0);
  }

  displayLine->SetLineWidth(1); /// RF cycles (bunch start)
  displayLine->SetLineStyle(2);
  displayLine->SetLineColor(kGray);
  iRF=0;
  for (double tc=EH->GetTimeGlobal(); tc<dummy->GetXaxis()->GetXmax(); tc+=EH->GetRFperiod()) {
    displayLine->DrawLine(tc, yMin-dy, tc, yMax+dy);
    if (iRF>0 && EH->GetBunchAction(iRF)==1 && tc>tFirst+5. && tc<tLast-5.) {
      sprintf(TitleC, "%d", EH->GetTrackCount(iRF));
      displayLatex->DrawLatex(tc+25., yMin+dy, TitleC);
    }
    iRF++;
  }

  iCol=1;
  displayLine->SetLineStyle(1);
  for (int iChan=0; iChan<4; iChan++) { // find boundaries for FFT singals
    TSignal* mySignal=Module->GetChannel(iChan)->GetSignalLong(iG);
    displayLine->SetLineWidth(2);
    displayLine->SetLineColor(iCol);
    int is0=mySignal->GetFirstSample(0);
    int is1=mySignal->GetLastSample(nPedWindows-1);
    if (is0==-1 || is1==-1) {
      printf("[DisplayModules] bad boundaries is0=%d is1=%d\n", is0, is1);
      continue; // do not show FFT correction if it is not there
    }
    for (int i=1; is0+i<is1; i++) {
      displayLine->DrawLine(Module->GetTime(is0+i-1), mySignal->GetFinalSignal(i-1), 
                            Module->GetTime(is0+i),   mySignal->GetFinalSignal(i));
    }
    iCol++;
  }

} // end Module loop
DMcanvas->Update();

//-----------------------------------------------------------
displayLine ->Delete();
displayLatex->Delete();
dummy       ->Delete();
return;
}
