struct EventHeader {
  int nSlice;
  int run;
  int burst;
  int event;
  int burstEvent;
  int pattern[6];
  int phase[4];
  int scaler10mhz;
  int scaler6mhz;
  int scaler10mhzOld; // value for previous event, 0 for first event in spill
  int scaler6mhzOld;  // value for previous event, 0 for first event in spill
  char *cTime;
  time_t Time;
  double TimeGlobal;  // event phase as found by GlobalPhase finder
  double SignalFactor;// signal/noise ratio in GlobalPhase finder
  double tms; // clock in milliseconds
  double integ[4]; // integrators raw
  double integL[4]; // integrators corrected, Log10
  double integC[4]; // integrators corrected for HV
  double GatedIntegral[4];
  bool isClone;
  bool isTrain;
  double Spiky;
  bool isSpiky;
  bool isEmpty;
  bool notEmpty;
  bool inHotZone;
  double TBunch[40]; // bunch times in current event;
  double RFperiod;
  double RFfreq;
  double RFamp;
  int firstBunch;
  int nTrig;
  double EmptyCut;
  double NotEmptyCut;
  bool S123inBeam;
  int  MIPcount[50];    /// number of MIPs found in the RF bunches
  int  MIPcount2[50];    /// sqrt(S1*S2*S3/Smax)
  int  MIPcount3[50];    /// (S1*S2*S3)^(1/3)
  int  BunchAction[50]; /// 2 for pedestal, 1 for filled bunch, 0 for 'tail zone'

  int Nbunches;// how many bunches are in the event //
  double Coherent[2][2][256]; // sums of LG and HG, 2 delays for unused FCAL
  
  void SetCoherent(int iG, int iD, int i, double a) {
    Coherent[iG][iD][i]=a;
    return;
  }

  double GetCoherent(int iG, int iD, int i) {
    return Coherent[iG][iD][i];
  }

  void SetHotZone(bool hot) {inHotZone=hot;}

  void SetSignalFactor(double SF) {
    if (!nTrig) nTrig=0;
    SignalFactor=SF;
    isEmpty=false;
    notEmpty=false;
    if (S123inBeam) {
      isEmpty = (nTrig==0) || (SF<EmptyCut && nTrig<3) || (SF<NotEmptyCut && nTrig<2);
      notEmpty= (nTrig==4) || (SF>NotEmptyCut && nTrig>2);
    } else {
      //isEmpty=(SF<EmptyCut     && nTrig<2);
      isEmpty=(SF<EmptyCut);
      notEmpty=(SF>NotEmptyCut && nTrig>1) || (SF>NotEmptyCut/2 && nTrig>3);
    }
  }

  void SetRFperiod(double Trf) {
    RFfreq=4.0*asin(1.0)/Trf;
    RFperiod=Trf;
  }
  double GetRFperiod() {
    return RFperiod;
  }
  double GetRFfreq() {
    return RFfreq;
  }

  void SetRFamp(double amp) {
    RFamp=amp;
  }
  double GetRFamp() {
    return RFamp;
  }


  void SetTimeGlobal(double TG) {
    TimeGlobal=TG;
  }
  double GetTimeGlobal() {
    return TimeGlobal;
  }

  double GetSinWave(double t) { 
// returns value of RF sin wave
    return RFamp*sin((t-TimeGlobal)*RFfreq);
  }

  void SetFirstBunch(int ib) {
    if (ib<0 || ib>29) {
      printf("[EventHeader->SetFirstBunch] bunch#%d is wrong!\n", ib);
      exit(-1);
    }
    firstBunch=ib;
  }
  int GetFirstBunch() {return firstBunch;}

  void SetGatedIntegral(int iMod, double Integral) {
    if (iMod>-1 && iMod<4) {
      GatedIntegral[iMod]=Integral;
    }
  }
  double GetGatedIntegral(int iMod) {
    double result=0.;
    if (iMod>-1 && iMod<4) {
      result=GatedIntegral[iMod];
    }
    return result;
  }

void SetNtrig(int n) {
  nTrig=n;
  return;
}

void SetTrackCount(int iRF, int nTracks) { // old version
  if (iRF<0 || iRF>49) {
    printf("[EventHeader->SetTrackCount] wrong RF cycle index %d, ciao!\n", iRF);
    exit(-1);
  }
  MIPcount[iRF]=nTracks;
  return;
}

void SetTrackCount(int iRF, int nTracks, int nTracks2, int nTracks3) {
  if (iRF<0 || iRF>49) {
    printf("[EventHeader->SetTrackCount] wrong RF cycle index %d, ciao!\n", iRF);
    exit(-1);
  }
  MIPcount[iRF]=nTracks;
  MIPcount2[iRF]=nTracks2;
  MIPcount3[iRF]=nTracks3;
  return;
}



int GetTrackCount(int iRF) {
  if (iRF<0 || iRF>49) {
    printf("[EventHeader->GetTrackCount] wrong RF cycle index %d, ciao!\n", iRF);
    exit(-1);
  }
  return MIPcount[iRF];
}
int GetTrackCount2(int iRF) {
  if (iRF<0 || iRF>49) {
    printf("[EventHeader->GetTrackCount2] wrong RF cycle index %d, ciao!\n", iRF);
    exit(-1);
  }
  return MIPcount2[iRF];
}
int GetTrackCount3(int iRF) {
  if (iRF<0 || iRF>49) {
    printf("[EventHeader->GetTrackCount3] wrong RF cycle index %d, ciao!\n", iRF);
    exit(-1);
  }
  return MIPcount3[iRF];
}

void SetBunchAction(int iRF, int action) {
  if (iRF<0 || iRF>49) {
    printf("[EventHeader->SetBunchAction] wrong RF cycle index %d, ciao!\n", iRF);
    exit(-1);
  }
  BunchAction[iRF]=action;
  return;
}
int GetBunchAction(int iRF) {
  if (iRF<0 || iRF>49) {
    printf("[EventHeader->GetBunchAction] wrong RF cycle index %d, ciao!\n", iRF);
    exit(-1);
  }
  return BunchAction[iRF];
}

void Print() {
  printf("[EventHeader] Run#%d Spill#%d Event#%d(%d) \n", run, burst, event, burstEvent);
}


///////////////////////////////////////////////////////////////////////////////////
};


