int LoadBeamFitResults (TModule* Setup, EventHeader* EH, ProcessingStatus* ProcStatus) {
// loads parameters of the 'bunch' fit for beam counters
  bool Dbg1=false;

  int xRun, xBurst, xEvent, xBunchs, xChan, xGain, xFit, xib;
  double Offset,  Time,  Time1,  Peak,  Peak1,  Width,  Width1, Chi2;
  double OffsetE, TimeE, Time1E, PeakE, Peak1E, WidthE, Width1E;
  double Txxx;

  fitdata >> xRun >> xBurst >> xEvent >> xBunchs;
  if (xRun != EH->run || xBurst!=EH->burst || xEvent!=EH->event) {
    printf("[LoadBeamFitResults] event header mismatch -> \n %5d %5d %5d \n %5d %5d %5d \n ",
	    xRun, xBurst, xEvent, EH->run, EH->burst, EH->event);
    return -1;
  }
  if (Dbg1) printf("[LoadBeamFitResults] -> found Event Header Run#%d Burst#%d Event#%d Nbunches=%d\n ",
	 xRun, xBurst, xEvent, xBunchs);

  ProcStatus->SetNbunches(xBunchs);

  for (int ib=0; ib<xBunchs; ib++) {
    fitdata >> xib >> Txxx;
    Setup[4].SetTbunch(ib, Txxx);
    if (Dbg1) printf("[LoadBeamFitResults] -> Fits from bunch#%d of %d\n", ib, xBunchs);
    for (int iChan=0; iChan<4; iChan++) {
      TChannel *Channel = Setup[4].GetChannel(iChan);
      if (Channel->GetStatus()) {
        for (int iGain=0;iGain<2;iGain++) {
          TSignal *Signal = Channel->GetSignalLong(iGain);
          
	  fitdata >> xChan >> xGain >> xFit;
          if (Dbg1) printf("[LoadBeamFitResults] -> Chan#%d Gain#%d FitLevel=%d\n", xChan, xGain, xFit);
          if (xChan != iChan || xGain!=iGain) {
            printf("[LoadBeamFitResults] channel indexes mismatch -> \n %5d %5d \n %5d %5d \n ",
	    xChan, xGain, iChan, iGain);
            return -1;
	  }
	  if (xFit==0) {
	    fitdata >> Offset  >> Time  >> Peak  >> Width >> Chi2;
	    fitdata >> OffsetE >> TimeE >> PeakE >> WidthE;
            Time1=0.0; Peak1=0.0; Width1=0.0; Time1E=0.0; Peak1E=0.0; Width1E=0.0; 
	  } else if (xFit==1) {
	    fitdata >> Offset  >> Time  >> Peak  >> Width  >> Time1  >> Peak1  >> Width1 >> Chi2;
	    fitdata >> OffsetE >> TimeE >> PeakE >> WidthE >> Time1E >> Peak1E >> Width1E;
	  } else {
            printf("[LoadBeamFitResults] wrong Ifit -> %5d \n ", xFit);
            return -1;
	  }
         
          double dT=Signal->GetTriggerOffset();
	  Signal->SetIfit(ib, xFit);
	  Signal->SetFitOffset(ib,  Offset);
	  Signal->SetFitOffsetE(ib, OffsetE);
	  Signal->SetFitTime(ib,    Time+dT);
	  Signal->SetFitTimeE(ib,   TimeE);
	  Signal->SetFitTime1(ib,   Time1);
	  Signal->SetFitTime1E(ib,  Time1E);
	  Signal->SetFitPeak(ib,    Peak);
	  Signal->SetFitPeakE(ib,   PeakE);
	  Signal->SetFitPeak1(ib,   Peak1);
	  Signal->SetFitPeak1E(ib,  Peak1E);
	  Signal->SetFitWidth(ib,   Width);
	  Signal->SetFitWidthE(ib,  WidthE);
	  Signal->SetFitWidth1(ib,  Width1);
	  Signal->SetFitWidth1E(ib, Width1E);
	  Signal->SetFitChi2(ib,    Chi2);

	}
      }
    }
  }
  return xBunchs;
}

