/********************************************************************/
bool LoadReferenseData(Int_t Run, TModule* Setup) {
  bool result=true;
  char PulseFile [100];      // file to save/load standard pulse shapes
  if      (Run<81)  sprintf(PulseFile, "../pulses/pulses_nov2007.pulse1a");
  else if (Run<100) sprintf(PulseFile, "../pulses/pulses_nov2007.pulse1a");
  else              sprintf(PulseFile, "../pulses/pulses_apr2008.pulse1a");
    
  std::ifstream loadPulseRef;
  loadPulseRef.open(PulseFile);
  if (!loadPulseRef.is_open()) {
    cout << "Error opening Pulses file " <<  PulseFile << endl;
    loadPulseRef.clear();
    return false;
  } else {
   
    for (int iMod=0;iMod<6;iMod++) {  // module (see below) // no reference for Integrator yet
      TModule *myModule = &Setup[iMod];
      for (int iChan=0;iChan<4;iChan++) {
        TChannel *myChannel = myModule->GetChannel(iChan);       
        for (int iGain=0;iGain<2;iGain++) {
          TReference *myReference = myChannel->GetReference(iGain);
          int iMod1, iChan1, iGain1, nBins;
          double x0, dx;
	  result = result && loadPulseRef >> iMod1 >> iChan1 >> iGain1 >> nBins >> x0 >> dx;

	  if (iMod!=iMod1 || iChan!=iChan1 || iGain!=iGain1) {
	    printf ("       wrong indexes iMod=%d iChan=%d iGain=%d\n", iMod, iChan, iGain);
	    return false;
	  } else {
	      //printf ("       loading indexes iMod=%d iChan=%d iGain=%d nBins=%d t0=%f dT=%f\n", iMod, iChan, iGain, nBins, x0, dx);
	  }
          myReference->SetReference(nBins, x0, dx);

	  double sw=0., checksum=0., checksum0=0.0, checksum1=0.0, wsum0=0.0, wsum1=0.0;
	  double a, w, s, smem=0.0;
          for (int ib=0;ib<nBins;ib++) {
            result = result && loadPulseRef  >> a >> s >> w;
	    if (a<0.9) {
	      myReference->LoadReference(ib, a, s);
	      smem=s;
	    } else {
	      myReference->LoadReference(ib, a, smem);
	    }
	    checksum+=a;
            if (ib<100.)     {checksum0+=a; wsum0+=1.0;}
            if (ib>nBins-50) {checksum1+=a; wsum1+=1.0;}
	    x0+=dx;
	  }

          double corr=(checksum0+checksum1)/(wsum0+wsum1);

	  checksum=0.; 
          checksum0=0.0;
          checksum1=0.0; 
          wsum0=0.0; 
          wsum1=0.0;
          for (int ib=0;ib<nBins;ib++) {
            a=myReference->GetRefPoint(ib)-corr;
            s=myReference->GetRerPoint(ib);
	    myReference->LoadReference(ib, a, s);
	    checksum+=a;
            if (ib<100.)      {checksum0+=a; wsum0+=1.0;}
            if (ib>nBins-100) {checksum1+=a; wsum1+=1.0;}
	  }

          for (int ib=0;ib<nBins;ib++) {
            a=myReference->GetRefPoint(ib);
            s=myReference->GetRerPoint(ib);
            if (ib<100) {
              a*=exp((ib-100.0)/35.);
	    } else if (ib>800.) {
              a*=exp((800.-ib)/35.);
	    }
	    myReference->LoadReference(ib, a, s);
          }          


          loadPulseRef >> sw;

          if (iMod==4 && iChan==3) { // Cherenkov, no 2007 pusle shape available - use average S123
            TReference *myRef0 = myModule->GetChannel(0)->GetReference(iGain);
            TReference *myRef1 = myModule->GetChannel(1)->GetReference(iGain);
            TReference *myRef2 = myModule->GetChannel(2)->GetReference(iGain);
              
	    for (int ib=0;ib<nBins;ib++) {
              a=(myRef0->GetRefPoint(ib) + myRef1->GetRefPoint(ib) + myRef2->GetRefPoint(ib))/3.0;
              s=(myRef0->GetRerPoint(ib) + myRef1->GetRerPoint(ib) + myRef2->GetRerPoint(ib))/3.0;
	      myReference->LoadReference(ib, a, s);
            }
	  }
          for (int iLimit=0;iLimit<5;iLimit++) {
            int iLimit1;
            double al, ah;
	    result = result && loadPulseRef >> iLimit1 >> al >> ah;

 	    for (int ib=1;ib<nBins+1;ib++) {
              result = result && loadPulseRef >> a >> s >> w;
	    }
            result = result && loadPulseRef >> sw;
	  }
	}
      }
    }
  }
  loadPulseRef.close();
  if (!result) printf("[ProcessingStatus->LoadReferenseData] Run#%d --> pulse shape load failed!\n", myRun);
  return result;
}


