//--------------------------------------------------------------------------------//
bool LoadTransferFunction(Int_t iOpt, TModule* Setup) {
  bool result=false;
  char myFile[300];
  char dumpFile[300];
  int  n1, n2;
  double Te, Tc, gain, phi;
  std::ifstream fftData;
  std::ofstream fftDump;
  if (iOpt==0) { // same for all channels
//
// download Transfer function for miniFEB
//
//    sprintf(myFile, "/home/savin/HiLum/analysis2012/FFT/TransitionFunction/FFTsample00.dat");
    sprintf(myFile, "/home/savin/HiLum/analysis2012/FFT/TransitionFunction/FFTsample20121213.dat");
    fftData.open(myFile, ios::in);

    sprintf(dumpFile, "/home/savin/HiLum/analysis2012/FFT/TransitionFunction/FFTsample20121213.dump");
    fftDump.open(dumpFile, ios::out);

    if (!fftData.is_open()) { // failed to open
      printf("[Main->LoadTransferFunction] failed to open file %s\n", myFile);
      result=false;
      fftData.clear();
    } else {
      if (!fftData.good()) { // opened but bad
        printf("[Main->LoadTransferFunction] bad file %s\n", myFile);
        result=false;
        fftData.close();
      } else { // all is fine
        fftData >> Te >> Tc;
        fftData >> n1 >> n2;
        printf("[Main->LoadTransferFunction] long file size = %d Te=%f, Tc=%f\n", n2, Te, Tc);
        fftDump << Te <<" "<< Tc << endl;
        fftDump << n1 << endl;
        for (int iMod=0; iMod<4; iMod++) {
          Setup[iMod].SetFFTsize(n1);
          Setup[iMod].SetFFTtailALL(Te, Tc);
        }
        for (int i=0; i<n2; i++) {
          fftData >> gain >> phi;
          fftDump << i <<" "<< gain <<" "<<  phi << endl;
          //printf("[Main->LoadTransferFunction] %10d %f\t %f\n", i, gain, phi);
          for (int iMod=0; iMod<4; iMod++) {
            Setup[iMod].SetFFTlongALL(i, gain, phi);
          }
        }
        fftData >> n1 >> n2;
        printf("[Main->LoadTransferFunction] short file size = %d\n", n2);
        for (int i=0; i<n2; i++) {
          fftData >> gain >> phi;
          //printf("[Main->LoadTransferFunction] %10d %f\t %f\n", i, gain, phi);
          for (int iMod=0; iMod<4; iMod++) {
            Setup[iMod].SetFFTshortALL(i, gain, phi);
          }
        }   
        result = true;
        fftData.close();     
        fftData.clear();
      } // end of all is fine
    }  // end of not open
    fftDump.close();     
    fftDump.clear();

  } else if (iOpt==1) { // same for all channels, but experimental FFT from Protvino data 

    printf("Loading transfer function, derived from Protvino scope data\n");
    
    sprintf(myFile, "/home/savin/HiLum/analysis2012/FFT/TransitionFunction/FFTsample20130717tails.dat");
    fftData.open(myFile, ios::in);
    if (!fftData.is_open()) { // failed to open
      printf("[Main->LoadTransferFunction] failed to open file %s\n", myFile);
      result=false;
      fftData.clear();
      return false;
    }
    if (!fftData.good()) { // opened but bad
      printf("[Main->LoadTransferFunction] bad file %s\n", myFile);
      result=false;
      fftData.close();
      fftData.clear();
      return false;
    }
    double T1, T2, T3, T4;
    fftData >> T1 >> T2;
    fftData >> T3 >> T4;
    Te=.5*(T1+T3);
    Tc=.5*(T2+T4);
    printf("[Main->LoadTransferFunction] tail parameters loaded, Te=%f, Tc=%f\n", Te, Tc);

    fftData.close();
    fftData.clear();


    sprintf(myFile, "/home/savin/HiLum/analysis2012/FFT/TransitionFunction/ProtvinoSite201211.dat");
    fftData.open(myFile, ios::in);

    sprintf(dumpFile, "/home/savin/HiLum/analysis2012/FFT/TransitionFunction/ProtvinoSite201211.dump");
    fftDump.open(dumpFile, ios::out);
    fftDump << Te <<" "<< Tc << endl;

    if (!fftData.is_open()) { // failed to open
      printf("[Main->LoadTransferFunction] failed to open file %s\n", myFile);
      result=false;
      fftData.clear();
      return false;
    }
    if (!fftData.good()) { // opened but bad
      printf("[Main->LoadTransferFunction] bad file %s\n", myFile);
      result=false;
      fftData.close();
      fftData.clear();
      return false;
    }

    int    nFileSamples;
    double dFmy=0.04;
    n1=int(80./dFmy);
    n2=n1/2+1;
    double xFs, xdF;
    double xF0, xF1; // frequency from file
    double xG0, xG1; // gain from file
    double xP0, xP1; // phase from file
    xG0=0.;
    xP0=2.0*asin(1.);
    xG1=0.;
    xP1=0.;
    xF0=0.;
    
    fftData >> nFileSamples >> xFs >> xdF;
    printf("[Main->LoadTransferFunction] long file size n1=%d n2=%d\n", n1, n2);
    fftDump << n1 << endl;
    for (int iMod=0; iMod<4; iMod++) {
      Setup[iMod].SetFFTsize(n1);
      Setup[iMod].SetFFTtailALL(Te, Tc);
    }
    
    fftData >> xF1 >> xG1 >> xP1;
    for (int i=0; i<n2; i++) {
      double myF=i*dFmy;
      while (myF>xF1) {// load new point from file //
        xF0=xF1;
        xG0=xG1;
        xP0=xP1;
        fftData >> xF1 >> xG1 >> xP1;
      }   

      if (xF0==0.) { // use parabola interpolation for gain//
        gain=xG1*myF*myF/xF1/xF1;
      } else { //use linear interpolation //
        gain=xG0*(xF1-myF)+xG1*(myF-xF0);
        gain/=(xF1-xF0);
      }
      phi=xP0*(xF1-myF)+xP1*(myF-xF0);
      phi/=(xF1-xF0);
     
      gain*=3.0;
     
      if (i==0) {gain=0.; phi=0.;}
//      printf("[Main->LoadTransferFunction] setting TF point#%5d f=%5.2f (%5.2f...%5.2f) gain=%5.2f (%5.2f...%5.2f) phi=%5.2f (%5.2f...%5.2f)\n",
//              i, myF, xF0, xF1, gain, 3*xG0, 3*xG1, phi, xP0, xP1); 
    
      fftDump << i <<" "<< gain <<" "<<  phi << endl;

      for (int iMod=0; iMod<4; iMod++) {
        Setup[iMod].SetFFTlongALL(i, gain, phi);
        if (i<(n2-1)/2+1) {
          Setup[iMod].SetFFTshortALL(i, gain, phi);
        }
      }
    }

    fftDump.close();     
    fftDump.clear();

    result = true;
    fftData.close();     
    fftData.clear();
////////////////////////////////////////////////////////////////////////////////////////////////////////////
  } else if (iOpt==2) { // bench FFT with theoretical 'cable' instead of oversize Cdet, same for all channes 

    printf("Loading transfer function, derived from Protvino scope data\n");
    
    sprintf(myFile, "/home/savin/HiLum/analysis2012/FFT/TransitionFunction/FFTsample20130717tails.dat");
    fftData.open(myFile, ios::in);
    if (!fftData.is_open()) { // failed to open
      printf("[Main->LoadTransferFunction] failed to open file %s\n", myFile);
      result=false;
      fftData.clear();
      return false;
    }
    if (!fftData.good()) { // opened but bad
      printf("[Main->LoadTransferFunction] bad file %s\n", myFile);
      result=false;
      fftData.close();
      fftData.clear();
      return false;
    }
    double T1, T2, T3, T4;
    fftData >> T1 >> T2;
    fftData >> T3 >> T4;
    Te=.5*(T1+T3);
    Tc=.5*(T2+T4);
    printf("[Main->LoadTransferFunction] tail parameters loaded, Te=%f, Tc=%f\n", Te, Tc);

    fftData.close();
    fftData.clear();


    sprintf(myFile, "/home/savin/HiLum/analysis2012/FFT/TransitionFunction/FFTsample20130717transferCabled.dat");
    fftData.open(myFile, ios::in);

    sprintf(dumpFile, "/home/savin/HiLum/analysis2012/FFT/TransitionFunction/FFTsample20130717transferCabled.dump");
    fftDump.open(dumpFile, ios::out);
    fftDump << Te <<" "<< Tc << endl;

    if (!fftData.is_open()) { // failed to open
      printf("[Main->LoadTransferFunction] failed to open file %s\n", myFile);
      result=false;
      fftData.clear();
      return false;
    }
    if (!fftData.good()) { // opened but bad
      printf("[Main->LoadTransferFunction] bad file %s\n", myFile);
      result=false;
      fftData.close();
      fftData.clear();
      return false;
    }

    double dFmy=0.04;
    n1=int(80./dFmy);
    n2=n1/2+1;

    double xF0, xF1; // frequency from file
    double xG0, xG1; // gain from file
    double xP0, xP1; // phase from file
    xG0=0.;
    xP0=2.0*asin(1.);
    xG1=0.;
    xP1=0.;
    xF0=0.;

    int n1f, n2f;
    
    fftData >> n1f >> n2f;
    printf("[Main->LoadTransferFunction] long file size n1=%d n2=%d\n", n1, n2);
    fftDump << n1 << endl;
    for (int iMod=0; iMod<4; iMod++) {
      Setup[iMod].SetFFTsize(n1);
      Setup[iMod].SetFFTtailALL(Te, Tc);
    }
    
    fftData >> xF1 >> xG1 >> xP1;
    for (int i=0; i<n2; i++) {
      double myF=i*dFmy;
      while (myF>xF1) {// load new point from file, until jumps over the desired value //
        xF0=xF1;
        xG0=xG1;
        xP0=xP1;
        fftData >> xF1 >> xG1 >> xP1;
      }   

      if (xF0==0.) { // use parabola interpolation for gain//
        gain=xG1*myF*myF/xF1/xF1;
      } else { //use linear interpolation //
        gain=xG0*(xF1-myF)+xG1*(myF-xF0);
        gain/=(xF1-xF0);
      }
      phi=xP0*(xF1-myF)+xP1*(myF-xF0);
      phi/=(xF1-xF0);
     
      //gain*=3.0;
     
      if (i==0) {gain=0.; phi=0.;}
      printf("[Main->LoadTransferFunction] setting TF point#%5d f=%5.2f (%5.2f...%5.2f) gain=%5.2f (%5.2f...%5.2f) phi=%5.2f (%5.2f...%5.2f)\n",
              i, myF, xF0, xF1, gain, 3*xG0, 3*xG1, phi, xP0, xP1); 
    
      fftDump << i <<" "<< gain <<" "<<  phi << endl;

      for (int iMod=0; iMod<4; iMod++) {
        Setup[iMod].SetFFTlongALL(i, gain, phi);
        if (i<(n2-1)/2+1) {
          Setup[iMod].SetFFTshortALL(i, gain, phi);
        }
      }
    }

    fftDump.close();     
    fftDump.clear();

    result = true;
    fftData.close();     
    fftData.clear();
////////////////////////////////////////////////////////////////////////////////////////////////////////////
  } else if (iOpt<0) {  // Attenuator Index -iOpt
    int    nFileSamples;
    double dFmy=0.04;
    n1=int(80./dFmy);
    n2=n1/2+1;
    double xdF;
    double xF0, xF1; // frequency from file
    double xG0, xG1; // gain from file
    double xP0, xP1; // phase from file

    int iAtt=-iOpt;
    if (iAtt !=   1 && 
        iAtt !=   3 && 
        iAtt !=  10 && 
        iAtt !=  30 && 
        iAtt != 100 && 
        iAtt != 300) {
      printf("[LoadTransferFunction] wrong attenuator value iOpt=%d iAtt=%d\n", iOpt, iAtt);
      exit(-1);
    }
    //if (iAtt==1) iAtt=3; // use 1:3 TF for 1:1
    printf("[Main->LoadTransferFunction] Loading transfer function, derived from 2013 Bench data for diffwrent attenuators\n");
    char dirTF[200];
    sprintf(dirTF, "/home/savin/HiLum/analysis2012/FFT/TransitionFunction/bench2013/");

    sprintf(myFile, "/home/savin/HiLum/analysis2012/FFT/TransitionFunction/FFTsample20130717tails.dat");
    fftData.open(myFile, ios::in);
    if (!fftData.is_open()) { // failed to open
      printf("[Main->LoadTransferFunction] failed to open file %s\n", myFile);
      result=false;
      fftData.clear();
      return false;
    }
    if (!fftData.good()) { // opened but bad
      printf("[Main->LoadTransferFunction] bad file %s\n", myFile);
      result=false;
      fftData.close();
      fftData.clear();
      return false;
    }
    double T1, T2, T3, T4; // time constants for the signal tails;
    fftData >> T1 >> T2;
    fftData >> T3 >> T4;
    Te=.5*(T1+T3);
    Tc=.5*(T2+T4);
    printf("[Main->LoadTransferFunction] tail parameters loaded, Te=%f, Tc=%f\n", Te, Tc);

    fftData.close();
    fftData.clear();

    for (int iChan=0; iChan<5; iChan++) {

      sprintf(myFile, "%schan%datt%d.dat", dirTF, iChan, iAtt);
      printf("[LoadTransferFunction] file %s\n", myFile);
      fftData.open(myFile, ios::in);
      if (!fftData.is_open()) { // failed to open
        printf("[Main->LoadTransferFunction] failed to open file %s\n", myFile);
        result=false;
        fftData.clear();
        return false;
      }
      if (!fftData.good()) { // opened but bad
        printf("[Main->LoadTransferFunction] bad file %s\n", myFile);
        result=false;
        fftData.close();
        fftData.clear();
        return false;
      }

      xF0=0.;
      xG0=0.;
      xP0=2.0*asin(1.);
      xG1=0.;
      xP1=0.;
      

      int jFile=0;
      
      //printf("[Main->LoadTransferFunction] long file size n1=%d n2=%d\n", n1, n2);
      for (int iMod=0; iMod<4; iMod++) {
        Setup[iMod].SetFFTsize(iChan, n1); // iChan must be here to keep all previously loaded data //
        Setup[iMod].SetFFTtailALL(Te, Tc);
      }
    
      fftData >> nFileSamples >> xdF >> xdF;
      //printf("[Main->LoadTransferFunction]  nFileSamples=%d xdF=%f\n", nFileSamples, xdF);  
      fftData >> xG1 >> xP1;
      //printf("[Main->LoadTransferFunction]  first point Freq=%f gain=%f phase=%f\n", jFile*xdF, xG1, xP1);
      xF1=jFile*xdF;
      for (int i=0; i<n2; i++) {
        double myF=i*dFmy;
        //printf("[Main->LoadTransferFunction] start seeking in file my Freq=%f\n", myF);
        while (myF>xF1 && jFile<nFileSamples-5) {// load new point from file, until jumps over the desired value //
          jFile++;
          xF0=xF1;
          xG0=xG1;
          xP0=xP1;
          fftData >> xG1 >> xP1;
          xF1=jFile*xdF;
          //printf("[Main->LoadTransferFunction] checking jFile=%d Freq=(%f...%f)<?>%f Gain=(%f...%f), Phase=(%f...%f)\n", jFile, xF0, xF1, myF, xG0, xG1, xP0, xP1);
        }   
//        printf("push->");
//        cin >> iqq;

        if (xF0==0.) { // use parabola interpolation for gain//
          gain=xG1*myF*myF/xF1/xF1;
        } else { //use linear interpolation //
          gain=xG0*(xF1-myF)+xG1*(myF-xF0);
          gain/=(xF1-xF0);
        }
        phi=xP0*(xF1-myF)+xP1*(myF-xF0);
        phi/=(xF1-xF0);
     
        if (i==0) {gain=0.; phi=0.;}

//        if (iChan==0) {
          //printf("[Main->LoadTransferFunction] setting TF point#%5d f=%5.2f (%5.2f...%5.2f) gain=%5.2f (%5.2f...%5.2f) phi=%5.2f (%5.2f...%5.2f)\n",
          //      i, myF, xF0, xF1, gain, xG0, xG1, phi, xP0, xP1); 
//        }
        gain/=(double)iAtt; // to have TF conversion in mV->uA
        for (int iMod=0; iMod<4; iMod++) {
          Setup[iMod].SetFFTlong(iChan, i, gain, phi);
          if (i<(n2-1)/2+1) {
            Setup[iMod].SetFFTshort(iChan, i, gain, phi);
          }
        }
      } // loop over wanted points  0 to n2

      fftData.close();
      fftData.clear();
        
    } // end of channel looper
    
  } else {
    printf("[Main->LoadTransferFunction] option#%d not implemented\n", iOpt);
    result=false;
    exit(-1);
  }

//
// download Transfer function for PMT shaper
//
  sprintf(myFile, "/home/savin/HiLum/analysis2012/FFT/TransitionFunctionPMT/hf42.dat");
  fftData.open(myFile, ios::in);
  if (!fftData.is_open()) { // failed to open
    printf("[Main->LoadTransferFunction] failed to open file %s\n", myFile);
    result=false;
    fftData.clear();
    return result;
  }
    
  if (!fftData.good()) { // opened but bad
    printf("[Main->LoadTransferFunction] bad file %s\n", myFile);
    result=false;
    fftData.close();
    return result;
  }

  double df;
  double fmax;
  fftData >> df >> fmax;
  n1=1+(int)(fmax/df);
  printf("[Main->LoadTransferFunction] PMT dF=%f Fmax=%f n=%d\n", df, fmax, n1);
  for (int iMod=4; iMod<6; iMod++) {
    Setup[iMod].SetTransferSizeALL(n1, df);
  }
  for (int i=0; i<n1; i++) {
    int j=0;
    fftData >> j >> gain >> phi;
      //printf("[Main->LoadTransferFunction] %10d %f\t %f\n", i, gain, phi);
    for (int iMod=4; iMod<6; iMod++) {
      Setup[iMod].SetTransferPointALL(i, gain, phi);
    }
  }
  result = true;
  fftData.close();     
  fftData.clear();
  
  for (int iMod=4; iMod<6; iMod++) {
    Setup[iMod].ConvertTransferFuction(); // converts transfer function to 
                                            //the signal lenght (ns2013)
  }
              
  return result;
}


