void SetModuleTiming(Int_t Run, TModule* Setup) {

  std::ifstream timingData;
  char FileName[100];
  sprintf(FileName, "../timing/Timing20090617_run%d.dat", Run);
  timingData.open(FileName, ios::in);
  if (!timingData.is_open()) {
    timingData.clear();  
    printf("[SetModuleTiming] timing file %s not found\n", FileName);
    if (Run>644) {
      sprintf(FileName, "../timing/Timing20090617_run001.dat");
      printf("[SetModuleTiming] trying default settings %s\n", FileName);
    } else {
      sprintf(FileName, "../timing/Timing20090617_run000.dat");
      printf("[SetModuleTiming] trying default settings %s\n", FileName);
    }
    timingData.open(FileName, ios::in);
    if (!timingData.is_open()) {
      timingData.clear();  
      printf("[SetModuleTiming] default timing file %s not found\n", FileName);
      exit(-1);
    }
  }

  for (int iMod=0; iMod<6; iMod++) {
    for (int iChan=0; iChan<4; iChan++) {
      TChannel *Channel = Setup[iMod].GetChannel(iChan);
      for (int iGain=0; iGain<2; iGain++) {
	TSignal *SignalShort0 = Channel -> GetSignalShort(2*iGain+0); //iDelay=0
	TSignal *SignalShort1 = Channel -> GetSignalShort(2*iGain+1); //iDelay=1
	TSignal *SignalLong   = Channel -> GetSignalLong(iGain);      //combined
        int im, ich, ig, id;
        double dT0, w0, err0;
        timingData >> im >> ich >> ig >> id >> dT0 >> w0 >> err0;
	SignalShort0->SetTriggerOffset(dT0);
	SignalShort0->SetTriggerOffsetW(w0);
	SignalShort0->SetTriggerOffsetE(err0);
        double dT1, w1, err1;
        timingData >> im >> ich >> ig >> id >> dT1 >> w1 >> err1;
	SignalShort1->SetTriggerOffset(dT1);
	SignalShort1->SetTriggerOffsetW(w1);
	SignalShort1->SetTriggerOffsetE(err1);
        double sT=0.0, sW=0.0;
        if (w0>10.) {
	  sT+=w0*dT0;
	  sW+=w0;
	}
        if (w1>10.) {
	  sT+=w1*dT1;
	  sW+=w1;
	}
        if (sW==0.0) {
          SignalLong -> SetTriggerOffset(0.5*(dT0+dT1));
	} else {
          SignalLong -> SetTriggerOffset(sT/sW);
	}
      }
    }
  }

  timingData.close();
  printf("[SetModuleTiming] Timing alignment file %s loaded\n", FileName);
  return;

}

