//#include <classes/TSignal.C>

class TChannel {
private:
  char Name[20];
  int id;
  int iChan;
  int iMod;
  int NsliceShort;
  int NsliceLong;
  bool Status;
  bool dumpIt;
  bool ChannelCanvasShown;

  double ChannelPosition[2]; // X,Y for channel centroid;

  //TCanvas *LocalCanvas;
  //TLine *dummyLine;
  //TH1F   *dummySignal;

  double PedWindow0[25], PedWindow1[25]; // pedestal windows, use for Pedestal calculation;
  int  NPedWindows;                      // total number of windows
  int  NGoodWindows;                     // total number of good windows in all channels
  bool GoodPedWindow[25];                // flags useful pedestal windows
  TSignal *SignalShort;
  TSignal *SignalLong;
  TSignal *SignalDummy;
  TReference *Reference;
  int ResponseStatus[50];  // response status per bunch -10 failed fit for all gains
                           //                           -2 low gain overflow
                           //                            0 low gain used
                           //                            1 high gain used
                           //                           10 low gain used, multiple peaks
                           //                           11 high gain used, multiple peaks
                           //                           12 low gain overflow (>HighLowSwitch)
  double Response[50];     // response for bunch, ADC
  double ResponseCal[50];  // response for bunch, corrected for calibration and attenuator
  double Response8[50];     // response for bunch, ADC
  double Response8cal[50];  // response for bunch, corrected for calibration and attenuator
  double BunchNoise[50];
  double Time[50];         // time for bunch from fit
  double Txxx[50];         // time for bunch from bunch finder
  double high[50];  // high and low gain response
  double low[50];
  double highCal[50];  // high and low gain response, corrected for attenuators 
  double lowCal[50];
  double high8[50];  // high and low gain response, sum8
  double low8[50];
  double high8cal[50];  // high and low gain response, sum8, corrected for attenuators 
  double low8cal[50];
  double chi2l[25], chi2h[25]; // fit quality
  double Atail[2][25], Ttail[2][25], chi2tail[2][25]; // for PedTailCorrection() [gain][PedWindow]

  double HighLow;
  double HighLowSwitch;
  double HighLowSwitch2013;
  double gainCorrection;
  int highStatus[25];
  int lowStatus[25];

  bool HighLowAlert[25];
  bool HGoverflowBunch[25];
  double SignalFactor; // derivative of signal/noise, used to 

//-- this section is a guide to High/Low crisis correction --//  
  double HLrc;     // RC for 'integrator'
  double HLrate;   // max Log10(rate) to use high gain 
  double HLeff;    // cut efficiensy
  double HLsigma;  // spread of high/low ratio, 10 3sigma iterations  
  double HLmean;   // mean high/low (peak, actually) after 10 3sigma iterations
  double HLcut;    // == HLmean-3.0*HLsigma;
  int    HLindex;  // index of correspondingtime constant in the RCtime 
  double rateLog10; // current bunch rate integrated by RCtime[HLindex];
  bool   HiLoCrisisFlag; // set for a channel even if a single bunch has hi/lo crisis condition 
  bool HGoverflow;
  bool LGoverflow;
  int  HLtotStat;

  double delayPMTmean; // additional alignment for PMT channels form HiLum2013mip0607.C and its Summary
  double delayPMTpeak; // located at reports/MIP/timing/

  double dip0pmt;    // preliminary limits for single track cut: dip-before, peak, dip-after
  double dip1pmt;    // PMT FFT units!
  double mip0pmt;

  double clusterPMTbunchAmp2[50][2]; // PMT-reconstructed cluster sum for the bunch, different gains
  double clusterPMTbunchAmp[50]; // PMT-reconstructed cluster sum for the bunch, optimal gain
  bool   clusterPMTbunchMIP[50]; // checks if current RF bunch is a MIP-like amplitude
  bool   clusterPMTbunchSET[50]; // checks if current RF bunch signal was computed
  bool   clusterPMTgain[50];     // gain to be used for bunch in this channel
// 2011 modified High/Low crisis handling
  double rateBin[200];
  int    statBin[200];
  double meanBin[200];
  double rmsUP[200];
  double rmsDW[200];
  double rateCut[200];
  double rateBin1;
  double rateBin2;
  int    rateBins; // shall be less than 200!
//--
//-- section with rate information --//
  double myRC;
  double myRateCut;
  int myIrc;
  bool rateCutSet;
  bool rateCutPassed[25];
//--
  double t1pmtRec;// time interval where PMT signal was reconstructed
  double t0pmtRec;


  ProcessingStatus* ProcStatus;

  TH1D *dummy1d;
  TH1D *FullEventSpectrum;
  TH1D *FullEventIntegral;
  TH1D *PedestalSpectrum;
  TH1D *highTOlow;
  TH2F *highVSlow;
  TH1D *signalFactor;   

  TH2F *RatioToCher;
  TH2F *RatioPlusMinus;
  TH2F *Tplus;
  TH2F *Tminus;
  
  bool Dbg;

public:
  TChannel() { 
    Status=false; 
    rateCutSet=false;
    dumpIt=false;
    for (int i=0; i<25; i++) rateCutPassed[i]=false;
    HighLow=10.; 
    id=-1;
    SignalShort = new TSignal[4];    // 2*Gain+Delay
    SignalLong  = new TSignal[2];    // Gain
    SignalDummy = new TSignal;
    Reference   = new TReference[2]; // Gain
    NPedWindows=0;
    NGoodWindows=0;
    gainCorrection=1.0;
    RatioToCher    = new TH2F[4];
    RatioPlusMinus = new TH2F[4];
    Tplus          = new TH2F[4];
    Tminus         = new TH2F[4];
    Dbg = false;
    ChannelCanvasShown=true;
    SetHLcorrection(0., 0., 0, 0., 0., 0.);
    HiLoCrisisFlag=false;
    HGoverflow=false;
    LGoverflow=false;
    SignalFactor=0.;

    t1pmtRec=0.;
    t0pmtRec=0.;

    delayPMTmean=23.;
    delayPMTpeak=23.;

    dip0pmt=0.;
    mip0pmt=0.;
    dip1pmt=0.;

    ChannelPosition[0]=0.;
    ChannelPosition[1]=0.;

 
    for (int i=0; i<50; i++) {
      clusterPMTbunchAmp[i]=0.0;
    }
  //LocalCanvas = new TCanvas();
  //LocalCanvas -> SetWindowSize(1200, 800);
  //LocalCanvas -> Iconify();    
  //dummyLine  = new TLine();
  //dummySignal = new TH1F();
  }

void SetPMTdelay(double t1, double t2) {
  delayPMTmean=t1;
  delayPMTpeak=t2;
  //printf("[TSignal->SetPMTdelay] %s mean=%.1fns peak=%.fns\n", Name, delayPMTmean, delayPMTpeak);
  return;
}

double GetPMTdelayMean() {
//printf("[TSignal->GetPMTdelayMean] %s mean=%.1fns peak=%.fns\n", Name, delayPMTmean, delayPMTpeak);
return delayPMTmean;
}

double GetPMTdelayPeak() {
//printf("[TSignal->GetPMTdelayPeak] %s mean=%.1fns peak=%.fns\n", Name, delayPMTmean, delayPMTpeak);
return delayPMTpeak;
}

void SetMIP2cuts(double dip0, double mip, double dip1) {
  dip0pmt=dip0;
  mip0pmt=mip;
  dip1pmt=dip1;
}

double GetMIPdip0() {return dip0pmt;}
double GetMIPdip1() {return dip1pmt;}
double GetMIPmip0() {return mip0pmt;}

int GetClusterPMTgain(int i) {
  if (i<0 || i>49) {
    printf("[TChannel->GetClusterPMTgain] wrong index %d\n", i);
    exit(-1);
  }
  return clusterPMTgain[i];
}

bool GetClusterPMTbunchSET(int i) {
  if (i<0 || i>49) {
    printf("[TChannel->GetClusterPMTbunchSET] wrong index %d\n", i);
    exit(-1);
  }
  return clusterPMTbunchSET[i];
}

bool GetClusterPMTbunchMIP(int i) {
  if (i<0 || i>49) {
    printf("[TChannel->GetClusterPMTbunchMIP] wrong index %d\n", i);
    exit(-1);
  }
  if (!clusterPMTbunchSET[i]) {
    printf("[TChannel->GetclusterPMTbunchMIP] %s not set for bunch#%d\n", Name, i);
    exit(-1);    
  }
  return clusterPMTbunchMIP[i];
}

double GetClusterPMTbunchAmp(int i) {
  if (i<0 || i>49) {
    printf("[TChannel->GetclusterPMTbunchAmp] wrong index %d\n", i);
    exit(-1);
  }
  if ( clusterPMTbunchSET[i]) {
    return clusterPMTbunchAmp[i];
  } else {
    printf("[TChannel->GetclusterPMTbunchAmp] %s not set for bunch#%d\n", Name, i);
    exit(-1);    
  }
  return 0.;
}


double GetClusterPMTbunchAmp2(int i, int iGain) {
  if (i<0 || i>49 || iGain<0 || iGain>1) {
    printf("[TChannel->GetclusterPMTbunchAmp2] wrong index#%d or gain#%d\n", i, iGain);
    exit(-1);
  }
  if ( clusterPMTbunchSET[i]) {
    return clusterPMTbunchAmp2[i][iGain];
  } else {
    printf("[TChannel->GetclusterPMTbunchAmp2] %s not set for bunch#%d\n", Name, i);
    exit(-1);    
  }
  return 0.;
}



double GetClusterPMTbunchAmpTracks(int i) {
  if (i<0 || i>49) {
    printf("[TChannel->GetclusterPMTbunchAmpTracks] wrong index %d\n", i);
    exit(-1);
  }
  if ( clusterPMTbunchSET[i]) {
    return clusterPMTbunchAmp[i]*gainCorrection;
  } else {
    printf("[TChannel->GetclusterPMTbunchAmpTracks] %s not set for bunch#%d\n", Name, i);
    exit(-1);    
  }
  return 0.;
}




void ComputePMTclusters() {
  double T00=ProcStatus->GetEventHeader()->GetTimeGlobal();
  double Trf=ProcStatus->GetEventHeader()->GetRFperiod();
  double tRFb=0.0;
  
  int myGain=1;
  if (SignalShort[2*myGain+0].OverflowDetected(HighLowSwitch) ||
      SignalShort[2*myGain+1].OverflowDetected(HighLowSwitch) ) {
    myGain=0;
//    printf("[Channel->ComputePMTclusters] gain set to LOW for %s \n", Name);
//    printf("[Channel->ComputePMTclusters] %s over=%d\n", 
//           SignalShort[2*myGain+0].GetName(), SignalShort[2*myGain+0].OverflowDetected(10000.));
//    printf("[Channel->ComputePMTclusters] %s over=%d\n", 
//           SignalShort[2*myGain+1].GetName(), SignalShort[2*myGain+1].OverflowDetected(10000.));
  } else if (SignalShort[0].GetMax0()>200. || 
             SignalShort[1].GetMax0()>200.) {
    myGain=0;
  } 

  SignalLong[0].ClearClusters2013();
  SignalLong[1].ClearClusters2013();
  for (int irf=0; irf<50; irf++) {
    clusterPMTbunchSET[irf]=false;
    clusterPMTbunchAmp2[irf][0]=0.;
    clusterPMTbunchAmp2[irf][1]=0.;
    clusterPMTbunchAmp[irf]=0.;
  }
  
  double tWin0=0.5*(PedWindow0[0]+PedWindow1[0]);
  double tWin1=0.5*(PedWindow0[NPedWindows-1]+PedWindow0[NPedWindows-1]);

  if (ProcStatus->GetReportMode()==1) {
    printf("[TChannel->ComputePMTclusters] %10s High/Low=%.2f\n", Name, HighLow);
  }

  for (int irf=0; irf<50 && tRFb<6200; irf++) {
    tRFb=T00+irf*Trf;
    if (tRFb<tWin0) continue;
    if (tRFb>tWin1) continue;

    clusterPMTbunchAmp2[irf][0]=SignalLong[0].ComputePMTcluster(tRFb+delayPMTpeak, irf);
    clusterPMTbunchAmp2[irf][1]=SignalLong[1].ComputePMTcluster(tRFb+delayPMTpeak, irf);
    SignalLong[0].ComputePMTclusterT01(tRFb+delayPMTpeak, irf);
    SignalLong[1].ComputePMTclusterT01(tRFb+delayPMTpeak, irf);
    
    high[irf]=clusterPMTbunchAmp2[irf][1];
    low[irf] =clusterPMTbunchAmp2[irf][0];
    //if (ProcStatus->GetReportMode()==1) printf("[Channel->ComputePMTclusters]              bunch#%3d low=%6.1f \t high=%6.1f\n", irf, low[irf], high[irf]);
    clusterPMTbunchAmp2[irf][0]*=HighLow;
    clusterPMTbunchAmp[irf]=clusterPMTbunchAmp2[irf][myGain];

    clusterPMTbunchSET[irf]=true;
    clusterPMTgain[irf]=myGain;
    if (ProcStatus->GetRunHeader()->S123inBeam) {
      clusterPMTbunchMIP[irf] = (clusterPMTbunchAmp[irf]<dip1pmt &&
                                 clusterPMTbunchAmp[irf]>dip0pmt);
    } else {
      clusterPMTbunchMIP[irf] = false;
    }
  }
}

void SetChannelPosition(double x, double y) {
  ChannelPosition[0]=x;
  ChannelPosition[1]=y;
}

double GetChannelPositionX() {return ChannelPosition[0];}
double GetChannelPositionY() {return ChannelPosition[1];}

  void ClearHiLoCrisisFlag() {HiLoCrisisFlag=false;}
  void SetHiLoCrisisFlag()   {HiLoCrisisFlag=true;}
  bool CheckHiLoCrisisFlag() {return HiLoCrisisFlag;}
  bool   GetDumpIt()    {return dumpIt;}
  void   SetDumpIt(bool b){
    printf("[TChannel->SetDumpIt] dump mode=%d set for %s\n", b, Name);
    dumpIt=b;
  }

  void SetGainCorrection(double gc) {gainCorrection=gc;}

  void SetProcStatus(ProcessingStatus *PS) {
    ProcStatus=PS;
    //printf("[TChannel]   setting ProcessingStatus object for %s\n", Name);
    for (int iGain=0; iGain<2; iGain++) {
      for (int iDelay=0; iDelay<2; iDelay++) {
        SignalShort[2*iGain+iDelay].SetProcStatus(ProcStatus);
      }
      SignalLong[iGain].SetProcStatus(ProcStatus);
    }
    SignalDummy->SetProcStatus(ProcStatus);
  }



  TH1D* GetSignalFactorTH1D() {
    return signalFactor;
  }
  void FillSignalFactorTH1D(double a) {
    signalFactor->Fill(a);
    return;
  }
  void ResetSignalFactorTH1D() {
    signalFactor->Reset();
    return;
  }

  void   SetSignalFactor(double f) {SignalFactor=f;}
  double GetSignalFactor()         {return SignalFactor;}


  TH1D* GetHighToLow() {return highTOlow;}
  TH2F* GetHighVsLow() {return highVSlow;}
  TH1D* GetFullEventSpectrum()  {return FullEventSpectrum;}
  TH1D* GetFullEventIntegral()  {return FullEventIntegral;}
  TH1D* GetPedestalSpectrum()   {return PedestalSpectrum;}
  
  void ResetFullEventSpectrum() {FullEventSpectrum->Reset();}
  void ResetPedestalSpectrum()  {PedestalSpectrum->Reset();}
  void ResetHighToLow()         {highTOlow->Reset();}
  void ResetHighVsLow()         {highVSlow->Reset();}

  TH2F* GetRatioToCher(int iChI)    {
    // printf("[GetRatioToCher] Integrator#%d requesting histogram %s from %s\n", iChI, RatioToCher[iChI].GetTitle(), Name);
    return &((TChannel*)this)->RatioToCher[iChI];
  }
  TH2F* GetRatioPlusMinus(int iChI) {return &((TChannel*)this)->RatioPlusMinus[iChI];}
  TH2F* GetTplus(int iChI)          {return &((TChannel*)this)->Tplus[iChI];}
  TH2F* GetTminus(int iChI)         {return &((TChannel*)this)->Tminus[iChI];}

  void SetModule(int Module) {
    iMod=Module;
    SignalDummy->SetModule(iMod);
    SignalDummy->SetGain(-1);
    SignalDummy->SetDelay(-1);
    SignalDummy->On();
    for (int ig=0;ig<2;ig++) {
      SignalLong[ig].SetModule(iMod);
      SignalLong[ig].SetGain(ig);
      SignalLong[ig].SetDelay(-1);
      SignalLong[ig].AssignReference(&Reference[ig]);
      if (iMod!=6) SignalLong[ig].On(); // no long signal for Integrator
      for (int idl=0;idl<2;idl++) {
        SignalShort[2*ig+idl].SetModule(iMod);
        SignalShort[2*ig+idl].SetGain(ig);
        SignalShort[2*ig+idl].SetDelay(idl);
        SignalShort[2*ig+idl].AssignReference(&Reference[ig]);
        if (iMod!=6 || (ig+idl==0)) SignalShort[2*ig+idl].On(); // only (0,0) signal for Integrator
        if (iMod==4 || iMod==5) { // set correction for delayed signal 1.0/0.965
          if (idl==0) SignalShort[2*ig+idl].SetSignalCorrection(0.965);
        }
      }
    }
  }
  int GetModule() {return iMod;}

  void   SetHighLowSwitch(double HL) {HighLowSwitch=HL;}
  double GetHighLowSwitch()          {return HighLowSwitch;}
  void   SetHighLowSwitch2013(double HL) {HighLowSwitch2013=HL;}
  double GetHighLowSwitch2013()          {return HighLowSwitch2013;}

  void SetChannel(int Channel) {
    iChan=Channel;
    SignalDummy->SetChannel(iChan);
    for (int i=0;i<4;i++) SignalShort[i].SetChannel(iChan);
    SignalLong[0].SetChannel(iChan);
    SignalLong[1].SetChannel(iChan);
//    HighLowSwitch=100000.; // dummy value
    if (iMod<4)  {HighLowSwitch=2000.; HighLowSwitch2013=1.0e10;}
    if (iMod==4) {HighLowSwitch=2000.; HighLowSwitch2013=20000.;}
    if (iMod==4 && iChan==3) {HighLowSwitch=2300.; HighLowSwitch2013=25000.;}
  }

double GetT0pmtRec() {return t0pmtRec;}
double GetT1pmtRec() {return t1pmtRec;}

void FindPMTsignalPeaks() {
for (int iGain=0; iGain<2; iGain++) {
  SignalLong[iGain].FindPMTsignalPeaks();
}
return;  
}
//==============================================================================
void RecoverPMTsignal() {
// special analysis for PMT chanell, using artificial 2.5ns sampling
//   and 'fast' shaper tramsfer function
    //EventHeader *EH = ProcStatus->GetEventHeader();
    double DbgL=false;
    DbgL=true;
    //SignalLong[0].RecoverPMTsignal();
    //SignalLong[1].RecoverPMTsignal();

//    if (DbgL && iMod==4) {
//      char TitleC[200];
//      sprintf(TitleC,"Run#%d Spill#%d Evt#%d(%d)  %s", EH->run, EH->burst, EH->event, EH->burstEvent, Name);
//      LocalCanvas->SetTitle(TitleC);
//      LocalCanvas->Clear();
//      LocalCanvas->Show();
//      LocalCanvas->Divide(1,2);
//    }

    t1pmtRec=0.;
    t0pmtRec=0.;

    for (int irf=0; irf<50; irf++) clusterPMTbunchSET[irf]=false;

    for (int iGain=0; iGain<2; iGain++) {
      SignalShort[2*iGain].RecoverPMTsignal();
      SignalShort[2*iGain+1].RecoverPMTsignal();
      int ns=SignalLong[0].GetNS2013();

      double t0[2]={0., 0.};
      double dt=0.;
      for (int iDelay=0; iDelay<2; iDelay++) t0[iDelay]=SignalShort[2*iGain+iDelay].GetTime2013(0);
      dt=SignalShort[2*iGain].GetTime2013(1)-SignalShort[2*iGain].GetTime2013(0);
      
      t0pmtRec+=SignalShort[2*iGain].GetT0pmtRec();
      t0pmtRec+=SignalShort[2*iGain+1].GetT0pmtRec();
      t1pmtRec+=SignalShort[2*iGain].GetT1pmtRec();
      t1pmtRec+=SignalShort[2*iGain+1].GetT1pmtRec();

      for (int i=0; i<ns; i++) {
        double tx=SignalLong[iGain].GetTime2013(i);
        int i0 = int((tx-t0[0])/dt);
        if (i0<0) i0=0;
        if (i0>ns-1) i0=ns-1;
        int i1 = int((tx-t0[1])/dt);
        if (i1<0) i1=0;
        if (i1>ns-1) i1=ns-1;
        //double tx1=0.5*(SignalShort[2*iGain].GetTime2013(i0)+SignalShort[2*iGain+1].GetTime2013(i1));
        double ax1=SignalShort[2*iGain].GetAmp2013rec(i0)+SignalShort[2*iGain+1].GetAmp2013rec(i1);
        SignalLong[iGain].SetAmp2013rec(i, ax1);
      }
      
    
/*
      if (DbgL && iMod==4) {
        char TitleC[200];
        sprintf(TitleC,"Run#%d Spill#%d Evt#%d(%d)  %s gain#%d", EH->run, EH->burst, EH->event, EH->burstEvent, Name, iGain);
        LocalCanvas->SetTitle(TitleC);
        dummySignal->SetNameTitle(Name, TitleC);
        dummySignal->SetBins(2000, SignalShort[2*iGain].GetTime2013(0)-10., 
                                   SignalShort[2*iGain+1].GetTime2013(ns-1)+10.);
 
        double scintMax=SignalLong[iGain].GetMax0();
        double scintMin=SignalLong[iGain].GetMin0();
        double dd=0.1*(scintMax-scintMin);
        dummySignal->GetYaxis()->SetRangeUser(scintMin-dd, scintMax+dd);

        LocalCanvas->cd(iGain+1);
        dummySignal->DrawCopy();

        dummyLine->SetLineWidth(32); /// pedestal windows from modules 
        dummyLine->SetLineStyle(1);
        dummyLine->SetLineColor(kYellow);
        for (int iw=0; iw<NPedWindows; iw++) {
          dummyLine->DrawLine(PedWindow0[iw], 0.0,
                              PedWindow1[iw], 0.0);
        }

        dummyLine->SetLineColor(kBlack);
        dummyLine->SetLineWidth(1);
        dummyLine->SetLineStyle(2);
        dummyLine->DrawLine(SignalShort[2*iGain].GetTime2013(0)-10.,       SignalLong[iGain].GetEmptyEventNoise(),
                            SignalShort[2*iGain+1].GetTime2013(ns-1)+10.,  SignalLong[iGain].GetEmptyEventNoise()); 
        dummyLine->DrawLine(SignalShort[2*iGain].GetTime2013(0)-10.,      -SignalLong[iGain].GetEmptyEventNoise(),
                            SignalShort[2*iGain+1].GetTime2013(ns-1)+10., -SignalLong[iGain].GetEmptyEventNoise()); 

        dummyLine->SetLineWidth(1); /// RF cycles (bunch start)
        dummyLine->SetLineStyle(2);
        dummyLine->SetLineColor(kGray);
        for (double tc=EH->GetTimeGlobal(); tc<dummySignal->GetXaxis()->GetXmax(); tc+=EH->GetRFperiod()) {
          dummyLine->DrawLine(tc, scintMin, tc, scintMax);
        }


        dummyLine->SetLineColor(kBlack);
        dummyLine->SetLineStyle(1);
        dummyLine->SetLineWidth(4);
        for (int i=1; i<SignalLong[iGain].GetNslice(); i++) {
          dummyLine->DrawLine(SignalLong[iGain].GetTime(i-1), SignalLong[iGain].GetAmp3(i-1), 
                              SignalLong[iGain].GetTime(i),   SignalLong[iGain].GetAmp3(i));
        }

        dummyLine->SetLineColor(kBlue);
        dummyLine->SetLineWidth(2);
        for (int i=1; i<ns; i++) {
          dummyLine->DrawLine(SignalLong[iGain].GetTime2013(i-1), SignalLong[iGain].GetAmp2013rec(i-1), 
                              SignalLong[iGain].GetTime2013(i),   SignalLong[iGain].GetAmp2013rec(i));
        }
        LocalCanvas->Update();
        
      } // show!
*/
  } // gain loop   
  t0pmtRec/=4.;
  t1pmtRec/=4.;
  SignalLong[0].SetT0pmtRec(t0pmtRec);
  SignalLong[1].SetT0pmtRec(t0pmtRec);
  SignalLong[0].SetT1pmtRec(t1pmtRec);
  SignalLong[1].SetT1pmtRec(t1pmtRec);
//==============================================================================================
}

  void SetFFTsize(int nSampleTail125) {
    int nShort=nSampleTail125/2+1;
    for (int i=0;i<4;i++) SignalShort[i].SetFFTsize(nShort);
    SignalLong[0].SetFFTsize(nSampleTail125);
    SignalLong[1].SetFFTsize(nSampleTail125);
  }

  int GetFFTsize() {
    return SignalLong[0].GetFFTsize();
  } 

  void SetFFTlongALL(int i, double gain, double phi) {
    if (iMod==1 && iChan==0 && i<10)printf("[TChannel->SetFFTlongALL] %15s -> %6d=(%f\t %f)\n",
           Name, i, gain, phi); 
    SignalLong[0].SetTransferFunction(i, gain, phi);
    SignalLong[1].SetTransferFunction(i, gain, phi);
  }
  void SetFFTshortALL(int i, double gain, double phi) {
    for (int is=0;is<4;is++) SignalShort[is].SetTransferFunction(i, gain, phi);
  }
  void SetFFTtailALL(double Te, double Tc) {
    for (int is=0;is<4;is++) SignalShort[is].SetFFTtail(Te, Tc);
    SignalLong[0].SetFFTtail(Te, Tc);
    SignalLong[1].SetFFTtail(Te, Tc);
  }  
  
  void SetTransferSizeALL(int n1, double df) {
    for (int is=0; is<4; is++) { 
      SignalShort[is].SetTransferSize(n1, df);
    }
    SignalLong[0].SetTransferSize(n1, df);        
    SignalLong[1].SetTransferSize(n1, df);     
  }

  void SetTransferPointALL(int i, double gain, double phi) {
    for (int is=0; is<4; is++) { 
      SignalShort[is].SetTransferPoint(i, gain, phi);
    }  
    SignalLong[0].SetTransferPoint(i, gain, phi);      
    SignalLong[1].SetTransferPoint(i, gain, phi);      
  }

  void ConvertTransferFuction() {
    for (int is=0; is<4; is++) { 
      SignalShort[is].ConvertTransferFuction();
    }  
    SignalLong[0].ConvertTransferFuction();      
    SignalLong[1].ConvertTransferFuction();      
  }

  void SetHLcorrection(double mean, double sigma, int irc, double rc, double rate, double eff) {
    HLrc=rc;
    HLrate=rate;
    HLeff=eff;
    HLmean=mean;
    HLsigma=sigma;
    HLindex=irc;
    HLcut=mean-1.5*sigma;
    return;
  }

  void SetHLcorrection2011(double mean, double sigma, int nSt, int irc, double rc, double rate, 
                           double eff, double bin1, double bin2, int nbins) {
    // same as in old version
    HLtotStat=nSt;
    HLrc=rc;
    HLrate=rate;
    HLeff=eff;
    HLmean=mean;
    HLsigma=sigma;
    HLindex=irc;
    HLcut=mean-1.5*sigma;
    // new - prepare the binned cuts!
    rateBin1=bin1;
    rateBin2=bin2;
    rateBins=nbins;
    return;
  }

  void SetHLcorrection2011bin(int ib1, double xb, int nstat, double mean, double rmsUp, double rmsDw) {
    int ib=ib1;
    rateBin[ib]=xb;
    statBin[ib]=nstat;
    meanBin[ib]=mean;
    rmsUP[ib]=rmsUp;
    rmsDW[ib]=rmsDw;
    rateCut[ib]=mean-3.0*rmsUp;
    return;
  }
  
  
  int GetHLindex() {return HLindex;}

  void SetPreampCut(int iRC, double RC, double Cut) {
    myIrc=iRC;
    myRC=RC;
    //myRateCut = Cut;
    myRateCut = exp(log(10.0)*Cut);
    rateCutSet=true;
    for (int i=0; i<25; i++) rateCutPassed[i]=false;
  }


  bool CheckRateCut(int bunch, double Rate) {
    if (!rateCutSet) {
      printf("[TChannel->CheckRateCut] no rate cut set for %s\n", Name);
      rateCutPassed[bunch]=false;
      return false;
    }
    rateCutPassed[bunch]=(Rate<myRateCut);
    return rateCutPassed[bunch];
  }

  bool GetRateCutPassed(int bunch) {
    return rateCutPassed[bunch];
  }

  bool GetRateCutSet()  {return rateCutSet;}
  int  GetRateCutIRC()  {return myIrc;}
  double GetRateCutRC() {return myRC;}
  double GetRateCut()   {return myRateCut;}

  int GetChannel() {return iChan;}

  void SetRate(double rateLOG) { // done for every even
    rateLog10=rateLOG;
    //printf("[TChannel->SetRate] %10s Log10(rate)=%.2f cut rate = %.2f\n", Name, rateLog10, HLrate);
    return;
  }
  
  bool GetHLcrisisFlag() {return (rateLog10>HLrate);}

  void UpdateEmptyEventPedestal() {
    for (int iGain=0; iGain<2; iGain++) {
      for (int iDelay=0; iDelay<2; iDelay++) {
        SignalShort[2*iGain+iDelay].UpdateEmptyEventPedestal();
      }
    }
  }

  void UpdateEmptyEventPedestalLong() {
    for (int iGain=0; iGain<2; iGain++) {
      SignalLong[iGain].UpdateEmptyEventPedestal();
    }
  }

   void ClearEmptyEventPedestal() {
    for (int iGain=0; iGain<2; iGain++) {
      SignalLong[iGain].ClearEmptyEventPedestal();
      for (int iDelay=0; iDelay<2; iDelay++) {
        SignalShort[2*iGain+iDelay].ClearEmptyEventPedestal();
      }
    }
  }

  bool CheckEmptyEventPedestals() {
    bool result=true;
    if (iMod==5 && iChan>1) return true;
    for (int iGain=0; iGain<2; iGain++) {
      for (int iDelay=0; iDelay<2; iDelay++) {
        bool ch=SignalShort[2*iGain+iDelay].CheckEmptyEventPedestal();
        if (!ch) {
          printf("[TChannel->CheckEmptyEventPedestals] failed for chan#%d %s delay=%d gain=%d\n", 
                 iChan, Name, iDelay, iGain);
          result=false;
        }
      }
    }
    return result;
  }

  void SetID(int idX) {id=idX;}
  int GetID() {return id;}

  void SetTbunch(int ib, double T) {
    Txxx[ib]=T;
    SignalLong[0].SetTbunch(ib, T);
    SignalLong[1].SetTbunch(ib, T);
    SignalDummy->SetTbunch(ib, T);
    for (int i=0;i<4;i++) SignalShort[i].SetTbunch(ib, T);
  }  
 
  void SetSlices(int n) {
    NsliceShort=n;
    NsliceLong=2*n;
    //printf("[Channel->SetSlices] %10s Setting slices to %d\n", Name, n);
    SignalDummy->SetSlices(n, false);
    //printf("[Channel->SetSlices] dummy {%s} is done\n", SignalDummy->GetName());
    for(int i=0;i<4;i++) SignalShort[i].SetSlices(n, false);
    //printf("[Channel->SetSlices] shorts are done\n");
    for(int i=0;i<2;i++) SignalLong[i].SetSlices(2*n, true);
    //printf("[Channel->SetSlices] longs are done\n");

    for(int i=0;i<n;i++) {
      //printf("[Channel->SetSlices] setting Dummy signal sample#%3d\n", i);
      SignalDummy->SetTime(i, 25.*i);
      //printf("[Channel->SetSlices] setting Dummy signal time set\n");
      SignalDummy->SetAmp1(i, 0.0);
      //printf("[Channel->SetSlices] setting Dummy signal amp1 set\n");
      SignalDummy->SetAmp2(i, 0.0);
      //printf("[Channel->SetSlices] setting Dummy signal amp2 set\n");
    }
    //printf("[Channel->SetSlices] dummy finished, bye\n");

    return;
  }

  void SetName(const char* xName) {
    char NameRef[40];
    char TitlC[200];
    char NameC[40];
    sprintf(Name, "%s", xName);
    //printf("[TChannel->SetName] setting names for shannel %s and its's signals\n", Name);
    //sprintf(TitlC, "%s_canvas", Name);    
    //LocalCanvas->SetName(TitlC);
    //LocalCanvas->SetTitle(TitlC);
    sprintf(NameRef, "%s low gain", xName);
    Reference[0].SetName(NameRef);
    sprintf(NameRef, "%s high gain", xName);
    Reference[1].SetName(NameRef);
    for (int iGain=0; iGain<2; iGain++) {
      for (int iDelay=0; iDelay<2; iDelay++) {
	SignalShort[2*iGain+iDelay].SetName(xName, iGain, iDelay); 
      }
      SignalLong[iGain].SetName(xName, iGain, -1);
    }
    SignalDummy->SetName(xName, -1, -1);
    sprintf(NameRef, "%s high to low ratio", xName);

    sprintf (NameC, "Signal %s", Name);
    dummy1d = new TH1D(NameC, NameC, 1000, -50., 6350.);


    double hMax=40000.;
    if (iChan==4) hMax*=4.;
    sprintf (NameC, "Spectrum %s", Name);
    int nBINspec=100+int(hMax);
    FullEventSpectrum = new TH1D(NameC, NameC, nBINspec, -100., hMax);
    FullEventSpectrum->GetXaxis()->SetTitle("Response peak, HG ADC");
    FullEventSpectrum->GetYaxis()->SetTitle("Bunches");
//    FullEventSpectrum->SetFillColor(kGray);

    sprintf (NameC, "Integral %s", Name);
    FullEventIntegral = new TH1D(NameC, NameC, nBINspec, -100., hMax);
    FullEventIntegral->GetXaxis()->SetTitle("Response Integral, HG ADC");
    FullEventIntegral->GetYaxis()->SetTitle("Bunches");
//    FullEventIntegral->SetFillColor(kGray);

    sprintf (NameC, "Noise %s", Name);
    PedestalSpectrum  = new TH1D(NameC, NameC, 200,  -100., 100.);
    PedestalSpectrum->GetXaxis()->SetTitle("Response peak, HG ADC");
    PedestalSpectrum->GetYaxis()->SetTitle("Bunches");
    PedestalSpectrum->SetLineColor(kBlue);

    sprintf(NameC, "Signal Factor %s", Name);
    signalFactor = new TH1D(NameC, NameC, 51, 0., 1.05);
    signalFactor -> GetXaxis() ->SetTitle("Signal Factor, a.u.");
    signalFactor -> GetYaxis() ->SetTitle("Events");

    if (iMod<4) {
      highTOlow = new TH1D(NameRef, NameRef, 200, 0., 20.); 
      highTOlow->GetXaxis()->SetTitle("High/Low ratio");
      highTOlow->GetYaxis()->SetTitle("Bunches");
      sprintf(NameRef, "%s high vs low", xName);
      highVSlow = new TH2F(NameRef, NameRef, 1000, -2000., 50000., 1000, -2000., 50000.); 
// old elsctronics
//      highVSlow = new TH2F(NameRef, NameRef, 600, 0., 400., 500, 0., 4000.); 

      highVSlow -> SetStats(0);
      highTOlow -> SetStats(0);
      highVSlow->GetXaxis()->SetTitle("Low Gain, ADC");
      highVSlow->GetYaxis()->SetTitle("High Gain, ADC");
      highVSlow->SetMarkerSize(0.2);
      highVSlow->SetMarkerStyle(20);

      for (int iChI=0; iChI<4; iChI++) {
	sprintf(TitlC, "%s vs Integrator-%d", xName, iChI);
	sprintf(NameC, "%s_vs_Integrator-%d",  xName, iChI);
        RatioToCher[iChI].SetNameTitle(NameC, TitlC);
        RatioToCher[iChI].SetBins(50, 0.0, 6.0, 50, 0., 1.0);

        sprintf(TitlC, "Negative/Positive %s vs Integrator-%d", xName,  iChI);
        sprintf(NameC, "Neg/Pos_%s_vs_Integr-%d", xName,  iChI);
        RatioPlusMinus[iChI].SetNameTitle(NameC, TitlC);
        RatioPlusMinus[iChI].SetBins(50, 0.0, 6.0, 50, 0., 2.);
       
        sprintf(TitlC, "Tneg %s vs Integrator-%d", xName,  iChI);
        sprintf(NameC, "Tneg_%s_vs_Integr-%d", xName,  iChI);
        Tminus[iChI].SetNameTitle(NameC, TitlC);
        Tminus[iChI].SetBins(50, 0.0, 6.0, 50, 0., 800.);
       
        sprintf(TitlC, "Tpos %s vs Integrator-%d", xName,  iChI);
        sprintf(NameC, "Tpos_%s_vs_Integr-%d", xName,  iChI);
        Tplus[iChI].SetNameTitle(NameC, TitlC);
        Tplus[iChI].SetBins(50, 0.0, 6.0, 50, 0., 200.);
      }
     
    } else {
      highTOlow = new TH1D(NameRef, NameRef, 200, 0.0, 20.0); 
      highTOlow->GetXaxis()->SetTitle("High/Low ratio");
      highTOlow->GetYaxis()->SetTitle("Bunches");
      sprintf(NameRef, "%s high vs low", xName);
      highVSlow = new TH2F(NameRef, NameRef, 1000, -2000., 50000., 1000, -2000., 50000.); 
//      highVSlow = new TH2F(NameRef, NameRef, 200, 0., 400., 300, 0., 3000.); 
      highVSlow->GetXaxis()->SetTitle("Low Gain, ADC");
      highVSlow->GetYaxis()->SetTitle("High Gain, ADC");
      highVSlow->SetMarkerSize(0.2);
      highVSlow->SetMarkerStyle(20);

      highVSlow -> SetStats(0);
      highTOlow -> SetStats(0);
 
      for (int iChI=0; iChI<4; iChI++) {
	sprintf(TitlC, "%s vs Integrator-%d", xName, iChI);
	sprintf(NameC, "%s_vs_Integrator-%d",  xName, iChI);
        RatioToCher[iChI].SetNameTitle(NameC, TitlC);
        RatioToCher[iChI].SetBins(50, 0.0, 6.0, 50, 0., 1.0);
        //printf("[TChannel->SetName] Histogram#%d %s was named!\n", iChI,  RatioToCher[iChI].GetTitle());

        sprintf(TitlC, "Negative/Positive %s vs Integrator-%d", xName,  iChI);
        sprintf(NameC, "Neg/Pos_%s_vs_Integr-%d", xName,  iChI);
        RatioPlusMinus[iChI].SetNameTitle(NameC, TitlC);
        RatioPlusMinus[iChI].SetBins(50, 0.0, 6.0, 50, 0., 2.);
       
        sprintf(TitlC, "Tneg %s vs Integrator-%d", xName,  iChI);
        sprintf(NameC, "Tneg_%s_vs_Integr-%d", xName,  iChI);
        Tminus[iChI].SetNameTitle(NameC, TitlC);
        Tminus[iChI].SetBins(50, 0.0, 6.0, 50, 0., 800.);
       
        sprintf(TitlC, "Tpos %s vs Integrator-%d", xName,  iChI);
        sprintf(NameC, "Tpos_%s_vs_Integr-%d", xName,  iChI);
        Tplus[iChI].SetNameTitle(NameC, TitlC);
        Tplus[iChI].SetBins(50, 0.0, 6.0, 50, 0., 200.);
      }
    }

  }

  void SetPedestalWindow(int iw, bool good, double tl, double th) {
    if (iw<0 || iw>24) {
      printf("[TChannel->SetPedestalWindow] wrong window index %d\n", iw);
      exit(-1);
      return;
    }
    if (dumpIt) {
      printf("[TChannel->SetPedestalWindow] %s setting pedestal window#%d\t good=%d t1=%.1f\t t2=%.1f\n",
            Name, iw, good, tl, th);
    }
    GoodPedWindow[iw]=good;
    NPedWindows=iw+1;
    PedWindow0[iw]=tl;
    PedWindow1[iw]=th;
    SignalDummy->SetPedestalWindow(iw, good, tl, th);
    for (int i=0;i<4;i++) {
      SignalShort[i].SetPedestalWindow(iw, good, tl, th);
    }
    SignalLong[0].SetPedestalWindow(iw, good, tl, th);
    SignalLong[1].SetPedestalWindow(iw, good, tl, th);

  }

  bool PerformTailFit(int iw) {
    bool result=true;
    //for (int i=0;i<4;i++) {
    //  result = result && SignalShort[i].PerformTailFit(iw);
    //}
    result = result && SignalLong[0].PerformTailFit(iw);
    result = result && SignalLong[1].PerformTailFit(iw);
    if (!result) printf("[TChannel->PerformTailFit] failed for %s PedWindow#%d\n", Name, iw);
    GoodPedWindow[iw]=result;
    return result;
  }

  void PrepareTailed() {
    //for (int i=0;i<4;i++) {
    //  SignalShort[i].PrepareTailed();
    //}
    SignalLong[0].PrepareTailed();
    SignalLong[1].PrepareTailed();
  }

  void DoFFTshape() {
    //for (int i=0;i<4;i++) {
    //  SignalShort[i].DoFFTshape();
    //}
    SignalLong[0].DoFFTshape();
    SignalLong[1].DoFFTshape();
  }

  void FixFFTbaseline() {
    //for (int i=0;i<4;i++) {
    //  SignalShort[i].FixFFTbaseline();
    //}
    SignalLong[0].FixFFTbaseline();
    SignalLong[1].FixFFTbaseline();
  }

void ComputeFFTAmpPhase() {
  //printf ("[TChannel->ComputeFFTAmpPhase] computing spectra for %s\n", Name);
  SignalLong[0].StraightFFTransform();
  SignalLong[1].StraightFFTransform();
  SignalLong[0].ComputeFFTAmpPhase();
  SignalLong[1].ComputeFFTAmpPhase();  
}

void SetPedestalWindowPre2012(int iw, double tl, double th) {
    if (iw<0 || iw>24) {
      printf("[TChannel->SetPedestalWindow] wrong window index %d\n", iw);
      exit(-1);
      return;
    }
    int nWin=1000;
    GoodPedWindow[iw]=true;
    NPedWindows=iw+1;
    PedWindow0[iw]=tl;
    PedWindow1[iw]=th;
    SignalDummy->SetPedestalWindow(iw, GoodPedWindow[iw], tl, th);
    for (int i=0;i<4;i++) {
      SignalShort[i].SetPedestalWindow(iw, GoodPedWindow[iw], tl, th);
      int n=SignalShort[i].CalcPedestalWindow(iw);
      if (Dbg) printf("[Channel->SetPedestalWindow] %s for signal#%d nWin=%d\n", Name, i, n);
      if (n<nWin) nWin=n;
    }
    SignalLong[0].SetPedestalWindow(iw, GoodPedWindow[iw], tl, th);
    SignalLong[1].SetPedestalWindow(iw, GoodPedWindow[iw], tl, th);
    if (nWin==0) GoodPedWindow[iw]=false; 
  }

  int CleanPedestalWindows() {
    bool Dbg1=false;
    int ReportMode = ProcStatus->GetReportMode();

    NGoodWindows=NPedWindows;
    for (int i=0;i<4;i++) {
      int n=SignalShort[i].CleanPedestalWindows(2);
      if (ReportMode==10003 || Dbg1) {
        printf("[Channel->CleanPedestalWindows] %s -> %d good pedWindows\n", SignalShort[i].GetName(), n);
      }
      if (n<NGoodWindows) NGoodWindows=n;
 
      for (int iw=0;iw<NPedWindows;iw++) {
        if (!SignalShort[i].GetPedWinX(iw)) GoodPedWindow[iw]=false; 
      }
    }
    if (ReportMode==10003 || Dbg1) {
      printf("[Channel->CleanPedestalWindows] %s -> %d good pedWindows\n\n", Name, NGoodWindows);
    }
    return NGoodWindows;
  }

  void SubtractPedestals() {
    for (int i=0;i<4;i++) {
      if (SignalShort[i].GetStatus()) SignalShort[i].SubtractPedestals();
    }    
  }

  void PedTailCorrection() { // does a+b*c*exp(-c*t) fit of a pedestal window area to account for the tails
    double time[2][100], ampx[2][100];
    int    nSamp[2], i0[2], i1[2];
    for (int iw=0; iw<NPedWindows; iw++) {
      for (int iGain=0; iGain<2; iGain++) {
//============================================================
        nSamp[0]=0;
        nSamp[1]=0;
        int nsmax=0;
        for (int iDelay=0; iDelay<2; iDelay++) {
          TSignal *Signal = &SignalShort[2*iGain+iDelay];
          i0[iDelay]=int((PedWindow0[iw]-Signal->GetTime(0))/Signal->GetdTsample())-2;
          i1[iDelay]=int((PedWindow1[iw]-Signal->GetTime(0))/Signal->GetdTsample())+2;
          if (i0[iDelay]<0) i0[iDelay]=0;
          if (i1[iDelay]>Signal->GetNslice()-1) i1[iDelay]=Signal->GetNslice()-1;
          int ns=0;
          for (int is=i0[iDelay]; is<i1[iDelay]; is++) {
            double t=Signal->GetTime(is);
            if (t>PedWindow0[iw] && 
                t<PedWindow1[iw]) {
              time[iDelay][ns]=t;
              ampx[iDelay][ns]=Signal->GetRaw(is);
              ns++;
            }
          }
          nSamp[iDelay]=ns;
          if (nSamp[iDelay]>nsmax) nsmax=nSamp[iDelay];
        }
        for (int iDelay=0; iDelay<2; iDelay++) {
          for (int is=nSamp[iDelay]; is<nsmax; is++) {
            time[iDelay][is]=-100.;
            ampx[iDelay][is]=-100.;
          }
        }
        printf("\[TChannel->PedTailCorrection] %s Gain=%d window#%d FitSamples->%d Window=(%6.1f, %6.1f)\n", 
               Name, iGain, iw, nsmax, PedWindow0[iw], PedWindow1[iw]);
        for (int is=0; is<nsmax; is++) {
          printf("    %3d Delay=0 t=%7.1f a=%6.1f Delay=1 t=%7.1f a=%6.1f\n",
                 is, time[0][is], ampx[0][is],  time[1][is], ampx[1][is]); 
        }
         
//============================================================
      } // gain loop
      printf("\n");
      //int iqq=0;
      //printf("Push! ->");
      //cin >> iqq;
      //if (iqq==0) exit(-1);
    } // window loop  
  }

  void PrepareAmpI() {
    for (int iGain=0;iGain<2;iGain++) {
      SignalLong[iGain].PrepareAmpI();    // prepare interpolated long signal ampI
    }
  }

  void SetLongPulses() {
    if (Status) {
      //if (ProcStatus->GetReportMode()==1) {
      //  printf("[TChannel->SetLongPulses] setting long pulses for %s\n", Name);
      //}
      //printf("[Channel->SetLongPulses] %s, nShort=%d\n", Name, NsliceShort);
      for (int iGain=0;iGain<2;iGain++) {
        double amax=-1.0e10,  amin=1.0e10;
        double amax0=-1.0e10, amin0=1.0e10;
        for (int iDelay=0; iDelay<2; iDelay++) {
          if (SignalShort[2*iGain+iDelay].GetMax()>amax)   amax=SignalShort[2*iGain+iDelay].GetMax();
          if (SignalShort[2*iGain+iDelay].GetMin()<amin)   amin=SignalShort[2*iGain+iDelay].GetMin();
          if (SignalShort[2*iGain+iDelay].GetMax0()>amax0) amax0=SignalShort[2*iGain+iDelay].GetMax0();
          if (SignalShort[2*iGain+iDelay].GetMin0()<amin0) amin0=SignalShort[2*iGain+iDelay].GetMin0();
	  for (int i=0; i<NsliceShort; i++) {
            double a=SignalShort[2*iGain+iDelay].GetAmp1(i);
            if (iDelay==1 && (iMod==4 || iMod==5) && !(iMod==5 && iChan==2)) a*=0.95; // adjust for delay cable losses;
            SignalShort[2*iGain+iDelay].SetAmp1(i, a);
            SignalShort[2*iGain+iDelay].SetAmpI(i, a); // interpolated signal for short is same as amp1
            SignalLong[iGain].SetAmp1(2*i+iDelay, a);
            SignalLong[iGain].SetAmp2(2*i+iDelay, a);
            SignalLong[iGain].SetOverf(2*i+iDelay, SignalShort[2*iGain+iDelay].GetOverf(i));
            SignalLong[iGain].SetUnder(2*i+iDelay, SignalShort[2*iGain+iDelay].GetUnder(i));
	  }
        } // end of delay loop
        SignalLong[iGain].SetMax(amax);
        SignalLong[iGain].SetMin(amin);
        SignalLong[iGain].SetMax0(amax0);
        SignalLong[iGain].SetMin0(amin0);
        SignalLong[iGain].SetGraph();
      }
    }
  }

  void SubtractRF(double scale, double phase) {
    if (ProcStatus->GetReportMode()==1) {
      printf("[TChannel->SubtractRF] subtracting RF for %s, shall not be here!\n", Name);
      exit(-1);
    }
    //printf("[TChannel->SubtractRF]subtracting RF for %s, scale=%.4f, phase=%.1fns\n", Name, scale, phase);
    for (int iGain=0;iGain<2;iGain++) {
      for (int iDelay=0; iDelay<2; iDelay++) {
        for (int i=0; i<NsliceShort; i++) {
          double a=double (SignalShort[2*iGain+iDelay].GetRaw(i));
          double t=SignalShort[2*iGain+iDelay].GetTime(i);
          double ax=scale*ProcStatus->GetEventHeader()->GetSinWave(t+phase);
          //if (iGain==1 && iDelay==1 && i<100) printf("   #%3d a0=%d\t rf=%4.1f\t", i, a, ax); 
          if (iGain==1) {
            a-=ax;
          } else {
            a-=ax/HighLow;
          }
          //if (iGain==1 && iDelay==1 && i<100) printf(" aNew=%d\n", a); 
          SignalShort[2*iGain+iDelay].SetRaw(i, a);
        }
      }
    }    
  }

  void SetShortPulses() {
    if (Status) {
      if (Dbg) printf("[Channel] Preparing short signals for %s, NSliceShort=%d\n", Name, NsliceShort);
      for (int i=0;i<4;i++) {
        SignalShort[i].SetGraph();
      }
    }
  }

  double Fit(double T, int N, int iFit) {
    //Dbg=true;
    int ReportMode  = ProcStatus->GetReportMode();
    double Chi2worst=0.0;
    chi2l[N]=-10000.;
    chi2h[N]=-10000.;
    if (Status) {

      if (Dbg || ReportMode==10021) printf("[Channel->Fit->LowGian] %s bunch#%d T=%.1f stage#%d\n", Name, N, T, iFit);
      if (SignalLong[0].GetStatus()) {  // if both delays in a good shape....
        chi2l[N]=SignalLong[0].Fit(T, N, iFit);
      } else {
        for (int iDelay=0; iDelay<2; iDelay++) { // find good delay signal if any
	  if (SignalShort[iDelay].GetStatus()) {
            chi2l[N]=SignalShort[iDelay].Fit(T, N, iFit);
	  }
	}
      }

      if (Dbg || ReportMode==10021) printf("[Channel->Fit->HighGian] %s bunch#%d T=%.1f stage#%d\n", Name, N, T, iFit);
      if (SignalLong[1].GetStatus()) {  // if both delays in a good shape....
        chi2h[N]=SignalLong[1].Fit(T, N, iFit);
      } else {
        for (int iDelay=0; iDelay<2; iDelay++) { // find good delay signal if any
	  if (SignalShort[2+iDelay].GetStatus()) {
            chi2h[N]=SignalShort[2+iDelay].Fit(T, N, iFit);
	  }
	}
      }
    }
    //Dbg=false;
    if (chi2h[N]>chi2l[N]) Chi2worst=chi2h[N];
    else Chi2worst=chi2l[N];
    return Chi2worst;
  }

  void SubtractFit(double T, int N, int iFit) {
    if (Status) {
      SignalLong[0].SubtractFit(T, N, iFit);
      SignalLong[1].SubtractFit(T, N, iFit);
    }
  }

  int GetResponseStatus(int bunch) {return  ResponseStatus[bunch];}

  void CalculateResponse(int bunch) {  
    printf("[Channel->CalculateResponse] shall not be here\n");
    exit(-1);
  }

  bool LowGainOverflowDetected() {
    LGoverflow = SignalLong[0].OverflowDetected(HighLowSwitch);
    //printf("[TChannel->LowGainOverflowDetected] %10s thr=%.1f LGoverflow=%d\n", Name, HighLowSwitch, LGoverflow);
    return LGoverflow;
  }

  bool HighGainOverflowDetected() {
    HGoverflow = SignalLong[1].OverflowDetected(HighLowSwitch);
    //printf("[TChannel->HighGainOverflowDetected] %10s thr=%.1f HGoverflow=%d\n", Name, HighLowSwitch, HGoverflow);
    return HGoverflow;
  }

  bool CheckHGoverflow() { // checks overflows in raw HG data, signal by signal 
    bool result=false;
    for (int iDelay=0; iDelay<2; iDelay++) {
      if (SignalShort[2+iDelay].OverflowDetected(1.0e10)) result=true;
    }
    return result;
  }

  int GetHiLoCrisis(int bunch) { // indicates that there is a problem with high/low ratio!
    //if (high[bunch]<HighLowSwitch && low[bunch]>5.0) {
    if (high[bunch]<HighLowSwitch) {
      if ((low[bunch]-5.)*0.8*HighLow>high[bunch]) return 1;
      if (low[bunch]*1.2*HighLow+50.<high[bunch]) return 1;
    }
    return 0;
  }

  bool GetHiLoCrisisCondition(int bunch) { // is there a rate-caused hi/low correction ?
    //int rep1=ProcStatus->GetReportMode();
    //printf("TChannel->GetHiLoCrisisCondition] %10s rateLog10=%.2e HLrate=%.2e high=%.2f low=%.2f low*(HL-2s)=%.2f",
    //       Name, rateLog10, HLrate, high[bunch], low[bunch], (low[bunch]-1.)*(HighLow-3.0*HLsigma));
    if (HLeff<0.3 || HLmean==0. || HLsigma==0.) { // no correction needed
      HiLoCrisisFlag=false;
      //return HiLoCrisisFlag;
    }
    if (rateLog10>HLrate) {
      HiLoCrisisFlag=true;
    } else if (low[bunch]>5. && high[bunch]<(low[bunch]-1.)*(HighLow-3.0*HLsigma)) {
      HiLoCrisisFlag=true;
    }
    if (HiLoCrisisFlag) {
      printf("[TChannel->GetHiLoCrisisCondition] crisis=%d\n", HiLoCrisisFlag);
      printf("             Eff=%.2f Log10(rate)=%4.2f HLrate=%4.2f\n", HLeff, rateLog10, HLrate);
      printf("             LG=%.2f  HG=%.2f\n", low[bunch], high[bunch]);
      printf("             HighLow-3.0*HLsigma=%.2f\n", HighLow-3.0*HLsigma);
//      ProcStatus->SetReportMode(1); 
    }
    return HiLoCrisisFlag;
  }

  bool GetHiLoCrisisCondition2011(int bunch) {
    HiLoCrisisFlag = false;
// cases of low intensity where no correction required ///
    if (HGoverflow) return false; // do not bother, low gain anyway
    if (HLtotStat<1000) return false;
    if (rateBin2<0.0)   return false;
    if (HLeff<0.05)     return false;
    if (HLmean==0. || HLsigma==0.) return false;
    if (ProcStatus->GetRunHeader()->AttnCorr==1.0  && HLrate<2.2) return false;
    if (ProcStatus->GetRunHeader()->AttnCorr==10.0 && HLrate<3.2) return false;      
    int rbin=1+int((rateLog10-rateBin[1])/(rateBin[2]-rateBin[1]));
    if (rbin==0 || rbin>=rateBins || rateLog10<rateBin1) return false; 
    
    if (rateLog10>HLrate) HiLoCrisisFlag=true;
    if (HGoverflowBunch[bunch]) HiLoCrisisFlag=true;
    if (low[bunch]>5. && high[bunch]<(low[bunch]-1.)*rateCut[rbin]) {
/*
      printf("[TChannel->GetHiLoCrisisCondition2011] %s crisis=%d HGover=%d HiLoSwitch=%.0f\n", 
             Name, HiLoCrisisFlag, HGoverflowBunch[bunch], HighLowSwitch);
      printf("             Eff=%.2f Log10(rate)=%5.3f HLrateCUT=%5.3f\n", HLeff, rateLog10, HLrate);
      printf("             LG=%.3f  HG=%.3f LG/HG=%.3f\n", low[bunch], high[bunch], high[bunch]/low[bunch]);
      printf("             bin#%d High/low cut=%.3f\n", rbin, rateCut[rbin]);
      //ProcStatus->SetReportMode(1); 
*/      
      HiLoCrisisFlag=true;
    }
    return HiLoCrisisFlag;
  }

   

  double GetResponseADC(int bunch) {
    return Response[bunch];
  }

  double GetResponse8(int bunch) {
    return Response8[bunch];
  }

  double GetResponse8cal(int bunch) {
    return Response8cal[bunch];
  }


  double GetResponseADCrate(int bunch) {

    double response=high[bunch];

    if (high[bunch]>HighLowSwitch) {
      response=low[bunch]*HighLow;
    }

    if (HiLoCrisisFlag) {
      response=low[bunch]*HighLow;
    } 

    return response;
  }

  double GetResponseCALrate(int bunch) { // calibrated for attenuator

    double response=highCal[bunch];

    if (high[bunch]>HighLowSwitch) { // low gain 
      return lowCal[bunch]*HighLow;
    }

    if (HiLoCrisisFlag) {
      response=lowCal[bunch]*HighLow;
    } 

    return response;
     
  }

  double GetResponseADClow(int bunch) {
    return low[bunch]*HighLow;
  }

  double GetResponseADClowAsIs(int bunch) {
    return low[bunch];
  }

  double GetResponseCal(int bunch) { // corrected for attenuator if applicable //
    return ResponseCal[bunch];
  }

  double GetResponseCorrected(int bunch) {
    return gainCorrection*ResponseCal[bunch];
  }
  double GetGainCorrection() {
    return gainCorrection;
  }

  double GetResponseTime(int bunch) {
    return Time[bunch];
  }

  double GetResponseHigh(int bunch)  {return high[bunch];}
  double GetResponseLow(int bunch)   {return low[bunch];}
  double GetResponseHighCal(int bunch)  {return highCal[bunch];}
  double GetResponseLowCal(int bunch)   {return lowCal[bunch];}
  double GetResponseHighCorrected(int bunch)  {return gainCorrection*highCal[bunch];}
  double GetResponseLowCorrected(int bunch)   {return gainCorrection*lowCal[bunch];}

  void FillHighLowHistsOld(int bunch) { // obsolete for old electronics
    highVSlow->Fill(low[bunch], high[bunch]);
    if (low[bunch]>5.0 && high[bunch]<HighLowSwitch) highTOlow->Fill(high[bunch]/low[bunch]);
  }

  void FillHighLowHists(int bunch) { 
    if (!clusterPMTbunchSET[bunch]) return;
    //printf("[Channel->FillHighLowHists] %10s bunch#%d low=%.1f(%.1f) \t high=%.1f(%.1f)\n", 
    //        Name, bunch, low[bunch]*HighLow, clusterPMTbunchAmp2[bunch][0], high[bunch], clusterPMTbunchAmp2[bunch][1]);
    highVSlow->Fill(low[bunch], high[bunch]);
    if (low[bunch]>1000.0 && high[bunch]<HighLowSwitch2013) highTOlow->Fill(high[bunch]/low[bunch]);
    if (low[bunch]<250.   && high[bunch]>4000.) {
      printf("\n\n\n***********************************\n");
      printf("[Channel->FillHighLowHists] %10s bunch#%d low=%.1f \t high=%.1f\n", Name, bunch, low[bunch], high[bunch]);
      printf("***********************************\n");
      //ProcStatus->SetReportMode(1);
    }
    if (low[bunch]>500.+0.1*high[bunch] && high[bunch]<HighLowSwitch2013) {
      printf("\n\n\n***********************************\n");
      printf("[Channel->FillHighLowHists] %10s bunch#%d low=%.1f \t high=%.1f\n", Name, bunch, low[bunch], high[bunch]);
      printf("***********************************\n");
      //ProcStatus->SetReportMode(1);
    }
  }

  void FillEventSpectrum(int bunch) {
    if (iMod<4) {
      FullEventSpectrum->Fill(Response[bunch]);
    } else {
      double aBunch=0.;
      if (SignalLong[1].GetOverflowStatus(bunch) ||
          SignalLong[1].GetUnderflowStatus(bunch)) aBunch=SignalLong[0].GetSum8(bunch)*HighLow;
      else aBunch=SignalLong[1].GetSum8(bunch);
      FullEventSpectrum->Fill(aBunch);
    }
  }



  void FillPedestalSpectrum(int bunch) {PedestalSpectrum->Fill(high[bunch]);}

  void FillEventIntegral(int bunch) {
    double aBunch=0.;
    if (SignalLong[1].GetOverflowStatus(bunch) ||
        SignalLong[1].GetUnderflowStatus(bunch)) aBunch=SignalLong[0].GetSum8(bunch)*HighLow;
    else aBunch=SignalLong[1].GetSum8(bunch);
    FullEventIntegral->Fill(aBunch);
  }

  void On()  {Status=true;}
  void Off() {Status=false;}
  void Print() {cout << "  " << Name << " Status=" << Status << endl;}
  double GetHighLow() {return HighLow;}
  void SetHighLow(double newHighLow) {
    HighLow=newHighLow;
    //printf("[TChannel->SetHighLow] for %s High-To-Low was set to %f\n", Name, HighLow);
  }

  bool  GetStatus() {return Status;}

  bool  GetPedWinX(int i) {
    if (i<0 || i>24) return false;
    return GoodPedWindow[i];
  }
  const char* GetName() {return Name;}

  TSignal* GetSignalShort(int i) { // returns pointer to the signal
    int i1=i;
    if (i1<0) {
      i1=0;
      cout << "*** Error in Channel " << Name << ", Short Signal called with index=" << i << endl; 
      exit(-2);
    } else if (i1>3) {
      i1=3;
      cout << "*** Error in Channel " << Name << ", Short Signal called with index=" << i << endl; 
      exit(-2);
    }
    return &((TChannel*)this)->SignalShort[i1];
  }

  TSignal* GetSignalShort(int iGain, int iDelay) {
    int i1=2*iGain+iDelay;
    if (i1<0) {
      printf("[TChannel->etSignalShort(int, int)] wrong index %d\n", i1);
      i1=0;
      exit(-2);
    } else if (i1>3) {
      printf("[TChannel->etSignalShort(int, int)] wrong index %d\n", i1);
      i1=3;
      exit(-2);
    }
    return &((TChannel*)this)->SignalShort[i1];
  }


  TSignal* GetSignalLong(int i) { // returns pointer to the channel
    int i1=i;
    if (i1<0) {
      i1=0;
      cout << "*** Error in Channel " << Name << ", Long Signal called with index=" << i << endl; 
      exit(-2);
    } else if (i1>1) {
      i1=1;
      cout << "*** Error in Channel " << Name << ", Long Signal called with index=" << i << endl; 
      exit(-2);
    }
    return &((TChannel*)this)->SignalLong[i1];
  }

  TSignal* GetGoodSignal(int iGain) { 
    // returns ponter to the Good Long (combined) signal; 
    // if not available - to good Short signal
    // if not available - to SignalDummy
    if (iGain<0 || iGain>1) {
      return SignalDummy;
    }
    if (SignalLong[iGain].GetStatus()) {
      return &((TChannel*)this)->SignalLong[iGain];
    } else {
      for (int iDelay=0; iDelay<2; iDelay++) {
        if (SignalShort[2*iGain+iDelay].GetStatus()) {
          return &((TChannel*)this)->SignalShort[2*iGain+iDelay];
        }
      }
    }
    return SignalDummy;
  }

  TReference* GetReference(int i) { // returns pointer to the reference (high or low gain)
    int i1=i;
    if (i1<0) {
      i1=0;
      cout << "*** Error in Channel " << Name << ", Long Signal called with index=" << i << endl; 
    } else if (i1>1) {
      i1=1;
      cout << "*** Error in Channel " << Name << ", Long Signal called with index=" << i << endl; 
    }
    return &((TChannel*)this)->Reference[i1];
  }

  void Dump(double Tbunch, int bunch) { // normalyzed on peak
    for (int i=0; i<2; i++) {
      SignalLong[i].Dump(Tbunch, bunch);
    }
  }

  double GetSampleTime(int i) {
    int iGain=1;
    if (HiLoCrisisFlag || HGoverflow) {
      iGain=0;
    }
    return SignalLong[iGain].GetTime(i);
  }
  double GetSampleSignalADC(int i) {
    if (HiLoCrisisFlag || HGoverflow) {
      return HighLow*SignalLong[0].GetAmp1(i);
    } else {
      return SignalLong[1].GetAmp1(i);
    }
  }



void FillPulseShapes(int bunch) { // normalyzed on peak
for (int i=0; i<4; i++) {
  if (SignalShort[i].GetStatus()) {
    if (iMod<4) { // use modules with baseline subtraction !
      SignalShort[i].FillPulseShapeX(bunch);
    } else {
      SignalShort[i].FillPulseShape1(bunch);
    }
  }
}

for (int i=0; i<2; i++) {
  if (SignalLong[i].GetStatus()) {
    if (iMod<4) { // use modules with baseline subtraction !
      SignalLong[i].FillPulseShapeX(bunch);
    } else {
      SignalLong[i].FillPulseShape1(bunch);
    }
  }
}
return;
}


  void FillPulseShapesModule(double Tbunch, int bunch, TProfile* Demo) { // normalyzed on peak

    bool Dbg1=false;

    if (Dbg1) {
      printf("[TChannel->FillPulseShapes] Long -> %s bunch#%d Tbunch=%.1f\n", Name, bunch, Tbunch); 
    }
    for (int i=0; i<2; i++) {
      if (Dbg1) printf("[TChannel->FillPulseShapes] Long#%d Status=%d\n", i, SignalShort[i].GetStatus());
      if (SignalLong[i].GetStatus()) {
        SignalLong[i].FillPulseShapeModule(Tbunch, bunch, Demo);
      }
    }
  }

   void FillPulseShapesA(double Tbunch) { // amplitude as is
    for (int i=0; i<2; i++) {
      SignalLong[i].FillPulseShapeA(Tbunch);
    }
  }

  void FillPulseShapesAped(int bunch) { // amplitude as is
    for (int i=0; i<2; i++) {
      SignalLong[i].FillPulseShapeAped(bunch);
    }
  }

  void FillPulseShapesI(double Tbunch, double Cherenkov, double CherIntegral, double BeamIntensityLog) {
    for (int i=0; i<4; i++) {
      if (SignalShort[i].GetStatus()) {
        SignalShort[i].FillPulseShapeI(Tbunch, Cherenkov, CherIntegral, BeamIntensityLog, HighLowSwitch);
      }
    }
    for (int i=0; i<2; i++) {
      if (SignalLong[i].GetStatus()) {
        SignalLong[i].FillPulseShapeI(Tbunch, Cherenkov, CherIntegral, BeamIntensityLog, HighLowSwitch);
      }
    }
  }

  void FillPulseShapeSplit(double Tbunch) { // high and low gain, long and short as well
    for (int i=0; i<2; i++) {
      if (SignalLong[i].GetStatus()) {
        SignalLong[i].FillPulseShapeSplit(Tbunch);
      }
    }
    
    for (int i=0; i<4; i++) {
      if (SignalShort[i].GetStatus()) {
        SignalShort[i].FillPulseShapeSplit(Tbunch);
      }
    }
    
  }

  void ClearPulseShapeSplit() { // high and low gain, long and short as well
    for (int i=0; i<2; i++) {
      SignalLong[i].ClearPulseShapeSplit();
    }
    if (iChan<4) {
      for (int i=0; i<4; i++) {
        SignalShort[i].ClearPulseShapeSplit();
      }
    }
  }

   void SavePulseShapeSplit() { // high and low gain, long and short as well
    for (int i=0; i<2; i++) {
      SignalLong[i].SavePulseShapeSplit();
    }
    
    for (int i=0; i<4; i++) {
      SignalShort[i].SavePulseShapeSplit();
    } 
  }  

  void ClearPulseShapes() {
    for (int i=0; i<4; i++) SignalShort[i].ClearPulseShape();
    for (int i=0; i<2; i++) SignalLong[i].ClearPulseShape();
  }

  void ResetSignalNoise() {
    for (int i=0; i<4; i++) SignalShort[i].ResetNoiseHist();
    for (int i=0; i<2; i++) SignalLong[i].ResetNoiseHist();
  }


  void SaveRaw() {
    for (int i=0; i<4; i++) {
      if (SignalShort[i].GetStatus()) SignalShort[i].SaveRaw();
    }
  }
  void CountTrains() {
    for (int i=0; i<4; i++) {
      if (SignalShort[i].GetStatus()) SignalShort[i].CountTrains();
    }
  }
  void ClearTrains() {
    for (int i=0; i<4; i++) {
      if (SignalShort[i].GetStatus()) SignalShort[i].ClearTrains();
    }
  }

  void PrintTrains(int ip) { // ip - details level 
                             // prints the trains of identical raw ADC readings with previous event
    for (int i=0; i<4; i++) {
      if (SignalShort[i].GetStatus()) {
        printf("[TChannel->PrintTrains] %s\t dump#%d\n", Name, ip);
        if (ip>0) SignalShort[i].PrintTrains0();
        if (ip>1) SignalShort[i].PrintTrains1();
        if (ip>2) SignalShort[i].PrintTrains2();
        if (ip>3) SignalShort[i].PrintTrains3();
        if (ip==-3) SignalShort[i].PrintTrains3();
      }
    }
  }
  int GetIdentical() {
    int isum=0;
    for (int i=0; i<4; i++) {
      if (SignalShort[i].GetStatus()) {
        isum+=SignalShort[i].GetIdentical();
      }
    }
    //printf("[TChannel->GetIdentical] Channel %s has %d matches\n", Name, isum);
    return isum;
  }

  bool IsClone() {
    for (int i=0; i<4; i++) {
      if (SignalShort[i].GetStatus()) {
        if(SignalShort[i].IsClone()) return true;
      }
    }
    return false;
  }

  bool IsTrain() {
    for (int i=0; i<4; i++) {
      if (SignalShort[i].GetStatus()) {
        if(SignalShort[i].IsTrain()) return true;
      }
    }
    return false;
  }

  void RecoverScintInput() { // recover input for S1, S2, S3, Ch, M1, M2 using RC2CR fit
    for (int iGain=0; iGain<2; iGain++) {
      TSignal *Signal = &SignalLong[iGain];
      if (!Signal->GetStatus()) { // if Long Signal has problems, pick one of the shorts which is good... if any!
	for (int iDelay=0; iDelay<2; iDelay++) {
	  Signal = &SignalShort[2*iGain+iDelay];
	  if (Signal->GetStatus()) break;
	}
      }
      if (Signal->GetStatus()) {
	Signal->RecoverScintInput();
      }
      int ReportMode = ProcStatus->GetReportMode();
      if (ReportMode<0) break; // exit loop if there is interest in the last plot
    }
    return;
  }

  void FindPulseParams2010(double Tbunch, int bunch) {
    int ReportMode = ProcStatus->GetReportMode();
    if (Dbg || ReportMode==10024) printf("[Channel->FindPulseParams2010] %s -- T=%.1f bunch#%d\n", Name, Tbunch, bunch);
    for (int iGain=0; iGain<2; iGain++) {
      TSignal *Signal = &SignalLong[iGain];
      if (!Signal->GetStatus()) { // if Long Signal has problems, pick one of the shorts which is good... if any!
	for (int iDelay=0; iDelay<2; iDelay++) {
	  Signal = &SignalShort[2*iGain+iDelay];
	  if (Signal->GetStatus()) break;
	}
      }
      if (Signal->GetStatus()) {
	Signal->FindPulseParams2010(Tbunch, bunch);
      }
      ReportMode = ProcStatus->GetReportMode();
      if (ReportMode<0) break; // exit loop if there is interest in the last plot
    }
  }

  void FindPulseParams2012(double Tbunch, int bunch) {
    int ReportMode = ProcStatus->GetReportMode();
    if (Dbg || ReportMode==10024) {
      printf("[Channel->FindPulseParams2012] %s -- T=%.1f bunch#%d\n", Name, Tbunch, bunch);
    }
    for (int iGain=0; iGain<2; iGain++) {
      TSignal *Signal = &SignalLong[iGain];
      if (!Signal->GetStatus()) { // if Long Signal has problems, pick one of the shorts which is good... if any!
	for (int iDelay=0; iDelay<2; iDelay++) {
	  Signal = &SignalShort[2*iGain+iDelay];
	  if (Signal->GetStatus()) break;
	}
      }
      if (Signal->GetStatus()) {
	Signal->FindPulseParams2012(Tbunch, bunch);
      }
      ReportMode = ProcStatus->GetReportMode();
      if (ReportMode<0) break; // exit loop if there is interest in the last plot
    }
  }

  void SetBunchNoise(int bunch, double noise) {
    BunchNoise[bunch]=noise;
  }

  double GetBunchNoise(int bunch) {
    return BunchNoise[bunch];
  }

void computeSinCross() {
  for (int iDelay=0; iDelay<2; iDelay++) {
    for (int iGain=0; iGain<2; iGain++) {
      SignalShort[2*iGain+iDelay].computeSinCross201212(); 
    }
  }  
}

void FillRFcorr() {
// fill RF frequency correction: A/A(RF), T0-T0(RF);
  for (int iDelay=0; iDelay<2; iDelay++) {
    for (int iGain=0; iGain<2; iGain++) {
      SignalShort[2*iGain+iDelay].FillRFcorr(); 
    }
  }  
}

void SubtractRFcross() {
  for (int iDelay=0; iDelay<2; iDelay++) {
    for (int iGain=0; iGain<2; iGain++) {
      //if (ProcStatus->GetReportMode()==1) {
      //  printf("[TChannel->SubtractRFcross] calling SubtractRFcross() for %s\n", SignalShort[2*iGain+iDelay].GetName());
      //}
      SignalShort[2*iGain+iDelay].SubtractRFcross(); 
    }
  }  
}


void ComputePrePed() {
  //printf("  [TChannel->ComputePrePed] %s\n", Name);
  for (int iDelay=0; iDelay<2; iDelay++) {
    for (int iGain=0; iGain<2; iGain++) {
      //printf("  [TChannel->ComputePrePed] delay=%d gain=%d\n", iGain, iDelay);
      SignalShort[2*iGain+iDelay].ComputePrePed(); 
    }
  }    
}

  bool ComputeVarianceByBunch() { // computes noise in a RAW data, delay by delay, gain by gain //
    if (Dbg) printf("[TChannel->ComputeVarianceByBunch] %s\n", Name);
    bool result=true;
    for (int iDelay=0; iDelay<2; iDelay++) {
      for (int iGain=0; iGain<2; iGain++) {
        bool zz=SignalShort[2*iGain+iDelay].ComputeVarianceByBunch();
        result = result && zz;
        if (Dbg) printf("[TChannel->ComputeVarianceByBunch] for signal %s, result=%d\n",
                        SignalShort[2*iGain+iDelay].GetName(), zz);
      }
    }
    return result;
  }

  void ComputeRFsig(int ib) {
// compute signal/noise for bunch offset ib
    for (int iGain=0; iGain<2; iGain++) {
      for (int iDelay=0; iDelay<2; iDelay++) {
        SignalShort[2*iGain+iDelay].ComputeRFsig(ib);
      }
    }
    return;
  }


  void DisplayHighLow(int bunch, double Tbunch) {
    TCanvas *ChannelCanvas = new TCanvas(Name, Name, 0, 0, 300, 300);
    EventHeader *EH = ProcStatus->GetEventHeader();
    TLine *LL = new TLine();
    //Dbg=true;
    char TitleC[200];
   
    sprintf(TitleC, "High/Low alert for Run #%d, Burst#%d, Event#%d(%d), Noise=%.2f", 
	  EH->run, EH->burst, EH->event, EH->burstEvent, BunchNoise[bunch]); 
    ChannelCanvas->SetTitle(TitleC);
    ChannelCanvas->SetCanvasSize(600, 600);
    ChannelCanvas->SetWindowSize(600, 600);
    ChannelCanvasShown=true;
    ChannelCanvas->Show();
    ChannelCanvas->Clear();
    ChannelCanvas->Divide(1,2);

    for (int iGain=0;iGain<2;iGain++) {
      char Title[60];
      double ama=0.;
      double ami=0.;

      if (iGain) sprintf(Title, "%s, High Gain", Name);
      else  sprintf(Title, "%s, Low Gain", Name);
 
      dummy1d->SetTitle(Title);
      ChannelCanvas->cd(1+iGain);
      gStyle->SetOptStat(0);
      dummy1d->SetTitleSize(0.04);
      gPad->SetBottomMargin(0.2);
      gPad->SetRightMargin(0.01);
      gPad->SetTopMargin(0.01);
      dummy1d->GetXaxis()->SetTitle("Time, ns");
      dummy1d->GetYaxis()->SetTitle("Response, ADC");
      double T0=Tbunch-200.;
      double T1=Tbunch+1000.;
      dummy1d->SetBins(1200, T0, T1);
      
      for (int i=0; i<SignalLong[iGain].GetNslice(); i++) {
        if (SignalLong[iGain].GetTime(i)>T0 && SignalLong[iGain].GetTime(i)<T1) {
          if (SignalLong[iGain].GetAmp1(i)>ama) ama=SignalLong[iGain].GetAmp1(i);
          if (SignalLong[iGain].GetAmp1(i)<ami) ami=SignalLong[iGain].GetAmp1(i);
        }
      }

      dummy1d->GetYaxis()->SetRangeUser(ami-0.05*(ama-ami), ama+0.05*(ama-ami));      
      dummy1d->Draw();

      for (int i=1; i<SignalLong[iGain].GetNslice(); i++) {
        if (SignalLong[iGain].GetTime(i-1)>T0 && SignalLong[iGain].GetTime(i)<T1) {
          LL->DrawLine(SignalLong[iGain].GetTime(i-1), SignalLong[iGain].GetAmp1(i-1),
                        SignalLong[iGain].GetTime(i),  SignalLong[iGain].GetAmp1(i));
        }
      }
    }
    LL->Delete();
    ChannelCanvas->Update();
    ChannelCanvas->Show();
    printf("Push!");
    int iqq=1;
    cin>>iqq;
    if (iqq==-1) {
      ProcStatus->SetReportMode(-1);
    } else if (iqq==0) {
      exit(-1);
    } else {
      ChannelCanvas->Close();
    }
    
  }

  void Display(ProcessingStatus *ProcStat, CherenkovBuffer *CS) {
    EventHeader *EH = ProcStat->GetEventHeader();
    Dbg=true;
    char TitleC[200];
    double integ0=EH->integ[0];
    double integ1=EH->integ[1];
    double integ2=EH->integ[2];
    double integ3=EH->integ[3];

    sprintf(TitleC, "Run #%d, Burst#%d, Event#%d(%d), 10MHz=%d Integator=(%.1f, %.1f, %.1f, %.1f)", 
	  EH->run, EH->burst, EH->event, EH->burstEvent, EH->scaler10mhz, integ0, integ1, integ2, integ3); 
    TCanvas *ChannelCanvas = new TCanvas(Name, TitleC, 0, 0, 1200, 600);
    printf("[TChannel->SetName] creating new canvas %s %s\n", Name, TitleC);
    ChannelCanvas->Clear();
    ChannelCanvas->Divide(1,2);

    for (int iGain=0;iGain<2;iGain++) {
      char Title[60];
      double ama=0.;
      double ami=0.;

      if (iGain) sprintf(Title, "%s, High Gain", Name);
      else  sprintf(Title, "%s, Low Gain", Name);
      if (Dbg) printf("[TChannel->Display] Gain=%d Title=%s\n", iGain, Title);

      dummy1d->SetTitle(Title);
      ChannelCanvas->cd(1+iGain);
      gStyle->SetOptStat(0);
      dummy1d->SetTitleSize(0.04);
      gPad->SetBottomMargin(0.2);
      gPad->SetRightMargin(0.01);
      gPad->SetTopMargin(0.01);
      dummy1d->Draw();
      dummy1d->GetXaxis()->SetTitle("Time, ns");
      dummy1d->GetYaxis()->SetTitle("Response, ADC");

      ChannelCanvas->Update();
  
      TSignal *Signal = &SignalLong[iGain];
      if (Signal->GetStatus()) {
        ama=Signal->GetMax();
        ami=Signal->GetMin();
        if (Dbg) printf("[TChannel->Display]      %s min=%.2f max=%.2f\n", Signal->GetName(), ami, ama);
      } else {
        for (int iDelay=0; iDelay<2; iDelay++) { 
          Signal = &SignalShort[2*iGain+iDelay];
          if (Signal->GetStatus()) {
            if (Signal->GetMax() > ama) ama=Signal->GetMax();
            if (Signal->GetMin() < ami) ami=Signal->GetMin();
            if (Dbg) printf("[TChannel->Display]      %s min=%.2f max=%.2f\n", Signal->GetName(), ami, ama);
         }
        }
      }
      if (iMod==4 && iGain==1 && iChan==3 && CS->GetCherenkovMax()>ama) ama=CS->GetCherenkovMax();
      if (iMod==4 && iGain==1 && iChan==3 && CS->GetCherenkovMin()<ami) ami=CS->GetCherenkovMin();

      if (iMod==4 && iGain==0 && iChan==3 && CS->GetCherenkovMax()/10.>ama) ama=CS->GetCherenkovMax()/10.;
      if (iMod==4 && iGain==0 && iChan==3 && CS->GetCherenkovMin()/10.<ami) ami=CS->GetCherenkovMin()/10.;

      dummy1d->GetYaxis()->SetRangeUser(1.05*ami, 1.05*ama);
      dummy1d->GetYaxis()->SetTitleSize(.1);
      dummy1d->GetYaxis()->SetTitleOffset(.5);
      dummy1d->GetYaxis()->SetLabelSize(.08);
      dummy1d->GetXaxis()->SetTitleSize(.1);
      dummy1d->GetXaxis()->SetTitleOffset(.75);
      dummy1d->GetXaxis()->SetLabelSize(.08);
      
      dummy1d->DrawCopy();

      for (int i=0; EH->TimeGlobal+i*989.772<50.+25.*EH->nSlice; i++) {
        TLine *lTimeGlobal = new TLine(EH->TimeGlobal+i*989.772, 1.05*ami, 
                                       EH->TimeGlobal+i*989.772, 1.05*ama);
        if (i==0) lTimeGlobal->SetLineWidth(2);
        else      lTimeGlobal->SetLineWidth(1);
        lTimeGlobal->SetLineColor(1);
        lTimeGlobal->SetLineStyle(2);
        lTimeGlobal->Draw("same");
      }
      Signal->SetGraph();
      TGraphErrors *Graph = Signal->GetGraph(); 
      if (Graph->GetN()) { 
       if (iMod==4 && iChan==3) {
          double cherScale=1.;
          if (iGain==0) cherScale=10.;
          for (int i=0; i<39; i++) {
            TLine *lCh = new TLine(CS->GetCherenkovTime(i)-82.481, CS->GetSample(i)/cherScale, 
                                   CS->GetCherenkovTime(i)+82.481, CS->GetSample(i)/cherScale);
            lCh->Draw();
            lCh->SetLineWidth(3);
            lCh->SetLineColor(kRed);
            TLine *lCh1 = new TLine(CS->GetCherenkovTime(i)+82.481, CS->GetSample(i)/cherScale, 
                                    CS->GetCherenkovTime(i)+82.481, CS->GetSample(i+1)/cherScale);
            lCh1->SetLineWidth(3);
            lCh1->SetLineColor(kRed);
            if (i<38) lCh1->Draw();
          } 
        }
        Graph->SetMarkerStyle(21);
        Graph->SetMarkerSize(0.15);
        Graph->SetLineColor(kBlue);
        Graph->SetLineWidth(2);
        Graph->Draw("LPZz same");
      }
      ChannelCanvas->Update();
    }
    Dbg=false;
    ChannelCanvas->Update();
    ChannelCanvas->Show();
    printf("Push!");
    int iqq=1;
    cin>>iqq;
    if (iqq==-1) {
      ProcStatus->SetReportMode(-1);
    } else if (iqq==0) {
      exit(-1);
    } else {
      ChannelCanvas->Close();
    }
 
  }

};


