//#include <classes/TChannel.C>
//#include <classes/TimingCuts.C>

double FitP3(Double_t *x, Double_t *par);

class TModule {
private:
  bool Dbg;
  char Name[40];
  int ID;
  Int_t nSlice;
  int nChannels; // number of channels: 5 for modules (4+sum), 4 for all other
  bool sizeSet;
  int nCut; /* how many is enough for good pedestal estimation */
  TCanvas *RefCanvas;
  TCanvas *FitCanvas;
  TCanvas *BunchCanvas;
  TCanvas *SignalCanvas;
  TCanvas *SignalCanvasNorm;
  TCanvas *HighTOlowCanvas;
  TCanvas *HighVSlowCanvas;
  TH1F    *dummyModule;
  TH1F    *dummyLocal;
  TH1F    *dummy0;        // Signal Display for module
  TGraphErrors *sumShape; // Signal shape for module Sum
  
  double time[512]; // time samples for sum signal;
  double wtim[512];
  bool   HGoverflow;
  bool   LGoverflow;
  double amp1max;
  double amp1min;
  double ampHmax;

  bool dummyModuleSet;
  bool dummyLocalSet;
  TChannel *Channel;
  double Tped0, Tped1; // pedestal window, loaded from ADC_to_CHAN map file;
  double PedWindow0[25], PedWindow1[25]; // pedestal windows, use for Pedestal calculation;
  int  NPedWindows;                      // total number of windows
  int  NGoodWindows;                     // total number of good windows in all channels
  bool GoodPedWindow[25];                // flags useful pedestal windows
  double PedAverageHG[25], PedRMSHG[25], NoiseHG;
  double PedAverageLG[25], PedRMSLG[25], NoiseLG;
  int    PedSamples[25];
  double Txxx[25];   // time from bunch finder;
  double Time[25];   // time form the fit;
  double Tnear[25];   // time form 4-point parabola fit ;
  double Thalf[25];  // width at half peak //
  double TRdown[25]; // half peak to the right //
  double TLdown[25]; // half peak to the left //
  double Anear[25];  // 'peak' near Tbunch //
  double Anois[25];  // 'noise' before Tbunch //
  double Anodd[25];  // 'odd noise' before Tbunch //
  double Sodd[25];  // 'odd pedestal' before Tbunch //
  double Aped[25];  // 'pedestal' before Tbunch //
  double Wbmp1[25];  // bunch weight from TimingCut1;
  bool   Bbmp1[25];  // bunch decision from TimingCut1;
  double Wbmp2[25];  // bunch weight from TimingCut2;
  bool   Bbmp2[25];  // bunch decision from TimingCut2;
  bool GoodBunch[25]; // final bunch decision

  double ChargeIntergationTime; // time for FFT signal intergal
  
  double SumHG[512], rmsHG[512], PedRMSHG0;
  double SumLG[512], rmsLG[512], PedRMSLG0;
  int Nbunchs; // number of bunchs fitted
  int nGoodBunches; // number of good bunches in event;
  double EmptyEventNoise[2];
  double EmptyEventNoiseS0[4];
  double EmptyEventNoiseS1[4];
  double EmptyEventNoiseS2[4];
  bool   isCold; // set to true if a candidate for a saturation //
  bool   HighActivity; 
  bool   HighActivity1; 
  double HighActivityT1; // high activity envelope
  double HighActivityT2;  
  double HighActivityA1; 
  double HighActivityA2; 
  double ResponseDefault; // ADC/proton without buildup 


//-- section with rate cut information --//
  double myRC;
  double myRateCut;
  int myIrc;
  bool rateCutSet;
  bool rateCutPassed[25];
//-- module chanel 'position'
  double channelX[4];
  double channelY[4];
  double CoGX, CoGY;
//--
  int nPulses;
  double attCorr; // correction for the attenuator
  ProcessingStatus* ProcStatus;
/*
  double *moduleSum0; // arrays size 3*nPulses (prehistory bins) case 0 - all events, 1 - warm peds, 2 - quiet past
  double *moduleSum1;
  double *moduleSum2;
  double *cherSum1;
  double *cherSum2;
  double *cherModSum;
*/
   
  TF1 *poly3peak;

  
public:
  TModule () {
    nPulses=0;
    sizeSet=false;
    Dbg=false;
    rateCutSet=false;
    ChargeIntergationTime = 0.;
    for (int i=0; i<25; i++) rateCutPassed[i]=false;
    Channel   = new TChannel[5]; // four readout channels and the Sum;
    nChannels=5;
    for (int i=0;i<5;i++) {
      Channel[i].Off();
      Channel[i].SetID(i);
    }
    for (int i=0;i<25;i++) {GoodPedWindow[i] = false;}
    NPedWindows = 0;
    Nbunchs=0;
    nGoodBunches=0;
    sprintf(Name, "%s", "default");
    dummyModuleSet=false;
    dummyLocalSet=false;
    isCold=false;
    LGoverflow=false;
    HighActivity=false;
    HighActivity1=false;
    HighActivityT1=0.;
    HighActivityT2=0.;
    HighActivityA1=1.0e10;
    HighActivityA2=1.0e10;
    attCorr=1.0;
  }

  void Print() {
    cout << "================" << endl;
    cout << Name << endl;
    cout << "================" << endl;
    for (int i=0;i<5;i++) {
      Channel[i].Print();
    }
  }
  
  int GetNslice() {return nSlice;}
  double GetIntegralTime() {return ChargeIntergationTime;}
 
  void SetID(int id)    {
    ID=id;
    if (id<4) nChannels=5;
    else      nChannels=4;

    ChargeIntergationTime = 125; //ns
    if (id<2) ChargeIntergationTime = 700.;

    for (int iChan=0; iChan<nChannels; iChan++) {
      Channel[iChan].SetModule(ID);
      Channel[iChan].SetChannel(iChan);
    }
    if (id==0) {        // HEC
      channelX[0]=-1.0; channelX[1]=1.0;  channelX[2]=-1.0; channelX[3]= 1.0;
      channelY[0]= 1.0; channelY[1]=1.0;  channelY[2]=-1.0; channelY[3]=-1.0;
      HighActivityT1=150.0e3; // 150us first boundary
      HighActivityT2=500.0e3;   // 2ms second boundary
      HighActivityA1=560.;   // MG ADC sum  first boundary
      HighActivityA2=5600.;  // MG ADC second boundary
      ResponseDefault=1.7;
    } else if (id==1) { // EMEC
      channelX[0]=-1.0; channelX[1]=-1.0; channelX[2]= 1.0; channelX[3]= 1.0;
      channelY[0]= 1.0; channelY[1]=-1.0; channelY[2]= 1.0; channelY[3]=-1.0;
      HighActivityT1=150.0e3; // 150us first boundary
      HighActivityT2=500.0e3;   // 2ms second boundary
      HighActivityA1=560.;   // MG ADC sum  first boundary
      HighActivityA2=5600.;  // MG ADC second boundary
      ResponseDefault=3.8;
    } else if (id==2) { // FCal025
      channelX[0]= 1.0; channelX[1]=-1.0; channelX[2]= 1.0; channelX[3]=-1.0;
      channelY[0]= 1.0; channelY[1]= 1.0; channelY[2]=-1.0; channelY[3]=-1.0;
      HighActivityT1=150.0e3; // 150us first boundary
      HighActivityT2=500.0e3;   // 2ms second boundary
      HighActivityA1=5600.;   // MG ADC sum  first boundary
      HighActivityA2=56000.;  // MG ADC second boundary
      ResponseDefault=8.0;
    } else if (id==3) { // FCal010
      channelX[0]= 1.0; channelX[1]=-1.0; channelX[2]= 1.0; channelX[3]=-1.0;
      channelY[0]= 1.0; channelY[1]= 1.0; channelY[2]=-1.0; channelY[3]=-1.0;
      HighActivityT1=150.0e3; // 150us first boundary
      HighActivityT2=500.0e3;   // 2ms second boundary
      HighActivityA1=10000.;   // MG ADC sum  first boundary
      HighActivityA2=100000.;  // MG ADC second boundary
      ResponseDefault=4.0;
    }
  }
  int  GetID()          {return ID;}

  int  GetNbunchs()      {return Nbunchs;}
  void SetNbunchs(int N) {Nbunchs=N;}

  void SetProcStatus(ProcessingStatus *PS) {
    ProcStatus=PS;
    //printf("[TModule] setting ProcessingStatus object for %s\n", Name);
    for (int i=0;i<nChannels;i++) {
      Channel[i].SetProcStatus(ProcStatus);
    } 
  }

  void ComputeCoG(int bunch) {
    CoGX=0.; 
    CoGY=0.;
    double Sum=0.;
    for (int iChan=0; iChan<nChannels; iChan++) {
      CoGX+=channelX[iChan]*Channel[iChan].GetResponseCALrate(bunch);
      CoGY+=channelY[iChan]*Channel[iChan].GetResponseCALrate(bunch);
      Sum+=Channel[iChan].GetResponseCALrate(bunch);
    }
    
    if (Sum>0.0) {
      CoGX/=Sum;
      CoGY/=Sum;
    } else {
      CoGX=-1000.;
      CoGY=-1000.;
    }
  }

  double GetCoGX() {return CoGX;}
  double GetCoGY() {return CoGY;}

  void SetPreampCut(int iRC, double RC, double rateCut) {
    myIrc=iRC;
    myRC=RC;
    myRateCut=rateCut;
    myRateCut = exp(log(10.0)*rateCut);
    rateCutSet=true;
    for (int i=0; i<25; i++) rateCutPassed[i]=false;
  }

  bool CheckRateCut(int bunch, double Rate) {
    if (!rateCutSet) {
      printf("[TChannel->CheckRateCut] no rate cut set for %s\n", Name);
      rateCutPassed[bunch]=false;
      return false;
    }
    rateCutPassed[bunch]=(Rate<myRateCut);
    return rateCutPassed[bunch];
  }

  bool GetRateCutPassed(int bunch) {
    return rateCutPassed[bunch];
  }

  bool GetRateCutSet()  {return rateCutSet;}
  int  GetRateCutIRC()  {return myIrc;}
  double GetRateCutRC() {return myRC;}
  double GetRateCut()   {return myRateCut;}


  double GetTbunch(int ib) {
    return Txxx[ib];
  }
  void   SetTbunch(int ib, double T) {
    Txxx[ib]=T;
    for (int iCh=0; iCh<nChannels; iCh++) {
      Channel[iCh].SetTbunch(ib, T);
    }
  }

  TCanvas* GetSignalCanvas()   {return SignalCanvas;}
  TCanvas* GetSignalCanvasNorm()   {return SignalCanvasNorm;}

  void     SetBunchCanvas(char *cName, char *Title) {
    BunchCanvas = new TCanvas(cName, Title, 2800, 0, 600, 800);
    printf("[TModule->SetBunchCanvas] creating new canvas %s %s\n", cName, Title);
    
  }
  void     CloseBunchCanvas()   {BunchCanvas->Close();}
  void     SetDummyModule(double x)    {
    dummyModule = new TH1F(Name, Name,  2500, -50., x);
  }
  bool     GetDummyModuleSet() {return dummyModuleSet;}
  void     SetDummyModuleSet(bool set) {dummyModuleSet=set;}




  void SetName(const char* xName) {
    char Name1[50];
    char Name2[50];
    //printf("\n[TModule->SetName] setting names for Module %s and it's channels\n", xName);
    for (int i=0;i<nChannels;i++) { // forward name to channels
      if (i<4) {
        sprintf(Name, "%s#%d", xName, i);
      } else {
        sprintf(Name, "%s Sum", xName);
      } 
      Channel[i].SetName(Name);
    }
    sprintf(Name, "%s", xName);
    sprintf(Name1, "%s_signals", xName);
    sprintf(Name2, "%s signals", xName);
    //printf("[Module->SetName] %s ID=%d\n", Name, ID);
    //int ID2=ID/2;
    if (ID<6) {
      //SignalCanvas = new TCanvas(Name1, Name2, 1900+600*ID2, 600*(ID-ID2*2), 600,  600);
      //SignalCanvas->Iconify();
    }
    sprintf(Name1, "%s_signals_norm", xName);
    sprintf(Name2, "%s signals, Normalized to peak", xName);
    if (ID<6) {
      //SignalCanvasNorm = new TCanvas(Name1, Name2, 600*ID2, 600*(ID-ID2*2), 900,  600);
      //SignalCanvasNorm->Iconify();
    }

    
    sprintf(Name1, "%s dummy", Name1);
    sprintf(Name2, "%s dummy", Name1);
    dummy0 = new TH1F(Name1, Name2, 3500, -250., 250.+25.*254.);

    sprintf(Name1, "%s-p3", Name);
    poly3peak = new TF1(Name1, FitP3, -50., 50., 4);

    dummy0->GetXaxis()->SetTitle("Time, ns");
    dummy0->GetYaxis()->SetTitle("ADC");
    dummy0->GetYaxis()->SetTitleSize(.07);
    dummy0->GetYaxis()->SetTitleOffset(.5);
    dummy0->GetYaxis()->SetLabelSize(.05);
    dummy0->GetXaxis()->SetTitleSize(.07);
    dummy0->GetXaxis()->SetTitleOffset(.75);
    dummy0->GetXaxis()->SetLabelSize(.05);
  }

  void ShowPulseShapes(int Run) {  /* and save relative delays */

    char TitleC[200];
    char NamecC[200];
    sprintf(TitleC, "Pulses for %s, Run#%d", Name, Run);
    sprintf(NamecC, "PulsesFor_%s_Run%d", Name, Run);
    SignalCanvas->Clear();
    SignalCanvas->SetName(NamecC);
    SignalCanvas->SetTitle(TitleC);
    int ndiv=0;
    for (int iChan=0;iChan<nChannels;iChan++) {
      if (Channel[iChan].GetStatus()) ndiv++;
    }
    SignalCanvas->Divide(2, ndiv, 0.01, 0.01);
    SignalCanvas->Show();
    for (int iGain=0; iGain<2; iGain++) {
      //double AmpMax=1.1, AmpMin=-0.5;
      double AmpMax=1.1, AmpMin=-0.2;
      double AmpMax1=1.10*AmpMax-0.10*AmpMin;
      double AmpMin1=1.15*AmpMin-0.15*AmpMax;
      dummy0->GetYaxis()->SetRangeUser(AmpMin1, AmpMax1);
      int iZone=0;

      for (int iChan=0; iChan<nChannels; iChan++) {
        if (Channel[iChan].GetStatus()) {

          TSignal  *SignalLong = Channel[iChan].GetSignalLong(iGain);
          TProfile *ShapeLong = SignalLong->GetPulseShape();
          //TH2D *ShapeLong = SignalLong->GetPulseShape2D();

	  SignalCanvas->cd(1+iZone*2+iGain);
          gStyle->SetOptStat(0);
          gPad->SetBottomMargin(0.2);
          gPad->SetRightMargin(0.01);
          gPad->SetTopMargin(0.01);

          double Tshape0=SignalLong->GetTshape0()-10.;
          double Tshape1=SignalLong->GetTshape1()+10.;
          //dummy0->GetXaxis()->SetRangeUser(Tshape0, Tshape1);
          dummy0->GetXaxis()->SetRangeUser(-50.,100.);
          dummy0->SetTitle(Channel[iChan].GetName());
          dummy0->DrawCopy();

          TLine *L0= new TLine(0., AmpMin1, 0., AmpMax1);
	  L0->SetLineStyle(2);
	  L0->Draw();
          TLine *L1= new TLine(Tshape0, 1., Tshape1, 1.);
	  L1->SetLineStyle(2);
	  L1->Draw();

          double PeakAverage[3]={0.0, 0.0, 0.0};
	  double PeakMax=0.0;
          PeakAverage[0]=SignalLong->GetPeakAverage();
	  PeakMax=PeakAverage[0];
          for (int iDelay=0; iDelay<2; iDelay++) {
            PeakAverage[iDelay+1] = Channel[iChan].GetSignalShort(2*iGain+iDelay)->GetPeakAverage();
	    if (PeakAverage[iDelay+1]>PeakMax) PeakMax=PeakAverage[iDelay+1];
	  }
          
          //printf("[ShowPulseShapes] %10s Chan#%d Gain=%d Peaks=(%6.2f, %6.2f, %6.2f) Max=%6.2f\n", 
	  //	 Name, iChan, iGain, PeakAverage[0], PeakAverage[1], PeakAverage[2], PeakMax);   

          for (int i=0;i<3;i++) PeakAverage[i]/=PeakMax;

          //printf("[ShowPulseShapes]                      new Peaks=(%6.2f, %6.2f, %6.2f)\n", 
	  //	 PeakAverage[0], PeakAverage[1], PeakAverage[2]);   

	  //ShapeLong->Scale(PeakAverage[0]);
	  ShapeLong->SetLineColor(kBlack);
	  ShapeLong->SetLineWidth(3);
	  ShapeLong->SetMarkerColor(kBlack);
	  ShapeLong->DrawCopy("same L");


          for (int iDelay=0; iDelay<2; iDelay++) {
            TSignal *SignalShort = Channel[iChan].GetSignalShort(2*iGain+iDelay);
            TProfile *ShapeShort = SignalShort->GetPulseShape();

            double wSum=0;
            for (int i=1; i<ShapeShort->GetNbinsX(); i++) {
	      wSum+=ShapeShort->GetBinEntries(i);
              if (ShapeShort->GetBinError(i)==0.0 || ShapeShort->GetBinEntries(i)<5.) ShapeShort->SetBinError(i, 0.75);
	    }
            wSum/=ShapeShort->GetNbinsX();

	    ShapeShort->SetLineColor(kBlack+iDelay+1);
	    ShapeShort->SetMarkerColor(kBlack+iDelay+1);
	    ShapeShort->DrawCopy("same");

            poly3peak->SetParameters(0., -1.0e-3, 0.0, 1.0);
	    //poly3peak->FixParameter(0, 0.);
	    poly3peak->SetParLimits(0, -5.0e-5, 5.0e-5);
	    poly3peak->SetParLimits(1, -8.0e-3, 0.0);
	    poly3peak->SetParLimits(2, -5.5e-2, 5.5e-2);
	    poly3peak->SetParLimits(3, 0.50,    1.5);


            double Tfit0=-100., Tfit1=-100.;
            for (int i=0;i<ShapeShort->GetNbinsX() && ShapeShort->GetBinCenter(i)<100.;i++) {
              if (ShapeShort->GetBinContent(i)>0.25 && ShapeShort->GetBinCenter(i)>-25.) {
                if (ShapeShort->GetBinContent(i)>0.30) Tfit1=ShapeShort->GetBinCenter(i);
                if (Tfit0==-100.) Tfit0=ShapeShort->GetBinCenter(i);
	      }
	    }
            
            ShapeShort->Fit("fit3p", "QNE", "", Tfit0, Tfit1);
            double parF[4], perF[4];
            for (int i=0;i<4;i++) {
              parF[i]=poly3peak->GetParameter(i);
              perF[i]=poly3peak->GetParError(i);
	    }
            double p0=poly3peak->GetParameter(0);
            double p1=poly3peak->GetParameter(1);
            double p2=poly3peak->GetParameter(2);
            double p3=poly3peak->GetParameter(3);
            //double p0e=poly3peak->GetParError(0);
            //double p1e=poly3peak->GetParError(1);
            //double p2e=poly3peak->GetParError(2);
            //double p3e=poly3peak->GetParError(3);

            poly3peak->SetParameters(p0, p1, p2, p3);
            poly3peak->SetLineColor(iDelay+2);
            poly3peak->SetLineWidth(1);
            poly3peak->DrawCopy("same");
            double T0=-15., Am=0.;
            for (double t=Tfit0; t<Tfit1; t+=0.001) {
              double f=p3+t*(p2+t*(p1+p0*t));
              if (f>Am) {Am=f; T0=t;}
	    }
            double Tp=T0;
            double Tm=T0;
            for (int i=0;i<4;i++) {
              double t0p=0., t0m=0., Am1=0.;
              parF[i]=parF[i]+perF[i];
              for (double t=Tfit0; t<Tfit1; t+=0.001) {
                double f=parF[3]+t*(parF[2]+t*(parF[1]+parF[0]*t));
                if (f>Am1) {Am1=f; t0p=t;}
	      }
              Am1=0.0;
              parF[i]=parF[i]-2.0*perF[i];
              for (double t=Tfit0; t<Tfit1; t+=0.001) {
                double f=parF[3]+t*(parF[2]+t*(parF[1]+parF[0]*t));
                if (f>Am1) {Am1=f; t0m=t;}
	      }
              parF[i]=parF[i]+perF[i]; // restore the parameter
              if (t0p>Tp) Tp=t0p;
              if (t0p<Tm) Tm=t0p;
              if (t0m>Tp) Tp=t0m;
              if (t0m<Tm) Tm=t0m;
              
	    }
            TLine *L2= new TLine(T0, AmpMin1, T0, AmpMax1);
	    L2->SetLineStyle(3);
	    L2->SetLineColor(iDelay+2);
	    L2->SetLineWidth(2);
	    L2->Draw();
            TLine *L2p= new TLine(Tm, 1.06+0.06*iDelay, Tp, 1.06+0.06*iDelay);
	    L2p->SetLineColor(iDelay+2);
	    L2p->SetLineWidth(2);
	    L2p->Draw();
            //for (int i=0;i<4;i++) {
            //  printf("[ShowPulseShapes]          par(%d)=%f+/-%f\n", i, parF[i], perF[i]);
	    //}
            printf("[ShowPulseShapes] %10s T0=%7.3f [%7.3f, %7.3f] W=%.1f\n", ShapeShort->GetName(), T0, Tm, Tp, wSum);
	  }
         
          SignalCanvas->Update();
	  iZone++;
	}
      }
     
    }
    sprintf(NamecC, "PulsesFor20090619_%s_Run%d.ps", Name, Run);
    //SignalCanvas->Print(NamecC);

  }

  void ClearTrains() { // clear the 'trains' 
    for (int iChan=0;iChan<4;iChan++) {
      if (Channel[iChan].GetStatus()) Channel[iChan].ClearTrains();
    }
  }

  void CountTrains() { // counts the trains of identical raw ADC readings with previous event;
    for (int iChan=0;iChan<4;iChan++) {
      if (Channel[iChan].GetStatus()) Channel[iChan].CountTrains();
    }
  }

  void SaveRaw() { // saves raw amplitudes to compare with next event
    for (int iChan=0;iChan<4;iChan++) {
      if (Channel[iChan].GetStatus()) Channel[iChan].SaveRaw();
    }
  }

  void PrintTrains(int ip) { // prints the trains of identical raw ADC readings with previous event;
                             // ip - details level
    for (int iChan=0;iChan<4;iChan++) {
      if (Channel[iChan].GetStatus()) Channel[iChan].PrintTrains(ip);
    }
  }

  int GetIdentical() {
    int isum=0;
    for (int iChan=0; iChan<4; iChan++) {
      if (Channel[iChan].GetStatus()) {
        isum+=Channel[iChan].GetIdentical();
      }
    }
    return isum;
  }

  bool IsClone() {
    for (int i=0; i<4; i++) {
      if (Channel[i].GetStatus()) {
        if(Channel[i].IsClone()) return true;
      }
    }
    return false;
  }

  bool IsTrain() {
    for (int i=0; i<4; i++) {
      if (Channel[i].GetStatus()) {
        if(Channel[i].IsTrain()) return true;
      }
    }
    return false;
  }

  int DisplayBunch(int bunch, ProcessingStatus *ProcStat) {
    /* displays all four signal from Module @ for a bunch */
    EventHeader *EH = ProcStat->GetEventHeader();
    int ReportMode  = ProcStat->GetReportMode();
    Dbg=false;   
    double Tbunch=GetBunchTime(bunch);

    printf("[DisplayBunch] %10s bunch=%d ============================\n", Name, bunch);

    char TitleC[200];
    char NamecC[200];
    sprintf(TitleC, "%s Run#%d, Burst#%d, Event#%d(%d), Bunch#%d Type=%02d Integator=(%.1f, %.1f, %.1f, %.1f)", 
          Name, 
	  EH->run, EH->burst, EH->event, EH->burstEvent, bunch, EH->pattern[0], 
          EH->integ[0], EH->integ[1], EH->integ[2], EH->integ[3]); 
    sprintf(NamecC, "%s_Run%d_Burst%d_Event%d_%d_%d", Name, 
          EH->run, EH->burst, EH->event, EH->burstEvent, bunch); 

    SignalCanvas->Clear();
    SignalCanvas->SetName(NamecC);
    SignalCanvas->SetTitle(TitleC);
    int ndiv=0;
    for (int iChan=0;iChan<nChannels;iChan++) {
      if (Channel[iChan].GetStatus()) ndiv++;
    }
    SignalCanvas->Divide(2, ndiv, 0.01, 0.01);
    SignalCanvas->Show();

    for (int iGain=0; iGain<2; iGain++) {
      printf("[DisplayBunch] Gain=%d\n", iGain);

      double T0=Tbunch-200.0;
      double T1=Tbunch+500.0;
      dummy0->GetXaxis()->SetRangeUser(T0, T1);
   
      int iZone=0;
      for (int iChan=0; iChan<nChannels; iChan++) {
        if (Channel[iChan].GetStatus()) {

	  SignalCanvas->cd(1+iZone*2+iGain);
          gStyle->SetOptStat(0);
          gPad->SetBottomMargin(0.2);
          gPad->SetRightMargin(0.01);
          gPad->SetTopMargin(0.01);

          TSignal *Signal = Channel[iChan].GetGoodSignal(iGain);
          double ama=Signal->GetMaxLocal(bunch);
          double ami=Signal->GetMinLocal(bunch);

          double Tpeak  = Signal->GetTpMax(bunch);     // parabola peak
          double Tlocal = Signal->GetTmaxLocal(bunch); // time of maximum sample
          double Tfoot  = Signal->GetTfoot(bunch);     // signal 'start'
          double Tcross = Signal->GetTcross(bunch);    // zero crossing after peak
          printf("[DisplayBunch]   GoodSignal->%25s Max=%6.1f Min=%6.1f Tup=%6.1f Tdown=%6.1f Parab=%d\n", 
                 Signal->GetName(), ama, ami, Tfoot-Tpeak, Tcross-Tpeak, Signal->GetParabStatus(bunch));


          if (ama<10.)  ama=10.;
          if (ami>-10.) ami=-10.;
          dummy0->GetYaxis()->SetRangeUser(1.1*ami, 1.1*ama);
          dummy0->SetTitle(Channel[iChan].GetName());
          dummy0->DrawCopy();

          if (iZone==0 && iGain==0) {
            sprintf(TitleC, "Run#%d, Burst#%d, Event#%d(%d), Type=%02d", 
                    EH->run, EH->burst, EH->event, EH->burstEvent, EH->pattern[0]); 
            TLatex *tl0 = new TLatex(30., 1.10*ami+0.02*ama, TitleC);
            tl0->SetTextSize(0.05);
	    tl0->Draw();
            sprintf(TitleC, "Integator=(%.1f, %.1f, %.1f, %.1f)", 
                    EH->integ[0], EH->integ[1], EH->integ[2], EH->integ[3]); 
            TLatex *tl1 = new TLatex(30., 1.10*ami-0.10*ama, TitleC);
            tl1->SetTextSize(0.05);
	    tl1->Draw();
	  }

          TLine *lBunch = new TLine(Tbunch, ami, Tbunch, ama);
          lBunch->SetLineColor(kBlue);
          lBunch->SetLineWidth(1);
          lBunch->Draw();
          TLine *lPeak = new TLine(Tpeak, ami/1.25, Tpeak, ama/1.25);
          lPeak->SetLineColor(kRed);
          lPeak->SetLineWidth(2);
          lPeak->Draw();
          TLine *lLocal = new TLine(Tlocal, ami/1.5, Tlocal, ama/1.5);
          lLocal->SetLineColor(kGreen);
          lLocal->SetLineWidth(2);
          lLocal->Draw();
          TLine *lFoot = new TLine(Tfoot, ami/1.75, Tfoot, ama/1.75);
          lFoot->SetLineColor(kGreen);
          lFoot->SetLineWidth(3);
          lFoot->Draw();
          TLine *lCross = new TLine(Tcross, ami/1.75, Tcross, ama/1.75);
          lCross->SetLineColor(kGreen);
          lCross->SetLineWidth(3);
          lCross->Draw();

          if (Dbg) printf("[DisplayBunch] %s parabola status=%d\n", 
                         Signal->GetName(), Signal->GetParabStatus(bunch));
  
          for (int ib=bunch-3; ib<bunch+6; ib++) {
            double tb=Tbunch+ib*989.772/6.;
            if (tb>T0 && tb<T1 && ib!=0) {
              TLine *lBlank= new TLine(tb, ami, tb, ama);
              lBlank->SetLineWidth(1);
              lBlank->SetLineStyle(3);
              lBlank->Draw();
 	    }
          }

          for (int iw=0;iw<NPedWindows; iw++) {
            int iCol;
            if (Signal->GetPedWinX(iw)) iCol=kYellow; else iCol=kRed;
            double t0=PedWindow0[iw];
            double t1=PedWindow1[iw];
            if  ((t1>T0 && t1<T1) || (t0>T0 && t0<T1)) { 
              if (t0<T0) t0=T0;
              if (t1>T1) t1=T1;
              TLine *lWin = new TLine(t0, 0., t1, 0.);
              lWin->SetLineWidth(30);
              lWin->SetLineColor(iCol);
              lWin->Draw("same");
            }
	  }

          TGraphErrors *Graph = Signal->GetGraph();
	  if (Graph->GetN()) {
            int iColor=1;
	    Graph->SetMarkerColor(iColor);
	    Graph->SetMarkerStyle(21);
	    Graph->SetMarkerSize(0.15);
	    Graph->SetLineColor(iColor);
            Graph->Draw("LPZz same");
	  } else {
	    if (Dbg || ReportMode==10006) printf("[DisplayBunch] Signal %s has no Graph!\n", Signal->GetName());
	  }
          SignalCanvas->Update();
          iZone++;
	}
      }
    }


    SignalCanvas->Update();
    int Result=ReportMode;
 
    Dbg=false;
    return Result;
  }

    
  int DisplaySignals(ProcessingStatus *ProcStat) {
    
    EventHeader *EH = ProcStat->GetEventHeader();
    int ReportMode  = ProcStat->GetReportMode();
    Dbg=false;   

    char TitleC[200];
    char NamecC[200];
    sprintf(TitleC, "%s Run#%d, Burst#%d, Event#%d(%d), Type=%02d Integator=(%.1f, %.1f, %.1f, %.1f)", 
          Name, 
	  EH->run, EH->burst, EH->event, EH->burstEvent, EH->pattern[0], 
          EH->integ[0], EH->integ[1], EH->integ[2], EH->integ[3]); 
    sprintf(NamecC, "%s_Run%d_Burst%d_Event%d_%d", Name, 
          EH->run, EH->burst, EH->event, EH->burstEvent); 

    if (Dbg) printf("[TModule->DisplaySignals] %s\n", TitleC);

    SignalCanvas->Clear();
    SignalCanvas->SetName(NamecC);
    SignalCanvas->SetTitle(TitleC);
    int ndiv=0;
    for (int iChan=0;iChan<nChannels;iChan++) {
      if (Channel[iChan].GetStatus()) ndiv++;
    }
    SignalCanvas->Divide(2, ndiv, 0.01, 0.01);
    SignalCanvas->Show();

    for (int iGain=0; iGain<2; iGain++) {
      double AmpMax=5.0, AmpMin=-5.0;
      for (int iChan=0; iChan<nChannels; iChan++) {
        TSignal *Signal = Channel[iChan].GetSignalLong(iGain);
        if (Channel->GetStatus()) {
          double ama=Signal->GetMax();
          double ami=Signal->GetMin();
          if (ama>AmpMax) AmpMax=ama; 
          if (ami<AmpMin) AmpMin=ami; 
	}
      }
      double AmpMax1=1.10*AmpMax-0.10*AmpMin;
      double AmpMin1=1.15*AmpMin-0.15*AmpMax;
      dummy0->GetYaxis()->SetRangeUser(AmpMin1, AmpMax1);
      dummy0->GetXaxis()->SetRangeUser(-50.0, 6400.0);
   
      int iZone=0;
      for (int iChan=0; iChan<nChannels; iChan++) {
        if (Channel[iChan].GetStatus()) {


	  SignalCanvas->cd(1+iZone*2+iGain);
          gStyle->SetOptStat(0);
          gPad->SetBottomMargin(0.2);
          gPad->SetRightMargin(0.01);
          gPad->SetTopMargin(0.01);


          TSignal *Signal = Channel[iChan].GetSignalLong(iGain);
          dummy0->SetTitle(Channel[iChan].GetName());
          dummy0->DrawCopy();

          if (iZone==0 && iGain==0) {
            sprintf(TitleC, "Run#%d, Burst#%d, Event#%d(%d), Type=%02d", 
                    EH->run, EH->burst, EH->event, EH->burstEvent, EH->pattern[0]); 
            TLatex *tl0 = new TLatex(30., 1.10*AmpMin+0.02*AmpMax, TitleC);
            tl0->SetTextSize(0.05);
	    tl0->Draw();
            sprintf(TitleC, "Integator=(%.1f, %.1f, %.1f, %.1f)", 
                    EH->integ[0], EH->integ[1], EH->integ[2], EH->integ[3]); 
            TLatex *tl1 = new TLatex(30., 1.10*AmpMin-0.10*AmpMax, TitleC);
            tl1->SetTextSize(0.05);
	    tl1->Draw();
	  }

          for (int iw=0;iw<NPedWindows; iw++) {
            int iCol;
            if (Signal->GetPedWinX(iw)) iCol=kYellow; else iCol=kRed;
            TLine *lWin = new TLine(PedWindow0[iw], 0., PedWindow1[iw], 0.);
            lWin->SetLineWidth(30);
            lWin->SetLineColor(iCol);
            lWin->Draw("same");
	  }

          TGraphErrors *Graph = Signal->GetGraph();
	  if (Graph->GetN()) {
            int iColor=1;
	    Graph->SetMarkerColor(iColor);
	    Graph->SetMarkerStyle(21);
	    Graph->SetMarkerSize(0.15);
	    Graph->SetLineColor(iColor);
            Graph->Draw("LPZz same");
	  } else {
	    if (Dbg || ReportMode==10006) printf("[DisplaySignals] %s has no Graph!\n", Signal->GetName());
            for (int iDelay=0; iDelay<2; iDelay++) {
              TSignal *Signal1 = Channel[iChan].GetSignalShort(2*iGain+iDelay);
	      Signal1->SetGraph();
              TGraphErrors *Graph1 = Signal1->GetGraph();
	      if (Graph1->GetN()) {
                int iColor=1+iDelay;
	        Graph1->SetMarkerColor(iColor);
	        Graph1->SetMarkerStyle(21);
	        Graph1->SetMarkerSize(0.15);
	        Graph1->SetLineColor(iColor);
                Graph1->Draw("LPZz same");
	      } else {
		if (Dbg || ReportMode==10006) printf("[DisplaySignals] %s has no Graph!\n", Signal1->GetName());
	      }
	    }
	  }

          iZone++;
	}
      }
    }


    SignalCanvas->Update();
    int Result=ReportMode;
    Dbg=false;
    return Result;
  }

  void SetTime(int i, double t) {
    if (i<0) return;
    if (i>511) return;
    time[i]=t;
    return;
  }

  double GetTime(int i) {
    if (i<0) {
      printf("[TModule->GetTime] %s time#%d requested, wrong index\n", Name, i);
      exit(-1);
      return -1000.;
    } else if (i>511) {
      printf("[TModule->GetTime] %s time#%d requested, wrong index\n", Name, i);
      exit(-1);
      return -1000.;
    }
    return time[i];
  }

  void AccumulateTime(int i, double t) {
    if (i<0) {
      printf("[TModule->AccumulateTime] %s time#%d requested, wrong index\n", Name, i);
      exit(-1);
      return;
    } else if (i>511) {
      printf("[TModule->AccumulateTime] %s time#%d requested, wrong index\n", Name, i);
      exit(-1);
      return;
    }
    time[i]+=t;
    wtim[i]+=1.0;
    return;
  }

  void ComputeTimeSlices() {
    //printf("[TModule->ComputeTimeSlices] %10s ", Name);
    for (int i=0; i<512; i++) {
      if (wtim[i]>0.) {
        time[i]/=wtim[i];
      }
    }
    
    for (int iG=0; iG<2; iG++) {
      TSignal* SignalLong   = Channel[4].GetSignalLong(iG);
      TSignal* SignalShort0 = Channel[4].GetSignalShort(2*iG);
      TSignal* SignalShort1 = Channel[4].GetSignalShort(2*iG+1);
      for (int i=0; i<SignalLong->GetNslice();   i++)  SignalLong  ->SetTime(i, time[i]);
      for (int i=0; i<SignalShort0->GetNslice(); i++)  SignalShort0->SetTime(i, time[2*i]);
      for (int i=0; i<SignalShort0->GetNslice(); i++)  SignalShort1->SetTime(i, time[2*i+1]);
    }
/*
    int iqq=0;
    printf("Push!");
    cin>>iqq;
    if (iqq==0) exit(-1);
*/
    return;
  }

  void ClearTime() {
    for (int i=0; i<512; i++) {
      time[i]=0.;
      wtim[i]=0.;
    }
    return;
  }

  void SetPedestalWindowsNew() { // uses RF-based pedestal windows //
    EventHeader *EH = ProcStatus->GetEventHeader();
    int Run = EH->run;
    if (ID==6) { // Integrator //
      NPedWindows=1;
      GoodPedWindow[0]=true;
      GoodPedWindow[1]=true;
      if (Run>300 && Run<344) {
        PedWindow0[0]=0.;
        PedWindow1[0]=5300.;
        PedWindow0[1]=5430.; // index #1 is used to store 'signal' interval for integrator
        PedWindow1[1]=5510.;
      } else if (Run<460) {
        PedWindow0[0]=0.;
        PedWindow1[0]=5300.;
        PedWindow0[1]=5450.; // index #1 is used to store 'signal' interval for integrator
        PedWindow1[1]=6400.;
      } else {
        PedWindow0[0]=0.;
        PedWindow1[0]=5300.;
        PedWindow0[1]=5700.; // index #1 is used to store 'signal' interval for integrator
        PedWindow1[1]=6400.;
      }
      for (int iChan=0;iChan<nChannels;iChan++) {
	Channel[iChan].SetPedestalWindow(0, true, PedWindow0[0], PedWindow1[0]);
      }
      NGoodWindows=NPedWindows;
      if (!NGoodWindows) {
        printf("[Module->SetPedestalWindowsNew] ***** No good windows found in %s, Run#%d Event#%d Burst#%d BurstEvent#%d\n", 
             Name, Run, EH->event, EH->burst, EH->burstEvent); 
      }
      return;
    }
// all other modules
    NPedWindows=ProcStatus->GetPedWindows();
//    int nBunches=ProcStatus->GetNbunches();
//    int nFilled =ProcStatus->GetNfilled();
    double tRF=EH->GetRFperiod();
    double t00=EH->GetTimeGlobal();
    int iOff=ProcStatus->GetEventHeader()->GetFirstBunch();
     //printf("[Module->SetPedestalWindowsNew] setting %d windows for %s t=[%6.1f, %6.1f], Tstart=%6.1f Tend=%6.1f\n", 
    //       NPedWindows, Name, time[0], time[nSlice-1], Tped0, Tped1);
    
    for (int iw=0; iw<NPedWindows; iw++) {
      GoodPedWindow[iw]=true;
      PedWindow0[iw]=ProcStatus -> GetPedWindow0(iw);
      PedWindow1[iw]=ProcStatus -> GetPedWindow1(iw);
      //if (ID==4) printf("[Module->SetPedestalWindowsNew] window#%2d initial  T1=%6.1f T2=%6.1f ", iw, PedWindow0[iw], PedWindow1[iw]);
      if (PedWindow1[iw]<time[0] || PedWindow0[iw]>time[nSlice-1]) {
        GoodPedWindow[iw]=false;
      }
      //if (ID==4) printf(" T0=%.0f\t T1=%.0f\t ", Tped0, Tped1);
      if (PedWindow0[iw]<time[0])        PedWindow0[iw]=2.0*time[0]-time[1];
      if (PedWindow1[iw]>time[nSlice-1]) PedWindow1[iw]=2.0*time[nSlice-1]-time[nSlice-2];
      if (PedWindow1[iw]-PedWindow0[iw] > Tped0-Tped1) {
        PedWindow0[iw]=PedWindow1[iw]-(Tped0-Tped1);
      }
 // extend pedestal window FORWARD if distance to PREVIOUS signal > 1200ns
      
      int irx=int((PedWindow0[iw]-t00)/tRF+0.5);
      int iFound=-1000;
      for (int irf=irx; irf>-30; irf--) {
        //double tb=t00+irf*tRF;
        int    ib=irf+iOff;
        while (ib<0)  ib+=30;
        while (ib>29) ib-=30;
        
        if (ProcStatus->GetBunchAction(ib)==1) { // filled bunch!
          iFound=irf;
          break;
        }
      }
      if (iFound==-1000) {
        printf("[Module->SetPedestalWindowsNew] ******* no filled bunch found in front of window#%d\n", iw);
        exit(-1);
      }
      double tFound=t00+iFound*tRF;
      //if (ID==1) printf("[Module->SetPedestalWindowsNew] -> found filled bunch #%d(%d) Tb=%6.1f for window#%d (T1=%6.1f T2=%6.1f)\n",
      //          iFound, (iFound+iOff)%30, t00+iFound*tRF, iw, PedWindow0[iw], PedWindow1[iw]);
      if (PedWindow0[iw]>1200.+tFound) {
        PedWindow0[iw]=(1200.+tFound+2.0*PedWindow0[iw])/3.0; 
        //if (ID==1) printf("[Module->SetPedestalWindowsNew]         window extended to (T1=%6.1f T2=%6.1f)\n",
        //                      PedWindow0[iw], PedWindow1[iw]);
        if (PedWindow0[iw]<time[0]-15.) PedWindow0[iw]=time[0]-15.;
      }

      if (PedWindow0[iw]-tFound<1000.)        PedWindow0[iw]=tFound+1000.; // too little room for signal
      if (PedWindow1[iw]-PedWindow0[iw]<310.) GoodPedWindow[iw]=false;               // ped window too short
      else if (iw==0 && PedWindow1[iw]-PedWindow0[iw]<400.) GoodPedWindow[iw]=false; // ped window too short 

      if (ProcStatus->GetReportMode()==1 && ID==0) {
        printf("[Module->SetPedestalWindowsNew] %2d t0=%6.0f width=%4.0f good=%d\n", 
               iw, PedWindow0[iw], PedWindow1[iw]-PedWindow0[iw], GoodPedWindow[iw]); 
      }
    }

    int jw=0;
    for (int iw=0; iw<NPedWindows; iw++) { // eliminate short pedestal windows from the list
      if (GoodPedWindow[iw]) {
        PedWindow0[jw]=PedWindow0[iw];
        PedWindow1[jw]=PedWindow1[iw];
        GoodPedWindow[jw]=true;
        jw++;
      }
    }
    NPedWindows=jw; 
    
    for (int iw=0; iw<NPedWindows; iw++) {  
      for (int iChan=0;iChan<nChannels;iChan++) {
        Channel[iChan].SetPedestalWindow(iw, GoodPedWindow[iw], PedWindow0[iw], PedWindow1[iw]);
      } // channel loop
    }
    ProcStatus->FilterBunches(PedWindow1[0]-5., PedWindow0[NPedWindows-1]+5.);
    return;
  }

void PerformTailFit() {
if (NPedWindows==0) {
  printf("\n\n[TModule->PerformTailFit] no pedestal windows found, quitting\n\n");
  exit(-1);
}
for (int iw=0; iw<NPedWindows; iw++) {
  for (int iCh=0; iCh<nChannels; iCh++) {
    GoodPedWindow[iw] = GoodPedWindow[iw] && Channel[iCh].PerformTailFit(iw);
  }
}
return;
}

void PrepareTailed() {
  for (int iCh=0; iCh<nChannels; iCh++) {
    Channel[iCh].PrepareTailed();
  }
}

void DoFFTshape() {
  for (int iCh=0; iCh<nChannels; iCh++) {
    Channel[iCh].DoFFTshape();
  }
}

void DoFFTshape(int iCh) {
  if (iCh<0 || iCh>4) {
    printf("[TModule->DoFFTshape(int)] wrong channel#%d\nBye!\n", iCh);
    exit(-1);
  }
  Channel[iCh].DoFFTshape();
}

void FixFFTbaseline() {
  for (int iCh=0; iCh<nChannels; iCh++) {
    Channel[iCh].FixFFTbaseline();
  }
}

   void SubtractCoherentNoiseFFT(TSignal* SignalX, int iCh0, int iCh1, double FFTwidthCut) {
      //if (ProcStatus->GetReportMode()==1) {
      //  printf("[TModule->SubtractCoherentNoiseFFT] for %s (source: %s), channels %d to %d\n", 
      //         Name, SignalX->GetName(), iCh0, iCh1);
      //}
// subtracts coherent noise determined by SignalX (long HG from FCal at HV=0)
// now get to the Signals we want to fix !
    for (int iCh=iCh0; iCh<iCh1; iCh++) {
      for (int iGain=0; iGain<2; iGain++) {
        TSignal *SignalLong=Channel[iCh].GetSignalLong(iGain);
/* transfer ModuleX signal to Module signal */
        for (int is=0; is<nSlice/2+1; is++) { // copy Amplitude and Phase from SignalX to Signal as noise reference signal
          SignalLong->SetFFTpointExtReal(is, SignalX->GetFFTamp(is), SignalX->GetFFTpha(is));
        }
        SignalLong->SetFFTwidthCut(FFTwidthCut);
        SignalLong->ComputeFFTcorrection();
/* subtract coherent noise from both LONG and SHORT signals!*/
        TSignal *SignalShort0=Channel[iCh].GetSignalShort(2*iGain);
        TSignal *SignalShort1=Channel[iCh].GetSignalShort(2*iGain+1);
        for (int i=0; i<nSlice/2; i++) {
          SignalShort0->SetAmp1(i, SignalLong->GetAmp1(i*2));
          SignalShort1->SetAmp1(i, SignalLong->GetAmp1(i*2+1));
        } // delay
      }  // gain
    }   // channel    
  }

  double GetRawVariance(int iGain) {
    if (iGain<0 || iGain>1) {
      printf("[TModule->GetRawVariance] wrong gain#%d requested", iGain);
      exit(-1); 
    }
    double result=0.0;
    double s0=0.0;
    double s2=0.0;
    double s1=0.0;
    for (int i=0; i<nSlice; i++) {
      double a=0.0;
      for (int iCh=0; iCh<4; iCh++) {
        a+=Channel[iCh].GetSignalLong(iGain)->GetAmp2(i);
      }
      s0+=1.0;
      s1+=a;
      s2+=a*a;
    }
    s1/=s0;
    s2/=s0;
    s2-=s1*s1;
    if (s2>0.0) s2=sqrt(s2);
    else        s2=0.0;
    result=s2;
    return result;
  }  

  double GetFFTVariance(int iGain) {
    if (iGain<0 || iGain>1) {
      printf("[TModule->GetRawVariance] wrong gain#%d requested", iGain);
      exit(-1); 
    }
    double result=0.0;
    double s0=0.0;
    double s2=0.0;
    double s1=0.0;
    for (int i=0; i<nSlice; i++) {
      double a=0.0;
      for (int iCh=0; iCh<4; iCh++) {
        a+=Channel[iCh].GetSignalLong(iGain)->GetAmp1(i);
      }
      s0+=1.0;
      s1+=a;
      s2+=a*a;
    }
    s1/=s0;
    s2/=s0;
    s2-=s1*s1;
    if (s2>0.0) s2=sqrt(s2);
    else        s2=0.0;
    result=s2;
    return result;
  }  

   int SetPedestalWindows(double TG, ProcessingStatus *ProcStat) {
    
    //if (ID==0) Dbg=true;
    //else Dbg=false;
    EventHeader *EH = ProcStat->GetEventHeader();
    int ReportMode  = ProcStat->GetReportMode();
    int Run = EH->run;
    if (ID==6) { // Integrator //
      NPedWindows=1;
      GoodPedWindow[0]=true;
      GoodPedWindow[1]=true;
      if (Run>300 && Run<344) {
        PedWindow0[0]=0.;
        PedWindow1[0]=5300.;
        PedWindow0[1]=5430.; // index #1 is used to store 'signal' interval for integrator
        PedWindow1[1]=5510.;
      } else if (Run<460) {
        PedWindow0[0]=0.;
        PedWindow1[0]=5300.;
        PedWindow0[1]=5450.; // index #1 is used to store 'signal' interval for integrator
        PedWindow1[1]=6400.;
      } else {
        PedWindow0[0]=0.;
        PedWindow1[0]=5300.;
        PedWindow0[1]=5700.; // index #1 is used to store 'signal' interval for integrator
        PedWindow1[1]=6400.;
      }
      for (int iChan=0;iChan<nChannels;iChan++) {
	Channel[iChan].SetPedestalWindow(0, true, PedWindow0[0], PedWindow1[0]);
      }
      NGoodWindows=NPedWindows;
      if (!NGoodWindows) {
        printf("[Module->SetPedestalWindows] ***** No good windows found in %s, Run#%d Event#%d Burst#%d BurstEvent#%d\n", 
             Name, Run, EH->event, EH->burst, EH->burstEvent); 
      }
    } else {
      NPedWindows=0;
      for (int i=-15; i<15; i++) {
        double th, tl;
        tl=TG-Tped0+989.772*i;
        th=TG-Tped1+989.772*i;

        if (i==0 && Dbg) printf (" [Module->SetPedestalWindows] *** Mod#%d t1-t0=%f t2-t0=%f\n", ID, tl-TG, th-TG);
        //printf (" [Module->SetPedestalWindows] *** Mod#%d nSlice=%d t1-t0=%f t2-t0=%f\n", ID, EH->nSlice, tl-TG, th-TG);
                

        if (th>0.0 && tl<25.*EH->nSlice) {
          if (tl<0.0) tl=0.0;
          if (th>25.*(EH->nSlice-1)) th=25.*(EH->nSlice-1);
          if (Dbg || ReportMode==10002) { 
            printf("[Module->SetPedestalWindows] %s  nSlice=%d ped window#%d %f-%f\n", Name, EH->nSlice, i, tl, th);           	   }
          PedWindow0[NPedWindows]=tl;
          PedWindow1[NPedWindows]=th;
          GoodPedWindow[NPedWindows]=true;

          for (int iChan=0;iChan<nChannels;iChan++) {
	    if (Channel[iChan].GetStatus()) {
              Channel[iChan].SetPedestalWindowPre2012(NPedWindows,tl,th);
	    }
	  }

          NPedWindows++;
        }
      }
      //NPedWindows--;

      if (Dbg || ReportMode==10012) printf("[Module->SetPedestalWindows] %d windows for Mod#%d\n", NPedWindows, ID);
      
    }

    NGoodWindows=NPedWindows;
    if (ID!=6) {
      int NPedWindowsLeft=NPedWindows;
      if (Dbg) printf ("[Module->SetPedestalWindows] %s starting with %d PedWindows\n", 
              Name, NGoodWindows);
      for (int iChan=0;iChan<nChannels;iChan++) {
	if (Channel[iChan].GetStatus()) {
          NPedWindowsLeft=Channel[iChan].CleanPedestalWindows();
          if (NPedWindowsLeft<NGoodWindows) {
            NGoodWindows=NPedWindowsLeft;
	  }
          if (Dbg) {
            printf(" [Module->SetPedestalWindows] %s after checking Chan#%d -> %d left\n", 
            Name, iChan, NGoodWindows);
            printf("------------------------\n");
	  }
	}
      }

      for (int iw=0;iw<NPedWindows;iw++) {
        GoodPedWindow[iw]=true;
        for (int iChan=0;iChan<nChannels;iChan++) {
	  if (Channel[iChan].GetStatus()) {
            if (!Channel[iChan].GetPedWinX(iw)) GoodPedWindow[iw]=false; 
	  }
	}
      }

    }

    if (!NGoodWindows) {
      printf("[Module->SetPedestalWindows] ***** No good windows found in %s, Run#%d Event#%d Burst#%d BurstEvent#%d\n", 
             Name, Run, EH->event, EH->burst, EH->burstEvent);
      //exit(-1); 
    }
    
    
    Dbg=false;
    return NGoodWindows;

  }

  void SetFFTsize(int nSampleTail125) {
    if (nSampleTail125%4 != 0) {
      printf("[Module->SetFFTsize] bad nSampleTail125=%d\n", nSampleTail125);
      exit(-1);
    }
    for (int iChan=0;iChan<nChannels;iChan++) {
      Channel[iChan].SetFFTsize(nSampleTail125);
    }    
  }

  void SetFFTsize(int iChan, int nSampleTail125) {
    if (nSampleTail125%4 != 0) {
      printf("[Module->SetFFTsize] bad nSampleTail125=%d\n", nSampleTail125);
      exit(-1);
    }
    if (iChan<0 || iChan>4) {
      printf("[Module->SetFFTsize] bad iChan=%d\n", iChan);
      exit(-1);
    }
    Channel[iChan].SetFFTsize(nSampleTail125);
  }

  int GetFFTsize(int iChan) {
    if (iChan<0 || iChan>4) {
      printf("[Module->GetFFTsize] bad channel number %d\n", iChan);
      exit(-1);
    }
    return Channel[iChan].GetFFTsize();
  }

  void SetFFTlongALL(int i, double gain, double phi) {
// sets all Transfer function point to the same value for 12.5ns sampling (long) signal 
    int NChan=5;
    if (ID>3) NChan=4; 
    for (int iChan=0; iChan<NChan; iChan++) {
//      printf("[TModule->SetFFTlongALL] %20s #%04d gain=%5.2f phase=%5.3f\n",
//             Channel[iChan].GetName(), i, gain, phi);
      Channel[iChan].SetFFTlongALL(i, gain, phi);
    }        
  }

  void SetFFTshortALL(int i, double gain, double phi) {
// sets all Transfer function point to the same value for 25ns sampling (short) signal  
    int NChan=5;
    if (ID>3) NChan=4; 
    for (int iChan=0; iChan<NChan; iChan++) {
      Channel[iChan].SetFFTshortALL(i, gain, phi);
    }        
  }

  void SetFFTlong(int iChan, int i, double gain, double phi) {
// sets one channel Transfer function point to the same value for 12.5ns sampling (long) signal 
    if (iChan<0 || iChan>4 || (ID>3 && iChan>3) ) {
      printf("[Module->SetFFTlong] wrong channel number for %s -> %d\n", Name, iChan);
      exit(-1);
    } 
    Channel[iChan].SetFFTlongALL(i, gain, phi);
  }
  void SetFFTshort(int iChan, int i, double gain, double phi) {
// sets one channel Transfer function point to the same value for 25ns sampling (long) signal 
    if (iChan<0 || iChan>4 || (ID>3 && iChan>3) ) {
      printf("[Module->SetFFTshort] wrong channel number for %s -> %d\n", Name, iChan);
      exit(-1);
    } 
    Channel[iChan].SetFFTshortALL(i, gain, phi);
  }


  void SetFFTtailALL(double Te, double Tc) {
// sets all Te and Tc for tail parametrization to same value 
    for (int iChan=0; iChan<nChannels; iChan++) {
      Channel[iChan].SetFFTtailALL(Te, Tc);
    }        
  }

  void SetTransferSizeALL(int n1, double df) {
    for (int iChan=0; iChan<nChannels; iChan++) { 
      Channel[iChan].SetTransferSizeALL(n1, df);
    }        
  }
  void SetTransferPointALL(int i, double gain, double phi) {
    for (int iChan=0; iChan<nChannels; iChan++) { 
      Channel[iChan].SetTransferPointALL(i, gain, phi);
    }        
  }

  void ConvertTransferFuction() {
    for (int iChan=0; iChan<nChannels; iChan++) { 
      Channel[iChan].ConvertTransferFuction();
    }        
  }

  void SubtractPedestals() {
    for (int iChan=0; iChan<nChannels; iChan++) {
      if (Channel[iChan].GetStatus()) {
        Channel[iChan].SubtractPedestals();
      }
    }
  }

  void ComputeFFTAmpPhase() {
    for (int iChan=0; iChan<nChannels; iChan++) {
      if (Channel[iChan].GetStatus()) {
        //printf("[TModule->ComputeFFTAmpPhase] computing spectrum for %s chan#%d\n", Name, iChan);
        Channel[iChan].ComputeFFTAmpPhase();
      }
    }
  }

   void PedTailCorrection() {
    for (int iChan=0;iChan<nChannels;iChan++) {
      if (Channel[iChan].GetStatus()) {
        Channel[iChan].PedTailCorrection();
      }
    }
  }

  bool CheckHGoverflow() {return HGoverflow;}
  bool CheckLGoverflow() {return LGoverflow;}

  void ComputeSumPulse() {
    //printf("[Module->ComputeSumPulse] computing sum signals for %s\n", Name);
    amp1max=-1000000.;
    ampHmax=-1000000.;
    amp1min=1000000.;
    attCorr=ProcStatus->GetRunHeader()->AttnCorr;
    
    TSignal *SignalSumH=Channel[4].GetSignalLong(1);
    TSignal *SignalSumL=Channel[4].GetSignalLong(0);

    HGoverflow=false;
    LGoverflow=false;

    for (int iChan=0; iChan<4; iChan++) {
      if (Channel[iChan].HighGainOverflowDetected()) HGoverflow=true;
      if (Channel[iChan].LowGainOverflowDetected())  LGoverflow=true;
    }
 
    //printf("[Module->ComputeSumPulse] HGoverfow=%d LGoverfow=%d\n", HGoverflow, LGoverflow);

    for (int is=0; is<nSlice; is++) {
      double ah=0.;
      double al=0.;
      double ahI=0.;
      double alI=0.;
      //printf("       %3d ", is); 
      for (int iChan=0; iChan<4; iChan++) {
        //printf("(%6.1f, %6.1f) ", Channel[iChan].GetSignalLong(1)->GetAmp1(is), Channel[iChan].GetSignalLong(0)->GetAmp1(is));
        ah+=Channel[iChan].GetSignalLong(1)->GetAmp1(is);
        al+=Channel[iChan].GetSignalLong(0)->GetAmp1(is) * Channel[iChan].GetHighLow();
        ahI+=Channel[iChan].GetSignalLong(1)->GetAmpI(is);
        alI+=Channel[iChan].GetSignalLong(0)->GetAmpI(is) * Channel[iChan].GetHighLow();
      }      
      if (amp1max<al)amp1max=al; 
      if (ampHmax<ah)ampHmax=ah; 
      if (amp1min>al)amp1min=al; 
      SignalSumH->SetAmp1(is, ah);
      SignalSumL->SetAmp1(is, al);
      SignalSumH->SetAmpI(is, ahI);
      SignalSumL->SetAmpI(is, alI);
    }
    //for (int ib=0; ib<25; ib++) Time[ib]=-1000.;
  }

//==========================================================================================
  void ComputeThalf() { // computes Thalf bunch-by-bunch, using self-timing
    //printf("[ComputeThalf]->\n");
    for (int ib=0; ib<Nbunchs; ib++) {
      //printf("[ComputeThalf] bunch#%d good=%d T=%.1f \n", ib, GoodBunch[ib], Time[ib]);
      if (!GoodBunch[ib]) continue;
      double TB=Time[ib];
      int nChan=4;
      if (ID==5) nChan=2;
      for (int iChan=0; iChan<nChan; iChan++) {
        for (int iGain=0; iGain<2; iGain++) {
          TSignal *Signal = Channel[iChan].GetSignalLong(iGain);
          Signal -> ComputeThalf(TB, ib);
        }
      }
    
      if (ID>3) return;
      int iGain=1;
      if (HGoverflow) {
        iGain=0;
      }

      TSignal *SumSignal = Channel[4].GetSignalLong(iGain);
      SumSignal -> ComputeThalf(TB, ib);
      TRdown[ib] = SumSignal -> GetTRdown(ib);
      TLdown[ib] = SumSignal -> GetTLdown(ib);
      Thalf[ib]  = SumSignal -> GetThalf(ib);
    }
  }
//==========================================================================================
  void ComputeThalf(int bunch, double Tbunch) { // computes Thalf for a given bunch, relying on external timing
                                                

      for (int iChan=0; iChan<4; iChan++) {
        for (int iGain=0; iGain<2; iGain++) {
          TSignal *Signal = Channel[iChan].GetSignalLong(iGain);
          Signal -> ComputeThalf(Tbunch, bunch);
        }
      }
    
    if (ID>3) return;
    int iGain=1;
    if (HGoverflow) {
      iGain=0;
    }

    TSignal *SumSignal = Channel[4].GetSignalLong(iGain); // pick a sum signal with a right gain
    SumSignal -> ComputeThalf(Tbunch, bunch);
    TRdown[bunch] = SumSignal -> GetTRdown(bunch);
    TLdown[bunch] = SumSignal -> GetTLdown(bunch);
    Thalf[bunch]  = SumSignal -> GetThalf(bunch);
  }
//==========================================================================================
   double GetThalf(int bunch) {
    if (bunch<0 || bunch>Nbunchs-1) {
      printf("[Module->GetThalf] requesting wrong bunch#%d of %d\n", bunch, Nbunchs);
      exit(-1);
    }
    return Thalf[bunch];
  }
//==========================================================================================
  double GetAnear(int bunch) {
    if (bunch<0 || bunch>Nbunchs-1) {
      printf("[Module->GetAnear] requesting wrong bunch#%d of %d\n", bunch, Nbunchs);
      exit(-1);
    }
    return Anear[bunch];
  }
//==========================================================================================
  double GetAnearCal(int bunch) {
    if (bunch<0 || bunch>Nbunchs-1) {
      printf("[Module->GetAnearCal] requesting wrong bunch#%d of %d\n", bunch, Nbunchs);
      exit(-1);
    }
    return Anear[bunch]*attCorr;
  }
///==========================================================================================
  double GetSumMin() {return amp1min;}
  double GetSumMax() {return amp1max;}

  double GetSumFinalMin(int iGain) {
    double result=Channel[4].GetSignalLong(iGain)->GetSignalFinalMin();
    return result;
  }
  double GetSumFinalMax(int iGain) {
    double result=Channel[4].GetSignalLong(iGain)->GetSignalFinalMax();
    return result;
  }
   
//==========================================================================================
  int GetSampleX0(int bunch) {
    int ir=0;
    for (int iChan=0; iChan<4; iChan++) {
      for (int iGain=0; iGain<2; iGain++) {
        int ix=Channel[iChan].GetSignalLong(iGain)->GetI0(bunch);
        if (ix!=0) ir=ix;
      }
    }
    return ir;
  }
  int GetSampleX1(int bunch) {
    int ir=0;
    for (int iChan=0; iChan<4; iChan++) {
      for (int iGain=0; iGain<2; iGain++) {
        int ix=Channel[iChan].GetSignalLong(iGain)->GetI1(bunch);
        if (ix!=0) ir=ix;
      }
    }
    return ir;
  }

  double GetSumSample(int is) {
    if (is<0) return 0.0;
    if (is>=nSlice) return 0.0;
    int iGain=1;
    if (HGoverflow) iGain=0;       
    return Channel[4].GetSignalLong(iGain)->GetAmp1(is);
  }
  double GetSumSampleH(int is) {
    if (is<0) return 0.0;
    if (is>=nSlice) return 0.0;
    return Channel[4].GetSignalLong(1)->GetAmp1(is);
  }
  double GetSumSampleX(int is) {
    printf("[TModule->GetSumSampleX] not implemented yet\n");
    exit(-1);
    if (is<0) return 0.0;
    if (is>=nSlice) return 0.0;


    return 0.;
  }
  double GetSumSampleHX(int is) {
    printf("[TModule->GetSumSampleHX] not implemented yet\n");
    exit(-1);
    if (is<0) return 0.0;
    if (is>=nSlice) return 0.0;
    //return ampXh[is];
    return 0.0;
  }
  double GetSumSampleCal(int is) {
    if (is<0) return 0.0;
    if (is>=nSlice) return 0.0;
    int iGain=1;
    if (HGoverflow) iGain=0;     
    return attCorr*Channel[4].GetSignalLong(iGain)->GetAmp1(is);
  }
  double GetSumSampleCalH(int is) {
    if (is<0) return 0.0;
    if (is>=nSlice) return 0.0;
    
    return attCorr*Channel[4].GetSignalLong(1)->GetAmp1(is);
  }

  double GetSignalVarianceOdd(double T0, double T1) {
    int i0=int((T0-time[0])/12.5-2);
    int i1=int((T1-time[0])/12.5+1);
    if (i0<0) i0=0;
    if (i0>=nSlice) i0=nSlice-1;
    if (i1<0) i1=0;
    if (i1>=nSlice) i1=nSlice-1;
    double s0=0.;
    double s1=0.;
    double s2=0.;
    bool HGou=false;
    bool LGou=false;
    for (int i=i0; i<i1; i++) {
      if (time[i]>T0 && time[i]<T1) {
        for (int iCh=0; iCh<4; iCh++) {
          if (Channel[iCh].GetSignalLong(1)->GetUnder(i) ||
              Channel[iCh].GetSignalLong(1)->GetOverf(i)) {
              HGou=true;
              break;
          } // HG overflow check
          if (Channel[iCh].GetSignalLong(0)->GetUnder(i) ||
              Channel[iCh].GetSignalLong(0)->GetOverf(i)) {
              LGou=true;
              break;
          } // HG overflow check
        } // channel loop
      } // sample check
    } // sample loop
    
    int iG=1;
    if (HGou) {
      printf("\n\n\n[TModule->GetSignalVarianceOdd] using LOW GAIN for noise estimation!\n\n\n");
      ProcStatus->SetReportMode(1);
      iG=0;
    }
    if (LGou) {
      printf("\n\n\n[TModule->GetSignalVarianceOdd] no even LOW GAIN available for noise estimation!\n\n\n");
      ProcStatus->SetReportMode(1);
      //exit(-1);
      iG=0;
    }

    for (int i=i0; i<i1; i++) {
      if (time[i]>T0 && time[i]<T1) {
        s0+=1.0;
        double s=0.;
        double aSig=1.0;
        for (int iCh=0; iCh<4; iCh++) {
          s+=aSig*Channel[iCh].GetSignalLong(iG)->GetAmp1(i);
          aSig=-aSig;
        }
        s1+=s;
        s2+=s*s;
      }
    }
    if (s0==0.0) {
      return 0.0;
    } 
    s1/=s0;
    s2/=s0;
    s2-=s1*s1;
    s2=sqrt(s2); 
    if (s2<=0.0) return 0.0;
    return s2;
  }

  double GetSignalVarianceSmoothOdd(double T0, double T1) {
    int i0=int((T0-time[0])/12.5-2);
    int i1=int((T1-time[0])/12.5+1);
    if (i0<1) i0=1;
    if (i0>=nSlice-1) i0=nSlice-2;
    if (i1<1) i1=1;
    if (i1>=nSlice-1) i1=nSlice-2;
    double s0=0.;
    double s1=0.;
    double s2=0.;
    bool HGou=false;
    bool LGou=false;
    for (int i=i0; i<i1; i++) {
      if (time[i]>T0 && time[i]<T1) {
        for (int iCh=0; iCh<4; iCh++) {
          if (Channel[iCh].GetSignalLong(1)->GetUnder(i) ||
              Channel[iCh].GetSignalLong(1)->GetOverf(i)) {
              HGou=true;
              break;
          } // HG overflow check
          if (Channel[iCh].GetSignalLong(0)->GetUnder(i) ||
              Channel[iCh].GetSignalLong(0)->GetOverf(i)) {
              LGou=true;
              break;
          } // HG overflow check
        } // channel loop
      } // sample check
    } // sample loop
    
    int iG=1;
    if (HGou) {
      printf("\n\n\n[TModule->GetSignalVarianSmoothceOdd] using LOW GAIN for noise estimation!\n\n\n");
      ProcStatus->SetReportMode(1);
      iG=0;
    }
    if (LGou) {
      printf("\n\n\n[TModule->GetSignalVarianceSmoothOdd] no even LOW GAIN available for noise estimation!\n\n\n");
      ProcStatus->SetReportMode(1);
      //exit(-1);
      iG=0;
    }

    for (int i=i0; i<i1; i++) {
      if (time[i]>T0 && time[i]<T1) {
        s0+=1.0;
        double s=0.0;
        double aSig=1.0;
        for (int iCh=0; iCh<4; iCh++) {
          double a=Channel[iCh].GetSignalLong(iG)->GetAmp1(i);
          a-=0.5*Channel[iCh].GetSignalLong(iG)->GetAmp1(i-1);
          a-=0.5*Channel[iCh].GetSignalLong(iG)->GetAmp1(i+1);
          s+=a*aSig;
          aSig=-aSig;
        }        
        s1+=s;
        s2+=s*s;
      }
    }
    
    if (s0==0.0) {
      return 0.0;
    } 
    s1/=s0;
    s2/=s0;
    s2-=s1*s1;
    if (s2<=0.0) return 0.0;
    s2=sqrt(s2/1.5); // 1.5 is re-normalization to a sample noise // 
    return s2;
  }

  double GetSignalVarianceSharpOdd(double T0, double T1) {
    int i0=int((T0-time[0])/12.5-2);
    int i1=int((T1-time[0])/12.5+1);
    if (i0<1) i0=1;
    if (i0>=nSlice-1) i0=nSlice-2;
    if (i1<1) i1=1;
    if (i1>=nSlice-1) i1=nSlice-2;
    double s0=0.;
    double s1=0.;
    double s2=0.;
    
    bool HGou=false;
    bool LGou=false;
    for (int i=i0; i<i1; i++) {
      if (time[i]>T0 && time[i]<T1) {
        for (int iCh=0; iCh<4; iCh++) {
          if (Channel[iCh].GetSignalLong(1)->GetUnder(i) ||
              Channel[iCh].GetSignalLong(1)->GetOverf(i)) {
              HGou=true;
              break;
          } // HG overflow check
          if (Channel[iCh].GetSignalLong(0)->GetUnder(i) ||
              Channel[iCh].GetSignalLong(0)->GetOverf(i)) {
              LGou=true;
              break;
          } // HG overflow check
        } // channel loop
      } // sample check
    } // sample loop
    
    int iG=1;
    if (HGou) {
      printf("\n\n\n[TModule->GetSignalVarianceSharpOdd] using LOW GAIN for noise estimation!\n\n\n");
      ProcStatus->SetReportMode(1);
      iG=0;
    }
    if (LGou) {
      printf("\n\n\n[TModule->GetSignalVarianceSharpOdd] no even LOW GAIN available for noise estimation!\n\n\n");
      ProcStatus->SetReportMode(1);
      //exit(-1);
      iG=0;
    }

    for (int i=i0; i<i1; i++) {
      if (time[i]>T0 && time[i]<T1) {
        s0+=1.0;
        double s=0.0;
        double aSig=1.0;
        for (int iCh=0; iCh<4; iCh++) {
          double a=Channel[iCh].GetSignalLong(iG)->GetAmp1(i);
          a+=0.5*Channel[iCh].GetSignalLong(iG)->GetAmp1(i-1);
          a+=0.5*Channel[iCh].GetSignalLong(iG)->GetAmp1(i+1);
          s+=a*aSig;
          aSig=-aSig;
        }        
        s1+=s;
        s2+=s*s;
      }
    }

    if (s0==0.0) {
      return 0.0;
    } 
    s1/=s0;
    s2/=s0;
    s2-=s1*s1;
    if (s2<=0.0) return 0.0;
    s2=sqrt(s2/1.5); // re-norm to a sample noise 
    return s2;
  }



  double GetSignalVarianceSmooth(int iGain, double T0, double T1) {
// variance for a(i)-0.5*(a(i-1)+a(i+1));
    bool DbgL=false;
    //if (ID<2) DbgL=true;
    //DbgL=true;

    double var0[2];
    double var1[2];
    double var2[2];
    if (DbgL) printf("[TModule->GetSignalVariance] noise for %s T0=%.1f T1=%.f\n", Name, T0, T1);
    for (int iDelay=0; iDelay<2; iDelay++) { 
      var0[iDelay]=0.;
      var1[iDelay]=0.;
      var2[iDelay]=0.;

      int is0=10000; 
      int is1=0;
      for (int iChan=0;iChan<4;iChan++) {
        if (Channel[iChan].GetStatus()) {
          TSignal* Signal = Channel[iChan].GetSignalShort(2*iGain+iDelay);
          int i0=int((T0-Signal->GetTime(0))/25.-1);
          if (i0<0) i0=0;
          if (i0<is0) is0=i0; 
          int i1=int((T1-Signal->GetTime(0))/25.);
          if (i1>Signal->GetNslice()-1) i1=Signal->GetNslice()-1;
          if (i1>is1) is1=i1;
        }
      }
      for (int is=is0+1; is<=is1-1; is++) {
        double sum=0.0;
        double sumOld=0.0;
        if (DbgL) printf("[TModule->GetSignalVariance] Delay=%d sample=%d t=%6.1f", 
                        iDelay, is, Channel[0].GetSignalShort(2*iGain+iDelay)->GetTime(is));
        for (int iChan=0; iChan<4; iChan++) {
          if (Channel[iChan].GetStatus()) {
            TSignal* Signal = Channel[iChan].GetSignalShort(2*iGain+iDelay);
// remove 'tails' from the past
            double sumCorr=Signal->GetAmp1(is)-0.5*(Signal->GetAmp1(is-1)+Signal->GetAmp1(is+1));
            if (DbgL) printf(" chan#%d %5.1f (%5.1f)", iChan, Signal->GetAmp1(is), sumCorr);
            sumOld+=Signal->GetAmp1(is);
            sum+=sumCorr;
          }
        }
        if (DbgL) printf(" Sum=%6.1f (%6.1f)\n", sumOld, sum);
        var0[iDelay]+=1.0;
        var1[iDelay]+=sum;
        var2[iDelay]+=sum*sum;
      }
      if (DbgL) printf("[TModule->GetSignalVariance] Sums for %s, Delay=%d -> %.3f, %.3f, %.3f\n", 
             Name, iDelay, var0[iDelay], var1[iDelay], var2[iDelay]);
      if (var0[iDelay]>0.) {
        var1[iDelay]/=var0[iDelay];
        var2[iDelay]/=var0[iDelay];
        var2[iDelay]-=var1[iDelay]*var1[iDelay];
        if (var2[iDelay]<=0.0) {
          var2[iDelay]=0.0;
        }
      } else {
        var1[iDelay]=0.0;
        var2[iDelay]=0.0;
      }
    }
    double var = sqrt(0.5*(var2[0]+var2[1]));
    if (DbgL) {
      printf("[TModule->GetSignalVariance] RMS=%5.2f (%5.2f+%5.2f)\n",
             var, sqrt(var2[0]), sqrt(var2[1]));
      //int iqq;
      //printf("Push->");
      //cin >> iqq;
      //if (iqq==0) exit(-1);
    }
    DbgL=false;
    return var;
  }

  double GetSignalVarianceSharp(int iGain, double T0, double T1) {
// variance for a(i)+0.5*(a(i-1)+a(i+1));
    bool DbgL=false;
    //if (ID<2) DbgL=true;
    //DbgL=true;

    double var0[2];
    double var1[2];
    double var2[2];
    if (DbgL) printf("[TModule->GetSignalVariance] noise for %s T0=%.1f T1=%.f\n", Name, T0, T1);
    for (int iDelay=0; iDelay<2; iDelay++) { 
      var0[iDelay]=0.;
      var1[iDelay]=0.;
      var2[iDelay]=0.;

      int is0=10000; 
      int is1=0;
      for (int iChan=0;iChan<4;iChan++) {
        if (Channel[iChan].GetStatus()) {
          TSignal* Signal = Channel[iChan].GetSignalShort(2*iGain+iDelay);
          int i0=int((T0-Signal->GetTime(0))/25.-1);
          if (i0<0) i0=0;
          if (i0<is0) is0=i0; 
          int i1=int((T1-Signal->GetTime(0))/25.);
          if (i1>Signal->GetNslice()-1) i1=i1>Signal->GetNslice()-1;
          if (i1>is1) is1=i1;
        }
      }
      for (int is=is0+1; is<=is1-1; is++) {
        double sum=0.0;
        double sumOld=0.0;
        if (DbgL) printf("[TModule->GetSignalVariance] Delay=%d sample=%d t=%6.1f", 
                        iDelay, is, Channel[0].GetSignalShort(2*iGain+iDelay)->GetTime(is));
        for (int iChan=0; iChan<4; iChan++) {
          if (Channel[iChan].GetStatus()) {
            TSignal* Signal = Channel[iChan].GetSignalShort(2*iGain+iDelay);
// remove 'tails' from the past
            double sumCorr=Signal->GetAmp1(is)+0.5*(Signal->GetAmp1(is-1)+Signal->GetAmp1(is+1));
            if (DbgL) printf(" chan#%d %5.1f (%5.1f)", iChan, Signal->GetAmp1(is), sumCorr);
            sumOld+=Signal->GetAmp1(is);
            sum+=sumCorr;
          }
        }
        if (DbgL) printf(" Sum=%6.1f (%6.1f)\n", sumOld, sum);
        var0[iDelay]+=1.0;
        var1[iDelay]+=sum;
        var2[iDelay]+=sum*sum;
      }
      if (DbgL) printf("[TModule->GetSignalVariance] Sums for %s, Delay=%d -> %.3f, %.3f, %.3f\n", 
             Name, iDelay, var0[iDelay], var1[iDelay], var2[iDelay]);
      if (var0[iDelay]>0.) {
        var1[iDelay]/=var0[iDelay];
        var2[iDelay]/=var0[iDelay];
        var2[iDelay]-=var1[iDelay]*var1[iDelay];
        if (var2[iDelay]<=0.0) {
          var2[iDelay]=0.0;
        }
      } else {
        var1[iDelay]=0.0;
        var2[iDelay]=0.0;
      }
    }
    double var = sqrt(0.5*(var2[0]+var2[1]));
    if (DbgL) {
      printf("[TModule->GetSignalVariance] RMS=%5.2f (%5.2f+%5.2f)\n",
             var, sqrt(var2[0]), sqrt(var2[1]));
      // int iqq;
      //printf("Push->");
      //cin >> iqq;
      //if (iqq==0) exit(-1);
    }
    DbgL=false;
    return var;
  }

   double GetSignalVariance(int iGain, double T0, double T1) {
    bool Dbg0=false;
    //if (ID<2) Dbg0=true;

    double var0[2]={0.0, 0.0};
    double var1[2]={0.0, 0.0};
    double var2[2]={0.0, 0.0};
    if (Dbg0) printf("[TModule->GetSignalVariance] noise for %s T0=%.1f T1=%.f\n", Name, T0, T1);
    for (int iDelay=0; iDelay<2; iDelay++) { 
      int is0=10000; 
      int is1=0;
      for (int iChan=0;iChan<4;iChan++) {
        if (Channel[iChan].GetStatus()) {
          TSignal* Signal = Channel[iChan].GetSignalShort(2*iGain+iDelay);
          int i0=int((T0-Signal->GetTime(0))/25.-1);
          if (i0<0) i0=0;
          if (i0<is0) is0=i0; 
          int i1=int((T1-Signal->GetTime(0))/25.);
          if (i1>Signal->GetNslice()-1) i1=i1>Signal->GetNslice()-1;
          if (i1>is1) is1=i1;
        }
      }
      for (int is=is0+1; is<=is1-1; is++) {
        double sum=0.0;
        if (Dbg0) printf("[TModule->GetSignalVariance] Delay=%d sample=%d t=%6.1f", 
                        iDelay, is, Channel[0].GetSignalShort(2*iGain+iDelay)->GetTime(is));
        for (int iChan=0; iChan<4; iChan++) {
          if (Channel[iChan].GetStatus()) {
            TSignal* Signal = Channel[iChan].GetSignalShort(2*iGain+iDelay);
// do not remove 'tails' from the past
//            double sumCorr=Signal->GetAmp1(is)-0.5*(Signal->GetAmp1(is-1)+Signal->GetAmp1(is+1));
            double sumCorr=Signal->GetAmp1(is);
            if (Dbg0) printf(" chan#%d %5.1f (%5.1f)", iChan, Signal->GetAmp1(is), sumCorr);
            sum+=sumCorr;
          }
        }
        if (Dbg0) printf(" Sum=%6.1f\n", sum);
        var0[iDelay]+=1.0;
        var1[iDelay]+=sum;
        var2[iDelay]+=sum*sum;
      }
      if (var0[iDelay]>0.) {
        var1[iDelay]/=var0[iDelay];
        var2[iDelay]/=var0[iDelay];
        var2[iDelay]-=var1[iDelay]*var1[iDelay];
        if (var2[iDelay]<=0.0) {
          var2[iDelay]=0.0;
        }
      } else {
        var1[iDelay]=0.0;
        var2[iDelay]=0.0;
      }
    }
    double var = sqrt(0.5*(var2[0]+var2[1]));
    if (Dbg0) {
      printf("[TModule->GetSignalVariance] RMS=%5.2f (%5.2f+%5.2f)\n",
             var, sqrt(var2[0]), sqrt(var2[1]));
      //int iqq;
      //printf("Push->");
      //cin >> iqq;
      //if (iqq==0) exit(-1);
    }
    return var;
  }

   double GetMySum(int bunch) {
    double sum=0.0;
    for (int iChan=0;iChan<4;iChan++) {
      if (Channel[iChan].GetStatus()) {
        sum+=Channel[iChan].GetResponseADC(bunch);
      }
    }
    return sum;
  }

  double GetMySumRate(int bunch) { // uses high/low gain crisis resolution
    double sum=0.0;
    for (int iChan=0;iChan<4;iChan++) {
      if (Channel[iChan].GetStatus()) {
        sum+=Channel[iChan].GetResponseADCrate(bunch);
      }
    }
    //printf("[TModule->GetMySumRate] Sum=%.1f\n", sum);
    return sum;
  }

  double GetMySumRateCal(int bunch) { // uses high/low gain crisis resolution
    double sum=0.0;
    for (int iChan=0;iChan<4;iChan++) {
      if (Channel[iChan].GetStatus()) {
        sum+=Channel[iChan].GetResponseCALrate(bunch);
      }
    }
    //printf("[TModule->GetMySumRate] Sum=%.1f\n", sum);
    return sum;
  }

  double GetMySumLOW(int bunch) { // same but low gain only
    double sum=0.0;
    for (int iChan=0;iChan<4;iChan++) {
      if (Channel[iChan].GetStatus()) {
        sum+=Channel[iChan].GetResponseADClow(bunch);
      }
    }
    return sum;
  }

   double GetMySumCal(int bunch) {
    double sum=0.0;
    for (int iChan=0;iChan<4;iChan++) {
      if (Channel[iChan].GetStatus()) {
        sum+=Channel[iChan].GetResponseCal(bunch);
      }
    }
    return sum;
  }

  bool GetCold()          {return isCold;}
  void SetCold(bool Cold) {isCold=Cold;}

  bool   GetHighActivity()        {return HighActivity;}
  bool   GetHighActivity1()       {return HighActivity1;}
  double GetHighActivityT1()      {return HighActivityT1;}
  double GetHighActivityT2()      {return HighActivityT2;}
  double GetHighActivityA1()      {return HighActivityA1;}
  double GetHighActivityA2()      {return HighActivityA2;}
  void   SetHighActivity(bool High)  {HighActivity=High;}
  void   SetHighActivity1(bool High) {HighActivity1=High;}
  double GetResponseDefault()     {return ResponseDefault;}

  void SetPulses() {
    if (Dbg) printf("[Module] ID=%d %s SetPulses ->\n", ID, Name);
    for (int iChan=0;iChan<4;iChan++) {
      if (Channel[iChan].GetStatus()) {
        if (ID==6) { // only short pulse in use for integrator !//
          Channel[iChan].SetShortPulses();
	} else {
          Channel[iChan].SetLongPulses();
          Channel[iChan].SetShortPulses();
	}
      }
    }
    return;
  }

  void PrepareAmpI() {
    for (int iChan=0; iChan<4; iChan++) {
      Channel[iChan].PrepareAmpI();    // prepare interpolated long signal ampI
    }
    return;
  }
  

  void CalculateBunchTime (int bunch) {
    double TT=0.0;
    double S=0.0;
    //Dbg=true;
    int ReportMode = ProcStatus->GetReportMode();
    if (ReportMode==10004 || Dbg) {
      printf("[CalculateBunchTime] %s bunch#%d\n", Name, bunch);
    }
    //for (int iChan=3; iChan<4; iChan++) { // use just a cherenkov for manual alignment
    for (int iChan=0; iChan<4; iChan++) {
      if (ID==5 && iChan==2) continue; // RF
      if (Channel[iChan].GetStatus()) {
	double A=Channel[iChan].GetResponseADC(bunch);
	double T=Channel[iChan].GetResponseTime(bunch);
        if (ReportMode==10004 || Dbg) {
	  printf("[CalculateBunchTime] Chan#%d A=%.1f T=%.1f\n", iChan, A, T);
	}
	if (A>10.0 && A<2000.) {
	  S+=A;
	  TT+=T*A;
	}
      }      
    }
    if (S>10.0) {
      TT/=S;
      Time[bunch]=TT;
    } else {
      TT=-1000.;
    }
    if (ReportMode==10004 || Dbg) {
      printf("[CalculateBunchTime]--------------------------------\n");
      printf("[CalculateBunchTime] %s       A=%.1f T=%.1f\n", Name, S, Time[bunch]);
    }

    
    if (ID==4) ProcStatus->GetEventHeader()->TBunch[bunch]=Time[bunch];

    //printf("[TModule->CalculateBunchTime] ModuleID=%d bunch#%d Tbunch=%.1f\n", ID, bunch,  Time[bunch]);
    //printf("push-->");
    //int iqq;
    //cin >> iqq;
    //Dbg=false;
  }

  double GetBunchTime (int bunch) {return Time[bunch];}
  void   SetBunchTime (int bunch, double t) {
    //printf("[Module->SetBunchTime] %10s setting bunch#%d time to %.1f\n", Name, bunch, t);
    Time[bunch]=t; 
    return;
  }

  void FilterBunchs1(TimingCut1* timingCut, ProcessingStatus *ProcStat) {
    int ReportMode = ProcStat->GetReportMode();

    if (Dbg || ReportMode==10007) printf("[FilterBunchs1]->filtering using %s Nbunch=%d\n", timingCut->GetName(), Nbunchs);
    for (int ib=0; ib<25; ib++) {Wbmp1[ib]=0.0; Bbmp1[ib]=false;}
    for (int ib1=0; ib1<Nbunchs; ib1++) {
      double T1=Time[ib1];
      for (int ib2=0; ib2<Nbunchs && ib2<6; ib2++) { // check against first 6 or less bunches to avoid self-
                                                    // supporting parazitic bunches and reflections!
        double dT=Time[ib2]-T1;
	if (dT<0) dT=-dT;
        for (int idiv=1; idiv<8; idiv++) {
	  double dW=timingCut->GetWeight(dT/idiv);
          Wbmp1[ib1]+=dW;
          //Wbmp1[ib2]+=dW;
	}
      }
      Bbmp1[ib1]=(Wbmp1[ib1]>0.1);    
    } 
    for (int ib=0; ib<Nbunchs && (Dbg || ReportMode==10007); ib++) {
      printf("[FilterBunchs1]->  bunch#%d T=%6.1f W=%.2f\n", ib, Time[ib], Wbmp1[ib]);
    }
  }

  void FilterBunchs2(TimingCut1* timingCut, ProcessingStatus *ProcStat) {
    int ReportMode = ProcStat->GetReportMode();
    if (Dbg || ReportMode==10007) printf("[FilterBunchs2]->filtering using %s Nbunch=%d\n", timingCut->GetName(), Nbunchs);
    for (int ib=0; ib<25; ib++) {Wbmp2[ib]=0.0; Bbmp2[ib]=false;}
    for (int ib=0; ib<Nbunchs; ib++) {
      Wbmp2[ib]=timingCut->GetWeight(Time[ib]);
      if (Dbg || ReportMode==10007) printf("[FilterBunchs2]->  bunch#%d T=%6.1f W=%.2f\n", ib, Time[ib], Wbmp2[ib]);
      Bbmp2[ib]=(Wbmp1[ib]>0.1);    
    }
  }

  void FilterBunchsFinal() {
    nGoodBunches=0;
    for (int ib=0; ib<Nbunchs; ib++) {
      GoodBunch[ib] = (Bbmp1[ib] && Bbmp2[ib] && Time[ib]>200. && Time[ib]<12.5*nSlice-700.);
      if (GoodBunch[ib]) nGoodBunches++;
      //GoodBunch[ib] = (Bbmp1[ib] && Time[ib]>100.);
    }
  }

  int    GetnGoodBunches() {return nGoodBunches;}
  double GetBunchWeight1 (int bunch) {
    if (bunch<0 || bunch>24) return 0.;
    return Wbmp1[bunch];
  }
  double GetBunchWeight2 (int bunch) {
    if (bunch<0 || bunch>24) return 0.;
    return Wbmp2[bunch];
  }
  bool   GetBunchFilter(int bunch)   {
    if (bunch<0 || bunch>24) return false;
    return GoodBunch[bunch];
  }

  void   SetBunchFilter(int bunch, bool filter) {
    if (bunch<0 || bunch>24) {
      printf("[Module->SetBunchFilter] attempt to set Filter for bunch with wrong number #%d\n", bunch);
      exit(-1);
    } else {
      GoodBunch[bunch]=filter;
    }
  }

  void Dump(double Tbunch, int bunch) { // peak normalyzed shapes
    if (bunch<0 || bunch>24) return;
    for (int iChan=0; iChan<nChannels; iChan++) {
      Channel[iChan].Dump(Tbunch, bunch);
    }
  }
  void FillPulseShapes(double Tbunch, int bunch) { // peak normalyzed shapes
    if (bunch<0 || bunch>24) return;
    for (int iChan=0; iChan<nChannels; iChan++) {
      if (Channel[iChan].GetStatus()) {
        if (Dbg) {
          printf("   filling shape for %s Tbunch#%6.1f\n", Name, Tbunch);
        }
        Channel[iChan].FillPulseShapes(bunch);
      }
    }
  }
  void FillPulseShapesModule(double Tbunch, int bunch, TProfile* Demo) { // peak normalyzed shapes
    if (bunch<0 || bunch>24) return;
    for (int iChan=0; iChan<nChannels; iChan++) {
      if (Channel[iChan].GetStatus()) Channel[iChan].FillPulseShapesModule(Tbunch, bunch, Demo);
    }
  }
  void FillPulseShapesA(double Tbunch) { // shapes in ADC as is
    for (int iChan=0; iChan<nChannels; iChan++) {
      if (Channel[iChan].GetStatus()) Channel[iChan].FillPulseShapesA(Tbunch);
    }
  }

  void FillPulseShapesI(double Tbunch, double Cherenkov, double CherIntegral, double BeamIntensityLog) {
    for (int iChan=0; iChan<nChannels; iChan++) {
      if (Channel[iChan].GetStatus()) Channel[iChan].FillPulseShapesI(Tbunch, Cherenkov, CherIntegral, BeamIntensityLog);
    }
  }

  void FillPulseShapeSplit(double Tbunch) {
    //printf("[TModule]     -> %s\t filling Tbunch=%.1f\n", Name, Tbunch);
    for (int iChan=0; iChan<nChannels; iChan++) {
      if (Channel[iChan].GetStatus()) Channel[iChan].FillPulseShapeSplit(Tbunch);
    }
  }

  void SavePulseShapeSplit() {
    for (int iChan=0; iChan<nChannels; iChan++) {
      if (Channel[iChan].GetStatus()) Channel[iChan].SavePulseShapeSplit();
    }
  }

  void ClearPulseShapes() {
    for (int iChan=0; iChan<nChannels; iChan++) {
      Channel[iChan].ClearPulseShapes();
    }
  }

  void ClearPulseShapesSplit() {
    for (int iChan=0; iChan<nChannels; iChan++) Channel[iChan].ClearPulseShapeSplit();
  }

  void  ResetSignalNoise() {
    for (int iChan=0; iChan<nChannels; iChan++) Channel[iChan].ResetSignalNoise();
  }

  void Fit(ProcessingStatus *ProcStat, int xBunch, int xChan) {
    Dbg=false;

    EventHeader *EH = ProcStat->GetEventHeader();
    int ReportMode  = ProcStat->GetReportMode();   
    char TitleC[40];
    char NamecC[40];
    char File1[100];
    char File2[100];
    bool BunchFound=true;
    double T=-10000.;
    double chi2worst=0.;
    int iFit=0;

 
    Nbunchs=0;
    if (Dbg || ReportMode==10008) {
      printf("\n\n[Module->FitPulses] Fitting pulses in %s Run#%d Burst#%d Evt#%d BurstEvt#%d\n", 
	      Name, EH->run, EH->burst, EH->event, EH->burstEvent);
    }
    if (ReportMode==10008) {
      sprintf(TitleC, "%s Run#%d Burst#%d Evt#%d BurstEvt#%d", 
	      Name, EH->run, EH->burst, EH->event, EH->burstEvent); 
      sprintf(NamecC, "%s_Run%d_Burst%d_Evt%d_BurstEvt%d", 
	      Name, EH->run, EH->burst, EH->event, EH->burstEvent);
      sprintf(File1, "ProblemEvents/ProblemEvent_%s_bunch%d.ps", NamecC, xBunch);
      sprintf(File2, "ProblemEvents/ProblemBunch_%s_bunch%d.ps", NamecC, xBunch);
      FitCanvas =   new TCanvas(NamecC, TitleC, 1920, 0, 800, 600);
      printf("[TChannel->Fit] creating new canvas %s %s\n", NamecC, TitleC);
      BunchCanvas = new TCanvas("BunchCanvas", "BunchCanvas", 2800, 0, 600, 800);
      printf("[TChannel->Fit] creating new canvas BunchCanvas \n");

      if (!dummyModuleSet) {
        dummyModule  = new TH1F("dummyModule", "dummyModule",  2500, -50., 50.+25.*EH->nSlice);
        dummyModuleSet=true;
      }
      FitCanvas->Divide(2, 4, 0.01, 0.01);
    }

    MakeSum();
    MakeSumNoise();
    do {
      MakeSum();
      T=FindBunch(EH->event, ProcStat);
      Txxx[Nbunchs]=T;
      if (T<-200.) {
        BunchFound=false;
      } else {
        chi2worst=0.0;
        iFit=0;
        for (int iChan=0;iChan<nChannels;iChan++) {
          double chi2=Channel[iChan].Fit(T, Nbunchs, iFit);
          if (chi2>chi2worst) chi2worst=chi2;
          if (Dbg) printf("[Module->FitPulses] Evt#%d Fit#0 %5s Chi2=%.1f Chi2Worst=%.1f\n", 
                          EH->event, Channel[iChan].GetName(), chi2, chi2worst);
	  if (ReportMode==10008) DisplayModuleGlobalFit(iChan, Nbunchs);
        }
        if (chi2worst>3.0) {
          iFit=1;
          chi2worst=0;
          for (int iChan=0;iChan<nChannels;iChan++) {
            double chi2=Channel[iChan].Fit(T, Nbunchs, iFit);
            if (chi2>chi2worst) chi2worst=chi2;
            if (Dbg) printf("[Module->FitPulses] Evt#%d Fit#1 %5s Chi2=%.1f Chi2Worst=%.1f\n", 
                            EH->event, Channel[iChan].GetName(), chi2, chi2worst);
	    if (ReportMode==10008) DisplayModuleGlobalFit(iChan, Nbunchs);
          }
	}

        for (int iChan=0;iChan<nChannels;iChan++) {
          Channel[iChan].SubtractFit(T, Nbunchs, iFit);
	}

        if (ReportMode==10008) DisplayModuleLocalFit(T, iFit, Nbunchs, ProcStat, xChan);
	if (Nbunchs==xBunch) {
          BunchCanvas->Print(File2, "ps Portrait");
	  break;
	}
        Nbunchs++;
      }
    } while (BunchFound && ReportMode>0 && Nbunchs<25);

    if (ReportMode==10008) {
      FitCanvas->Update();
      BunchCanvas->Update();
      ReportMode=ProcStat->WhatToDo("[Module->FitPulses]",  10008);
      if (ReportMode>0) {
	FitCanvas->Print(File1, "ps Landscape");
        FitCanvas->Close();
        BunchCanvas->Close();
      }
    } 
    Dbg=false;
  }

  void DisplayModuleGlobalFit(int iChan, int bunch) {
    for (int ig=0;ig<2;ig++) {
      double chi2=Channel[iChan].GetSignalLong(ig)->GetFitChi2(bunch);
      if (chi2<0) return;
      FitCanvas->cd(1+2*iChan+ig);
      double amin=Channel[iChan].GetSignalLong(ig)->GetMin();
      double amax=Channel[iChan].GetSignalLong(ig)->GetMax();
      if (bunch==0) {
        gStyle->SetOptStat(0);
	dummyModule->GetYaxis()->SetRangeUser(amin-0.05*(amax-amin), amax+0.05*(amax-amin));
        dummyModule->SetTitle(Channel[iChan].GetSignalLong(ig)->GetName());
        dummyModule->GetYaxis()->SetTitleSize(.1);
        dummyModule->GetYaxis()->SetTitleOffset(.5);
        dummyModule->GetYaxis()->SetLabelSize(.08);
        dummyModule->GetXaxis()->SetTitleSize(.1);
        dummyModule->GetXaxis()->SetTitleOffset(.75);
        dummyModule->GetXaxis()->SetLabelSize(.08);
        dummyModule->DrawCopy();
        Channel[iChan].GetSignalLong(ig)->GetGraph()->SetLineColor(kBlack);
        Channel[iChan].GetSignalLong(ig)->GetGraph()->SetMarkerColor(kBlack);
        Channel[iChan].GetSignalLong(ig)->GetGraph()->Draw("LPZz same");
      }
      int iFit=Channel[iChan].GetSignalLong(ig)->GetIfit(bunch);
      if (iFit==0) {
        TF1 *pulseFunc1 = Channel[iChan].GetSignalLong(ig)->GetFunc1();
        pulseFunc1->SetParameters(Channel[iChan].GetSignalLong(ig)->GetFitOffset(bunch),
                                 Channel[iChan].GetSignalLong(ig)->GetFitPeak(bunch),
                                 Channel[iChan].GetSignalLong(ig)->GetFitTime(bunch),
                                 Channel[iChan].GetSignalLong(ig)->GetFitWidth(bunch));
        pulseFunc1->SetLineWidth(1);
        pulseFunc1->DrawCopy("same");
      } else if (iFit==1) {
         TF1 *pulseFunc2 = Channel[iChan].GetSignalLong(ig)->GetFunc2();
         pulseFunc2->SetParameters(Channel[iChan].GetSignalLong(ig)->GetFitOffset(bunch),
                                  Channel[iChan].GetSignalLong(ig)->GetFitPeak(bunch),
                                  Channel[iChan].GetSignalLong(ig)->GetFitTime(bunch),
                                  Channel[iChan].GetSignalLong(ig)->GetFitWidth(bunch),
                                  Channel[iChan].GetSignalLong(ig)->GetFitPeak1(bunch),
                                  Channel[iChan].GetSignalLong(ig)->GetFitTime1(bunch),
                                  Channel[iChan].GetSignalLong(ig)->GetFitWidth1(bunch));
        pulseFunc2->SetLineWidth(1);
        pulseFunc2->DrawCopy("same");
      }
      TLine *lbunch = new TLine(Time[bunch], 0.92*amax+0.08*amin, Time[bunch], amin);
      lbunch->SetLineStyle(2);
      lbunch->Draw();
      char BunchC[3]; 
      sprintf(BunchC, "%d", bunch);
      TLatex *blat = new TLatex(Time[bunch]+20., 0.95*amax+0.05*amin, BunchC);
      blat->Draw("same");

      FitCanvas->Update();
    }
  }

  void DisplayModuleLocalFit(double T, int iFit, int iBunch, ProcessingStatus *ProcStat, int xChan) {
  
    EventHeader *EH = ProcStat->GetEventHeader();
    int ReportMode  = ProcStat->GetReportMode();
    char TitleC[200];
    char NamecC[200];
    if (!dummyLocalSet) {
      sprintf(TitleC, "%s Local ", Name);
      sprintf(NamecC, "%s_Local ", Name);
      dummyLocal  = new TH1F(NamecC,  TitleC,   2500, -50., 50.+25.*EH->nSlice);
      dummyLocalSet=true;
    }
  
    printf("[DisplayModuleLocalFit]  Fit %s, bunch#%d T=%.1fns Fit#%d\n", Name, iBunch, T, iFit); 
  
    sprintf(TitleC, "Fit %s, bunch#%d T=%.1fns Fit#%d", Name, iBunch, T, iFit); 
    sprintf(NamecC, "Fit_%s_bunch_%d_T=%.1fns Fit#%d", Name, iBunch, T, iFit); 
    BunchCanvas->SetTitle(TitleC);
    BunchCanvas->SetName(NamecC);

    BunchCanvas->Divide(2, 4, 0.01, 0.01);

    double Tspan=800.;
    double Tmarg=50.;
    if (ID<4) { 
      Tspan=400.;
      Tmarg=25.;
    }

    for (int iChan=0;iChan<nChannels;iChan++) {
      if (Channel[iChan].GetStatus()) {
        for (int ig=0;ig<2;ig++) {
          double ampmax=5., ampmin=-5.;
          TSignal* Signal=Channel[iChan].GetSignalLong(ig);
          bool gotSignal=true;
          if (!Signal->GetStatus()) {
            gotSignal=false;
            for (int iDelay=0; iDelay<2; iDelay++) { // find good delay signal if any
              if (Channel[iChan].GetSignalShort(2*ig+iDelay)->GetStatus()) {
		Signal=Channel[iChan].GetSignalShort(2*ig+iDelay);
                gotSignal=true;
	      }
	    }
	  }

          if (gotSignal) {

            double dT=Signal->GetdTsample();
            int i1=int((T-250.)/dT)-1;
            if (i1<0) i1=0;
            int i2=i1+int(Tspan/dT)+1;
            if (i2>nSlice-1) {
              i2=nSlice-1;
              i1=i2-int(Tspan/dT)-1;
            }


            double chi2=Signal->GetFitChi2(iBunch);
	    int jFit=Signal->GetIfit(iBunch);
            double t1=Signal->GetTime(i1)-Tmarg;
            double t2=Signal->GetTime(i2)+Tmarg;
            sprintf(TitleC, "%s", Signal->GetName());
	    dummyLocal->SetTitle(TitleC);
            dummyLocal->GetXaxis()->SetRangeUser(t1, t2);
	    BunchCanvas->cd(1+2*iChan+ig);
            for (int i=i1;i<i2;i++) {
              double a=Signal->GetAmp3(i);
	      if (a>ampmax) ampmax=1.05*a;
	      if (a<ampmin) ampmin=1.05*a;
            }
            dummyLocal->GetYaxis()->SetRangeUser(ampmin, ampmax);
	    dummyLocal->DrawCopy();
            for (int i=i1;i<i2;i++) {
	      if (!Signal->GetDoFit(i)) {
                double t=Signal->GetTime(i);
	        TLine *l1 = new TLine(t-7., 0., t+7., 0.);
                l1->SetLineColor(kYellow);
                l1->SetLineWidth(7);
                l1->Draw();
	      }
            }
          
            Signal->GetGraphOld()->Draw("LPZz same");
            if (chi2>0.0) Signal->DrawGraphFit(iBunch);
            if (jFit==0) {
              TF1 *pulseFunc1 = Signal->GetFunc1();
              pulseFunc1->SetParameters(Signal->GetFitOffset(iBunch),
                                       Signal->GetFitPeak(iBunch),
                                       Signal->GetFitTime(iBunch),
                                       Signal->GetFitWidth(iBunch));
              sprintf(TitleC, "A=%.1f+/-%.1f", 
                      Signal->GetFitPeak(iBunch)/Signal->GetFitWidth(iBunch),
                      Signal->GetFitPeakE(iBunch));
              pulseFunc1->SetLineWidth(2);
              pulseFunc1->DrawCopy("same");
	    } else if (jFit==1) {
              TF1 *pulseFunc2 = Signal->GetFunc2();
              pulseFunc2->SetParameters(Signal->GetFitOffset(iBunch),
                                        Signal->GetFitPeak(iBunch),
                                        Signal->GetFitTime(iBunch),
                                        Signal->GetFitWidth(iBunch),
                                        Signal->GetFitPeak1(iBunch),
                                        Signal->GetFitTime1(iBunch),
                                        Signal->GetFitWidth1(iBunch));
	      double a1 =Signal->GetFitPeak(iBunch);
	      double t1r =Signal->GetFitTime(iBunch);
	      double t2r =Signal->GetFitTime1(iBunch);
	      //double a1e=Signal->GetFitPeakE(iBunch);
	      double s1 =Signal->GetFitWidth(iBunch);
	      double s2 =Signal->GetFitWidth1(iBunch);
	      double ww=Signal->GetFitPeak1(iBunch);
              sprintf(TitleC, "#splitline{A1=%.1f, T1=%.1f}{A2=%.1f, T2=%.1f}", (1.0-ww)*a1/s1, t1r, ww*a1/s2, t2r);
              pulseFunc2->SetLineWidth(2);
              pulseFunc2->DrawCopy("same");
            }
	    TLatex *l1 = new TLatex(0.45*t1+0.55*t2, 0.90*ampmax+0.10*ampmin, TitleC);
	    if (iChan==xChan) l1->SetTextColor(kRed);
            l1->Draw("same");
            sprintf(TitleC, "Chi2=%.1f", chi2);
	    TLatex *l2 = new TLatex(0.45*t1+0.55*t2, 0.80*ampmax+0.20*ampmin, TitleC);
	    if (iChan==xChan) l2->SetTextColor(kRed);
            l2->Draw("same");

            BunchCanvas->Update();
          }
        }
      }
    }

    ReportMode=ProcStat->WhatToDo("DisplayModuleLocalFit", 10009);
    if (ReportMode==10009) {
      //BunchCanvas->Close();
      //dummyLocal->Delete();
    }
  }


  double FindBunch(int evt, ProcessingStatus *ProcStat) {
    //Dbg=true;
    //Dbg=false;
    Dbg=(evt<0);
    //Dbg=(evt==15);
    int ReportMode = ProcStat->GetReportMode();

    int Nbunch=0;
    double Abunch=0., Rbunch=0., Tbunch=-10000.;
    double S[512], S1[512], S2[512], R[512], Tb[512];
    for (int i=1;i<nSlice-1; i++) {
      int i0=i-1;
      int i1=i+2;
      int i2=i+9;
      int i3=i+12;
      double t1=0.0, tx=0.0, r1=0.0, r2=0.0, r=0.0;
      S1[i]=0.0;
      S2[i]=0.0;
      if (i3>nSlice) i3=nSlice;
      if (i2>nSlice-1) i2=nSlice-1;
      bool Apos=true;
      for (int j=i0;j<i1;j++) {
        double a=SumHG[j]+SumLG[j];
        double t=Channel[0].GetSignalLong(0)->GetTime(j);
        if (a<0.) Apos=false;
        S1[i]+=a; 
	r1+=rmsHG[j]*rmsHG[j]+rmsLG[j]*rmsLG[j];
        t1+=t*a; 
        tx+=t; 
        //if (Dbg) printf("[Module->FindBunch] t=%.1f A=%.1f, t1acc=%.1f tX=%.1f S=%.1f, Positive=%d\n", 
	//               t, a, t1, tx, s1, Apos);
      } 
      if (Apos) {
        t1/=S1[i];
      } else {
        t1=tx/(i1-i0);
      }
      S1[i]/=(i1-i0);
      r1=r1/(i1-i0);      
      for (int j=i2;j<i3;j++) {
        S2[i]+=SumHG[j]+SumLG[j];
	r2+=rmsHG[j]*rmsHG[j]+rmsLG[j]*rmsLG[j];
      }
      S2[i]/=(i3-i2);      
      r2=r2/(i1-i0);
           
      S[i]=S1[i]-S2[i];
      R[i]=sqrt(r1+r2);
      Tb[i]=t1;
      if (Dbg && i<50) { 
        printf("[Module->FindBunch] i=%3d t=%6.1f A=%6.1f S=%6.1f r=%6.2f",
               i, Channel[0].GetSignalLong(0)->GetTime(i),SumHG[i]+SumLG[i], S[i], R[i]);
	for (int iChan=0;iChan<nChannels;iChan++) {
	  if (Channel[iChan].GetStatus()) {
	    printf(" ");
	    for (int iGain=0;iGain<2;iGain++) {
              printf("%d%d%d ", 
                     Channel[iChan].GetSignalLong(iGain)->GetUnder(i), 
                     Channel[iChan].GetSignalLong(iGain)->GetOverf(i), 
		     Channel[iChan].GetSignalLong(iGain)->GetDoFit(i));
	    }
	  } else {
            printf("   ");
	  }
	}
        if (S1[i]>1.5+3.0*r && S1[i]>0.0 && S2[i]<0.0) {
          printf("  *\n");
        } else {
	  printf("\n");
        }
      }
    }
    for (int i0=1;i0<nSlice;i0++) { // check how many significunt samples are sitting together
      int n=0;
      for (int i=i0;i>1;i--) {
        if (S[i]>1.5+3.0*R[i] && S1[i]>0.0 && S2[i]<0.0) {
	  n++;
        } else {
	  break;
        }
      }
      for (int i=i0+1;i<nSlice-1;i++) {
        if (S[i]>1.5+3.0*R[i] && S1[i]>0. && S2[i]<0.0) {
	  n++;
        } else {
	  break;
        }
      }
      if (n>1 && S[i0]>Abunch && S1[i0]>0. && S2[i0]<0.0) {
        Abunch=S[i0];
	Tbunch=Tb[i0];
        Rbunch=R[i0];
        Nbunch=n;
      }
    }

    if ((Dbg || ReportMode==10010) && Tbunch>-500.) printf("[Module->FindBunch] bunch->(t=%6.1f A=%6.1f r=%.2f N=%d)\n",
				      Tbunch, Abunch, Rbunch, Nbunch);
    Dbg=false;

    return Tbunch;
    
  }

  void FindBunchs(ProcessingStatus *ProcStat) {
    MakeSum();
    MakeSumNoise();
    Dbg=true;
    int ReportMode = ProcStat->GetReportMode();

    for (int i=1;i<nSlice-1; i++) {
      int i0=i-1;
      int i1=i+2;
      int i2=i+9;
      int i3=i+12;
      double t1=0.0, tx=0.0, s1=0.0, s2=0.0, r1=0.0, r2=0.0, s=0.0, r=0.0;
      if (i3>nSlice) i3=nSlice;
      if (i2>nSlice-1) i2=nSlice-1;
      bool Apos=true;
      for (int j=i0;j<i1;j++) {
        double a=SumHG[j]+SumLG[j];
        double t=Channel[0].GetSignalLong(0)->GetTime(j);
        if (a<0.) Apos=false;
        s1+=a; 
        t1+=t*a; 
        tx+=t; 
      } 
      if (Apos) {
        t1/=s1;
      } else {
        t1=tx/(i1-i0);
      }
      s1/=(i1-i0);
      r1=(NoiseHG+NoiseLG)/sqrt(i1-i0);      
      for (int j=i2;j<i3;j++) s2+=SumHG[j]+SumLG[j];
      s2/=(i3-i2);      
      r2=(NoiseHG+NoiseLG)/sqrt(i1-i0);
           
      s=s1-s2;
      r=sqrt(r1*r1+r2*r2);
      if (Dbg || ReportMode==10011) printf("[Module->FindBunchs] i=%3d t=%6.1f A=%6.1f S1=%6.1f S2=%6.1f S1-S2=%6.1f r=%.2f",
                      i, Channel[0].GetSignalLong(0)->GetTime(i),SumHG[i]+SumLG[i], s1, s2, s, r);
      if (s>3.0+3.*r) {
        if (Dbg) printf("  *\n");
      } else {
	if (Dbg) printf("\n");
      }
    }
    if (Dbg || ReportMode==10011) ReportMode=ProcStat->WhatToDo("[Module->FindBunchs]", 11);
    Dbg=false;
  }

  void MakeSumNoise() {
    Dbg=false;
    int iw=0;
    bool inWindow=false;
    for (int i=0; i<nSlice; i++) {
      double t=Channel[0].GetSignalLong(1)->GetTime(i);
      if (t<PedWindow1[iw] && t>=PedWindow0[iw]) {
	if (!inWindow) { // first sample for this window //
          inWindow=true;
          PedSamples[iw]=0;
          PedAverageHG[iw]=0.0;
          PedRMSHG[iw]=0.0;
          PedAverageLG[iw]=0.0;
          PedRMSLG[iw]=0.0;
	}
        PedSamples[iw]++;
        PedAverageHG[iw]+=SumHG[i];
        PedRMSHG[iw]+=SumHG[i]*SumHG[i];
        PedAverageLG[iw]+=SumLG[i];
        PedRMSLG[iw]+=SumLG[i]*SumLG[i];
      } else {
        if (inWindow) { // just left the Pedestal Window, do the math //
          inWindow=false;
          PedAverageHG[iw]/=PedSamples[iw];
          PedRMSHG[iw]/=PedSamples[iw];
          PedRMSHG[iw]-=PedAverageHG[iw]*PedAverageHG[iw];
          PedRMSHG[iw]=sqrt(PedRMSHG[iw]);
          PedAverageLG[iw]/=PedSamples[iw];
          PedRMSLG[iw]/=PedSamples[iw];
          PedRMSLG[iw]-=PedAverageLG[iw]*PedAverageLG[iw];
          PedRMSLG[iw]=sqrt(PedRMSLG[iw]);
          if (Dbg) printf(" [TModule->MakeSumNoise] window #%d T=(%6.1f, %6.1f) Low=(%.1f+/-%.1f) High=(%.1f+/-%.1f)\n",
			  iw, PedWindow0[iw], PedWindow1[iw], PedAverageLG[iw], PedRMSLG[iw], 
                         PedAverageHG[iw], PedRMSHG[iw]);
          iw++;
	}
      }
    }
    for (int iw1=0; iw1<NPedWindows; iw1++) {
      if (GoodPedWindow[iw1]) {
        NoiseHG+=PedRMSHG[iw1];
        NoiseLG+=PedRMSHG[iw1];
      }
    }
    NoiseHG/=NGoodWindows;
    NoiseLG/=NGoodWindows;
    if (Dbg) printf("[TModule->MakeSumNoise]----------  NoiseHG=%.2f NoiseLG=%.2f\n", NoiseHG, NoiseLG);
    Dbg=false;
  }

  void MakeSum() {
    Dbg=false;   
    //if (ID==4) Dbg=true;

    if (Dbg) printf("[Module->MakeSum] Preparing Channel Sums for %s, Nslice=%d\n", 
	            Name, nSlice);

    for (int i=0; i<nSlice; i++) {
      SumHG[i]=0.0;
      SumLG[i]=0.0;
      rmsHG[i]=0.0;
      rmsLG[i]=0.0;
      for (int iChan=0; iChan<4; iChan++) {
        if (Channel[iChan].GetStatus()) {
	  if (Channel[iChan].GetSignalLong(1)->GetDoFit(i)) {
            SumHG[i]+=Channel[iChan].GetSignalLong(1)->GetAmp2(i);
            rmsHG[i]+=Channel[iChan].GetSignalLong(1)->GetErr2(i);
	  }
	  if (Channel[iChan].GetSignalLong(0)->GetDoFit(i)) {
            SumLG[i]+=Channel[iChan].GetSignalLong(0)->GetAmp2(i); 
            rmsLG[i]+=Channel[iChan].GetSignalLong(0)->GetErr2(i);         
	  }        
	}
      }
      if (Dbg) {
        double t=Channel[0].GetSignalLong(1)->GetTime(i);
        printf("[Module->MakeSum] %3d %6.1f Low=%7.1f High=%7.1f\n", 
               i, t, SumLG[i], SumHG[i]);
      }   
    } 
    Dbg=false;
    return;   
  }


  const char* GetName() {return Name;}

  void SetSlices(int n) { // called once per run
    printf("[TModule->SetSlices] setting %10s slices to %d\n", Name, n);
    if (sizeSet) {
    } else {
      if (ID==6) nSlice=n; 
      else  nSlice=2*n;
      sizeSet=true;
      for (int i=0;i<nChannels;i++) {
        //printf("[TModule->SetSlices] setting %10s slices to %d\n", Channel[i].GetName(), n);
        Channel[i].SetSlices(n);
        //printf("[TModule->SetSlices]         %10s done\n", Channel[i].GetName()); 
      }
    }
    return;
  }

  void   SetPedWindow(double t0, double t1) {
    Tped0=t0; 
    Tped1=t1;
    for (int iCh=0; iCh<nChannels; iCh++) {
      for (int iGain=0; iGain<1; iGain++) {
        Channel[iCh].GetSignalLong(iGain)->SetPedWindow(Tped0, Tped1);
        for (int iDelay=0; iDelay<2; iDelay++) {
          Channel[iCh].GetSignalShort(iGain, iDelay)->SetPedWindow(Tped0, Tped1);
        }
      }
    }
  }
  double GetPedWindow0() {return Tped0;}
  double GetPedWindow1() {return Tped1;}
  double GetPedWin0(int i) {return PedWindow0[i];}
  double GetPedWin1(int i) {return PedWindow1[i];}
  bool   GetPedWinX(int i) {return GoodPedWindow[i];}
  int    GetNPedWindows()  {return NPedWindows;}

  TChannel* GetChannel(int i) { // returns pointer to the channel
    return &((TModule*)this)->Channel[i];
  }
/*
  void SetSlopeArrays(int n) {
    nPulses=n;
    moduleSum0 = (double*) calloc(3*nPulses, sizeof(double));
    moduleSum1 = (double*) calloc(3*nPulses, sizeof(double));
    moduleSum2 = (double*) calloc(3*nPulses, sizeof(double));
    cherSum1   = (double*) calloc(3*nPulses, sizeof(double));
    cherSum2   = (double*) calloc(3*nPulses, sizeof(double));
    cherModSum = (double*) calloc(3*nPulses, sizeof(double));
  }

  void ClearSlopeArrays() {
    for (int i=0; i<3*nPulses; i++) {
      moduleSum0[i]=0.0;
      moduleSum1[i]=0.0;
      moduleSum2[i]=0.0;
      cherSum1[i]  =0.0;
      cherSum2[i]  =0.0;
      cherModSum[i]=0.0;
    }
  }

  void AddToSlopeArrays(int iCase, int i, double Mod, double Cher) {
    if (iCase<0 || iCase>2 || i<0 || i>=nPulses) {
      printf("[TModule->AddToSlopeArrays] bad input parameters Case=%d Bin=%d\n", iCase, i);
      exit(-1);
    }
    int j=nPulses*iCase+i;
    moduleSum0[j]+=1.0;
    moduleSum1[j]+=Mod;
    moduleSum2[j]+=Mod*Mod;
    cherSum1[j]+=Cher;
    cherSum2[j]+=Cher*Cher;
    cherModSum[j]+=Mod*Cher;
  }

  double GetModule0(int iCase, int i) {
    if (iCase<0 || iCase>2 || i<0 || i>=nPulses) {
      printf("[TModule->GetModule0] wrong case=%d index=%d, nPulses set to %d\n", iCase, i, nPulses);
      exit(-1);
    } else {
      return moduleSum0[nPulses*iCase+i];
    }
  }
  double GetModule1(int iCase, int i) {
    if (iCase<0 || iCase>2 || i<0 || i>=nPulses) {
      printf("[TModule->GetModule0] wrong case=%d index=%d, nPulses set to %d\n", iCase, i, nPulses);
      exit(-1);
    } else {
      if (moduleSum0[nPulses*iCase+i]==0.0) {
        return 0.0;
      } else {
        return moduleSum1[nPulses*iCase+i]/moduleSum0[nPulses*iCase+i];
      }
    }
  }
  double GetModule2(int iCase, int i) {
    if (iCase<0 || iCase>2 || i<0 || i>=nPulses) {
      printf("[TModule->GetModule0] wrong case=%d index=%d, nPulses set to %d\n", iCase, i, nPulses);
      exit(-1);
    } else {
      if (moduleSum0[nPulses*iCase+i]==0.0) {
        return 0.0;
      } else {
        return moduleSum2[nPulses*iCase+i]/moduleSum0[nPulses*iCase+i];
      }
    }
  }
  double GetCher1(int iCase, int i) {
    if (iCase<0 || iCase>2 || i<0 || i>=nPulses) {
      printf("[TModule->GetModule0] wrong case=%d index=%d, nPulses set to %d\n", iCase, i, nPulses);
      exit(-1);
    } else {
      if (moduleSum0[nPulses*iCase+i]==0.0) {
        return 0.0;
      } else {
        return cherSum1[nPulses*iCase+i]/moduleSum0[nPulses*iCase+i]; 
      }
    }
  }
  double GetCher2(int iCase, int i) {
    if (iCase<0 || iCase>2 || i<0 || i>=nPulses) {
      printf("[TModule->GetModule0] wrong case=%d index=%d, nPulses set to %d\n", iCase, i, nPulses);
      exit(-1);
    } else {
      if (moduleSum0[nPulses*iCase+i]==0.0) {
        return 0.0;
      } else {
        return cherSum2[nPulses*iCase+i]/moduleSum0[nPulses*iCase+i];
      }
    }
  }
  double GetCherMod(int iCase, int i) {
    if (iCase<0 || iCase>2 || i<0 || i>=nPulses) {
      printf("[TModule->GetModule0] wrong case=%d index=%d, nPulses set to %d\n", iCase, i, nPulses);
      exit(-1);
    } else {
      if (moduleSum0[nPulses*iCase+i]==0.0) {
        return 0.0;
      } else {
        return cherModSum[nPulses*iCase+i]/moduleSum0[nPulses*iCase+i];
      }
    }
  }
*/
  void SetIntegrator(int Run) {
    if (ID<6) return;
    if (Run<0) printf("[TModule->SetIntegrator] bad run number %d\n", Run);
    for (int iChan=0; iChan<4; iChan++) {
      if (Channel[iChan].GetStatus()) {
        if (Run<460) {
          if (iChan<2) {
            Channel[iChan].GetSignalShort(0)->SetIntegrator(5430., 5530.);
          } else {
            Channel[iChan].GetSignalShort(0)->SetIntegrator(5470., 5530.);
          }
        } else {
          if (iChan<2) {
            Channel[iChan].GetSignalShort(0)->SetIntegrator(5700., 6300.);
          } else {
            Channel[iChan].GetSignalShort(0)->SetIntegrator(5700., 6300.);
          }
        }
      }
    }
  }

  void UpdateEmptyEventPedestal() {

    for (int iChan=0; iChan<4; iChan++) {
      if (Channel[iChan].GetStatus() && !(ID==5 && iChan==2)) { // do not do it for RF!
        Channel[iChan].UpdateEmptyEventPedestal();
      }      
    }

    if (ID>3) return; // Sum Noise only for modules! //

    for (int iGain=0; iGain<2; iGain++) {
      double rr=0.;
      for (int iDelay=0; iDelay<2; iDelay++) {
        int indx=2*iGain+iDelay;
        int NS=Channel[0].GetSignalShort(0)->GetNslice();
        for (int is=0; is<NS; is++) {
          double sum=0.;
          for (int iChan=0; iChan<4; iChan++) {
            sum+=Channel[iChan].GetSignalShort(indx)->GetRaw(is);
          }
          EmptyEventNoiseS0[indx]+=1.0;
          EmptyEventNoiseS1[indx]+=sum;
          EmptyEventNoiseS2[indx]+=sum*sum;
        }
        double S1=EmptyEventNoiseS1[indx]/EmptyEventNoiseS0[indx];
        double S2=EmptyEventNoiseS2[indx]/EmptyEventNoiseS0[indx];
        S2-=S1*S1;
        rr+=S2;
        //printf("[UpdateEmptyEventPedestal] %10s Gain#%d noise#%d=%6.3f\n", Name, iGain, iDelay, sqrt(S2));
      }
      EmptyEventNoise[iGain]=sqrt(0.5*rr);
      //printf("[UpdateEmptyEventPedestal] %10s Gain#%d noise sum = %6.3f\n", Name, iGain, EmptyEventNoise[iGain]);
    }
    return;
  }

  void UpdateEmptyEventPedestalLong() {
    for (int iChan=0; iChan<4; iChan++) {
      if (Channel[iChan].GetStatus() && !(ID==5 && iChan==2)) { // do not do it for RF!
        Channel[iChan].UpdateEmptyEventPedestalLong();
      }      
    }    
  }

  void ClearEmptyEventPedestal() {
    for (int iChan=0; iChan<4; iChan++) {
      if (Channel[iChan].GetStatus()) {
        Channel[iChan].ClearEmptyEventPedestal();
      }
    }
    for (int iGain=0; iGain<2; iGain++) {
      EmptyEventNoiseS0[iGain]=0.0;
      EmptyEventNoiseS1[iGain]=0.0;
      EmptyEventNoiseS2[iGain]=0.0;      
    }
  }

  bool CheckEmptyEventPedestals() {
    bool result=true;
    int  iChan1=4;
    if (ID==5) iChan1=3;
    for (int iChan=0; iChan<iChan1; iChan++) {
      if (Channel[iChan].GetStatus()) {
        bool ch=Channel[iChan].CheckEmptyEventPedestals();
        if (!ch) {
          printf("[TModule->CheckEmptyEventPedestals] failed for %s ch#%d\n", Name, iChan);
          result=false;
        }
      }
    }
    return result;
  }

  double GetEmptyEventNoise(int iGain) {
    if (iGain<0 || iGain>1) return 0.;
    return EmptyEventNoise[iGain];
  }


//---------------------------------------------------------------------------------------
  void ComputePrePed() {
    //printf("[TModule->ComputePrePed] %s\n", Name);
    for (int iChan=0; iChan<4; iChan++) {
      if (ID==5 && iChan==2) continue; // RF
      //printf("[TModule->ComputePrePed] chan#%d %s\n", iChan, Channel[iChan].GetName());
      Channel[iChan].ComputePrePed();
    }
  }

  void SubtractRF() {
    if (ID==4) {
      Channel[3].SubtractRF(0.04, 25.);
    } else if (ID==5) {
      Channel[1].SubtractRF(0.05, -25.);
    }  
  }

  void ComputeCoherent() {
    for (int iG=0; iG<2; iG++) {
      for (int iD=0; iD<2; iD++) {
        for (int i=0; i<nSlice/2; i++) {
          double coh=0.0;
          for (int iCh=0; iCh<4; iCh++) {
            coh+=Channel[iCh].GetSignalShort(2*iG+iD)->GetRaw(i);
          }
          coh/=4.0;
          ProcStatus->GetEventHeader()->SetCoherent(iG, iD, i, coh);
        }
      }
    }
    return;
  }


  void SubtractCoherent(int iModX) { // 
    if (ID>3) return;
    if (ID==iModX) return;
    double scale=0.;
    double phase=0.;
    if      (ID==0 || ID==1) {scale=0.6; phase=50;}
    else if (ID==2 || ID==3) {scale=1.0; phase=0.;}
    for (int iG=0; iG<2; iG++) {
      for (int iD=0; iD<2; iD++) {
        for (int iCh=0; iCh<4; iCh++) {
          for (int i=0; i<nSlice/2; i++) {
            int i1=i+int(phase/25.);
            if (i1<0)         i1+=nSlice/2;
            if (i1>=nSlice/2) i1-=nSlice/2;
            double a = Channel[iCh].GetSignalShort(2*iG+iD)->GetRaw(i);
            a-=ProcStatus->GetEventHeader()->GetCoherent(iG, iD, i1);
            Channel[iCh].GetSignalShort(2*iG+iD)->SetRaw(i, a);
          }
        }
      }
    }

    return;
  }
//---------------------------------------------------------------------------------------
  double GetIntegral(int iCh, int iG, double T0, double T1, bool &good) {
// integral is in fC, since scale is MPI ADC mV/ADC, after FFT (V/mA) FinalSignal is in uA
    double myIntegral=0.;
    good =false;
    if (ID>3) {
      printf("[Module->GetIntegral] use GetClusterPMTbunchAmp for beam, dummy!\n");
      exit(-1);
    }
    TSignal* mySignal=Channel[iCh].GetSignalLong(iG);
//    int nPedWindows=ProcStatus->GetPedWindows();
    int is0=mySignal->GetFirstSample(0);
    int is1=mySignal->GetLastSample(NPedWindows-1);
    if (time[is0+1]>T0) return 0.; // final signal starts too early 
    if (time[is1]<T1)   return 0.; // final signal ends too late
//    if (ProcStatus->GetReportMode()==1) {
//      printf("\n [Module->GetIntegral]  %s npw=%d t=%6.0f...%6.0f samples=%d...%d\n", 
//                 mySignal->GetName(), NPedWindows, T0, T1, is0, is1);
//    }
    for (int i=1; is0+i<is1; i++) {
      if (time[is0+i]<T0) continue;
      if (time[is0+i]>T1) break;
      myIntegral+=mySignal->GetFinalSignal(i); // [uA]
//      if (ID>1 || ProcStatus->GetReportMode()==1) {
//        printf("[Module->GetIntegral]  %3d t=%7.0f A=%6.1f\n", is0+i, time[is0+i], mySignal->GetFinalSignal(i)*12.5);
//      }
    }
    myIntegral*=12.5; // uA*ns=fC
    good =true;
    return myIntegral;
  }
//---------------------------------------------------------------------------------------
  void PreCompute2013() {
    if (ID==ProcStatus->GetRunHeader()->AntennaModule) return;
    if (ID<4) {
      PerformTailFit(); // do the pedestal interval fitting - prepare for the FFT reconstruction
      PrepareTailed();  //  append the LAST window tail and 
                        //  subtract the FIRST one prepare for the FFT reconstruction
      DoFFTshape();     //  recover current signal shape
      FixFFTbaseline(); // fits polynomial through pedestal intervals
    } else if (ID<5) {
      int iCh0=0;
      int iCh1=4;
      if (ID==5) iCh1=2;
      for (int iCh=iCh0; iCh<iCh1; iCh++) {
        Channel[iCh].RecoverPMTsignal();
        Channel[iCh].ComputePMTclusters();
//      Channel[iCh].FindPMTsignalPeaks();
      }
    }
    return;
  }
//---------------------------------------------------------------------------------------
void PreCompute() {

/* prepares bunch times for a module based on FitResults from file or GlobalPhase */

  LGoverflow=false;
  for (int iChan=0; iChan<4; iChan++) {
    if (ID==5 && iChan==2) continue; // RF, ignore
    Channel[iChan].ClearHiLoCrisisFlag();
    if (Channel[iChan].LowGainOverflowDetected()) {
      LGoverflow=true;
    }
  }
 // derive pulse parameters from the Global Phase and Spill Structure

  int Nbunches = ProcStatus->GetNbunches();
  if (ProcStatus->GetReportMode()==1) printf("[TModule->PreCompute] processing %d bunches\n", Nbunches);
  for (int bunch=0; bunch<Nbunches; bunch++) {
    double Tbunch=ProcStatus->GetBunchTime(bunch);
    FindPulseParams2012(Tbunch, bunch);
  }
  return;
}

void FindPulseParams2012(double Tbunch, int bunch) {

if (ID==ProcStatus->GetAntennaModule()) return; // no analysis for unplugged module //
int iCh0=0;
int iCh1=5;             // 4 channels plus sum
if (ID==4) iCh1=4;      // no sum for scintillators!
else if (ID==5) iCh1=2; // only 2 monitors
for (int iChan=iCh0; iChan<iCh1; iChan++) {
  if (Channel[iChan].GetStatus()) {
    Channel[iChan].FindPulseParams2012(Tbunch, bunch);
    Channel[iChan].CalculateResponse(bunch);
  }
  if (ProcStatus->GetReportMode()<0) break; // exit loop if there is interest in the last plot
} 
return;   
}

//---------------------------------------------------------------------------------------
void PreCompute2010() {

/* prepares bunch times for a module based on FitResults from file or GlobalPhase */

  LGoverflow=false;
  for (int iChan=0; iChan<4; iChan++) {
    if (ID==5 && iChan==2) continue; // RF
    Channel[iChan].ClearHiLoCrisisFlag();
    if (Channel[iChan].LowGainOverflowDetected()) {
      LGoverflow=true;
    }
  }



 // derive pulse parameters from the Global Phase and Spill Structure

  int Nbunches = ProcStatus->GetNbunches();
  for (int bunch=0; bunch<Nbunches; bunch++) {
    double Tbunch=ProcStatus->GetBunchTime(bunch);
    FindPulseParams2010(Tbunch, bunch);
  }

  return;
}

void computeSinCross() {
// fit with the RF frequency where applicable;
  if (ID<4 || ID>5) return;
  int iChan0=0;
  int iChan1=4;
  if (ID==5) iChan1=2;
  for (int iChan=iChan0; iChan<iChan1; iChan++) {
    Channel[iChan].computeSinCross();
  }
}  

void FillRFcorr() {
// fill RF frequency correction: A/A(RF), T0-T0(RF);
  if (ID<4 || ID>5) return;
  int iChan0=0;
  int iChan1=4;
  if (ID==5) iChan1=2;
  for (int iChan=iChan0; iChan<iChan1; iChan++) {
    Channel[iChan].FillRFcorr();
  }
}  

void SubtractRFcross() {
  if (ID<4 || ID>5) return;
  int iChan0=0;
  int iChan1=4;
  if (ID==5) iChan1=2;
  for (int iChan=iChan0; iChan<iChan1; iChan++) {
    //if (ProcStatus->GetReportMode()==1) {
    //  printf("[TModule->SubtractRFcross] calling SubtractRFcross() for %s\n", Channel[iChan].GetName());
    //}
    Channel[iChan].SubtractRFcross();
  }
  
}




  void FindPulseParams2010(double Tbunch, int bunch) {
    //Dbg=true;
    int ReportMode = ProcStatus->GetReportMode();
    if (Dbg || ReportMode==10015 || ReportMode==10024) {
      printf("\n[Module->FindPulseParams2010] %s --> T=%.1f bunch#%d\n", Name, Tbunch, bunch);
    }
    for (int iChan=0; iChan<4; iChan++) {
      if (Channel[iChan].GetStatus()) {
        Channel[iChan].FindPulseParams2010(Tbunch, bunch);
        Channel[iChan].CalculateResponse(bunch);
        if (Dbg || ReportMode==10015) {
          printf("[Module->FindPulseParams2010] %s\t Chan#%d A=%.1f\n", 
                 Name, iChan, Channel[iChan].GetResponseADC(bunch));
        } 
      }
      if (ProcStatus->GetReportMode()<0) break; // exit loop if there is interest in the last plot
    } 
    //Dbg=false;
    return;   
  }

  void ShowSumShapesI(int c0, int c1, int c2, int c3) {  
    char TitleC[200];
    char NamecC[200];
 
    bool UsedChan[4];
    UsedChan[0]=(c0==1);
    UsedChan[1]=(c1==1);
    UsedChan[2]=(c2==1);
    UsedChan[3]=(c3==1);
    int nUsed=0;
    for (int iChan=0; iChan<4; iChan++) if (UsedChan[iChan]) nUsed++;

    sprintf(TitleC, "Pulses for %s", Name);
    sprintf(NamecC, "PulsesFor_%s", Name);
    SignalCanvas->Clear();
    SignalCanvas->SetName(NamecC);
    SignalCanvas->SetTitle(TitleC);

    SignalCanvas->Divide(1, 2, 0.01, 0.01);
    SignalCanvas->Show();
 
    for (int iGain=0; iGain<2; iGain++) {
      SignalCanvas->cd(iGain+1);
      gStyle->SetOptStat(0);
      gStyle->SetOptTitle(0);
      gPad->SetBottomMargin(0.2);
      gPad->SetRightMargin(0.01);
      gPad->SetTopMargin(0.01);

      bool firstBin=true;
      int iCol=1;
      double Amax=0.0, Amin=0.0;

      double T0=0.0, T1=0.0, wChan=0.0;
      double binSum0=0.0;
      double binSum1=0.0;

      int iStep=1;
      int jBin=0;
      for (int iBin=0; iBin<5; iBin+=iStep) {
        if (iBin==4) iStep=2;
        bool first=true;
        binSum0=0.0;
        binSum1=0.0;

        TProfile *ShapeSumI = new TProfile;
        char Title[200];
        sprintf(Title, "%s rate bin#%d", Name, iBin);
        T0=0.0; 
        T1=0.0; 
        wChan=0.0;

        for (int iChan=0; iChan<4; iChan++) {
          if (Channel[iChan].GetStatus() && UsedChan[iChan]) {
            TSignal  *Signal = Channel[iChan].GetGoodSignal(iGain);            
            binSum0+=Signal->GetAccRate0(iBin);
            binSum1+=Signal->GetAccRate1(iBin);
            wChan+=1.0;
            printf("        iChan=%d binSum0=%f binSum1=%f wChan=%f\n", iChan, binSum0, binSum1, wChan);
            if (binSum0>0.0) {
              if (first) {
                first=false;
                ShapeSumI = (TProfile *)Signal->GetPulseShapeProfileI(iBin)->Clone(Title);
                T0=Signal->GetTshape0();
                T1=Signal->GetTshape1();
              } else {
                ShapeSumI->Add(Signal->GetPulseShapeProfileI(iBin), 1.0);
              }
            }
          }
        }

        printf("   iBin=%d wChan=%f binSum0=%f\n", iBin, wChan, binSum0);
        if (binSum0>1000.0) {
          ShapeSumI->Scale(wChan);
 	  ShapeSumI->SetLineColor(iCol);
	  ShapeSumI->SetMarkerColor(iCol);
          for (int i=0; i<ShapeSumI->GetNbinsX(); i++) {
            if (ShapeSumI->GetBinCenter(i)>-100.0 && ShapeSumI->GetBinCenter(i)<500.0) {
              double a=ShapeSumI->GetBinContent(i);
              if (a>Amax) Amax=a;
              if (a<Amin) Amin=a;
              //printf("i=%d x=%.1f cont=%.1f min=%.1f max=%.1f\n", 
              //       i, ShapeSumI->GetBinCenter(i), a, Amin, Amax);
            }
          }
          if (firstBin) {
            firstBin=false;
            ShapeSumI->GetYaxis()->SetRangeUser(1.1*Amin, 1.1*Amax);
            sprintf(Title, "%s Response, low gain ADC/proton", Name);
            ShapeSumI->GetYaxis()->SetTitle(Title);
            T1=700.;
            ShapeSumI->GetXaxis()->SetRangeUser(T0, T1);
            ShapeSumI->GetXaxis()->SetTitle("Time, ns");
            ShapeSumI->DrawCopy("L");
            TLine *lz = new TLine(T0, 0.0, T1, 0.0);
            lz->Draw();
          } else {
            ShapeSumI->GetYaxis()->SetRangeUser(1.1*Amin, 1.1*Amax);
            ShapeSumI->DrawCopy("same L");
          }
          double yy=Amax-(Amax-Amin)*0.08*jBin;
          char TLTEXT[200];
          sprintf(TLTEXT, "Rate = %.1e/bunch", binSum1/binSum0);
          TLatex *lRate = new TLatex(250., yy, TLTEXT);
          lRate->Draw();
          TLine *zRate = new TLine(150., yy+(Amax-Amin)*0.04, 230., yy+(Amax-Amin)*0.04);
          zRate->SetLineColor(iCol);
          zRate->SetLineWidth(5);
          zRate->Draw(); 

          iCol++;
          if (iCol==5) iCol++;
          if (iCol>7)  iCol=1;
          jBin++;
          printf(" %s iGain=%d iBin=%d stat=%f AverageRate=%.1e wChan=%f\n", 
                 Name, iGain, iBin, binSum0, binSum1/binSum0, wChan); 
        }
      }


    }
  }

  void ShowPulseShapesI() {  
    char TitleC[200];
    char NamecC[200];
    sprintf(TitleC, "Signals for %s, run#%d", Name, ProcStatus->GetEventHeader()->run);
    sprintf(NamecC, "PulsesFor_%s", Name);
    SignalCanvas->Clear();
    SignalCanvas->SetName(NamecC);
    SignalCanvas->SetTitle(TitleC);

    int ndiv=0;
    for (int iChan=0;iChan<4;iChan++) {
      if (Channel[iChan].GetStatus()) ndiv++;
    }
    SignalCanvas->Divide(2, ndiv, 0.01, 0.01);
    SignalCanvas->Show();
 
    for (int iGain=0; iGain<2; iGain++) {
      int iZone=0;
      for (int iChan=0; iChan<4; iChan++) {
        if (Channel[iChan].GetStatus()) {
          //printf("[ShowPulseShapesI] reaching for channel %s\n", Channel[iChan].GetName());
          TSignal  *Signal = Channel[iChan].GetGoodSignal(iGain);
          //printf("[ShowPulseShapesI] found good signal %s\n", Signal->GetName());
 
	  SignalCanvas->cd(1+iZone*2+iGain);
          gStyle->SetOptStat(0);
          gPad->SetBottomMargin(0.2);
          gPad->SetRightMargin(0.01);
          gPad->SetTopMargin(0.01);

          int iCol=0;
          for (int iBin=0; iBin<5; iBin++) {
            //printf("[ShowPulseShapesI] reaching for shape#%d\n", iBin);
            TProfile *Shape = Signal->GetPulseShapeProfileI(iBin);
            //printf("[ShowPulseShapesI] got shape %s\n", Shape->GetName());
	    Shape->SetLineColor(iCol);
	    Shape->SetMarkerColor(iCol);
            if (iBin==0) Shape->DrawCopy();
	    else Shape->DrawCopy("same");
            iCol++;
            if (iCol>7) iCol=0;
          }
          for (int iBin=5; iBin<20; iBin+=2) {
            //printf("[ShowPulseShapesI] reaching for shape#%d\n", iBin);
            TProfile *Shape = Signal->GetPulseShapeProfileI(iBin);
            //printf("[ShowPulseShapesI] got shape %s\n", Shape->GetName());
	    Shape->SetLineColor(iCol);
	    Shape->SetMarkerColor(iCol);
            if (iBin==0) Shape->DrawCopy();
	    else Shape->DrawCopy("same");
            iCol++;
            if (iCol>7) iCol=0;
          }
          SignalCanvas->Update();
          iZone++;
        }
      }
    }


  }

  void ShowPulseShapeSplit() {  
    char TitleC[200];
    char NamecC[200];
    sprintf(TitleC, "Signals for %s, run#%d", Name, ProcStatus->GetEventHeader()->run);
    sprintf(NamecC, "PulsesFor_%s", Name);
    SignalCanvas->Clear();
    SignalCanvas->SetName(NamecC);
    SignalCanvas->SetTitle(TitleC);
    sprintf(TitleC, "Normalized pulses for %s, Run#%d", Name, ProcStatus->GetEventHeader()->run);
    sprintf(NamecC, "Norm_PulsesFor_%s_Run%d", Name, ProcStatus->GetEventHeader()->run);
    SignalCanvasNorm->Clear();
    SignalCanvasNorm->SetName(NamecC);
    SignalCanvasNorm->SetTitle(TitleC);

    int ndiv=0;
    int nStatCut=100;

    for (int iChan=0;iChan<4;iChan++) {
      if (Channel[iChan].GetStatus()) ndiv++;
    }
    SignalCanvas->Divide(2, ndiv, 0.01, 0.01);
    SignalCanvas->Show();
    SignalCanvasNorm->Divide(3, 2);
    SignalCanvasNorm->Show();
 
    for (int iGain=0; iGain<2; iGain++) {
      int iZone=0;
      for (int iChan=0; iChan<4; iChan++) {
        if (Channel[iChan].GetStatus()) {
          TSignal  *Signal = Channel[iChan].GetGoodSignal(iGain);
 
	  SignalCanvas->cd(1+iZone*2+iGain);
          gStyle->SetOptStat(0);
          gPad->SetGridx(1);
          gPad->SetBottomMargin(0.2);
          gPad->SetRightMargin(0.01);
          gPad->SetTopMargin(0.01);

          int iCol=1;
          bool started=false;
          int istep=1;
          int fullCount=0;
          int istart=Signal->GetNshapeSplit()-1;
          
          bool foundLargestBin=false;
  
          while (true) {
            fullCount=0;
            for (int iBin=istart; iBin>-1; iBin-=istep) {
              if (Signal->GetSplitRate(iBin)>nStatCut) {
                fullCount++;
                if (!foundLargestBin) {istart=iBin;  foundLargestBin=true;}
              }          
            }
            if (fullCount<7) break;
            istep++;
          }
          printf("[ShowPulseShapeSplit] %s plotting Start=%d step=%d\n", Signal->GetName(), istart, istep); 
          
          for (int iBin=istart; iBin>-1; iBin-=istep) {
            double A=0.0;
            if (Signal->GetSplitRate(iBin)>0) A=Signal->GetSplitSum(iBin)/Signal->GetSplitRate(iBin);
            printf("[ShowPulseShapeSplit]         bin#%3d of %d, Entries=%.1f <A>=%.2f", 
                   iBin, Signal->GetNshapeSplit(), Signal->GetSplitRate(iBin), A);
            if (Signal->GetSplitRate(iBin)>nStatCut) {
              printf(" plotting ------------->\n");
              TProfile *Shape = Signal->GetPulseShapeProfileSplit(iBin);
	      Shape->SetLineColor(iCol);
	      Shape->SetLineColor(0);
	      Shape->SetMarkerColor(iCol);
	      Shape->SetMarkerColor(0);
              if (!started) {
                sprintf(TitleC, "%s, run#%d", Signal->GetName(), ProcStatus->GetEventHeader()->run);
                Shape->SetTitle(TitleC); 
                Shape->GetXaxis()->SetTitle("Time, ns"); 
                Shape->GetYaxis()->SetTitle("Signal, ADC"); 
                Shape->DrawCopy(); 
                started=true;
              } /// make fake plots ///
	      else Shape->DrawCopy("same");
              iCol++;
              if (iCol==3) iCol++;
              if (iCol==5) iCol++;
              if (iCol>8) iCol=1;
            } else {
              printf("\n");
            }
          }

          iCol=1;

          for (int iBin=istart; iBin>-1; iBin-=istep) {
 
           if (Signal->GetSplitRate(iBin)>nStatCut) {

              TProfile *Shape = Signal->GetPulseShapeProfileSplit(iBin);
 
              for (int i=0; i<Shape->GetNbinsX()-1; i++) {
                TLine *ll = new TLine (Shape->GetBinCenter(i), Shape->GetBinContent(i), 
                                       Shape->GetBinCenter(i+1), Shape->GetBinContent(i+1));
                ll->SetLineWidth(2);
                ll->SetLineColor(iCol);
                ll->Draw();
      
              }
              iCol++;
              if (iCol==3) iCol++;
              if (iCol==5) iCol++;
              if (iCol>8) iCol=1;
            }
          }

          TLine *l0 = new TLine (Signal->GetTshape0(), 0., Signal->GetTshape2(), 0.);
          l0->SetLineColor(1);
          l0->SetLineStyle(2);
          l0->Draw();
           

          SignalCanvas->Update();
          iZone++;
        }
      }
    }
    
    int iChan1=3;
    int iChan2=4;
    if (ID==5) {iChan1=2; iChan2=3;}


    for (int iGain=0; iGain<2; iGain++) {
      for (int iChan=iChan1; iChan<iChan2; iChan++) {
        if (Channel[iChan].GetStatus()) {
          TSignal  *Signal = Channel[iChan].GetGoodSignal(iGain);
 
	  SignalCanvasNorm->cd(1+3*iGain);
          gStyle->SetOptStat(0);
          gPad->SetGridx(1);
          gPad->SetBottomMargin(0.2);
          gPad->SetRightMargin(0.01);
          gPad->SetTopMargin(0.01);

          int iCol=1;
          bool started=false;
          int istep=1;
          int fullCount=0;
          int istart=Signal->GetNshapeSplit()-1;
          
          bool foundLargestBin=false;
  
          while (true) {
            fullCount=0;
            for (int iBin=istart; iBin>-1; iBin-=istep) {
              if (Signal->GetSplitRate(iBin)>100) {
                fullCount++;
                if (!foundLargestBin) {istart=iBin;  foundLargestBin=true;}
              }          
            }
            if (fullCount<7) break;
            istep++;
          }
          printf("[ShowPulseShapeSplit] %s plotting Start=%d step=%d\n", Signal->GetName(), istart, istep); 
          
          for (int iBin=istart; iBin>-1; iBin-=istep) {
            double A=0.0;
            if (Signal->GetSplitRate(iBin)>0) A=Signal->GetSplitSum(iBin)/Signal->GetSplitRate(iBin);
            printf("[ShowPulseShapeSplit]         bin#%3d of %d, Entries=%.1f <A>=%.2f", 
                   iBin, Signal->GetNshapeSplit(), Signal->GetSplitRate(iBin), A);
            if (Signal->GetSplitRate(iBin)>nStatCut) {
              printf(" plotting ------------->\n");
              TProfile *Shape = Signal->GetPulseShapeProfileSplit(iBin);
	      Shape->SetLineColor(iCol);
	      Shape->SetLineColor(0);
	      Shape->SetMarkerColor(iCol);
	      Shape->SetMarkerColor(0);
	      Shape->GetXaxis()->SetRangeUser(-100., 250.);
              if (!started) {
                sprintf(TitleC, "%s, run#%d", Signal->GetName(), ProcStatus->GetEventHeader()->run);
                Shape->SetTitle(TitleC); 
                Shape->GetXaxis()->SetTitle("Time, ns"); 
                Shape->GetYaxis()->SetTitle("Signal, ADC"); 
                Shape->DrawCopy(); 
                Signal->GetSelectSplit()->DrawCopy("same"); 
                started=true;
              } /// make fake plots ///
	      else Shape->DrawCopy("same");
              iCol++;
              if (iCol==3) iCol++;
              if (iCol==5) iCol++;
              if (iCol>8) iCol=1;
            } else {
              printf("\n");
            }
          }

          iCol=1;

          for (int iBin=istart; iBin>-1; iBin-=istep) {
 
           if (Signal->GetSplitRate(iBin)>nStatCut) {

              TProfile *Shape = Signal->GetPulseShapeProfileSplit(iBin);
 
              for (int i=0; i<Shape->GetNbinsX()-1 && Shape->GetBinCenter(i+1)<250.; i++) {
                TLine *ll = new TLine (Shape->GetBinCenter(i), Shape->GetBinContent(i), 
                                       Shape->GetBinCenter(i+1), Shape->GetBinContent(i+1));
                ll->SetLineColor(iCol);
                ll->SetLineWidth(2);
                ll->Draw();
      
              }
              iCol++;
              if (iCol==3) iCol++;
              if (iCol==5) iCol++;
              if (iCol>8) iCol=1;
            }
          }

          TLine *l0 = new TLine (Signal->GetTshape0(), 0., 250., 0.);
          l0->SetLineColor(1);
          l0->SetLineStyle(2);
          l0->Draw();
           
          SignalCanvasNorm->Update();

	  SignalCanvasNorm->cd(2+3*iGain);
          gStyle->SetOptStat(0);
          gStyle->SetOptTitle(0);
          gPad->SetGridx(1);
          gPad->SetBottomMargin(0.2);
          gPad->SetRightMargin(0.01);
          gPad->SetTopMargin(0.01);

          iCol=1;
          started=false;
          istep=1;
          fullCount=0;
          istart=Signal->GetNshapeSplit()-1;
          
          foundLargestBin=false;
          double y1=0.;
          double y0=0.;
          while (true) {
            fullCount=0;
            for (int iBin=istart; iBin>-1; iBin-=istep) {
              if (Signal->GetSplitRate(iBin)>100) {
                fullCount++;
                if (!foundLargestBin) {istart=iBin;  foundLargestBin=true;}
              }          
            }
            if (fullCount<7) break;
            istep++;
          }
          printf("[ShowPulseShapeSplit] %s plotting Start=%d step=%d\n", Signal->GetName(), istart, istep); 
          
          for (int iBin=istart; iBin>-1; iBin-=istep) {
            double A=0.0;
            if (Signal->GetSplitRate(iBin)>0) A=Signal->GetSplitSum(iBin)/Signal->GetSplitRate(iBin);
            printf("[ShowPulseShapeSplit]         bin#%3d of %d, Entries=%.1f <A>=%.2f", 
                   iBin, Signal->GetNshapeSplit(), Signal->GetSplitRate(iBin), A);
            if (Signal->GetSplitRate(iBin)>nStatCut) {
              printf(" plotting ------------->\n");
              TProfile *Shape = Signal->GetPulseShapeProfileSplit(iBin);
	      Shape->SetLineColor(iCol);
	      Shape->SetLineColor(0);
	      Shape->SetMarkerColor(iCol);
	      Shape->SetMarkerColor(0);
	      Shape->GetXaxis()->SetRangeUser(250., 900.);
              if (!started) {
                sprintf(TitleC, "%s, run#%d", Signal->GetName(), ProcStatus->GetEventHeader()->run);
                Shape->SetTitle(TitleC); 
                Shape->GetXaxis()->SetTitle("Time, ns"); 
                Shape->GetYaxis()->SetTitle("Signal, ADC"); 
                Shape->DrawCopy();
                started=true;
              } /// make fake plots ///
	      else Shape->DrawCopy("same");
              iCol++;
              if (iCol==3) iCol++;
              if (iCol==5) iCol++;
              if (iCol>8) iCol=1;
            } else {
              printf("\n");
            }
          }

          double xline[4];
          if (iGain==0) {
            xline[0]=330.;
            xline[1]=380.;
            xline[2]=490.;
            xline[3]=570.;
          } else {
            xline[0]=320.;
            xline[1]=370.;
            xline[2]=460.;
            xline[3]=560.;
          }

          
          for (int iBin=istart; iBin>-1; iBin-=istep) { 
            if (Signal->GetSplitRate(iBin)>nStatCut) {
              TProfile *Shape = Signal->GetPulseShapeProfileSplit(iBin); 
              for (int i=0; i<Shape->GetNbinsX(); i++) {
                if (Shape->GetBinCenter(i)>250.)  {
                  if (y0>Shape->GetBinContent(i)) y0=Shape->GetBinContent(i);
                  if (y1<Shape->GetBinContent(i)) y1=Shape->GetBinContent(i);
                }
              }
            }
          }

          for (int i=0; i<4; i++) {
            TLine *ll = new TLine (xline[i], y0, xline[i], y1);
            ll->SetLineColor(5);
            ll->SetLineWidth(10);
            ll->Draw();
            char KTS[100];
            sprintf(KTS, "%d", i+1);
            TLatex *tl0 = new TLatex(xline[i], y0, KTS);
            tl0->SetTextSize(0.05);
            tl0->SetTextAlign(23);
	    tl0->Draw();
          }
          iCol=1;
          Signal->GetSelectSplit()->DrawCopy("same"); 

          for (int iBin=istart; iBin>-1; iBin-=istep) {
 
           if (Signal->GetSplitRate(iBin)>nStatCut) {

              TProfile *Shape = Signal->GetPulseShapeProfileSplit(iBin);
 
              for (int i=0; i<Shape->GetNbinsX()-1; i++) {
                if (Shape->GetBinCenter(i)>250. && Shape->GetBinCenter(i+1)<900.)  {
                  TLine *ll = new TLine (Shape->GetBinCenter(i), Shape->GetBinContent(i), 
                                         Shape->GetBinCenter(i+1), Shape->GetBinContent(i+1));
                  ll->SetLineColor(iCol);
                  ll->SetLineWidth(2);
                  ll->Draw();
                }
              }
              iCol++;
              if (iCol==3) iCol++;
              if (iCol==5) iCol++;
              if (iCol>8) iCol=1;
            }
          }

          TLine *l0a = new TLine (250., 0., Signal->GetTshape2(), 0.);
          l0a->SetLineColor(1);
          l0a->SetLineStyle(2);
          l0a->Draw();

          y0=0.;
          y1=0.;
          for (int iBin=0; iBin<Signal->GetNshapeSplit(); iBin++) {
            TProfile *Shape = Signal->GetPulseShapeProfileSplit(iBin);
            if (Signal->GetSplitRate(iBin)>20) {
              for (int i=0; i<Shape->GetNbinsX(); i++) {
                if (Shape->GetBinCenter(i)>xline[0]-5.) {
                  if (1.05*Shape->GetBinContent(i)>y1) y1=1.05*Shape->GetBinContent(i);
                  if (1.05*Shape->GetBinContent(i)<y0) y0=1.05*Shape->GetBinContent(i);
                }
              }
            }
          }
         

	  SignalCanvasNorm->cd(3+3*iGain);
          gStyle->SetOptStat(0);
          gStyle->SetOptTitle(0);
          gPad->SetGridx(1);
          gPad->SetBottomMargin(0.2);
          gPad->SetRightMargin(0.01);
          gPad->SetTopMargin(0.01);

          dummy0->SetBins(2800, 0., 2800.);
          dummy0->GetYaxis()->SetRangeUser(y0, y1);
          dummy0->GetYaxis()->SetTitle("Wiggle, ADC");
          dummy0->GetXaxis()->SetTitle("Signal Peak, ADC");
          dummy0->DrawCopy();

          
          iCol=1;
          for (int ip=0; ip<4; ip++) {
            double Xold=0.0; 
            double Yold=0.0;
            double Xnew=0.0;
            double Ynew=0.0;
            double Xd=0.0;
            double Yd=0.0;
            double Wnew=0.0;
            printf("Peak-dependance @ %.1fns\n", xline[ip]);
            for (int iBin=0; iBin<Signal->GetNshapeSplit()-1; iBin++) {
              Xnew=0.0;
              Ynew=0.0;
              Wnew=0.0;
              printf("       bin=%3d N=%.1f \n", iBin, Signal->GetSplitRate(iBin));
              if (Signal->GetSplitRate(iBin)>20.0) {
                Xnew=Signal->GetSplitSum(iBin)/Signal->GetSplitRate(iBin);
                TProfile *Shape = Signal->GetPulseShapeProfileSplit(iBin);
                Wnew=0.0;
                Ynew=0.0;
                for (int i=0; i<Shape->GetNbinsX(); i++) {
                  double x=Shape->GetBinCenter(i);
                  if (-5.0<x-xline[ip] && x-xline[ip]<5.0) {
                    Wnew+=Shape->GetBinEntries(i);
                    Ynew+=Shape->GetBinContent(i)*Shape->GetBinEntries(i);
                  }
                }
                printf("        Wnew=%.1f", Wnew);
                if (Wnew>0.0 && iBin>0) {
                  Ynew/=Wnew;
                  TLine *ls = new TLine(Xold, Yold, Xnew, Ynew);
                  if (1.05*Ynew>y1) y1=1.05*Ynew;
                  if (1.05*Ynew<y0) y0=1.05*Ynew;
                  if (1.05*Yold>y1) y1=1.05*Yold;
                  if (1.05*Yold<y0) y0=1.05*Yold;
                  Xd=Xnew;
                  Yd=Ynew;
                  ls->SetLineColor(iCol);
                  ls->SetLineWidth(2);
                  ls->Draw();
                  Xold=Xnew;
                  Yold=Ynew;
                  printf("     Xnew=%.1f Ynew=%f\n", Xnew, Ynew);
                } else {
                  printf("\n");
                }
              }
            }
            char KTS[100];
            sprintf(KTS, "%d", ip+1);
            TLatex *tl0 = new TLatex(Xd, Yd, KTS);
            tl0->SetTextSize(0.05);
            tl0->SetTextAlign(12);
            tl0->SetTextColor(iCol);
	    tl0->Draw();
            iCol++;
            if (iCol==3) iCol++;
            if (iCol==5) iCol++;
            if (iCol>8) iCol=1;
          }
          dummy0->GetYaxis()->SetRangeUser(y0, y1);

           
          SignalCanvasNorm->Update();
        }
      }


    }


  }



  void DisplayHighToLow() {
    HighTOlowCanvas->Clear();
    HighTOlowCanvas->Divide(2,2);
    for (int iChan=0; iChan<4; iChan++) {
      HighTOlowCanvas->cd(iChan+1);
      gStyle->SetOptStat(111110);
      Channel[iChan].GetHighToLow()->DrawCopy();
    //Setup[4].GetChannel(iChan)->GetHighToLow()->Reset();
      double HighLow=Channel[iChan].GetHighLow();
      TLine *l1 = new TLine(15., HighLow, 265., HighLow);
      l1->SetLineWidth(3);
      l1->Draw();
    }  
    HighTOlowCanvas->Update();
  }

  void DisplayHighVsLow() {
    HighVSlowCanvas->Clear();
    HighVSlowCanvas->Divide(2,2);
    for (int iChan=0; iChan<4; iChan++) {
      HighVSlowCanvas->cd(iChan+1);
      gStyle->SetOptStat(111110);
      Channel[iChan].GetHighVsLow()->DrawCopy();
    //Setup[4].GetChannel(iChan)->GetHighToLow()->Reset();
      double HighLow=Channel[iChan].GetHighLow();
      TLine *l1 = new TLine(0., 0., 265., 265.*HighLow);
      l1->SetLineWidth(1);
      l1->Draw();
    }  
    HighVSlowCanvas->Update();
  }

};


