void FoldHists(TH1D *h0, TH1D *h1, TH1D *h2);
void SmoothTail(TH1D *h0, double pFrac);
void SmoothTail1(TH1D *h0);
void SmoothTail1full(TH1D *h0); // with show
 // full version - with 'runninig' display in ShowCanvas
bool   SubtractZeroes(TH1D *emptyHist, TH1D *dataHist, TH1D *signalHist, double &zeroRate, double &signalRate, TCanvas *showCanvas);
double ComputeFee(TH1D *h0, double wFee, int zeroBin, TH1D* h2);
double ComputeChi2(TH1D* h0, TH1D* h1, TH1D* h2);

// short version without intermediate grafics;
bool   SubtractZeroes(TH1D *emptyHist, TH1D *dataHist, TH1D *signalHist, double &zeroRate, double &signalRate);
double ComputeFee(TH1D *h0, double wFee, int zeroBin);
double ComputeChi2(TH1D* h0, TH1D* h1);
//---------------------------------------------------------------------

// full version //
bool SubtractZeroes(TH1D *emptyHist, TH1D *dataHist, TH1D *signalHist, double &zeroRate, double &signalRate, TCanvas *showCanvas) {
bool result=true;
int iqq=2;
char title[200];
int nBins=emptyHist->GetNbinsX();
double wFee=1.;
if (dataHist->GetNbinsX() != nBins) {
  printf("[SubtractZeroes] wrong nBins=%d for data histogram %s\n",
          dataHist->GetNbinsX(), dataHist->GetName());
  return false;
}
if (signalHist->GetNbinsX() != nBins) {
  printf("[SubtractZeroes] wrong nBins=%d for output histogram %s\n",
          signalHist->GetNbinsX(), signalHist->GetName());
  return false;
}
TH1D *stepHist = (TH1D*) dataHist->Clone("stepHist");
TH1D *newHist  = (TH1D*) dataHist->Clone("newHist");
TH1D *errHist  = (TH1D*) dataHist->Clone("errHist");
TH1D *pedHist  = (TH1D*) emptyHist->Clone("pedHist");
TH1D *diffHist = (TH1D*) emptyHist->Clone("diffHist");
TH1D *feeHist  = (TH1D*) emptyHist->Clone("feeHist");

double wEmpty=emptyHist->GetEntries();
pedHist->Scale(1.0/wEmpty);

double sigmaNoise=emptyHist->GetRMS();
double data0=0.;
double nois0=0.;
for (int i=0; i<nBins+1; i++) {
  if (emptyHist->GetBinCenter(i)>0.5*sigmaNoise) break;
  data0+=dataHist->GetBinContent(i);
  nois0+=emptyHist->GetBinContent(i);
}
double scale0=data0/nois0;
zeroRate=scale0*emptyHist->GetEntries();
signalRate=dataHist->GetEntries()-zeroRate;
int zeroBin=0;

//=============================
for (int iTer=0; iTer<30; iTer++) {
//=============================
stepHist->Reset();
newHist->Reset();
errHist->Reset();
signalHist->Reset();
feeHist   ->Reset();
diffHist  ->Reset();
newHist->SetLineColor(kRed);
newHist->SetLineWidth(1);
stepHist->SetLineColor(kBlue);
stepHist->SetLineWidth(1);
feeHist->SetLineColor(kRed);
feeHist->SetLineWidth(1);
diffHist->SetLineColor(kGreen);
diffHist->SetLineWidth(1);



zeroBin=0;
for (int i=0; i<nBins+1; i++) {
  double w=dataHist->GetBinContent(i)-scale0*emptyHist->GetBinContent(i);
  if (emptyHist->GetBinCenter(i)<=0.) zeroBin=i;
  if (emptyHist->GetBinCenter(i)<0.) w=0.;
  else if (w<1.0) w=1.0;
  signalHist->SetBinContent(i, w);
  w=sqrt(1.+dataHist->GetBinContent(i));
  errHist->SetBinContent(i, w);
}
signalHist->SetBinContent(zeroBin, zeroRate);
for (int i=0; i<nBins+1; i++) {
  if (emptyHist->GetBinCenter(i)<0.) {
    stepHist->SetBinContent(i, 0.0);
  } else {
    double w=0.1*signalHist->GetBinContent(i);
    if (w<5.) w=5.;
    stepHist->SetBinContent(i, w);
  }
}

FoldHists(pedHist, signalHist, newHist);

TH1D *stepHist0 = (TH1D*) stepHist->Clone("stepHist0");
showCanvas->Clear();
showCanvas->cd(0);
//emptyHist->DrawCopy();
dataHist->DrawCopy("e");
signalHist->DrawCopy("same");
newHist->DrawCopy("e same");
diffHist->Draw("same");
feeHist->Draw("same");
showCanvas->Update();

double chi2old=0.;
double chi2new=0.;
double chi2=0.;
int    iLoop=0;
double nPosData=0.;
double nPosSign=0.;
double stepOld=0.;
double stepNew=0.;

for (int i=1; i<nBins+1; i++) {
  if (dataHist->GetBinContent(i)>0.0)   nPosData+=1.0;
  if (signalHist->GetBinContent(i)>0.0) nPosSign+=1.0;
  double s=stepHist->GetBinContent(i);
  stepOld+=s*s;
  //double err=errHist->GetBinContent(i);
  if ((i>zeroBin-5 && i<zeroBin+5) ||
      i>nBins-5) {
    printf("iBin=%3d x=%.1f w=%.0f step=%.2f\n",
           i, signalHist->GetBinCenter(i), signalHist->GetBinContent(i), stepHist->GetBinContent(i));
  }
}
chi2=ComputeChi2(newHist, dataHist, diffHist);

//double nDF=nPosData-nPosSign;
double fee=ComputeFee(signalHist, wFee, zeroBin, feeHist);
chi2old=chi2+fee;
stepOld=sqrt(stepOld);
//double fee0=fee;
double feeNew=fee;

//printf("loop#%04d chi2old=%.2f\t chi2new=%.2f\t fee=%f\n", iLoop, chi2, chi2, feeNew);
stepNew=1.0;

while (iLoop<1000 && stepNew>0.01) {
  iLoop++;
  for (int is=1; is<nBins+1; is++) {
    double step=stepHist->GetBinContent(is);
    if (step==0.) continue;
    double wOld=signalHist->GetBinContent(is);
    double wNew=wOld+step;
    if (wNew<0.) {
      stepHist->SetBinContent(is, 0.66*step);
      continue;
    }
    signalHist->SetBinContent(is, wNew);
    newHist->Reset();
    FoldHists(pedHist, signalHist, newHist);
    chi2=ComputeChi2(newHist, dataHist, diffHist);
    fee=ComputeFee(signalHist, wFee, zeroBin, feeHist);
    chi2new=chi2+fee;

    //chi2new*=fee/fee0;
    //chi2new/=nDF;

    if (chi2new<chi2old) {
      stepHist->SetBinContent(is, 1.33*step);
      chi2old=chi2new;
      feeNew=fee;
    } else {
      stepHist->SetBinContent(is, -0.66*step);
      signalHist->SetBinContent(is, wOld); // return old spectrum value
    }

  } // sample loop //

  for (int i=1; i<nBins+2; i++) {
    if (i>nBins-5) {
        printf("iBin=%3d\t x=%.1f\t w=%.0f\t step=%.2f\n",
           i, signalHist->GetBinCenter(i), signalHist->GetBinContent(i), stepHist->GetBinContent(i));
    }
  }

  showCanvas->Clear();
  showCanvas->cd(0);
  sprintf(title, "iter#%d chi2=%.3f\t fee=%f\t", iLoop, chi2new-feeNew, feeNew);
  dataHist->SetTitle(title);
  //dataHist->GetXaxis()->SetRangeUser(dataHist->GetXaxis()->GetXmax()-20., dataHist->GetXaxis()->GetXmax()+5.);
  //printf("%s new/old max=%f\n", title, stepNew);
  dataHist->DrawCopy("e");
  signalHist->DrawCopy("same");
  newHist->DrawCopy("e same");
  diffHist->Draw("same");
  feeHist->Scale(wFee);
  feeHist->Draw("same");
  showCanvas->Update();
  stepNew=0.0;
  for (int i=1; i<nBins+1; i++) {
    double s1=stepHist ->GetBinContent(i);
    double s0=stepHist0->GetBinContent(i);
    if (s1==0.0 || s0==0.0) continue;
    if (stepNew<fabs(s1/s0)) stepNew=fabs(s1/s0);
  }   
  if (iqq==1 && iTer%10==0) {
    printf("push->");
    cin >> iqq;
  }
  if (iqq==0) break;
//  break;
}

stepHist0->Delete();
if (feeNew>0.2*(chi2new-feeNew) && feeNew<0.3*(chi2new-feeNew)) break;
double ratio=(chi2new-feeNew)/feeNew;
wFee*=.25*ratio; 
printf(" chi2=%f\t fee=%f\t new wFee=%f\n", chi2new-feeNew, feeNew, wFee);
if (iqq==1) {
  printf("push->");
  cin >> iqq;
}
if (iqq==0) break;
} // iTer loop

stepHist ->Delete();
newHist  ->Delete();
errHist  ->Delete();
pedHist  ->Delete();

zeroRate=signalHist->GetBinContent(zeroBin);
signalRate=signalHist->GetEntries()-zeroRate;

return result;
}
///====================================== short version
bool SubtractZeroes(TH1D *emptyHist, TH1D *dataHist, TH1D *signalHist, double &zeroRate, double &signalRate) {
bool result=true;
int iqq=2;
int nBins=emptyHist->GetNbinsX();
double wFee=1.;
if (dataHist->GetNbinsX() != nBins) {
  printf("[SubtractZeroes] wrong nBins=%d for data histogram %s\n",
          dataHist->GetNbinsX(), dataHist->GetName());
  return false;
}
if (signalHist->GetNbinsX() != nBins) {
  printf("[SubtractZeroes] wrong nBins=%d for output histogram %s\n",
          signalHist->GetNbinsX(), signalHist->GetName());
  return false;
}
TH1D *stepHist = (TH1D*) dataHist->Clone("stepHist");
TH1D *newHist  = (TH1D*) dataHist->Clone("newHist");
TH1D *errHist  = (TH1D*) dataHist->Clone("errHist");
TH1D *pedHist  = (TH1D*) emptyHist->Clone("pedHist");

double wEmpty=emptyHist->GetEntries();
pedHist->Scale(1.0/wEmpty);

double sigmaNoise=emptyHist->GetRMS();
double data0=0.;
double nois0=0.;
for (int i=0; i<nBins+1; i++) {
  if (emptyHist->GetBinCenter(i)>0.5*sigmaNoise) break;
  data0+=dataHist->GetBinContent(i);
  nois0+=emptyHist->GetBinContent(i);
}
double scale0=data0/nois0;
zeroRate=scale0*emptyHist->GetEntries();
signalRate=dataHist->GetEntries()-zeroRate;
int zeroBin=0;

//=============================
for (int iTer=0; iTer<30; iTer++) {
//=============================
stepHist->Reset();
newHist->Reset();
errHist->Reset();
signalHist->Reset();
newHist->SetLineColor(kRed);
newHist->SetLineWidth(1);
stepHist->SetLineColor(kBlue);
stepHist->SetLineWidth(1);


for (int i=0; i<nBins+1; i++) {
  double w=dataHist->GetBinContent(i)-scale0*emptyHist->GetBinContent(i);
  if (emptyHist->GetBinCenter(i)<0.) w=0.;
  else if (w<1.0) w=1.0;
  signalHist->SetBinContent(i, w);
  w=sqrt(1.+dataHist->GetBinContent(i));
  errHist->SetBinContent(i, w);
}

zeroBin=0;
for (int i=0; i<nBins+1; i++) {
  if (emptyHist->GetBinCenter(i)<=0.) zeroBin=i;
}

signalHist->SetBinContent(zeroBin, zeroRate);
for (int i=0; i<nBins+1; i++) {
  if (emptyHist->GetBinCenter(i)<0.) {
    stepHist->SetBinContent(i, 0.0);
  } else {
    double w=0.1*signalHist->GetBinContent(i);
    if (w<5.) w=5.;
    stepHist->SetBinContent(i, w);
  }
}

FoldHists(pedHist, signalHist, newHist);

TH1D *stepHist0 = (TH1D*) stepHist->Clone("stepHist0");

double chi2old=0.;
double chi2new=0.;
double chi2=0.;
int    iLoop=0;
double nPosData=0.;
double nPosSign=0.;
double stepOld=0.;
double stepNew=0.;

for (int i=1; i<nBins+1; i++) {
  if (dataHist->GetBinContent(i)>0.0)   nPosData+=1.0;
  if (signalHist->GetBinContent(i)>0.0) nPosSign+=1.0;
  double s=stepHist->GetBinContent(i);
  stepOld+=s*s;
  //double err=errHist->GetBinContent(i);
}
chi2=ComputeChi2(newHist, dataHist);

//double nDF=nPosData-nPosSign;
double fee=ComputeFee(signalHist, wFee, zeroBin);
chi2old=chi2+fee;
stepOld=sqrt(stepOld);
//double fee0=fee;
double feeNew=fee;

//printf("loop#%04d chi2old=%.2f\t chi2new=%.2f\t fee=%f\n", iLoop, chi2, chi2, feeNew);
stepNew=1.0;

while (iLoop<1000 && stepNew>0.01) {
  iLoop++;
  for (int is=1; is<nBins+1; is++) {
    double step=stepHist->GetBinContent(is);
    if (step==0.) continue;
    double wOld=signalHist->GetBinContent(is);
    double wNew=wOld+step;
    if (wNew<0.) {
      stepHist->SetBinContent(is, 0.66*step);
      continue;
    }
    signalHist->SetBinContent(is, wNew);
    newHist->Reset();
    FoldHists(pedHist, signalHist, newHist);
    chi2=ComputeChi2(newHist, dataHist);
    fee=ComputeFee(signalHist, wFee, zeroBin);
    chi2new=chi2+fee;

    //chi2new*=fee/fee0;
    //chi2new/=nDF;

    if (chi2new<chi2old) {
      stepHist->SetBinContent(is, 1.33*step);
      chi2old=chi2new;
      feeNew=fee;
    } else {
      stepHist->SetBinContent(is, -0.66*step);
      signalHist->SetBinContent(is, wOld); // return old spectrum value
    }

  } // sample loop //

  stepNew=0.0;
  for (int i=1; i<nBins+1; i++) {
    double s1=stepHist ->GetBinContent(i);
    double s0=stepHist0->GetBinContent(i);
    if (s1==0.0 || s0==0.0) continue;
    if (stepNew<fabs(s1/s0)) stepNew=fabs(s1/s0);
  }   
//  break;
}

stepHist0->Delete();
if (feeNew>0.2*(chi2new-feeNew) && feeNew<0.3*(chi2new-feeNew)) break;
double ratio=(chi2new-feeNew)/feeNew;
wFee*=.25*ratio; 
printf(" chi2=%f\t fee=%f\t new wFee=%f\n", chi2new-feeNew, feeNew, wFee);
if (iqq==1) {
  printf("push->");
  cin >> iqq;
}
if (iqq==0) break;
} // iTer loop

stepHist ->Delete();
newHist  ->Delete();
errHist  ->Delete();
pedHist  ->Delete();

zeroRate=signalHist->GetBinContent(zeroBin);
signalRate=signalHist->GetEntries()-zeroRate;
return result;
}





///=========================================== full version
double ComputeFee(TH1D *h0, double wFee, int zeroBin, TH1D* h1) {
  double fee=0.;
  double a1, a2, a3, y0, y22;
  int nBins=h0->GetNbinsX();   
  h1->Reset();
  for (int i=zeroBin+2; i<nBins; i++) {
    a1=h0->GetBinContent(i-1);
    a2=h0->GetBinContent(i);
    a3=h0->GetBinContent(i+1);
    y0=.25*(a1+2.0*a2+a3);
    y22=a1+a3-2.0*a2;
    double dfee=wFee*y22*y22/sqrt(1.0+fabs(y0));
    fee+=dfee;
    h1->SetBinContent(i, dfee);
  }
  fee=sqrt(fee);
  return fee;
}

double ComputeFee(TH1D *h0, double wFee, int zeroBin) { // no-plots version
  double fee=0.;
  double a1, a2, a3, y0, y22;
  int nBins=h0->GetNbinsX();   
  for (int i=zeroBin+2; i<nBins; i++) {
    a1=h0->GetBinContent(i-1);
    a2=h0->GetBinContent(i);
    a3=h0->GetBinContent(i+1);
    y0=.25*(a1+2.0*a2+a3);
    y22=a1+a3-2.0*a2;
    fee+=wFee*y22*y22/sqrt(1.0+fabs(y0));
   }
  fee=sqrt(fee);
  return fee;
}
//====================================================================
double ComputeChi2(TH1D* h0, TH1D* h1, TH1D* h2) {
double chi2=0.;
int nBin=h0->GetNbinsX();
h2->Reset();
for (int i=1; i<nBin+1; i++) {
  double err=h0->GetBinError(i)*h0->GetBinError(i)+
             h1->GetBinError(i)*h1->GetBinError(i);
  if (err==0.) continue;
  double diff=h0->GetBinContent(i)-h1->GetBinContent(i);
  h2->SetBinContent(i, 5.0*fabs(diff));
  chi2+=diff*diff/err;
}
return chi2;
}

double ComputeChi2(TH1D* h0, TH1D* h1) { // no-plots version
double chi2=0.;
int nBin=h0->GetNbinsX();
for (int i=1; i<nBin+1; i++) {
  double err=h0->GetBinError(i)*h0->GetBinError(i)+
             h1->GetBinError(i)*h1->GetBinError(i);
  if (err==0.) continue;
  double diff=h0->GetBinContent(i)-h1->GetBinContent(i);
  chi2+=diff*diff/err;
}
return chi2;
}



void FoldHists(TH1D *h0, TH1D *h1, TH1D *h2) {
int n0=h0->GetNbinsX()+1;
int n1=h1->GetNbinsX()+1;
h2->Reset();
for (int i0=1; i0<n0; i0++) { 
  double x0=h0->GetBinCenter(i0);
  double w0=h0->GetBinContent(i0);
  if (w0==0.) continue;
  for (int i1=1; i1<n1; i1++) {  
    double x1=h1->GetBinCenter(i1);
    double w1=h1->GetBinContent(i1);
    if (w1==0.) continue;
    h2->Fill(x0+x1, w0*w1);
  } 
}
}

void SmoothTail(TH1D *h0, double pFrac) {
int    n0=h0->GetNbinsX()+1;
double w0=pFrac*h0->GetMaximum();
TH1D* h1 = (TH1D*) h0->Clone("smoothTail");
h1->Reset();
double wPrev=0.;
double xPeak=0.;
double wPeak=0.;
for (int i=1; i<n0; i++) {
  double w1=h0->GetBinContent(i);
  double x1=h0->GetBinCenter(i);
  if (w1>wPeak) {
    wPeak=w1;
    xPeak=x1;
  }
}
//bool smoothHere=false;
//printf("[SmoothTail] %s xPeak=%.2f\t wPeak=%f\t w0=%f\n", h0->GetName(), xPeak, wPeak, w0);
for (int i=1; i<n0; i++) {
  double w1=h0->GetBinContent(i);
  double x1=h0->GetBinCenter(i);

  if (x1<xPeak) {
    h1->Fill(x1, w1);
    continue;
  }

  double ws=exp(-w1/w0)/3.; 
//  ws=(1.0-w1/w0)/3.0;
//  if (ws<0.) ws=0.;

  if (i>1 && i<n0 ) {
//    h1->Fill(h0->GetBinCenter(i-1), w1/4.-wPrev/4.);    
//    h1->Fill(x1, w1/2.+wPrev/4.);
//    h1->Fill(h0->GetBinCenter(i+1), w1/4.);    
    h1->Fill(h0->GetBinCenter(i-1), w1*ws);    
    h1->Fill(x1, w1*(1.0-2.0*ws));
    h1->Fill(h0->GetBinCenter(i+1), w1*ws);    
  } else {
    h1->Fill(x1, w1);
  } 
  wPrev=w1;
}
h0->Reset();
for (int i=1; i<n0; i++) {
  h0->SetBinContent(i, h1->GetBinContent(i));
}
h1->Delete(); 
//int iqq=0;
//printf("push->");
//cin >> iqq;

}


//====================================================================================================
void SmoothTail1(TH1D *h0) {
//int iqq=1;
int    n0=h0->GetNbinsX();
//double x0=h0->GetXaxis()->GetXmin();
//double x1=h0->GetXaxis()->GetXmax();
TH1D* h1 = (TH1D*) h0->Clone("smoothTail1");
h1->Reset();

double w=0.;
double wSum=0.;

for (int i=n0; i>0; i--) {
  if (h0->GetBinContent(i)>0.0) {
    int ir=i;
    int il=i;
    for (int j=i+1; j<n0+1; j++) {
      if (h1->GetBinContent(j)>0.0) break;
      ir=j;
    }
    for (int j=i-1; j>0; j--) {
      if (h0->GetBinContent(j)>0.0) break;
      il=(j+i)/2+1;
    }
    if (h0->GetBinCenter(il)<0) il=i;

    if (i==il && i==ir) {
      w=h0->GetBinContent(i);
      h1->Fill(h0->GetBinCenter(i), w);
    } else {
      w=h0->GetBinContent(i)/(ir-il+1.);
      wSum=0.;
      for (int j=il; j<ir+1; j++) {
        h1->Fill(h0->GetBinCenter(j), w);
        wSum+=w;
      }
    }
  }
}
h0->Reset();
for (int i=0; i<n0+1; i++) {
  h0->Fill(h0->GetBinCenter(i), h1->GetBinContent(i));
}

h1->Delete(); 
}



//====================================================================================================
void SmoothTail1full(TH1D *h0) {
int iqq=1;
int    n0=h0->GetNbinsX();
double x0=h0->GetXaxis()->GetXmin();
double x1=h0->GetXaxis()->GetXmax();
TH1D* h1 = (TH1D*) h0->Clone("smoothTail1");
h1->SetLineColor(kRed);
h1->Reset();

TCanvas *showCanvas = new TCanvas("SmoothTailShow", "SmoothTailShow", 3000, 0, 800, 800);
double w=0.;

for (int i=n0; i>0; i--) {
  if (h0->GetBinContent(i)>0.0) {
    int ir=i;
    int il=i;
    for (int j=i+1; j<n0+1; j++) {
      if (h1->GetBinContent(j)>0.0) break;
      ir=j;
    }
    for (int j=i-1; j>0; j--) {
      if (h0->GetBinContent(j)>0.0) break;
      il=(j+i)/2+1;
    }
    if (i==il && i==ir) {
      w=h0->GetBinContent(i);
      h1->Fill(h0->GetBinCenter(i), w);
    } else {
      w=h0->GetBinContent(i)/(ir-il+1.);
      double wSum=0.;
      for (int j=il; j<ir+1; j++) {
        h1->Fill(h0->GetBinCenter(j), w);
        wSum+=w;
      }
      printf("[mySpectraTricks->SmoothTail1] %s, bin %d\t w=%lf\t wNew=%lf\t wSum=%lf\t iil=%d\t ir=%d\t Range=(%.2f, %.2f)\n",
           h0->GetTitle(), i, h0->GetBinContent(i), w, wSum, il, ir, h0->GetBinCenter(il), h0->GetBinCenter(ir));
    }
      showCanvas->Clear();
      showCanvas->Divide(1,2);
      showCanvas->cd(1);
      gPad->SetLogy(1);
      gPad->SetGridy(1);
      gPad->SetGridx(1);
      h0->GetXaxis()->SetRangeUser(x0, x1);
      h0->DrawCopy();

      showCanvas->cd(2);
      gPad->SetLogy(0);
      gPad->SetGridy(1);
      gPad->SetGridx(1);
      int ir1=ir+5;
      int il1=il-3;
      if (ir1>n0) ir1=n0;
      if (il1<1)  il1=1;
      h0->GetXaxis()->SetRangeUser(h0->GetBinCenter(il1), h0->GetBinCenter(ir1));
      h0->DrawCopy();
      h1->DrawCopy("same");

      showCanvas->cd(1);
      h1->DrawCopy("same");
      showCanvas->Update();
      
      printf("push 1->");
      cin>>iqq;
      if (iqq==0) break;
      if (iqq==-1) exit(-1);
  }
}

showCanvas->Close();
h1->Delete(); 
}
