struct CherenkovCalibration2015 {

private:
int    HVcurve;
double GADCmipMAX;
double HVmax;
double corrMax;
double prepMIP;
double currentMIP;
//==============================================================================
public:

void SetMax(int curve, double GADCmip, double HV) {
  GADCmipMAX=GADCmip;
  HVmax=HV;
  HVcurve=curve;
  if (HVcurve==0) {
    corrMax=Curve2008(HVmax);
  } else if (HVcurve==1) {
    corrMax=Curve2009(HVmax);
  } else if (HVcurve==2) {
    corrMax=Curve2013(HVmax);
  } else {
    printf("[CherenkovCalibration2015->SetMax] unknow HV curve#%d, Bye!\n", HVcurve);
    exit(-1);
  }
  prepMIP=GADCmipMAX/corrMax;
}

void SetMax(int curve, int myRun) {

  if (myRun<860) {
    printf("[CherenkovCalibration2015->SetMax] Sorry no service for runs prior to year 2012. Bye.\n");
    exit(-1);
  } else if (myRun<1070) {
    GADCmipMAX=10.09;
    HVmax=2.550;
  } else if (myRun<1195) {
    GADCmipMAX=41.64;
    HVmax=2.601;
  } else {
    printf("[CherenkovCalibration2015->SetMax] Sorry no service for runs taken after year 2013. Bye.\n");
    exit(-1);
  }
  HVcurve=curve;
  if (HVcurve==0) {
    corrMax=Curve2008(HVmax);
  } else if (HVcurve==1) {
    corrMax=Curve2009(HVmax);
  } else if (HVcurve==2) {
    corrMax=Curve2013(HVmax);
  } else {
    printf("[CherenkovCalibration2015->SetMax] unknow HV curve#%d, Bye!\n", HVcurve);
    exit(-1);
  }
  prepMIP=GADCmipMAX/corrMax;
  printf("[CherenkovCalibration2015->SetMax] HV curve#%d set for run#%d, 1p=%.3f GADC/MIP at %6.3fkV\n",
          HVcurve, myRun, GADCmipMAX, HVmax);
  return;
}

double Curve2008(double HV) {
  double result=10.966589+(-0.102387-0.066478*HV)*HV;
  return exp(result*log(HV));
}
double Curve2009(double HV) {// A.Savine, LED
  double result=9.926711+(0.351145-0.131120*HV)*HV; // is 1 at 1kV. absolute calibration needed!!!!
  result=exp(result*log(HV));
  return result;
}
double Curve2013(double HV) {// A.Kozelov, LED
  double result=10.09+(0.016-0.065*HV)*HV; // is 1 at 1kV. absolute calibration needed!!!!
  result=exp(result*log(HV));
  return result;
}

double ComputeMIP(double HV) {
  currentMIP=0.;
  if (HV<=0.) {
    printf("[CherenkovCalibration2015->ComputeMIP]  Bad HV=%f\n", HV);
    exit(-1);    
  }
  if (HVcurve==0) {
    currentMIP=prepMIP*Curve2008(HV);
  } else if (HVcurve==1) {
    currentMIP=prepMIP*Curve2009(HV);
  } else if (HVcurve==2) {
    currentMIP=prepMIP*Curve2013(HV);
  } else {
    printf("[CherenkovCalibration2015->ComputeMIP] %d not implemented\n", HVcurve);
    exit(-1);
  }
  printf("[CherenkovCalibration2015->ComputeMIP]  HV=%f 1proton=%fGADC\n", HV, currentMIP);

  return currentMIP;  
}

double GetCurrentMIP() {
  return currentMIP;
}
  
};



