struct CherenkovCalibration2015a {

private:
int    HVcurve;
double GADCmipMAX;
double MPImipMAX;
double HVmax;
double corrMax;
double GADCprepMIP;
double MPIprepMIP;
double currentMIP;
//==============================================================================
public:

void SetMax(int curve, double GADCmip, double MPImip, double HV) {
  GADCmipMAX=GADCmip;
  MPImipMAX =MPImip;
  HVmax=HV;
  HVcurve=curve;
  if (HVcurve==0) {
    corrMax=Correction2008(HVmax);
  } else if (HVcurve==1) {
    corrMax=Correction2009(HVmax);
  } else if (HVcurve==2) {
    corrMax=Correction2013(HVmax);
  } else {
    printf("[CherenkovCalibration2015->SetMax] unknow HV curve#%d, Bye!\n", HVcurve);
    exit(-1);
  }
  GADCprepMIP=GADCmipMAX/corrMax;
  MPIprepMIP =MPImipMAX /corrMax;
}

void SetMax(int curve, int myRun) {

  if (myRun<860) {
    printf("[CherenkovCalibration2015->SetMax] Sorry no service for runs prior to year 2012. Bye.\n");
    exit(-1);
  } else if (myRun<1070) {
    GADCmipMAX=41.64;
    MPImipMAX=168.85;
    HVmax=2.601;
  } else if (myRun<1095) {
    GADCmipMAX=10.09;
    MPImipMAX =43.04;
    HVmax=2.550;
  } else {
    printf("[CherenkovCalibration2015->SetMax] Sorry no service for runs taken after year 2013. Bye.\n");
    exit(-1);
  }
  HVcurve=curve;
  if (HVcurve==0) {
    corrMax=Correction2008(HVmax);
  } else if (HVcurve==1) {
    corrMax=Correction2009(HVmax);
  } else if (HVcurve==2) {
    corrMax=Correction2013(HVmax);
  } else {
    printf("[CherenkovCalibration2015->SetMax] unknow HV curve#%d, Bye!\n", HVcurve);
    exit(-1);
  }
  GADCprepMIP=GADCmipMAX/corrMax;
  MPIprepMIP =MPImipMAX /corrMax;
}

double Correction2008(double HV) {
  double result=10.966589+(-0.102387-0.066478*HV)*HV;
  return exp(result*log(HV));
}
double Correction2009(double HV) {// A.Savine, LED
  double result=9.926711+(0.351145-0.131120*HV)*HV; // is 1 at 1kV. absolute calibration needed!!!!
  result=exp(result*log(HV));
  return result;
}
double Correction2013(double HV) {// A.Kozelov, LED
  double result=10.09+(0.016-0.065*HV)*HV; // is 1 at 1kV. absolute calibration needed!!!!
  result=exp(result*log(HV));
  return result;
}

void ComputeMIP(double HV) {
  currentMIP=0.;
  if (HV<=0.) {
    printf("[CherenkovCalibration2015->ComputeMIP]  Bad HV=%f\n", HV);
    exit(-1);    
  }
  if (HVcurve==0) {
    GADCcurrentMIP=GADCprepMIP*Correction2008(HV);
    MPIcurrentMIP =MPIprepMIP*Correction2008(HV);
  } else if (HVcurve==1) {
    GADCcurrentMIP=GADCprepMIP*Correction2009(HV);
    MPIcurrentMIP =MPIprepMIP*Correction2009(HV);
  } else if (HVcurve==2) {
    GADCcurrentMIP=GADCprepMIP*Correction2013(HV);
    MPIcurrentMIP =MPIprepMIP*Correction2013(HV);
  } else {
    printf("[CherenkovCalibration2015->ComputeMIP] HVcurve#%d not implemented\n", HVcurve);
    exit(-1);
  }
}

double GetGADCMIP() {
  return GADCcurrentMIP;
}
double GetMPIMIP() {
  return MPIcurrentMIP;
}
  
};



