// structure, holding MPI signal, FFT signal, time, samples, ped windows data for PMT counter (S1, S2, S3 or Ch)
struct myCounter {
  char Name[200];
  char myText[200];
  int id;
  int nPedWindows;
  int firstSample[25];
  int lastSample[25];
  double t0PMTrec;
  double t1PMTrec;
  double t0window;
  double t1window;
  double time2013start;
  double time2013step;
  int    ns2013;
  int    HVgroup;
  double mipCut1;
  double mipCut0;
  double HVset;
  float hilow;
  float lg2013[3000];
  float hg2013[3000];
  float ampLG[512];
  float ampHG[512];
  double t0;
  double t1;
  double TphaseRF;

  double peakAlignment;      // position of peak and mean relative to RF cycle,
  double meanAlignment;      // loaded run-by-run
  double peakAlignmentSigma; // spread of peak position and mean relative to RF cycle,
  double meanAlignmentSigma; // loaded run-by-run

  double pulseHalfWidh;       // average pulse Half-Width; 
  double pulseHalfWidhSigma;  // pulse Half-Width variance
  double pulseWidhSigma;      // average pulse Sigma; 
  double pulseWidhSigmaSigma; // pulse Sigma variance;


  int    npnt;
  int    iBinI[300];  // Integral bin for iRF
  double binsI[300]; // integral (argument)
  double mean1[300]; // mean vs integral
  double mean2[300]; // mean variance vs integral;

  double peak1[300]; // peak vs integral
  double peak2[300]; // peak variance vs integral;

  double sigma1[300]; // sigma vs integral
  double sigma2[300]; // sigma variance vs integral;

  double half1[300]; // half-width vs integral
  double half2[300]; // half-width variance vs integral;
  
  double cutFactor; // how many sigma's to cut

//
  bool   noiseSet;
  double pedHG0, noiseHG0; // blank events in the beginning of spill
  double pedLG0, noiseLG0;
  double pedHG1, noiseHG1; // empty bunches in the spill
  double pedLG1, noiseLG1;
// peak search variables
  int   iGainSearch;
  int   nPeaks;
  bool  useThis[3000];
  int   peakIndex[100];
  int   peakRFindex[100]; // has RF cycle# for the peak
  int   peakGain[100];
  int   peakIndex0[100];
  int   peakIndex1[100];
  double peakT00[100];
  double peakT2U[100];
  double peakT2D[100];
  double peakTRMS[100];
  double peakTmn[100];
  double peakAmp[100];
  double peakMean[100];

  double peakIntegralHG[40];
  double peakIntegralLG[40];
  double peakIntegralXX[40];
  double X[40];
  int    nX[40];

  int   peakIndexHG[40]; // peak search by RF cycle !
  int   peakIndexLG[40];
  int   peakFoundRF[40]; // RF cycle, where peak was found
  bool  filledRF[40];
  double peakAmpHG[40];
  double peakAmpLG[40];

  double delayRFold;
  double twIN;
  double twOUT;

  double maxLG, maxHG;
  double cutLG, cutHG;
  double cutLG1, cutHG1;
//------------------------------------------------------------------------------
  void Clear() {
    cutFactor=3.0;
    twIN=70.;
    twOUT=35.;
    maxHG=0.;
    maxLG=0.;
    cutHG=pedHG0+3.*noiseHG0;
    cutLG=pedLG0+3.*noiseLG0;
    cutHG1=pedHG0+20.*noiseHG0;
    cutLG1=pedLG0+20.*noiseLG0;
    nPeaks=0;
    iGainSearch=0;
    t0window=firstSample[0]*12.5+50.;
    t1window=lastSample[nPedWindows-1]*12.5-50.;
    for (int i=0; i<40; i++) {
      iBinI[i]=-1;
      peakIntegralHG[i]=0.;
      peakIntegralLG[i]=0.;
      peakIntegralXX[i]=0.;
      peakAmpHG[i]=-1000.;
      peakAmpLG[i]=-1000.;
      peakRFindex[i]=-1;
      peakIndexHG[i]=-1;
      peakIndexLG[i]=-1;
      filledRF[i]=false;
      peakGain[i]=-1;
      X[i]=0.;
      nX[i]=0;
    }

    for (int i=0; i<100; i++) {
      peakAmp[i]=0.;
      peakMean[i]=0.;
      peakT00[i]=0.;
      peakTmn[i]=0.;
      peakT2U[i]=-1000.;
      peakT2D[i]=-1000.;
      peakTRMS[i]=0.;
      peakIndex[i]=-1;
      peakIndex0[i]=-1;
      peakIndex1[i]=-1;
      peakGain[i]=-1;
    }
    for (int i=0; i<3000; i++) {
      double t=time2013start+i*time2013step;
      useThis[i]=(i<ns2013   &
                  t>t0PMTrec && 
                  t<t1PMTrec && 
                  t>t0window &&
                  t<t1window &&
                  hg2013[i]>cutHG);
    }
  }



//======================================================================
  void PrintPulseInfo(int iRF) {
    printf("%s iRF=%d =====================\n", Name, iRF);
    printf("       gain#%d integrals: %7.1f %7.1f (%5.1f, %5.1f)",  
           peakGain[iRF], peakIntegralLG[iRF], peakIntegralHG[iRF], 
           mipCut0, mipCut1);
    checkDynamicPeak(iRF);
    checkDynamicThalf(iRF);
    checkDynamicMean(iRF);
    checkDynamicSigma(iRF); 
    printf("  Peak-> %5.1f %5.1fns (%5.2f, %5.2f)",
           peakT00[iRF]-TphaseRF-iRF*165., peakT2D[iRF]-peakT2U[iRF],
           checkDynamicPeak(iRF), checkDynamicThalf(iRF));
    printf("  Mean-> %5.1f %5.1fns (%5.2f, %5.2f)",
           peakTmn[iRF]-TphaseRF-iRF*165., peakTRMS[iRF],
           checkDynamicMean(iRF), checkDynamicSigma(iRF)); 
    printf(" X=(%2d, %5.1f)\n", nX[iRF], X[iRF]);
  }

  void SetPhaseRF(double t) {
    TphaseRF=t;
  }

  void FindPeaks(int iRep) {
    if (!noiseSet) {
      printf("\n\n=============================================================\n");
      printf("[myCounter->FindPeaks] noise not set for %s, bye!\n", Name);
      printf("=============================================================\n\n");
      exit(-1);
    }
    nPeaks=0;
    bool found=true;
    while (found && nPeaks<98) {
      found=false;
      double aMax=0.;
      int    iMax=-1;
      if (iGainSearch==0) {
        for (int i=0; i<ns2013; i++) {
          if (!useThis[i]) continue;
          if (lg2013[i]<cutLG) continue;
          if (aMax<lg2013[i]) {
            aMax=lg2013[i];
            iMax=i;
          }
        }

        if (aMax<150.) { // continue using HG
          if (iRep==1) printf("[%s->FindPeaks()]      maxLG=%.1f switching to HG\n", Name, aMax);
          iGainSearch=1;
          aMax=0.;
          iMax=-1;
        } else if (iMax != -1) { // add as a peak!
          if (iRep==1) {
            double t0=time2013start+iMax*time2013step;
            printf("[%s->FindPeaks()] found LG peak#%d maxLG=%.1f t=%.1f\n", Name, nPeaks, aMax*hilow, t0);
          }
          found=true;
          peakGain[nPeaks]=iGainSearch;
          peakIndex[nPeaks]=iMax;
          peakAmp[nPeaks]=aMax*hilow;
        }
      }
 
      if (iGainSearch==1) {
        for (int i=0; i<ns2013; i++) {
          if (!useThis[i]) continue;
          if (hg2013[i]<cutHG) continue;
          if (aMax<hg2013[i]) {
            aMax=hg2013[i];
            iMax=i;
          }
        }

        if (iMax != -1) { // add as a peak!
          if (iRep==1) {
            double t0=time2013start+iMax*time2013step;
            printf("[%s->FindPeaks()] found HG peak#%d maxHG=%.1f t=%.1f\n", Name, nPeaks, aMax, t0);
          }
          found=true;
          peakGain[nPeaks]=iGainSearch;
          peakIndex[nPeaks]=iMax;
          peakAmp[nPeaks]=aMax;
        }
      }

      if (found) {
        int i0=peakIndex[nPeaks]-15;
        int i1=peakIndex[nPeaks]+30;
        if (i0<0) i0=0;
        if (i1>ns2013) i1=ns2013;
        for (int i=i0; i<i1; i++) useThis[i]=false;
        nPeaks++;
        if (nPeaks>100) {
          printf("\n\n[%s->FindPeaks()] found peak#%d. Too many, ciao!\n", Name, nPeaks);
          exit(-1);
        }
      } 
    }

    if (iRep==1) {
      printf("[%s->FindPeaks()] found %d peaks\n", Name, nPeaks);
    }

// order chronologicaly //
    bool moved=true;
    int nLaps=0;
    while (moved) {
      moved=false;
      for (int i=0; i<nPeaks-1; i++) {
        if (peakIndex[i]>peakIndex[i+1]) {// swap!
          moved=true;
          int pgain  =peakGain[i];
          int pindx  =peakIndex[i];
          double pamp=peakAmp[i];
          peakGain[i] =peakGain[i+1];
          peakIndex[i]=peakIndex[i+1];
          peakAmp[i]  =peakAmp[i+1];
          peakGain[i+1] =pgain;
          peakIndex[i+1]=pindx;
          peakAmp[i+1]  =pamp;
        }
      }
 
      nLaps++;
    }
    if (iRep==1) {
      printf("[%s->FindPeaks()] ordered, %d laps\n", Name, nLaps);
    }

    for (int ip=0; ip<nPeaks; ip++) {
      peakT00[ip]=time2013start+peakIndex[ip]*time2013step;
      double st0=0.;
      double st1=0.;
      double st2=0.;
      double st0w=0.;
      double st1w=0.;
      double st2w=0.;
      int n0=0;
      int n1=0;
      int nw=0;
      int i0=peakIndex[ip]-15;
      int i1=peakIndex[ip]+15;
      if (i0<0) i0=0;
      if (i1>ns2013-1) i1=ns2013-1;

      double t0=time2013start+peakIndex[ip]*time2013step;
      peakT2D[ip]=t0;
      peakT2U[ip]=t0;

      if (peakGain[ip]==0) { // low gain 

        for (int i=i0; i<i1; i++) {
          double t=time2013start+i*time2013step;
          if (lg2013[i]>0.) {
            st0w+=lg2013[i];
            st1w+=lg2013[i]*t;
            st2w+=lg2013[i]*t*t;
            nw++;
          }
        }

        for (int i=peakIndex[ip]; i>-1; i--) {
          double t=time2013start+i*time2013step;
          if (t<t0window) {
            peakT2U[ip]=0.5*(t+peakT2U[ip]);
            break;
          }
          if (lg2013[i]*hilow>0.5*peakAmp[ip]) {
            peakT2U[ip]=t;
          } 
          if (lg2013[i]>0.) {
            st0+=lg2013[i];
            st1+=lg2013[i]*t;
            st2+=lg2013[i]*t*t;
          }
          n0++;
          if (lg2013[i]<cutLG) {
            peakT2U[ip]=0.5*(t+peakT2U[ip]);
            break;
          }
        }

        for (int i=peakIndex[ip]+1; i<ns2013; i++) {
          double t=time2013start+i*time2013step;
//          if (iRep==1) {
//            printf("[%s->FindPeaks()] search-right LG lg2013[%5d]*hilow=%7.1f HalfPeak=%7.1f cutLG=%5.1f t=%7.1f t1window=%7.1f \n",
//                   Name, i, lg2013[i]*hilow, .5*peakAmp[ip], cutLG*hilow, t, t1window);
//          }
          
          if (t>t1window) {
            peakT2D[ip]=0.5*(t+peakT2D[ip]);
            break;
          }
          if (lg2013[i]*hilow>0.5*peakAmp[ip]) {
            peakT2D[ip]=t;
          } 
          if (lg2013[i]>0.) {
            st0+=lg2013[i];
            st1+=lg2013[i]*t;
            st2+=lg2013[i]*t*t;
          }
          n1++;
          if (lg2013[i]<cutLG) {
            peakT2D[ip]=0.5*(t+peakT2D[ip]);
            break;
          }
        }
//----------------------------------------------------------------------------
      } else {             // high gain 
//----------------------------------------------------------------------------
        for (int i=i0; i<i1; i++) {
          double t=time2013start+i*time2013step;
          if (hg2013[i]>0.) {
            st0w+=hg2013[i];
            st1w+=hg2013[i]*t;
            st2w+=hg2013[i]*t*t;
            nw++;
          }
        }

        for (int i=peakIndex[ip]; i>-1; i--) {
          double t=time2013start+i*time2013step;
          if (t<t0window) {
            peakT2U[ip]=0.5*(t+peakT2U[ip]);
            break;
          }
//          if (hg2013[i]<0.5*peakAmp[ip] && peakT2U[ip]<0.) {
//            peakT2U[ip]=t;
//          } 
          if (hg2013[i]>0.5*peakAmp[ip]) {
            peakT2U[ip]=t;
          } 
          if (hg2013[i]>0.) {
            st0+=hg2013[i];
            st1+=hg2013[i]*t;
            st2+=hg2013[i]*t*t;
          }
          n0++;
          if (hg2013[i]<cutHG) {
            peakT2U[ip]=0.5*(t+peakT2U[ip]);
            break;
          }
        }

        for (int i=peakIndex[ip]+1; i<ns2013; i++) {
          double t=time2013start+i*time2013step;
//          if (iRep==1) {
//            printf("[%s->FindPeaks()] search-right HG hg2013[%5d]=%7.1f HalfPeak=%7.1f cutHG=%5.1f t=%7.1f t1window=%7.1f \n", 
//                    Name, i, hg2013[i], .5*peakAmp[ip], cutHG, t, t1window);
//          }
          if (t>t1window) {
            peakT2D[ip]=0.5*(t+peakT2D[ip]);
            break;
          }
          if (hg2013[i]>0.5*peakAmp[ip]) {
            peakT2D[ip]=t;
          } 
          if (hg2013[i]>0.) {
            st0+=hg2013[i];
            st1+=hg2013[i]*t;
            st2+=hg2013[i]*t*t;
          }
          n1++;
          if (hg2013[i]<cutHG) {
            peakT2D[ip]=0.5*(t+peakT2D[ip]);
            break;
          }
        }

      }

      st1/=st0;
      st2/=st0;
      st2-=st1*st1;
      st2=sqrt(st2);
      
//      peakTRMS[ip]=st2;
//      peakTmn[ip]=st1;
//      peakMean[ip]=st0/(n0+n1);

      st1w/=st0w;
      st2w/=st0w;
      st2w-=st1w*st1w;
      st2w=sqrt(st2w);
      
      peakTRMS[ip]=st2w;
      peakTmn[ip]=st1w;
      peakMean[ip]=st0w/nw;

      peakRFindex[ip]=int((time2013start+peakIndex[ip]*time2013step-TphaseRF)/165.);

      if (iRep==1) {
        printf("[%s->FindPeaks()] peak%2d RF#%2d gain#%1d A=%5.1f <A>=%5.1f Tup=%7.1f Tpk=%7.1f Tmn=%7.1f Tdw=%7.1f tRMS=%.1f n=(%d, %d, %d)\n", 
             Name, ip, peakRFindex[ip], peakGain[ip], peakAmp[ip], peakMean[ip], peakT2U[ip], peakT00[ip], 
             peakTmn[ip], peakT2D[ip], peakTRMS[ip], n0, n1, nw);
      }

    }
   
    return;
  }

void FindPeaksByCycle(int iRep) {
  if (!noiseSet) {
    printf("\n\n=============================================================\n");
    printf("[myCounter->FindPeaksByCycle] noise not set for %s, bye!\n", Name);
    printf("=============================================================\n\n");
    exit(-1);
  }

  int iRFold=-1;
  int iRFnew=0;
  double maxLG=0.;
  double maxHG=0.;
  int    iMaxH=-1;
  int    iMaxL=-1;
  for (int i=0; i<ns2013; i++) {
    double t=time2013start+i*time2013step;
    iRFnew=int((t-TphaseRF)/165.);
    if (iRFnew != iRFold) {
//    tansfer found peak info to data storage, if it iRFold != -1
      if (iRFold != -1) {
        peakIndexHG[iRFold]=iMaxH;
        peakIndexLG[iRFold]=iMaxL;
        peakAmpHG[iRFold]=maxHG;
        peakAmpLG[iRFold]=maxLG;
        if (maxHG>cutHG) {
          peakRFindex[nPeaks]=iRFold;
          if (iRep==1) {
            printf("[%s->FindPeaksByCycle] found peak#%d RF#%d indx=(%4d, %4d) max=(%8.1f, %8.1f) \n", 
                   Name, nPeaks, iRFold, iMaxL, iMaxH, maxLG, maxHG);
          }
//  transfer data to global peak search format
          if(maxLG<150.) { // use HG for this RF cycle
            peakGain[nPeaks]=1;
            peakIndex[nPeaks]=iMaxH;
            peakAmp[nPeaks]=maxHG;
          } else {         // use LG for this RF cycle
            peakGain[nPeaks]=0;
            peakIndex[nPeaks]=iMaxL;
            peakAmp[nPeaks]=maxLG*hilow;
          }
          
          nPeaks++;
        }
      }
      maxLG=0.;
      maxHG=0.;
      iMaxH=-1;
      iMaxL=-1; 
      iRFold=iRFnew;     
    }
    if (!useThis[i]) continue;
    
    if (maxLG<lg2013[i]) {
      maxLG=lg2013[i];
      iMaxL=i;
    }
    if (maxHG<hg2013[i]) {
      maxHG=hg2013[i];
      iMaxH=i;
    }
  }

  if (iRep==1) {
    printf("[%s->FindPeaks()] found %d peaks\n", Name, nPeaks);
  }

  for (int ip=0; ip<nPeaks; ip++) {
    peakT00[ip]=time2013start+peakIndex[ip]*time2013step;
    double st0=0.;
    double st1=0.;
    double st2=0.;
    double st0w=0.;
    double st1w=0.;
    double st2w=0.;
    int n0=0;
    int n1=0;
    int nw=0;
    int i0=peakIndex[ip]-15;
    int i1=peakIndex[ip]+15;
    if (i0<0) i0=0;
    if (i1>ns2013-1) i1=ns2013-1;

    if (peakGain[ip]==0) { // low gain 

      for (int i=i0; i<i1; i++) {
        double t=time2013start+i*time2013step;
        if (lg2013[i]>0.) {
          st0w+=lg2013[i];
          st1w+=lg2013[i]*t;
          st2w+=lg2013[i]*t*t;
          nw++;
        }
      }

      for (int i=peakIndex[ip]; i>-1; i--) {
        double t=time2013start+i*time2013step;
        if (t<t0window) {
          peakT2U[ip]=t;
          break;
        }
        if (lg2013[i]>0.5*peakAmp[ip]) {
          peakT2D[ip]=t;
        } 
        if (lg2013[i]>0.) {
          st0+=lg2013[i];
          st1+=lg2013[i]*t;
          st2+=lg2013[i]*t*t;
        }
        n0++;
        if (lg2013[i]<cutLG) break;
      }

      for (int i=peakIndex[ip]+1; i<ns2013; i++) {
        double t=time2013start+i*time2013step;
        if (t>t1window) {
          peakT2D[ip]=t;
          break;
        }
        if (lg2013[i]>0.5*peakAmp[ip]) {
          peakT2D[ip]=t;
        } 
        if (lg2013[i]>0.) {
          st0+=lg2013[i];
          st1+=lg2013[i]*t;
          st2+=lg2013[i]*t*t;
        }
        n1++;
        if (lg2013[i]<cutLG) break;
      }
//=============================================
    } else {             // same for high gain 
//=============================================
      for (int i=i0; i<i1; i++) {
        double t=time2013start+i*time2013step;
        if (hg2013[i]>0.) {
          st0w+=hg2013[i];
          st1w+=hg2013[i]*t;
          st2w+=hg2013[i]*t*t;
          nw++;
        }
      }

      for (int i=peakIndex[ip]; i>-1; i--) {
        double t=time2013start+i*time2013step;
        if (t<t0window) {
          peakT2U[ip]=t;
          break;
        }
        if (hg2013[i]>0.5*peakAmp[ip]) {
          peakT2U[ip]=t;
        } 
        if (hg2013[i]>0.) {
          st0+=hg2013[i];
          st1+=hg2013[i]*t;
          st2+=hg2013[i]*t*t;
        }
        n0++;
        if (hg2013[i]<cutHG) break;
      }

      for (int i=peakIndex[ip]+1; i<ns2013; i++) {
        double t=time2013start+i*time2013step;
        if (t>t1window) {
          peakT2D[ip]=t;
          break;
        }
        if (hg2013[i]>0.5*peakAmp[ip]) {
          peakT2D[ip]=t;
        } 
        if (hg2013[i]>0.) {
          st0+=hg2013[i];
          st1+=hg2013[i]*t;
          st2+=hg2013[i]*t*t;
        }
        n1++;
        if (hg2013[i]<cutHG) break;
      }

    }

    st1/=st0;
    st2/=st0;
    st2-=st1*st1;
    st2=sqrt(st2);
      
    st1w/=st0w;
    st2w/=st0w;
    st2w-=st1w*st1w;
    st2w=sqrt(st2w);
      
    peakTRMS[ip]=st2w;
    peakTmn[ip]=st1w;
    peakMean[ip]=st0w/nw;

    if (iRep==1) {
      printf("[%s->FindPeaksByCycle] peak%2d RF#%2d gain#%1d A=%5.1f <A>=%5.1f Tup=%7.1f Tpk=%7.1f Tmn=%7.1f Tdw=%7.1f tRMS=%.1f n=(%d, %d, %d)\n", 
          Name, ip, peakRFindex[ip], peakGain[ip], peakAmp[ip], peakMean[ip], peakT2U[ip], peakT00[ip], 
          peakTmn[ip], peakT2D[ip], peakTRMS[ip], n0, n1, nw);
    }

  }
   
  
}

  void DisplayMPI(TCanvas *myCanvas, TH1F *myDummy, TLine *myLine, int trapIt) {
    myCanvas->cd(id+1);
    double maxlow = 0.5;
    double maxhigh= 5.0;
    double minlow =-0.5;
    double minhigh=-5.0;

    double tPlot0=-200.;
    double tPlot1=512*12.5;

    if (trapIt != -1 && trapIt!=1000) {
       tPlot0=TphaseRF+(trapIt-2)*165;
      if (tPlot0<-200.) tPlot0=-200.;
      tPlot1=tPlot0+5.*165;
      if (tPlot1>512*12.5) {
        tPlot1=512*12.5;
        tPlot0=tPlot1-7.*165;
      }
    }

    for (int i=0; i<512; i++) {
      if (i*12.5<tPlot0) continue;
      if (i*12.5>tPlot1) break;
      if (maxlow<ampLG[i]) maxlow=ampLG[i];
      if (minlow>ampLG[i]) minlow=ampLG[i];
      if (maxhigh<ampHG[i]) maxhigh=ampHG[i];
      if (minhigh>ampHG[i]) minhigh=ampHG[i];
    }
    maxlow*=hilow;
    minlow*=hilow;
    if (maxhigh<maxlow) maxhigh=maxlow;
    if (minhigh>minlow) minhigh=minlow;
    myDummy->GetXaxis()->SetRangeUser(tPlot0, tPlot1);
    myDummy->GetYaxis()->SetRangeUser(1.05*minhigh, 1.05*maxhigh);
    myDummy->GetYaxis()->SetTitle("MPI ADC, mV");
    sprintf(myText, "%s high/low=%.2f", Name, hilow);
    myDummy->SetTitle(myText);
    myDummy->SetStats(0);
    myDummy->DrawCopy();

    myLine->SetLineWidth(13);
    myLine->SetLineStyle(1);
    myLine->SetLineColor(kYellow);
    for (int i=0; i<nPedWindows; i++) {
      double tt0=12.5*firstSample[i];
      double tt1=12.5*lastSample[i];
      if (tt1<tPlot0) continue;
      if (tt0>tPlot1) break;
      if (tt0<tPlot0) tt0=tPlot0;
      if (tt1>tPlot1) tt1=tPlot1;
      myLine->DrawLine(tt0, 0., tt1, 0.);
    }

    myLine->SetLineWidth(2);
    myLine->SetLineStyle(2);
    myLine->SetLineColor(kGray);
    for (double t=TphaseRF; t<6400.; t+=165.) {
      if (t<tPlot0) continue;
      if (t>tPlot1) break;
      myLine->DrawLine(t, 1.05*minhigh, t, 1.05*maxhigh);
    }

    myLine->SetLineStyle(1);
    myLine->SetLineColor(kBlack);
    for (int i=1; i<512; i++) {
      if (i*12.5<tPlot0) continue;
      if (i*12.5>tPlot1) break;
      myLine->DrawLine(12.5*(i-1),       ampHG[i-1], 12.5*i,       ampHG[i]);
    }
    myLine->SetLineColor(kBlue);
    for (int i=1; i<512; i++) {
      if (i*12.5<tPlot0) continue;
      if (i*12.5>tPlot1) break;
      myLine->DrawLine(12.5*(i-1), hilow*ampLG[i-1], 12.5*i, hilow*ampLG[i]);
    }
    myCanvas->Update();
  }

  bool GetRFbunchSample(int is, int& iRF, float& ampHG, float& ampLG, float& tsamp) {
// returns true, sets "amp" and "iRF", for given FFT sample
    bool result=true;
    if (is<0 || is >= ns2013-1) {
      return false;
    }
    tsamp=time2013start+time2013step*(is-1);
    if (tsamp<12.5*(firstSample[0]+0.5) || tsamp>12.5*(lastSample[nPedWindows-1]-0.5)) {
      return false;
    }
    iRF=int((tsamp-TphaseRF)/165.);
    ampHG=hg2013[is];
    ampLG=lg2013[is];
    return result;
  }
/*
  void DisplayFFT(TCanvas *myCanvas, TH1F *myDummy, TLine *myLine, int trapIt) {
    TLatex* myLatex;
    myLatex = new TLatex;
    myCanvas->cd(id+1);
    double maxlow = 0.5;
    double maxhigh= 5.0;
    double minlow =-0.5;
    double minhigh=-5.0;

    double tPlot0=-200.;
    double tPlot1=512*12.5;

    if (trapIt != -1 && trapIt!=1000) {
      tPlot0=TphaseRF+(trapIt-3)*165;
      if (tPlot0<-200.) tPlot0=-200.;
      tPlot1=tPlot0+7.*165;
      if (tPlot1>512*125.) {
        tPlot1=512*125.;
        tPlot0=tPlot1-7.*165;
      }
    }

    for (int i=0; i<ns2013; i++) {
      t0=time2013start+time2013step*(i-1);
      if (t0<tPlot0) continue;
      if (t0>tPlot1) break;
      if (maxlow<lg2013[i])  maxlow=lg2013[i];
      if (minlow>lg2013[i])  minlow=lg2013[i];
      if (maxhigh<hg2013[i]) maxhigh=hg2013[i];
      if (minhigh>hg2013[i]) minhigh=hg2013[i];
    }
    maxlow*=hilow;
    minlow*=hilow;
    if (maxhigh<maxlow) maxhigh=maxlow;
    if (minhigh>minlow) minhigh=minlow;
    myDummy->GetXaxis()->SetRangeUser(tPlot0, tPlot1);
    myDummy->GetYaxis()->SetRangeUser(1.05*minhigh, 1.05*maxhigh);
    myDummy->GetYaxis()->SetTitle("FFT Reconstruction, a.u.");
    myDummy->SetTitle(Name);
    myDummy->SetStats(0);
    myDummy->DrawCopy();

    if (nPeaks>0) {
      myLine->SetLineWidth(2);
      myLine->SetLineStyle(1);
      myLine->SetLineColor(kRed-9);
      for (int i=0; i<nPeaks; i++) {
        double tp=time2013start+peakIndex[i]*time2013step;
        myLine->DrawLine(tp, 1.05*minhigh, tp, 1.05*maxhigh);
      }
    }


//    myLine->SetLineWidth(13);
//    myLine->SetLineStyle(1);
//    myLine->SetLineColor(kYellow);
//    for (int i=0; i<nPedWindows; i++) myLine->DrawLine(12.5*firstSample[i], 0., 12.5*lastSample[i], 0.);

    double tCut0=12.5*firstSample[0];
    double tCut1=12.5*lastSample[nPedWindows-1];

    myLine->SetLineWidth(2);
    myLine->SetLineStyle(2);
    myLine->SetLineColor(kGray);
    for (int i=0; i<40; i++) {
      double t=TphaseRF+i*165.;
      if (t>6400.) break;
      myLine->DrawLine(t, 1.05*minhigh, t, 1.05*maxhigh);
      sprintf(myText, "%d", i);
      myLatex->DrawLatex(t, 1.15*maxhigh, myText);
    }

    myLine->SetLineWidth(7);
    myLine->SetLineStyle(1);
    myLine->SetLineColor(kGray);
    for (int i=0; i<40; i++) {
      double t=TphaseRF+i*165.;
      if (t>6400.) break;
      if (filledRF[i]) {
        myLine->SetLineColor(kRed);
        myLine->DrawLine(t+delayRFold-0.5*twIN-twOUT, 0., t+delayRFold-0.5*twIN, 0.);
        myLine->DrawLine(t+delayRFold+0.5*twIN+twOUT, 0., t+delayRFold+0.5*twIN, 0.);
        myLine->SetLineColor(kGreen);
        myLine->DrawLine(t+delayRFold-0.5*twIN, 0., t+delayRFold+0.5*twIN, 0.);
      }      
    }

    myLine->SetLineStyle(1);
    printf("[myCounter->DisplayFFT]  %s  time2013start=%.3f,\t time2013step=%.3f\n", 
            Name, time2013start, time2013step);
    printf("[myCounter->DisplayFFT]    t0PMTrec=%.3f,\t t1PMTrec=%.3f\n", t0PMTrec, t1PMTrec);
    double t0, t1;
    for (int i=1; i<ns2013; i++) {
      t0=time2013start+time2013step*(i-1);
      t1=t0+time2013step;
      if (t0<tPlot0) continue;
      if (t0>tPlot1) break;
      if (t0<t0PMTrec ||  t0<tCut0) continue;
      if (t1>t1PMTrec ||  t1>tCut1) break;
      myLine->SetLineColor(kBlack);
      myLine->SetLineWidth(3);
      myLine->DrawLine(t0,     hg2013[i-1], t1,     hg2013[i]);
      myLine->SetLineColor(kBlue);
      myLine->SetLineWidth(1);
      myLine->DrawLine(t0, hilow*lg2013[i-1], t1, hilow*lg2013[i]);
    }
    myCanvas->Update();
    myCanvas->Show();
    myLatex->Delete();
  }
*/
//======================================================================
  void DisplayFFT(TCanvas *myCanvas, TH1F *myDummy, TLine *myLine, int trapIt) {
    TLatex* myLatex;
    myLatex = new TLatex;
    myCanvas->cd(id+1);
    double maxlow = 0.5;
    double maxhigh= 5.0;
    double minlow =-0.5;
    double minhigh=-5.0;

    double tPlot0=-200.;
    double tPlot1=512*12.5;

    if (trapIt != -1 && trapIt!=1000) {
      tPlot0=TphaseRF+(trapIt-2)*165;
      if (tPlot0<-200.) tPlot0=-200.;
      tPlot1=tPlot0+5.*165;
      if (tPlot1>512*12.5) {
        tPlot1=512*12.5;
        tPlot0=tPlot1-7.*165;
      }
    }

    for (int i=0; i<ns2013; i++) {
      t0=time2013start+time2013step*(i-1);
      if (t0<tPlot0) continue;
      if (t0>tPlot1) break;
      if (maxlow<lg2013[i])  maxlow=lg2013[i];
      if (minlow>lg2013[i])  minlow=lg2013[i];
      if (maxhigh<hg2013[i]) maxhigh=hg2013[i];
      if (minhigh>hg2013[i]) minhigh=hg2013[i];
    }
    maxlow*=hilow;
    minlow*=hilow;
    if (maxhigh<maxlow) maxhigh=maxlow;
    if (minhigh>minlow) minhigh=minlow;
    myDummy->GetXaxis()->SetRangeUser(tPlot0, tPlot1);
    myDummy->GetYaxis()->SetRangeUser(1.05*minhigh, 1.05*maxhigh);
    myDummy->GetYaxis()->SetTitle("FFT Reconstruction, a.u.");
    myDummy->SetTitle(Name);
    myDummy->SetStats(0);
    myDummy->DrawCopy();

    if (nPeaks>0) {
      myLine->SetLineWidth(2);
      myLine->SetLineStyle(1);
      myLine->SetLineColor(kRed-9);
      for (int i=0; i<nPeaks; i++) {
        double tp=time2013start+peakIndex[i]*time2013step;
        myLine->DrawLine(tp, 1.05*minhigh, tp, 1.05*maxhigh);
      }
    }

    myLine->SetLineWidth(13);
    myLine->SetLineStyle(1);
    myLine->SetLineColor(kYellow);
//    for (int i=0; i<nPedWindows; i++) myLine->DrawLine(12.5*firstSample[i], 0., 12.5*lastSample[i], 0.);

    double tCut0=12.5*firstSample[0];
    double tCut1=12.5*lastSample[nPedWindows-1];

    myLine->SetLineWidth(2);
    myLine->SetLineStyle(2);
    myLine->SetLineColor(kGray);
    for (int i=0; i<40; i++) {
      double t=TphaseRF+i*165.;
      if (t<tPlot0+80.) continue;
      if (t<tCut0+80.)  continue;
      if (t>tCut1-80.)  break;
      if (t>tPlot1-80.) break;
      if (t>6400.) break;
      myLine->DrawLine(t, 1.05*minhigh, t, 1.05*maxhigh);
      if (trapIt == -1 || trapIt==1000) {      
        sprintf(myText, "%d", i);
      } else {
        sprintf(myText, "%d nX=%d X=%.1f", i, nX[i], X[i]);
      }
      myLatex->DrawLatex(t, 1.15*maxhigh, myText);
    }

    myLine->SetLineWidth(7);
    myLine->SetLineStyle(1);
    myLine->SetLineColor(kGray);

    for (int i=0; i<40; i++) {
      double t=TphaseRF+i*165.;
      if (t>6400) break;
      printf("%d", filledRF[i]);
    }
    printf("\n");

    if (trapIt != -1 && trapIt!=1000) {
// show pulse info //
      for (int i=0; i<40; i++) {
        double t=TphaseRF+i*165.;
        if (t<tPlot0+80.) continue;
        if (t<tCut0+80.)  continue;
        if (t>tCut1-80.)  break;
        if (t>tPlot1-80.) break;
        if (filledRF[i]) {
          double tx=t+delayRFold;
          tx=peakT00[i];
          myLine->SetLineWidth(7);
          myLine->SetLineColor(kRed);
          myLine->DrawLine(tx-0.5*twIN-twOUT, 0., tx-0.5*twIN, 0.);
          myLine->DrawLine(tx+0.5*twIN+twOUT, 0., tx+0.5*twIN, 0.);
          myLine->SetLineColor(kGreen);
          myLine->DrawLine(tx-0.5*twIN, 0., tx+0.5*twIN, 0.);

          myLine->SetLineWidth(1);
          myLine->DrawLine(peakT2U[i], 0., peakT2U[i], 1.05*maxhigh); 
          myLine->DrawLine(peakT2D[i], 0., peakT2D[i], 1.05*maxhigh);
          sprintf(myText, "#splitline{T_{HALF}=%.1fns}{#splitline{#sigma=%.1fns}{I=%.1f}}",
                  peakT2D[i]-peakT2U[i], peakTRMS[i], peakIntegralXX[i]);
          double yy=0.85*peakAmp[i];
          if (yy<mipCut0) yy=mipCut0; 
          myLatex->DrawLatex(peakT2D[i]+7.5, yy, myText);
        }      
      }
    }


    myLine->SetLineStyle(1);
    printf("[myCounter->DisplayFFT]  %s  time2013start=%.3f,\t time2013step=%.3f\n", 
            Name, time2013start, time2013step);
    printf("[myCounter->DisplayFFT]    t0PMTrec=%.3f,\t t1PMTrec=%.3f\n", t0PMTrec, t1PMTrec);
    double t0, t1;
    for (int i=1; i<ns2013; i++) {
      t0=time2013start+time2013step*(i-1);
      t1=t0+time2013step;
      int jRF=int((0.5*(t0+t1)-TphaseRF)/165.);
      if (t0<tPlot0) continue;
      if (t0>tPlot1) break;
      if (t0<t0PMTrec ||  t0<tCut0) continue;
      if (t1>t1PMTrec ||  t1>tCut1) break;
      myLine->SetLineColor(kBlack);
      if (peakGain[jRF]!=0) myLine->SetLineWidth(3);
      else                  myLine->SetLineWidth(1);
      myLine->DrawLine(t0,     hg2013[i-1], t1,     hg2013[i]);
      myLine->SetLineColor(kBlue);
      if (peakGain[jRF]==0) myLine->SetLineWidth(3);
      else                  myLine->SetLineWidth(1);
      myLine->DrawLine(t0, hilow*lg2013[i-1], t1, hilow*lg2013[i]);
    }
    myCanvas->Update();
    myCanvas->Show();
    myLatex->Delete();
  }

//========================================================================================================= 
// integral arount time T, adding delay relative to RF cycle;
//
  bool getDiffIntegral(double Trf, float &integralL, float &integralH) {
    bool result = true;
    double T0i=Trf+delayRFold;
    integralL=0.; integralH=0.;
    if (T0i<12.5*firstSample[0]+100.)            return false;
    if (T0i>12.5*lastSample[nPedWindows-1]-100.) return false;

    float S1H=0.;
    float S2H=0.;
    float S3H=0.;
    float S1L=0.;
    float S2L=0.;
    float S3L=0.;
    result = result && simpleIntegral(T0i-0.5*twIN-twOUT, T0i-0.5*twIN,       S1L, S1H); 
    result = result && simpleIntegral(T0i-0.5*twIN,       T0i+0.5*twIN,       S2L, S2H); 
    result = result && simpleIntegral(T0i+0.5*twIN,       T0i+0.5*twIN+twOUT, S3L, S3H);

    integralL=S2L-S1L-S3L; 
    integralH=S2H-S1H-S3H; 

    return result;
  }

//========================================================================================================= 
// integral arount time T
//
   bool getDiffIntegralBasic(double T, float &integralL, float &integralH) {
    bool result = true;
    double T0i=T;
    integralL=0.; integralH=0.;
    if (T0i<12.5*firstSample[0]+100.)            return false;
    if (T0i>12.5*lastSample[nPedWindows-1]-100.) return false;

    float S1H=0.;
    float S2H=0.;
    float S3H=0.;
    float S1L=0.;
    float S2L=0.;
    float S3L=0.;
    result = result && simpleIntegral(T0i-0.5*twIN-twOUT, T0i-0.5*twIN,       S1L, S1H); 
    result = result && simpleIntegral(T0i-0.5*twIN,       T0i+0.5*twIN,       S2L, S2H); 
    result = result && simpleIntegral(T0i+0.5*twIN,       T0i+0.5*twIN+twOUT, S3L, S3H);

    integralL=S2L-S1L-S3L; 
    integralH=S2H-S1H-S3H; 

    return result;
  }

//========================================================================================================= 
// integral arount time T, adding delay relative to RF cycle;
// also computes parameters of a pulse and stores them in peakT00, peakT2U etc arrays
//
  bool getDiffIntegral(int iRF, double Trf, float &integralL, float &integralH) {
    bool result = true;
    double st0n=0.;
    double st1n=0.;
    double st2n=0.;
    double st0w=0.;
    double st1w=0.;
    double st2w=0.;
    int nn=0;
    int nw=0;
    double T0i=Trf+delayRFold;

    integralL=0.; integralH=0.;
    if (T0i<12.5*firstSample[0]+100.)            return false;
    if (T0i>12.5*lastSample[nPedWindows-1]-100.) return false;

    float S1H=0.;
    float S2H=0.;
    float S3H=0.;
    float S1L=0.;
    float S2L=0.;
    float S3L=0.;
    result = result && simpleIntegral(T0i-0.5*twIN-twOUT, T0i-0.5*twIN,       S1L, S1H); 
    result = result && simpleIntegral(T0i-0.5*twIN,       T0i+0.5*twIN,       S2L, S2H); 
    result = result && simpleIntegral(T0i+0.5*twIN,       T0i+0.5*twIN+twOUT, S3L, S3H);

    integralL=S2L-S1L-S3L; 
    integralH=S2H-S1H-S3H; 

    peakIntegralHG[iRF]=integralH;
    peakIntegralLG[iRF]=integralL;

    int i0w=int(0.5+(T0i-0.5*twIN-0.5*twOUT-time2013start)/time2013step);    
    int i0n=int(0.5+(T0i-0.5*twIN-time2013start)/time2013step);    
    int i1w=int(0.5+(T0i+0.5*twIN+0.5*twOUT-time2013start)/time2013step);    
    int i1n=int(0.5+(T0i+0.5*twIN-time2013start)/time2013step);   
   
    if (i0w<0) i0w=0;
    if (i0n<0) i0n=0;
    if (i1w>ns2013-1) i1w=ns2013-1;
    if (i1n>ns2013-1) i1n=ns2013-1;

    peakIndex[iRF]=-1;
    peakIndexHG[iRF]=-1;
    peakIndexLG[iRF]=-1;
    peakGain[iRF]=-1;

    for (int i=i0w; i<=i1w; i++) { // find peaks to select gain 
      if (peakAmpLG[iRF]<lg2013[i]) {
        peakAmpLG[iRF]=lg2013[i];
        peakIndexLG[iRF]=i;
      }        
      if (peakAmpHG[iRF]<hg2013[i]) {
        peakAmpHG[iRF]=hg2013[i];
        peakIndexHG[iRF]=i;
      }        
    }
    
    if (integralL*hilow >2000. || peakAmpLG[iRF]*hilow>2000.) {
      peakGain[iRF]=0;
      peakAmp[iRF]=peakAmpLG[iRF];
      peakIndex[iRF]=peakIndexLG[iRF];
    } else {
      peakGain[iRF]=1;
      peakAmp[iRF]=peakAmpHG[iRF];
      peakIndex[iRF]=peakIndexHG[iRF];
    } 

//    i0w=peakIndex[iRF]-15;
//    i1w=peakIndex[iRF]+15;
//    if (i0w<0) i0w=0.;
//    if (i1w>ns2013-1) i1w=ns2013-1;

    if (peakGain[iRF]==0) { // use LG
      for (int i=i0w; i<=i1w; i++) { // accumulate sums for <T> and Sigma(T);
        double t=time2013start+i*time2013step;
        if (lg2013[i]>0.) {
          st0w+=lg2013[i];
          st1w+=lg2013[i]*t;
          st2w+=lg2013[i]*t*t;
          nw++;
          if (i>=i0n && i<=i1n) {
            st0n+=lg2013[i];
            st1n+=lg2013[i]*t;
            st2n+=lg2013[i]*t*t;
            nn++;
          }
        }

      }


      for (int i=peakIndex[iRF]; i>0; i--) {
        double t=time2013start+i*time2013step;
        if (t<t0window) break;
        if (lg2013[i]<0.5*peakAmp[iRF]) break;
        peakT2U[iRF]=t;
        if (lg2013[i]<cutLG) {
          peakT2U[iRF]=0.5*(t+peakT2U[iRF]);
          break;
        }
      }
      for (int i=peakIndex[iRF]; i<ns2013; i++) {
        double t=time2013start+i*time2013step;
        if (t>t1window) break;
        if (lg2013[i]<0.5*peakAmp[iRF]) break;
        peakT2D[iRF]=t;
        if (lg2013[i]<cutLG) {
          peakT2D[iRF]=0.5*(t+peakT2D[iRF]);
          break;
        }
      }
      peakAmp[iRF]*=hilow;
//..................................................
    } else { // use HG
      for (int i=i0w; i<=i1w; i++) { // accumulate sums for <T> and Sigma(T);
        double t=time2013start+i*time2013step;
        if (hg2013[i]>0.) {
          st0w+=hg2013[i];
          st1w+=hg2013[i]*t;
          st2w+=hg2013[i]*t*t;
          nw++;
          if (i>=i0n && i<=i1n) {
            st0n+=hg2013[i];
            st1n+=hg2013[i]*t;
            st2n+=hg2013[i]*t*t;
            nn++;
          }
        }
      }

 
      for (int i=peakIndex[iRF]; i>0; i--) {
        double t=time2013start+i*time2013step;
        if (t<t0window) break;
        if (hg2013[i]<0.5*peakAmp[iRF]) break;
        peakT2U[iRF]=t;
        if (hg2013[i]<cutHG) {
          peakT2U[iRF]=0.5*(t+peakT2U[iRF]);
          break;
        }
      }
      for (int i=peakIndex[iRF]; i<ns2013; i++) {
        double t=time2013start+i*time2013step;
        if (t>t1window) break;
        if (hg2013[i]<0.5*peakAmp[iRF]) break;
        peakT2D[iRF]=t;
        if (hg2013[i]<cutHG) {
          peakT2D[iRF]=0.5*(t+peakT2D[iRF]);
          break;
        }
      }
    }

    st1w/=st0w;
    st2w/=st0w;
    st2w-=st1w*st1w;
    if (st2w<0.) st2w=0.;
    else st2w=sqrt(st2w);

    st1n/=st0n;
    st2n/=st0n;
    st2n-=st1n*st1n;
    if (st2n<0.) st2n=0.;
    else st2n=sqrt(st2n);

    peakTmn[iRF]=st1w;
    peakTRMS[iRF]=st2w;
    
    peakT00[iRF]=time2013start+peakIndex[iRF]*time2013step;
    return result;
  }


//========================================================================================================= 
// integral around peak, found around time Trf, adding delay relative to RF cycle;
// also computes parameters of a pulse and stores them in peakT00, peakT2U etc arrays
//
  bool getDiffIntegralPeak(int iRF, double Trf, float &integralL, float &integralH) {
    bool result = true;
    double st0n=0.;
    double st1n=0.;
    double st2n=0.;
    double st0w=0.;
    double st1w=0.;
    double st2w=0.;
    int nn=0;
    int nw=0;
    double T0i=Trf+delayRFold;
    double Tinit=T0i;

//    printf("[getDiffIntegralPeak] initial time=%.1f, twIN=%.1f twOUT=%.1f\n", Tinit, twIN, twOUT);

    if (twIN==0. || twOUT==0.) {
      printf("[getDiffIntegralPeak] error, %s iRF=%d initial time=%.1f, twIN=%.1f twOUT=%.1f\n", 
              Name, iRF, Tinit, twIN, twOUT);
      exit(-1);
    }

    integralL=0.; integralH=0.;

    if (T0i<12.5*firstSample[0]+100.) {
      printf("[getDiffIntegralPeak] failed initial T0i=%.1f low cut=%.1f\n",
             T0i, 12.5*firstSample[0]+100.);
      return false;
    }
    if (T0i>12.5*lastSample[nPedWindows-1]-100.) {
      printf("[getDiffIntegralPeak] failed initial T0i=%.1f high cut=%.1f\n", 
             T0i, 12.5*lastSample[nPedWindows-1]-100.);
      return false;
    }

    int i0w=int(0.5+(T0i-0.5*twIN-0.5*twOUT-time2013start)/time2013step);    
    int i1w=int(0.5+(T0i+0.5*twIN+0.5*twOUT-time2013start)/time2013step);    
//    int i0w=int(0.5+(T0i-0.5*twIN-twOUT-time2013start)/time2013step);    
//    int i1w=int(0.5+(T0i+0.5*twIN+twOUT-time2013start)/time2013step); 

    int i0n=int(0.5+(T0i-0.5*twIN-time2013start)/time2013step);    
    int i1n=int(0.5+(T0i+0.5*twIN-time2013start)/time2013step);   
   

    if (i0w<0) i0w=0;
    if (i0n<0) i0n=0;
    if (i1w>ns2013-1) i1w=ns2013-1;
    if (i1n>ns2013-1) i1n=ns2013-1;

    peakIndex[iRF]=-1;
    peakIndexHG[iRF]=-1;
    peakIndexLG[iRF]=-1;
    peakGain[iRF]=-1;

    peakAmpLG[iRF]=-1.0e10;
    peakAmpHG[iRF]=-1.0e10;

    for (int i=i0w; i<=i1w; i++) { // find peaks to select gain 
      if (peakAmpLG[iRF]<lg2013[i]) {
        peakAmpLG[iRF]=lg2013[i];
        peakIndexLG[iRF]=i;
      }        
      if (peakAmpHG[iRF]<hg2013[i]) {
        peakAmpHG[iRF]=hg2013[i];
        peakIndexHG[iRF]=i;
      }        
    }
    
    if (peakAmpLG[iRF]*hilow>2000. || peakAmpHG[iRF]>2000.) {
      peakGain[iRF]=0;
      peakAmp[iRF]=peakAmpLG[iRF];
      peakIndex[iRF]=peakIndexLG[iRF];
    } else {
      peakGain[iRF]=1;
      peakAmp[iRF]=peakAmpHG[iRF];
      peakIndex[iRF]=peakIndexHG[iRF];
    } 
    peakT00[iRF]=time2013start+peakIndex[iRF]*time2013step;

//------------------------------------------------

    if (peakT00[iRF]<12.5*firstSample[0]+100.) {
      printf("[getDiffIntegralPeak] failed peak %s RF#%d T0i=%.1f low cut=%.1f\n",
             Name, iRF, peakT00[iRF], 12.5*firstSample[0]+100.);
      printf("                      initial time=%.1f, twIN=%.1f twOUT=%.1f\n", Tinit, twIN, twOUT);
      printf("                      peak index=%d gain=%d maxLG=%6.1f maxHG=%6.1f iHG=%5d iLG=%5d\n", 
              peakIndex[iRF], peakGain[iRF], peakAmpLG[iRF], peakAmpHG[iRF], peakIndexLG[iRF], peakIndexHG[iRF]);
      printf("                      i0w=%4d i0n=%4d i1n=%4d i1w=%4d\n", i0w, i0n, i1n, i1w);
      for (int i=i0w; i<=i1w; i++) {
        printf("               #%4d lg=%6.1f hg=%6.1f\n", i, lg2013[i], hg2013[i]); 
      }
      return false;
    }
    if (peakT00[iRF]>12.5*lastSample[nPedWindows-1]-100.) {
      printf("[getDiffIntegralPeak] failed peak T0i=%.1f high cut=%.1f\n", 
             peakT00[iRF], 12.5*lastSample[nPedWindows-1]-100.);
      printf("[getDiffIntegralPeak] initial time=%.1f, twIN=%.1f twOUT=%.1f\n", Tinit, twIN, twOUT);
      printf("[getDiffIntegralPeak] i0w=%4d i0n=%4d i1n=%4d i1w=%4d\n", i0w, i0n, i1n, i1w);
      return false;
    }

    T0i=peakT00[iRF]; /// redefine signal center

    i0w=int(0.5+(T0i-0.5*twIN-0.5*twOUT-time2013start)/time2013step);    
    i1w=int(0.5+(T0i+0.5*twIN+0.5*twOUT-time2013start)/time2013step);    
//    i0w=int(0.5+(T0i-0.5*twIN-twOUT-time2013start)/time2013step);    
//    i1w=int(0.5+(T0i+0.5*twIN+twOUT-time2013start)/time2013step);    

    i0n=int(0.5+(T0i-0.5*twIN-time2013start)/time2013step);    
    i1n=int(0.5+(T0i+0.5*twIN-time2013start)/time2013step);   
//------------------------------------------------

    float S1H=0.;
    float S2H=0.;
    float S3H=0.;
    float S1L=0.;
    float S2L=0.;
    float S3L=0.;
    result = result && simpleIntegral(T0i-0.5*twIN-twOUT, T0i-0.5*twIN,       S1L, S1H); 
    result = result && simpleIntegral(T0i-0.5*twIN,       T0i+0.5*twIN,       S2L, S2H); 
    result = result && simpleIntegral(T0i+0.5*twIN,       T0i+0.5*twIN+twOUT, S3L, S3H);

    if (!result) {
      printf("[getDiffIntegralPeak] integral failure for %s RF#%d\n\n", Name, iRF);
      return false;
    }

    integralL=S2L-S1L-S3L; 
    integralH=S2H-S1H-S3H; 

    peakIntegralHG[iRF]=integralH;
    peakIntegralLG[iRF]=integralL;
    
    if (peakGain[iRF]==0) { // use LG
      peakIntegralXX[iRF]=integralL*hilow;
      for (int i=i0w; i<=i1w; i++) { // accumulate sums for <T> and Sigma(T);
        double t=time2013start+i*time2013step;
        if (lg2013[i]>0.) {
          st0w+=lg2013[i];
          st1w+=lg2013[i]*t;
          st2w+=lg2013[i]*t*t;
          nw++;
          if (i>=i0n && i<=i1n) {
            st0n+=lg2013[i];
            st1n+=lg2013[i]*t;
            st2n+=lg2013[i]*t*t;
            nn++;
          }
        }

      }


      for (int i=peakIndex[iRF]; i>0; i--) {
        double t=time2013start+i*time2013step;
        if (t<t0window) break;
        if (lg2013[i]<0.5*peakAmp[iRF]) break;
        peakT2U[iRF]=t;
        if (lg2013[i]<cutLG) {
          peakT2U[iRF]=0.5*(t+peakT2U[iRF]);
          break;
        }
      }
      for (int i=peakIndex[iRF]; i<ns2013; i++) {
        double t=time2013start+i*time2013step;
        if (t>t1window) break;
        if (lg2013[i]<0.5*peakAmp[iRF]) break;
        peakT2D[iRF]=t;
        if (lg2013[i]<cutLG) {
          peakT2D[iRF]=0.5*(t+peakT2D[iRF]);
          break;
        }
      }
      peakAmp[iRF]*=hilow;
//..................................................
    } else { // use HG
      peakIntegralXX[iRF]=integralH;
      for (int i=i0w; i<=i1w; i++) { // accumulate sums for <T> and Sigma(T);
        double t=time2013start+i*time2013step;
        if (hg2013[i]>0.) {
          st0w+=hg2013[i];
          st1w+=hg2013[i]*t;
          st2w+=hg2013[i]*t*t;
          nw++;
          if (i>=i0n && i<=i1n) {
            st0n+=hg2013[i];
            st1n+=hg2013[i]*t;
            st2n+=hg2013[i]*t*t;
            nn++;
          }
        }
      }

 
      for (int i=peakIndex[iRF]; i>0; i--) {
        double t=time2013start+i*time2013step;
        if (t<t0window) break;
        if (hg2013[i]<0.5*peakAmp[iRF]) break;
        peakT2U[iRF]=t;
        if (hg2013[i]<cutHG) {
          peakT2U[iRF]=0.5*(t+peakT2U[iRF]);
          break;
        }
      }
      for (int i=peakIndex[iRF]; i<ns2013; i++) {
        double t=time2013start+i*time2013step;
        if (t>t1window) break;
        if (hg2013[i]<0.5*peakAmp[iRF]) break;
        peakT2D[iRF]=t;
        if (hg2013[i]<cutHG) {
          peakT2D[iRF]=0.5*(t+peakT2D[iRF]);
          break;
        }
      }
    }

    st1w/=st0w;
    st2w/=st0w;
    st2w-=st1w*st1w;
    if (st2w<0.) st2w=0.;
    else st2w=sqrt(st2w);

    st1n/=st0n;
    st2n/=st0n;
    st2n-=st1n*st1n;
    if (st2n<0.) st2n=0.;
    else st2n=sqrt(st2n);

    peakTmn[iRF]=st1w;
    peakTRMS[iRF]=st2w;
/*
    iBinI[iRF]=-1;    
    for (int ib=2; ib<npnt; ib++) { // find bin;
      if (peakIntegralXX[iRF]<binsI[ib] && mipCut1<binsI[ib]) {
        iBinI[iRF]=ib;
        break;
      }
    }
    if (iBinI[iRF]==-1) {
      iBinI[iRF]=npnt-1;
    }
    X[iRF]=0;
    X[iRF]+=checkDynamicPeak(iRF);
    X[iRF]+=checkDynamicThalf(iRF);
    X[iRF]+=checkDynamicMean(iRF);
    X[iRF]+=checkDynamicSigma(iRF);
     
    nX[iRF]=0;
    if (checkDynamicPeak(iRF)>2.)  nX[iRF]++;
    if (checkDynamicMean(iRF)>2.)  nX[iRF]++;
    if (checkDynamicSigma(iRF)>2.) nX[iRF]++;
    if (checkDynamicThalf(iRF)>2.) nX[iRF]++;
*/
    return result;
  }

//=============================================================================================== 
// compute bunch Quality data;
bool computeBunchQuality(int iRF) {
  bool result=true; 
  iBinI[iRF]=-1;    
  for (int ib=2; ib<npnt; ib++) { // find bin;
    if (peakIntegralXX[iRF]<binsI[ib] && mipCut1<binsI[ib]) {
      iBinI[iRF]=ib;
      break;
    }
  }
  if (iBinI[iRF]==-1) {
    iBinI[iRF]=npnt-1;
  }

  if (iBinI[iRF]==-1) {
    printf("\n\n[computeBunchQuality] Integral intervals not loaded properly\n");
    printf("npnt=%d\n", npnt);
    printf("iRF=%d\n", iRF);
    printf("Bye!\n");
    exit(-1);
  }

  X[iRF]=0;
  X[iRF]+=checkDynamicPeak(iRF);
  X[iRF]+=checkDynamicThalf(iRF);
  X[iRF]+=checkDynamicMean(iRF);
  X[iRF]+=checkDynamicSigma(iRF);
     
  nX[iRF]=0;
  if (checkDynamicPeak(iRF)>2.)  nX[iRF]++;
  if (checkDynamicMean(iRF)>2.)  nX[iRF]++;
  if (checkDynamicSigma(iRF)>2.) nX[iRF]++;
  if (checkDynamicThalf(iRF)>2.) nX[iRF]++;
  
return result;
}

//=============================================================================================== 
// check pulse position and width;
//
  bool checkPulseMean(int iRF) {
    return (peakTmn[iRF]<meanAlignment+TphaseRF+iRF*165.+cutFactor*meanAlignmentSigma) &&
           (peakTmn[iRF]>meanAlignment+TphaseRF+iRF*165.-cutFactor*meanAlignmentSigma);  
  }
  bool checkPulsePeak(int iRF) {
    return (peakT00[iRF]<peakAlignment+TphaseRF+iRF*165.+cutFactor*peakAlignmentSigma) &&
           (peakT00[iRF]>peakAlignment+TphaseRF+iRF*165.-cutFactor*peakAlignmentSigma); 
  }

  bool checkPulseSigma(int iRF) {
    printf(" %s #%d sigma=%.1f expected=%.1f cut=%.1f\n", 
           Name, iRF, peakTRMS[iRF], pulseWidhSigma, pulseWidhSigma+cutFactor*pulseWidhSigmaSigma);
    return peakTRMS[iRF]<pulseWidhSigma+cutFactor*pulseWidhSigmaSigma;
  }

  bool checkPulseThalf(int iRF) {
    return peakT2D[iRF]-peakT2U[iRF]<pulseHalfWidh+cutFactor*pulseHalfWidhSigma;
  }
//=============================================================================================== 
// check signal-depending pulse position and width;
//
double checkDynamicMean(int iRF) {
  double result=0.;
  if (peakGain[iRF]==-1) {
    printf("\n\n[checkDynamicMean] %s gain not set\n\n", Name);
    exit(-1); 
  }
  int ib=iBinI[iRF];
  if (ib>299 || ib<0) {
    printf("\n\n[checkDynamicMean] %s RF#%d bad bin#%d\n\n", Name, iRF, ib);
    exit(-1); 
  }
  result=peakTmn[iRF]-TphaseRF-iRF*165.-mean1[ib];
  result/=mean2[ib];
  result=result*result;
  return result;
}
double checkDynamicPeak(int iRF) {
  double result=0.;
  if (peakGain[iRF]==-1) {
    printf("\n\n[checkDynamicPeak] %s gain not set\n\n", Name);
    exit(-1); 
  }
  int ib=iBinI[iRF];
  if (ib>299 || ib<0) {
    printf("\n\n[checkDynamicPeak] %s RF#%d bad bin#%d\n", Name, iRF, ib);
    for (int i=0; i<40; i++) printf("       iBinI[%2d]=%d \n", i, iBinI[i]);  
    exit(-1); 
  }

  result=peakT00[iRF]-TphaseRF-iRF*165.-peak1[ib];
  result/=peak2[ib];
  result=result*result;
  return result;
}
double checkDynamicSigma(int iRF) {
  double result=0.;
  if (peakGain[iRF]==-1) {
    printf("\n\n[checkDynamicSigma] %s gain not set\n\n", Name);
    exit(-1); 
  }
  int ib=iBinI[iRF];
  if (ib>299 || ib<0) {
    printf("\n\n[checkDynamicSigma] %s RF#%d bad bin#%d\n\n", Name, iRF, ib);
    exit(-1); 
  }
  result=peakTRMS[iRF]-sigma1[ib];
  result/=sigma2[ib];
  result=result*result;
  return result;
}
double checkDynamicThalf(int iRF) {
  double result=0.;
  if (peakGain[iRF]==-1) {
    printf("\n\n[checkDynamicThalf] %s gain not set\n\n", Name);
    exit(-1); 
  }
  int ib=iBinI[iRF];
  if (ib>299 || ib<0) {
    printf("\n\n[checkDynamicThalf] %s RF#%d bad bin#%d\n\n", Name, iRF, ib);
    exit(-1); 
  }
  result=peakT2D[iRF]-peakT2U[iRF]-half1[ib];
  result/=half2[ib];
  result=result*result;
  return result;
}
//==================================================================================================== 
// integral around time T
//
  bool getDiffIntegral00(double T, float &integralL, float &integralH) {
    bool result = true;
    double T0i=T;
    integralL=0.; integralH=0.;
    if (T0i<12.5*firstSample[0]+100.)            return false;
    if (T0i>12.5*lastSample[nPedWindows-1]-100.) return false;

    float S1H=0.;
    float S2H=0.;
    float S3H=0.;
    float S1L=0.;
    float S2L=0.;
    float S3L=0.;
    result = result && simpleIntegral(T0i-0.5*twIN-twOUT, T0i-0.5*twIN,       S1L, S1H); 
    result = result && simpleIntegral(T0i-0.5*twIN,       T0i+0.5*twIN,       S2L, S2H); 
    result = result && simpleIntegral(T0i+0.5*twIN,       T0i+0.5*twIN+twOUT, S3L, S3H);

    integralL=S2L-S1L-S3L; 
    integralH=S2H-S1H-S3H; 

    return result;
  }

//====================================================================================================== 
  bool simpleIntegral(double T1, double T2,  float &integralL, float &integralH) {
    bool result = true;
    integralL=0.; integralH=0.;
    if (T1<12.5*firstSample[0]+100.)            return false;
    if (T2>12.5*lastSample[nPedWindows-1]-100.) return false;
    int i0=int((T1-time2013start)/time2013step);
    if (i0<0) return false;
    int i1=int((T2-time2013start)/time2013step);
    if (i1>ns2013-1) return false;
    double t00=time2013start+i0*time2013step;
    double t01=t00+time2013step;
    double t10=time2013start+i1*time2013step;
    double t11=t10+time2013step;

    if (t00>T1 || t01<T1 || t10 >T2 || t11<T2) {
      printf("[myCounter->simpleIntegral] bad timing (t00=%.1f T1=%.1f t01=%.1f) (t10=%.1f T2=%.1f t11=%.f)\n",
              t00, T1, t01, t10, T2, t11);
      printf("                            tStart=%.1f tStep=%.1f i0=%d i1=%d N=%d\n",
              time2013start, time2013step, i0, i1, ns2013);
//      exit(-1);
      return false;
    }

    float A1h=(hg2013[i0]*(t01-T1)+hg2013[i0+1]*(T1-t00))/time2013step;
    float A2h=(hg2013[i1]*(t11-T2)+hg2013[i1+1]*(T2-t10))/time2013step;
    float A1l=(lg2013[i0]*(t01-T1)+lg2013[i0+1]*(T1-t00))/time2013step;
    float A2l=(lg2013[i1]*(t11-T2)+lg2013[i1+1]*(T2-t10))/time2013step;

    integralH=((t01-T1)*(A1h+hg2013[i0+1])+(T2-t10)*(A2h+hg2013[i1-1]));
    integralL=((t01-T1)*(A1l+lg2013[i0+1])+(T2-t10)*(A2l+lg2013[i1-1]));
    for (int i=i0+1; i<i1-1; i++) {
      integralL+=time2013step*(lg2013[i]+lg2013[i+1]);
      integralH+=time2013step*(hg2013[i]+hg2013[i+1]);
    }

    integralL/=50.; // close to the peak scale...
    integralH/=50.;

    return result;    
  }




//==========================================================================================================================  
  bool getIntegral(int control, double T1, double T2, float &integralL, float &integralH, float &peakLG, float &peakHG) {
    bool result = true;
    integralL=0.; integralH=0.;
    peakHG=-1.0e10; 
    peakLG=-1.0e10;
    if (T1<12.5*firstSample[0]+100.)            return false;
    if (T2>12.5*lastSample[nPedWindows-1]-100.) return false;

    int i0=int((T1-time2013start)/time2013step);
    if (i0<0) return false;
    int i1=int((T2-time2013start)/time2013step);
    if (i1>ns2013-1) return false;
    double t00=time2013start+i0*time2013step;
    double t01=t00+time2013step;
    double t10=time2013start+i1*time2013step;
    double t11=t10+time2013step;

    if (t00>T1 || t01<T1 || t10 >T2 || t11<T2) {
      printf("[myCounter->getIntegral] bad timing (t00=%f T1=%f t01=%f) (t10=%f T2=%f t11=%f)\n",
              t00, T1, t01, t10, T2, t11);
      printf("[myCounter->getIntegral] bad timing tStart=%f tStep=%f i0=%d i1=%d N=%d\n",
              time2013start, time2013step, i0, i1, ns2013);
      exit(-1);
    }

    float A1h=(hg2013[i0]*(t01-T1)+hg2013[i0+1]*(T1-t00))/time2013step;
    float A2h=(hg2013[i1]*(t11-T2)+hg2013[i1+1]*(T2-t10))/time2013step;
    float A1l=(lg2013[i0]*(t01-T1)+lg2013[i0+1]*(T1-t00))/time2013step;
    float A2l=(lg2013[i1]*(t11-T2)+lg2013[i1+1]*(T2-t10))/time2013step;
    if (control==1) {
      printf("[getIntegral] %s i0=%d t00=%.1f T1=%.1f t01=%.1f --- i1=%d t10=%.1f T2=%.1f t11=%.1f\n",
             Name, i0, t00, T1, t01, i1, t10, T2, t11);
      printf("Aleft=%.2f sLeft=%.2f\n", A1h, (t01-T1)*(A1h+hg2013[i0+1])/2.);
    }
    if (peakHG<hg2013[i0]) peakHG=hg2013[i0];
    if (peakLG<lg2013[i0]) peakLG=lg2013[i0];
    if (peakHG<hg2013[i1+1]) peakHG=hg2013[i1+1];
    if (peakLG<lg2013[i1+1]) peakLG=lg2013[i1+1];
    integralH=((t01-T1)*(A1h+hg2013[i0+1])+(T2-t10)*(A2h+hg2013[i1-1]));
    integralL=((t01-T1)*(A1l+lg2013[i0+1])+(T2-t10)*(A2l+lg2013[i1-1]));
    for (int i=i0+1; i<i1-1; i++) {
      if (control==1) printf(" s[%d]=%.2f\n", i, time2013step*(hg2013[i]+hg2013[i+1])/2.);
      integralL+=time2013step*(lg2013[i]+lg2013[i+1]);
      integralH+=time2013step*(hg2013[i]+hg2013[i+1]);
      if (peakHG<hg2013[i]) peakHG=hg2013[i];
      if (peakLG<lg2013[i]) peakLG=lg2013[i];
    }
    if (control==1) {
      printf("Aright=%.2f sRight=%.2f s\n\n\n", A2h, (T2-t10)*(A2h+hg2013[i1-1])/2.);
    }
//    integralL/=2.;
//    integralH/=2.;
    integralL/=50.; // close to the peak scale...
    integralH/=50.;

    return result;    
  }

//--------------------------- load integral-depending cuts from file----------------------
bool loadDynamicCuts(char* fileName) {
  bool result = false;
  printf("loading %s cuts from %s\n", Name, fileName);
  std::ifstream indata;
//  indata.open(fileName, ios::in);
  indata.open(fileName, std::ifstream::in);
  bool loaded[4];
  int idCounter, idPlot, np;
  double s0, sx, s1, s2;
  
  for (int j=0; j<4; j++) loaded[j]=false;
 
  for (int j=0; j<4; j++) {
//peak
    indata >> idCounter >> idPlot >> np;
    if (idCounter==id) {loaded[0]=true; npnt=np;}
    if (npnt>300) {
      printf("[loadDynamicCuts] %s too many points for dynamic cuts = %d\n",
             Name, npnt);
      exit(-1);
    }
    for (int i=0; i<np; i++) {
      indata >> s0 >> sx >> s1 >> s2;
      if (loaded[0]) {
        binsI[i]=sx;
        peak1[i]=s1;
        peak2[i]=s2;
      } 
    }
//mean
    indata >> idCounter >> idPlot >> np;
    if (idCounter==id) loaded[1]=true;
    for (int i=0; i<np; i++) {
      indata >> s0 >> sx >> s1 >> s2;
      if (loaded[1]) {
        mean1[i]=s1;
        mean2[i]=s2;
      } 
    }
    
//half-width
    indata >> idCounter >> idPlot >> np;
    if (idCounter==id) loaded[2]=true;
    for (int i=0; i<np; i++) {
      indata >> s0 >> sx >> s1 >> s2;
      if (loaded[2]) {
        half1[i]=s1;
        half2[i]=s2;
      } 
    }

//sigma
    indata >> idCounter >> idPlot >> np;
    if (idCounter==id) loaded[3]=true;
    for (int i=0; i<np; i++) {
      indata >> s0 >> sx >> s1 >> s2;
      if (loaded[2]) {
        sigma1[i]=s1;
        sigma2[i]=s2;
      } 
    }
   
    result = loaded[0] && loaded[1] && loaded[2] && loaded[3];

    if (result) break;   
  }

  for (int i=0; i<10; i++) {
    printf("     %4d of %dI=%6.1f peak=%5.1f+/-%5.1f mean=%5.1f+/-%5.1f halfW=%5.1f+/-%5.1f sigma=%5.1f+/-%5.1f\n",
           i, np, binsI[i], peak1[i], peak2[i], mean1[i],  mean2[i], 
                        half1[i], half2[i], sigma1[i], sigma2[i]);      
  }

  indata.close();
  indata.clear();
  return result;
}

}; 


