struct Module { 
  int iMod;
  char Name[20];
  char gName[50];
  Channel channel[5]; // 4 readout outputs and the sum
  double X;        // X center of gravity,  entire event
  double Y;        // Y center of gravity,  entire event
  double Q;        // module Sum, entire event;
  double Xf;       // X center of gravity,  entire event - full integral
  double Yf;       // Y center of gravity,  entire event - full integral
  double Xcl[5];   // X center of gravity, per cluster
  double Ycl[5];   // Y center of gravity, per cluster
  double Qcl[5];
  bool computed;
  bool sumShownFFT;
  bool HGoverflow;
  bool LGoverflow;
  int nPedWindows;
  int firstSample[25];
  int lastSample[25];
  float time[512];
  double TphaseRF;
  double aGain[2];

  double eventFullIntegral[5];

//===========================================================================================
void TranserPedWindows() {
  for (int ich=0; ich<5; ich++) {
    for (int ig=0; ig<2; ig++) {
      channel[ich].signal[ig].nPedWindows=nPedWindows;
      for (int iw=0; iw<nPedWindows; iw++) {
        channel[ich].signal[ig].firstSample[iw]=firstSample[iw];
        channel[ich].signal[ig].lastSample[iw] =lastSample[iw];
      }
    }
  }
}
//===========================================================================================
   void SetPhaseRF(double t) {
    TphaseRF=t;
    for (int i=0; i<5; i++) channel[i].SetPhaseRF(TphaseRF);
    SetTime();
  }
  
//===========================================================================================
  void SetTime() {
    for (int i=0; i<512; i++) time[i]=TphaseRF+12.5*i;
  }

//===========================================================================================
void ComputeFullIntegrals() {
  for (int iCh=0; iCh<5; iCh++) {
    eventFullIntegral[iCh]=0.;
    for (int iCl=0; iCl<nPedWindows-1; iCl++) {
      for (int iG=0; iG<2; iG++) {
        channel[iCh].signal[iG].ComputeClusterIntegralFull(iCl);
      }
      double add=0.;
      if (HGoverflow) {
        add=channel[iCh].signal[0].GetClusterIntegralFull(iCl);
      } else {
        add=channel[iCh].signal[1].GetClusterIntegralFull(iCl);
      }
      eventFullIntegral[iCh]+=add;
    }
  }
}

//===========================================================================================
double GetFullIntegral(int iCl) {
  if (HGoverflow) {
    return channel[4].signal[0].GetClusterIntegralFull(iCl);
  } else {
    return channel[4].signal[1].GetClusterIntegralFull(iCl);
  }
}
//===========================================================================================
  void Add(int iCl, int iGain) {
    for (int iCh=0; iCh<4; iCh++) {
      float A=aGain[iGain]*channel[iCh].signal[iGain].clusterIntegral[iCl];
      float x=channel[iCh].X;
      float y=channel[iCh].Y;
      //printf("[Module->Add] %s#%d gain=%d x=%5.2f y=%5.2f  A=%.1f\n", Name, iCh, iGain, x, y, A);
      X+=A*x;
      Y+=A*y;
      Q+=A;
      Xcl[iCl]+=A*x;
      Ycl[iCl]+=A*y;
      Qcl[iCl]+=A;
    }    
  }

//===========================================================================================
  void Compute() {
    computed=true;
    if (Q>0.) {
      X/=Q;
      Y/=Q;
    } else {
      X=-1.e10;
      Y=-1.e10;
    }
    for (int i=0; i<5; i++) {
      if (Qcl[i]>0.) {
        Xcl[i]/=Qcl[i];
        Ycl[i]/=Qcl[i];
      } else {
        Xcl[i]=-1.e10;
        Ycl[i]=-1.e10;
      }      
    }
  }

//===========================================================================================
  void Clear() {
    sumShownFFT=false;
    computed=false;
    X=0.;
    Y=0.;
    Q=0.;
    for (int i=0; i<5; i++) {
      channel[i].Clear();
      Xcl[i]=0.; 
      Ycl[i]=0.; 
      Qcl[i]=0.;
    }
    aGain[0]=10.5;
    aGain[1]=1.0;
  }

//===========================================================================================
void AdjustLG() {
  for (int i=0; i<4; i++) {
    channel[i].AdjustLG(aGain[0]);
  }
}
//===========================================================================================
  void ClearSpill() { // clear noises and other spill info
    Clear();
    for (int i=0; i<5; i++) channel[i].ClearSpill();
  }

//===========================================================================================
  void FillEmptyPed(int iw) { // fill empty event pedestal window
    for (int i=0; i<5; i++) channel[i].FillEmptyPed(firstSample[iw], lastSample[iw]);
  }
 
//===========================================================================================
  void FillPedWindow(int iw) { // fill filled  event pedestal window
    for (int i=0; i<5; i++) channel[i].FillPedWindow(iw, firstSample[iw], lastSample[iw]);
  }
 
  //===========================================================================================
  void DisplayMPI(TCanvas* myCanvas, TH1F *myDummy, TLine *myLine, int iZone) {
    float amax;
    float amin;
    float dd;
    int    iG=1;
    double tPlot0=-200.;
    double tPlot1=512*12.5;
    if (HGoverflow) iG=0;
    for (int iChan=0; iChan<5; iChan++) {
      channel[iChan].doMaxMinMPI();
    }
    amax=channel[4].maxMPI;
    amin=channel[4].minMPI;
    dd=amax-amin;
    amax+=0.05*dd;
    amin-=0.05*dd;

    myCanvas->cd(iZone)->SetGrid(1);
    myDummy->GetXaxis()->SetRangeUser(tPlot0, tPlot1);
    myDummy->GetYaxis()->SetRangeUser(amin, amax);
    myDummy->GetYaxis()->SetTitle("MPI ADC, mV");
    if (iG==1) {
      sprintf(gName, "%s, High Gain", Name);
    } else {
      sprintf(gName, "%s, Low Gain", Name);
    }    
    myDummy->SetTitle(gName);
    myDummy->GetXaxis()->SetTitle("Time, ns");
    myDummy->DrawCopy();

    myLine->SetLineWidth(20);
    myLine->SetLineColor(kYellow);
    for (int iWin=0; iWin<nPedWindows; iWin++) {
      myLine->DrawLine(time[firstSample[iWin]], 0., 
                       time[lastSample[iWin]],  0.);
    }

    myLine->SetLineWidth(2);
    myLine->SetLineColor(kBlack);
    for (int i=0; i<511; i++) {
      if (time[i+1]>6200.) break;
      myLine->DrawLine(time[i],   channel[4].signal[iG].ampI[i],
                       time[i+1], channel[4].signal[iG].ampI[i+1]);
    }
    myCanvas->Update();    
  }


//===========================================================================================
  void DisplayFFT(TCanvas* myCanvas, TH1F *myDummy, TLine *myLine, int iZone) {
    float amax;
    float amin;
    float dd;
    int    iG=1;
    double tPlot0=-200.;
    double tPlot1=512*12.5;
    if (HGoverflow) iG=0;
    for (int iChan=0; iChan<5; iChan++) {
      channel[iChan].doMaxMinFFT();
    }
    sumShownFFT=true;
    amax=channel[4].maxFFT;
    amin=channel[4].minFFT;
    dd=amax-amin;
    amax+=0.05*dd;
    amin-=0.05*dd;

    myCanvas->cd(iZone)->SetGrid(1);
    myDummy->GetXaxis()->SetRangeUser(tPlot0, tPlot1);
    myDummy->GetYaxis()->SetRangeUser(amin/1000., amax/1000.);
    myDummy->GetYaxis()->SetTitle("Current, mA");
    if (iG==1) {
      sprintf(gName, "%s High Gain, Trf=%.2fns", Name, TphaseRF);
    } else {
      sprintf(gName, "%s Low Gain, Trf=%.2fns", Name, TphaseRF);
    }    
    myDummy->SetTitle(gName);
    myDummy->GetXaxis()->SetTitle("Time, ns");
    myDummy->DrawCopy();

    myLine->SetLineWidth(20);
    myLine->SetLineColor(kYellow);
    for (int iWin=0; iWin<nPedWindows; iWin++) {
      myLine->DrawLine(time[firstSample[iWin]], 0., 
                       time[lastSample[iWin]],  0.);
      if (iWin<nPedWindows-1) {
      }
    }

    myLine->SetLineWidth(3);
    myLine->SetLineColor(kBlack);
    int is0=firstSample[0];
    int is1=lastSample[nPedWindows-1];
    for (int i=0; i<is1-is0-1; i++) {
      myLine->DrawLine(time[i+is0],   channel[4].signal[iG].ampF[i]/1000.,
                       time[i+is0+1], channel[4].signal[iG].ampF[i+1]/1000.); //// mA, not uA!
    }
    myCanvas->Update();    
  }


//===========================================================================================
  void DisplayFFT(TCanvas* myCanvas, TH1F *myDummy, TLine *myLine, int iZone, int iChan, double scale) {
//    float amax;
//    float amin;
//    float dd;
    int    iG=1;
//    double tPlot0=-200.;
//    double tPlot1=512*12.5;
    if (iChan<0 || iChan>3) return;
    if (HGoverflow) iG=0;

    double gainCorr=1.;
    if (iChan<4) gainCorr=aGain[iG];    

    if (sumShownFFT) {
      myCanvas->cd(iZone);
    } else {
      sumShownFFT=true;
      DisplayFFT(myCanvas, myDummy, myLine, iZone);
    }

    myLine->SetLineWidth(2);
    myLine->SetLineColor(iChan+1);
    int is0=firstSample[0];
    int is1=lastSample[nPedWindows-1];
    for (int i=0; i<is1-is0-1; i++) {
      myLine->DrawLine(time[i+is0],   scale*channel[iChan].signal[iG].ampF[i]/1000.,
                       time[i+is0+1], scale*channel[iChan].signal[iG].ampF[i+1]/1000.);
    }
    myCanvas->Update();    
  }
};



