struct parabolaFit {
private:
double time[5000];
double gadc[5000];
double sigm[5000];
double w0, xc, x1, x2, x3, x4, y1, y2, xy, x2y, chi2;
double a,  b,  c;
double a1, b1, c1;
double x0;
unsigned int  nFill;
unsigned int  ip;
unsigned int  nLast;
bool fitDone;
public:
void Clear() {
  for (int i=0; i<5000; i++) {
    time[i]=0.;
    gadc[i]=0.;
    sigm[i]=0.;
  }
  w0=0.;
  x1=0.;
  x2=0.;
  x3=0.;
  x4=0.;
  y1=0.;
  y2=0.;
  xy=0.;
  x2y=0.;
  chi2=0.;
  nFill=0;
  nLast=0;
  ip=0;
  fitDone=false;
}

void SetNpoints(int n) {
  if (n>4999) {
    printf("\n\n[parabolaFit->SetNpoints] too many ponts %d requested, bye!\n\n", n);
    exit(-1);
  } 
  nFill = n;
}
int  GetNpoints() {
  if (nLast<nFill) return nLast;
  return nFill;
}

void AddPoint(double t, double g, double s) {
  fitDone=false;
  if (nLast<nFill) {
    time[ip]=t;
    gadc[ip]=g;
    sigm[ip]=s;
    ip++;
    nLast++;
  } else {
    if (ip == nFill) ip=0;
    time[ip]=t;
    gadc[ip]=g;
    sigm[ip]=1./s/s;
    ip++;
  }
}

double DoFit() {
  fitDone=true;
  w0=0.;
  x1=0.;
  x2=0.;
  x3=0.;
  x4=0.;
  y1=0.;
  y2=0.;
  xy=0.;
  x2y=0.;
  chi2=0.;
  for (unsigned int i=0; i<nFill; i++) {
    double w=sigm[ip];
    double x=time[i];
    w0+=w;
    x1+=w*x; 
  }
  xc=x1/w0;
  w0=0.;
  x1=0.;
  for (unsigned int i=0; i<nFill; i++) {
    double w=sigm[i];
    double x=time[i]-xc;
    double g=gadc[i];
    w0+=w;
    x1+=w*x;
    x2+=w*x*x;
    x3+=w*x*x*x;
    x4+=w*x*x*x*x;
    xy+=w*x*g;
    x2y+=w*x*x*g;
    y1+=w*g;
    y2+=w*g*g;
  }

  x1/=w0;
  x2/=w0;
  x3/=w0;
  x4/=w0;
  y1/=w0;
  y2/=w0;
  xy/=w0;
  x2y/=w0;
  double det0=x4*x2    + 2.0*x3*x2*x1 - x2*x2*x2  - x3*x3    - x4*x1*x1;
  double detA=x2y*x2   + y1*x3*x1     + xy*x2*x1  - y1*x2*x2 - xy*x3     - x2y*x1*x1;
  double detB=xy*x4    + x2y*x2*x1    + y1*x3*x2  - xy*x2*x2 - x2y*x3    - y1*x4*x1;
  double detC=y1*x4*x2 + xy*x3*x2     + x2y*x3*x1 - y1*x3*x3 - x2y*x2*x2 - xy*x4*x1;
  a1=detA/det0;
  b1=detB/det0;
  c1=detC/det0;
  chi2=a1*a1*x4     + b1*b1*x2     + c1*c1      + y2 + 
       2.0*a1*b1*x3 + 2.0*a1*c1*x2 - 2.0*a1*x2y + 
       2.0*b1*c1*x1 - 2.0*b1*xy    - 2.0*c1*y1;
//  printf("[DoFit] chi2=%.2f <x>=%f a=%f b=%f c=%f t0=%f\n",
//         chi2, xc, a1, b1, c1, xc-0.5*b1/a1);
//  double xp=xc-0.5*b1/a1;
//  a=a1;
//  b=0.;
//  c=c1+a1*(xp-xc)*(xp-xc)+b1*(xp-xc);
//  printf("[DoFit]                a=%f b=%f c=%f tPeak=%f\n",
//         a, b, c, xp);
  return chi2;
}

double getA() {
  if (!fitDone) DoFit();
  return a1;
}

double getB() {
  if (!fitDone) DoFit();
  return b1;
}

double getC() {
  if (!fitDone) DoFit();
  return c1;
}

double getCenter() {
  if (!fitDone) DoFit();
  return -0.5*b/a;
}

double getXc() {
  if (!fitDone) DoFit();
  return xc;
}

double getAverageGADC() {
  if (!fitDone) DoFit();
  return y1;
}

int getIndex(int i) {
  if (i<0 || i>=nFill) {
    printf("[parabolaFit->getT] wrong index %d, nFill=%d; bye\n",
           i, nFill);
  }
  int j=ip+i;
  if (j>=nFill) j-=nFill;
  return j;
}


double getTime(int i) { // get time 0...nFill in straight order
  if (i<0 || i>=nFill) {
    printf("[parabolaFit->getT] wrong index %d, nFill=%d; bye\n",
           i, nFill);
  }
  int j=ip+i;
  if (j>=nFill) j-=nFill;
  return time[j];
}

double getGADC(int i) { // get time 0...nFill in straight order
  if (i<0 || i>=nFill) {
    printf("[parabolaFit->getT] wrong index %d, nFill=%d; bye\n",
           i, nFill);
  }
  int j=ip+i;
  if (j>=nFill) j-=nFill;
  return gadc[j];
}


double getFunc(int i) { // get time 0...nFill in straight order
  if (!fitDone) DoFit();
  if (i<0 || i>=nFill) {
    printf("[parabolaFit->getT] wrong index %d, nFill=%d; bye\n",
           i, nFill);
  }
  int j=ip+i;
  if (j>=nFill) j-=nFill;
  double x=time[j]-xc;
  return (a1*x+b1)*x+c1;
}
}; 
