struct prehistoryBin {
private:
  int    integralOption; // 0 - flat; 1 - triangle;
  int    module; // HEC, EMEC, FCal-250, FCal-100
  int    channel;
  int     myID;

  double integrationTime;
  double rateLow;
  double rateHigh;

  double chrMax;
  double chrMin;
  double sigMax;
  double sigMin;

  double ped0;
  double chrPed1;
  double chrPed2;
  double sigPed1;
  double sigPed2;
  double currentCherPed;
  double currentCherPedSigma;
  double currentModPed;
  double currentModPedSigma;
  

  double modCalibration; // must be set manualy by SetModCalibration(double c); tracks/fC
                         // used for "intelligent fit";
  double slopeSmart;

  double sx1;
  double sx2;
  double sy1;
  double sy2;
  double sxy;

  bool sumsReady;
  bool report;

  std::vector<int> run;
  std::vector<int> spill;
  std::vector<int> event;
  std::vector<int> sevnt;
  std::vector<double> history;
  std::vector<double> cherenkov;
  std::vector<double> signalMod;
  std::vector<double> signalModCalib;
  std::vector<double> xBal;
  std::vector<double> yBal;


public:
//-------------------------------------------------------------
void Clear() {
 
  run.clear();
  spill.clear();
  event.clear();
  sevnt.clear();
  history.clear();
  cherenkov.clear();
  signalMod.clear();
  signalModCalib.clear();
  xBal.clear();
  yBal.clear();
  
  chrMax= 0.;
  chrMin= 0.;
  sigMax= 0.;
  sigMin= 0.;

  ped0=0.;
  chrPed1=0.;
  chrPed2=0.;
  sigPed1=0.;
  sigPed2=0.;

  currentCherPed=0.;
  currentModPed=0.;
  currentCherPedSigma=0.;
  currentModPedSigma=0.;


  sx1=0.;
  sx2=0.;
  sy1=0.;
  sy2=0.;
  sxy=0.;
  
  slopeSmart=0.0;

  sumsReady=false;
  report=false;

  modCalibration=1.0;
}

//-------------------------------------------------------------
void ClearPedestals() {
  ped0=0.;
  chrPed1=0.;
  chrPed2=0.;
  sigPed1=0.;
  sigPed2=0.;
} 

//-------------------------------------------------------------
void Set(int mod, int chn, int iOpt, double ti, double rateL, double rateH) {
  integralOption  = iOpt;
  integrationTime = ti;
  module   = mod;
  channel  = chn;
  rateLow  = rateL;
  rateHigh = rateH;
}

//-------------------------------------------------------------
void Set(int id, int mod, int chn, int iOpt, double ti, double rateL, double rateH) {
  myID=id;
  integralOption  = iOpt;
  integrationTime = ti;
  module   = mod;
  channel  = chn;
  rateLow  = rateL;
  rateHigh = rateH;
}

//-------------------------------------------------------------
void SetModCalibration(double c) {
//  printf(" module calibration set to %f fC/track\n", c);
  modCalibration=1.0/c; // tracks/fC
}

//-------------------------------------------------------------
void SetReport(bool b) {
  report=b;
}

//-------------------------------------------------------------
void Fill(int spl, int evt, int svt, double hst, double cher, double signal, double xb, double yb) {
  spill.push_back(spl);
  event.push_back(evt);
  sevnt.push_back(svt);
  history.push_back(hst);
  cherenkov.push_back(cher   - currentCherPed);
  signalMod.push_back(signal - currentModPed);
  signalModCalib.push_back(signal*modCalibration);
  xBal.push_back(xb);
  yBal.push_back(yb);
  if (chrMax<cher)   chrMax=cher;
  if (chrMin>cher)   chrMin=cher;
  if (sigMax<signal) sigMax=signal;
  if (sigMin>signal) sigMin=signal;

}

//-------------------------------------------------------------
void Fill(int rn, int spl, int evt, int svt, double hst, double cher, double signal, double xb, double yb) {
  run  .push_back(rn);
  spill.push_back(spl);
  event.push_back(evt);
  sevnt.push_back(svt);
  history.push_back(hst);
  cherenkov.push_back(cher   - currentCherPed);
  signalMod.push_back(signal - currentModPed);
  signalModCalib.push_back(signal*modCalibration);
  xBal.push_back(xb);
  yBal.push_back(yb);
  if (chrMax<cher)   chrMax=cher;
  if (chrMin>cher)   chrMin=cher;
  if (sigMax<signal) sigMax=signal;
  if (sigMin>signal) sigMin=signal;

 }

void ReportPedestals() {
  int ilast=event.size()-1;
  printf(" run#%04d spill#%03d event#%04d(%03d) pedCher=%5.2f+/-%5.2f ped(Mod#%d)=%5.2f-%5.2f\n",
         run[ilast], spill[ilast], event[ilast], sevnt[ilast], currentCherPed, currentCherPedSigma, 
          module, currentModPed, currentModPedSigma);    
}

//-------------------------------------------------------------
void FillPedestal(double cher, double signal) { 
  ped0+=1.0;
  chrPed1+=cher;
  chrPed2+=cher*cher;
  sigPed1+=signal;
  sigPed2+=signal*signal;

  currentCherPed=chrPed1/ped0;
  currentCherPedSigma=chrPed2/ped0;
  currentCherPedSigma=currentCherPedSigma*currentCherPedSigma-
                      currentCherPed*currentCherPed;
  currentCherPedSigma=sqrt(currentCherPedSigma);

  currentModPed=sigPed1/ped0;
  currentModPedSigma=sigPed2/ped0;
  currentModPedSigma=currentModPedSigma*currentModPedSigma-
                     currentModPed*currentModPed;
  currentModPedSigma=sqrt(currentModPedSigma);
    
}
//-------------------------------------------------------------
double GetRateLow() {
  return rateLow;
}

//-------------------------------------------------------------
double GetRateHigh() {
  return rateHigh;
}

//-------------------------------------------------------------
double GetCherMax() {
  return chrMax;
}

//-------------------------------------------------------------
double GetCherMin() {
  return chrMin;
}

//-------------------------------------------------------------
double GetSigMax() {
  return sigMax;
}

//-------------------------------------------------------------
double GetSigMin() {
  return sigMin;
}


//-------------------------------------------------------------
double GetIntegrationTime() {
  return integrationTime;
}

//-------------------------------------------------------------
int GetIntegralOption() {
  return integralOption;
}

//-------------------------------------------------------------
int GetStat() {
  return event.size();
}
//-------------------------------------------------------------
void Dump(int n) {
  int Ndump=n;
  if (Ndump <= 0 || Ndump > int(event.size())) Ndump=event.size();
  printf("[prehistoryBin->Dump] - >\n");
  printf(" entry#  cherenkov(p) module(fC) xBal  yBal\n");
  for (int i=0; i<Ndump; i++) {
    printf(" %5d   %5.2e     %5.2e   %5.2f   %5.2f\n",
           i,  cherenkov[i], signalMod[i], xBal[i], yBal[i]);      
  }
  printf("---------------------------------------------\n");
}
//-------------------------------------------------------------
int GetStatX(double x0, double x1) { // statistics within X0, X1 window
  int stat=0;
//  printf("[prehistoryBin->GetStatX] -> [%5.2f, %5.2f]\n", x0, x1);
//  printf(" entry#  cherenkov(p) module(fC) xBal  yBal\n");
  for (unsigned int i=0; i<event.size(); i++) {
    if (xBal[i]>x0 && xBal[i]<=x1) stat++;
//    if (i>50) continue;
//    printf(" %5d   %5.2e     %5.2e   %5.2f  %5.2f %d\n",
//           i,  cherenkov[i], signalMod[i], xBal[i], yBal[i], stat);  
  }
//  printf("---------------------------------------------\n");
  return stat;
}

///-------------------------------------------------------------
int GetEvent(int i) {
  if (i<0 || i>=int(event.size())) {
    printf("[prehistoryBin->GetEvent] bad index#%d\n", i);
    exit(-1);
  }
  return event[i];
}

//-------------------------------------------------------------
int GetSpillEvent(int i) {
  if (i<0 || i>=int(event.size())) {
    printf("[prehistoryBin->GetSpillEvent] bad index#%d\n", i);
    exit(-1);
  }
  return sevnt[i];
}
  
//-------------------------------------------------------------
int GetSpill(int i) {
  if (i<0 || i>=int(event.size())) {
    printf("[prehistoryBin->GetSpill] bad index#%d\n", i);
    exit(-1);
  }
  return spill[i];
}

//-------------------------------------------------------------
double GetCherenkov(int i) {
  if (i<0 || i>=int(event.size())) {
    printf("[prehistoryBin->GetCherenkov bad index#%d\n", i);
    exit(-1);
  }
  return cherenkov[i];
}

//-------------------------------------------------------------
double GetHistory(int i) {
  if (i<0 || i>=int(event.size())) {
    printf("[prehistoryBin->GetHistory bad index#%d\n", i);
    exit(-1);
  }
  return history[i];
}

//-------------------------------------------------------------
double GetAverageHistory() {
  double aHist=0.;
  if (history.size()==0) return 0.;

  for (int i=0; i<int(history.size()); i++) {
    aHist+=history[i];
  }
  return aHist/(double)history.size();
}

//-------------------------------------------------------------
double GetSignal(int i) {
  if (i<0 || i>=int(event.size())) {
    printf("[prehistoryBin->GetSignal bad index#%d\n", i);
    exit(-1);
  }
  return signalMod[i];
}

//-------------------------------------------------------------
double GetXbal(int i) {
  if (i<0 || i>=int(event.size())) {
    printf("[prehistoryBin->GetXbal bad index#%d\n", i);
    exit(-1);
  }
  return xBal[i];
}

//-------------------------------------------------------------
double GetYbal(int i) {
  if (i<0 || i>=int(event.size())) {
    printf("[prehistoryBin->GetYbal bad index#%d\n", i);
    exit(-1);
  }
  return yBal[i];
}

//-------------------------------------------------------------
double GetAverageCherenkov() {
  double s1=0.;
  for (unsigned int i=0; i<event.size(); i++) {
    s1+=cherenkov[i];
  }
  if (event.size()>0) s1/=double(event.size());
  return s1;
}

double GetAverageCherenkov(double x0, double x1) { // average in the (x0, x1) interval of Xbalance
  double s1=0.;
  double s0=0.;
  for (unsigned int i=0; i<event.size(); i++) {
    if (xBal[i]>x0 && xBal[i]<=x1) {
      s1+=cherenkov[i];
      s0+=1.0;
    }
  }
  if (s0>0.) s1/=s0;
  else s1=0.;
  return s1;
}

double GetAverageCherenkov(int iSet, int nSet) { // subset #iSet out of nSet
  double s1=0.;
  double s0=0.;
  if (iSet>nSet-1) {
    printf("[prehistoryBin->GetAverageCherenkov(int, int) iSet=%d shall be less than nSet=%d\n", iSet, nSet);
    exit(-1);
  }
  for (unsigned int i=iSet; i<event.size(); i+=nSet) {
    s1+=cherenkov[i];
    s0+=1.0;
  }
  if (s0>0.) s1/=s0;
  else return 0.;

  return s1;
}

double GetAverageCherenkov(int iSet, int nSet, double x0, double x1) {
// subset #iSet out of nSet
// average in the (x0, x1) interval of Xbalance
  double s1=0.;
  double s0=0.;
  if (iSet>nSet-1) {
    printf("[prehistoryBin->GetAverageCherenkov(int, int) iSet=%d shall be less than nSet=%d\n", iSet, nSet);
    exit(-1);
  }
  for (unsigned int i=iSet; i<event.size(); i+=nSet) {
    if (xBal[i]>x0 && xBal[i]<=x1) {
      s1+=cherenkov[i];
      s0+=1.0;
    }
  }
  if (s0>0.) s1/=s0;
  else return 0.;

  return s1;
}



//-------------------------------------------------------------
double GetSigmaCherenkov() {
  double s1=0.;
  double s2=0.;
  for (unsigned int i=0; i<event.size(); i++) {
    s1+=cherenkov[i];
    s2+=cherenkov[i]*cherenkov[i];
  }
  if (event.size()>1) {
    s1/=double(event.size());
    s2/=double(event.size());
    s2-=s1*s1;
    s2=sqrt(s2);
  } else {
    s2=0.;
  }
  return s2;
}

//-------------------------------------------------------------
double GetAverageSignal() {
  double s1=0.;
  for (unsigned int i=0; i<event.size(); i++) {
    s1+=signalMod[i];
  }
  if (event.size()>0) {
    s1/=double(event.size());
  } else {
    return 0.0;
  }
  return s1;
}

//-------------------------------------------------------------
double GetAverageSignal(double x0, double x1) { // average in the (x0, x1) interval of Xbalance
  double s1=0.;
  double s0=0.;
  for (unsigned int i=0; i<event.size(); i++) {
    if (xBal[i]>x0 && xBal[i]<=x1) {
      s1+=signalMod[i];
      s0+=1.0;
    }
  }
  if (s0>0) s1/=s0;
  else s1=0.;
  return s1;
}

//-------------------------------------------------------------
double GetAverageSignal(int iSet, int nSet) { // subset #iSet out of nSet
  double s1=0.;
  double s0=0.;
  if (iSet>nSet-1) {
    printf("[prehistoryBin->GetAverageSignal(int, int) iSet=%d shall be less than nSet=%d\n", iSet, nSet);
    exit(-1);
  }
  for (unsigned int i=iSet; i<event.size(); i+=nSet) {
    s1+=signalMod[i];
    s0+=1.0;
  }
  if (s0>0) s1/=s0;
  else return 0.;

  return s1;
}
//-------------------------------------------------------------
double GetAverageSignal(int iSet, int nSet, double x0, double x1) { 
// subset #iSet out of nSet
// average in the (x0, x1) interval of Xbalance
  double s1=0.;
  double s0=0.;
  if (iSet>nSet-1) {
    printf("[prehistoryBin->GetAverageSignal(int, int) iSet=%d shall be less than nSet=%d\n", iSet, nSet);
    exit(-1);
  }
  for (unsigned int i=iSet; i<event.size(); i+=nSet) {
    if (xBal[i]>x0 && xBal[i]<=x1) {
      s1+=signalMod[i];
      s0+=1.0;
    }
  }
  if (s0>0) s1/=s0;
  else return 0.;

  return s1;
}
//-------------------------------------------------------------
double GetSignalToCherenkov() {
  return GetAverageSignal() / GetAverageCherenkov();
}

//-------------------------------------------------------------
double GetSignalToCherenkov(double x0, double x1) {// average in the (x0, x1) interval of Xbalance
  return GetAverageSignal(x0, x1) / GetAverageCherenkov(x0, x1);
}

//-------------------------------------------------------------
double GetSignalToCherenkovErr(int nSet) { // split data into nSet groups and vompute variance;
  if (nSet<2) {
    printf("[prehistoryBin->GetSignalToCherenkovVar(int) nSet=%d is too small\n", nSet);
    exit(-1);    
  }
  double s1=0.;
  double s2=0.;
  for (int iSet=0; iSet<nSet; iSet++) {
    double rat=GetAverageSignal(iSet, nSet)/GetAverageCherenkov(iSet, nSet);
    s1+=rat;
    s2+=rat*rat;
  }
  s1/=(double)nSet;
  s2/=(double)nSet;
  s2-=s1*s1;
  if (s2<0.) s2=0.;
  else s2=sqrt(s2/(nSet-1.0));
  return s2;
}

//-------------------------------------------------------------
double GetSignalToCherenkovErr(int nSet, double x0, double x1) { // split data into nSet groups and vompute variance;
                                                                 // in the (x0, x1) interval of Xbalance
  if (nSet<2) {
    printf("[prehistoryBin->GetSignalToCherenkovVar(int) nSet=%d is too small\n", nSet);
    exit(-1);    
  }
  double s1=0.;
  double s2=0.;
  for (int iSet=0; iSet<nSet; iSet++) {
    double rat=GetAverageSignal(iSet, nSet, x0, x1)/GetAverageCherenkov(iSet, nSet, x0, x1);
    s1+=rat;
    s2+=rat*rat;
  }
  s1/=(double)nSet;
  s2/=(double)nSet;
  s2-=s1*s1;
  if (s2<0.) s2=0.;
  else s2=sqrt(s2/(nSet-1.0));
  return s2;
}



//-------------------------------------------------------------
double GetSigmaSignal() {
  double s1=0.;
  double s2=0.;
  for (unsigned int i=0; i<event.size(); i++) {
    s1+=signalMod[i];
    s2+=signalMod[i]*signalMod[i];
  }
  if (event.size()>1) {
    s1/=double(event.size());
    s2/=double(event.size());
    s2-=s1*s1;
    s2=sqrt(s2);
  } else {
    s2=0.;
  }
  return s2;
}

//-------------------------------------------------------------
double GetPedMeanCherenkov() {
  return currentCherPed;
/*
  if (ped0>0.) {
    return chrPed1/ped0;
  } else {
//    printf("****************************************\n");
//    printf("  [prehistoryBin->GetPedMeanCherenkov]  \n");
//    printf("   alert - no pedestals for cherenkov   \n");
//    printf("****************************************\n");
//    printf(" ID=%d option=%d integralTime=%f channel=%d module=%d rateLow=%f rateHigh=%f\n",
//            myID, integralOption, integrationTime, channel, module, rateLow, rateHigh);
//    exit(-1);
  }
  return 0.;
*/
}

//-------------------------------------------------------------
double GetPedMeanSignal() {
  return currentModPed;
/*
  if (ped0>0.) {
    return sigPed1/ped0;
  } else {
//    printf("****************************************\n");
//    printf("  [prehistoryBin->GetPedMeanSignal]     \n");
//    printf("   alert - no pedestals for cherenkov   \n");
//    printf("****************************************\n");
//    printf(" ID=%d option=%d integralTime=%f channel=%d module=%d rateLow=%e rateHigh=%e\n",
//            myID, integralOption, integrationTime, channel, module, rateLow, rateHigh);
//    exit(-1);
  }
  return 0.;
*/
}

//-------------------------------------------------------------
void ComputeAverages() {
  if (sumsReady) {
    printf("[prehistoryBin->ComputeAverages] sorry, pal, u 4got to cear sums!\n");
    exit(-1);
  }
  for (unsigned int i=0; i<event.size(); i++) {
    sx1+=cherenkov[i];
    sx2+=cherenkov[i]*cherenkov[i];
    sy1+=signalMod[i];
    sy2+=signalMod[i]*signalMod[i];
    sxy+=signalMod[i]*cherenkov[i];
  }
  if (event.size()>1) {
    sx1/=double(event.size());
    sx2/=double(event.size());
    sx2-=sx1*sx1;
    sx2=sqrt(sx2);
    sy1/=double(event.size());
    sy2/=double(event.size());
    sy2-=sy1*sy1;
    sy2=sqrt(sy2);
    sxy/=double(event.size());
  } else {
    printf("********************************************\n");
    printf("   alert - too few events for correlation   \n");
    printf("           and slopes                       \n");
    printf("********************************************\n");
  }
  sumsReady=true;
}


//-------------------------------------------------------------
double GetCorrelation() {
  double corr=0.;
  if (!sumsReady)  ComputeAverages();
  if (event.size()>1) {
    corr=(sxy-sx1*sy1)/sx2/sy2;
  } 
  return corr;
}

//-------------------------------------------------------------
double GetSlope2() {
  double result=0.;
  if (event.size()<3) {
    printf("********************************************\n");
    printf("   alert - too few events for slope         \n");
    printf("********************************************\n");
    return result;
  }
  if (!sumsReady) ComputeAverages();
  double x=sx2*sx2-sy2*sy2;
  double y=2.0*(sxy-sx1*sy1);
  double alpha=0.5*atan2(y, x);
  result=tan(alpha);
  return result;
}

double SmartFit00() {
  double alphaMIN=0.;
  double alphaMAX=asin(1.0);
//  double angle=0.;
  double dist1, dist2, distS;
//  double distCut;
  double da=(alphaMAX-alphaMIN)/180.;
  double distMIN=1.0e10;
  double sw00=(double)cherenkov.size();
  double aBest=0.0;
  double sBest;
  double slopeSmart0;
  
//  printf("alphaMIN=%f, alphaMAX=%.f\n", alphaMIN, alphaMAX);
  for (double a=alphaMIN; a<=alphaMAX; a+=da) {
    dist2=0.;
    dist1=0.;
    for (unsigned int i=0; i<cherenkov.size(); i++) {
      double d=distToLine00(a, cherenkov[i], signalModCalib[i]);
      dist2+=d*d;
      dist1+=d;
    }
    dist1/=sw00;
    dist2/=sw00;
    distS=dist2-dist1*dist1;
    if (distMIN>sqrt(dist2)) {
      distMIN=sqrt(dist2);
      aBest=a;
      sBest=sqrt(distS);
    }
//    printf("\t\ta=%f\t dist2=%f distMIN=%f\n", a, dist2, distMIN); 
  }
  slopeSmart0=sin(aBest)/cos(aBest);
  printf("best initial angle = %.4frad, slope=%.4f\n", aBest, slopeSmart0);


  return slopeSmart0/modCalibration;
}

double bestToLine00(double angle, double x, double y) {
// best parameter t for line {t*cos(angle), t*sin(angle)} and point (x,y)
  return x*cos(angle)+y*sin(angle);
}

// "signed" distance from point to line!
double distToLine00(double angle, double x, double y) { 
  return x*sin(angle)-y*cos(angle);
}
//=================================================================
};


