// ---- 2012 online with RF event display ----//
int DisplayAllCounters2012(TModule* Setup, ProcessingStatus* ProcStatus, CherenkovBuffer* CherBuff) {

  bool Dbg=false;
  char TitleC[200];
  //char titleC[200];
  char NamecC[200];
  int Rep=0;

  EventHeader *EH = ProcStatus->GetEventHeader();
  RunHeader   *RH = ProcStatus->GetRunHeader();
  BurstHeader *BH = ProcStatus->GetBurstHeader();
  int dWidth  = ProcStatus->GetDisplayWidth();
  //int dHeight = ProcStatus->GetDisplayHeight();
  int Nbunches = ProcStatus->GetNbunches();

  TLine  * displayLine  = new TLine(); 
  TLatex * displayLatex = new TLatex();


  if (dWidth>1800) dWidth=1800;
  Dbg=false;
  //Dbg=true;


  sprintf(TitleC, "Run #%d, Burst#%d, Event#%d(%d), T(10MHz)=%.1fms Phase=%.1fns RFoffset=%d iMax=%d Type=%02d SignalFactor=%.2f", 
	  EH->run, EH->burst, EH->event, EH->burstEvent, EH->scaler10mhz/10000., EH->GetTimeGlobal(),
          (EH->scaler6mhz)%30, EH->GetFirstBunch(), EH->pattern[0], EH->SignalFactor); 
  sprintf(NamecC, "CountersRun%d_Burst%d_Event%d_%d", EH->run, EH->burst, EH->event, EH->burstEvent); 

  TH1D    *dummy = new TH1D("dummy_display_new", "", 2500, -150., 150.+25.*EH->nSlice);
  dummy->GetXaxis()->SetTitle("Time, ns");

  EventCanvas->SetTitle(TitleC);
  EventCanvas->SetName(NamecC);
  EventCanvas->Clear();
  EventCanvas->Divide(1, 5, 0.002, 0.002);
  EventCanvas->Update();
  EventCanvas->Show();

  
  dummy->SetTitleSize(0.04);
  dummy->SetStats(0);
  dummy->GetYaxis()->SetTitleSize(.1);
  dummy->GetYaxis()->SetTitleOffset(.4);
  dummy->GetYaxis()->SetLabelSize(.08);
  dummy->GetXaxis()->SetTitleSize(.1);
  dummy->GetXaxis()->SetTitleOffset(.75);
  dummy->GetXaxis()->SetLabelSize(.08);
  dummy->SetLineStyle(2);


  
// show scitillatorios in next division

  double scintMax=-1.0e10;
  double scintMin=1.0e10;
  int iGain=1;
  int iZone=1;
  double errFactor=2.0;

  for (int iMod=4; iMod<5; iMod++) {
//  for (int iMod=4; iMod<6; iMod++) {
    int nSlice=Setup[iMod].GetNslice();
    int iCh0=0;
    int iCh1=4;
    if (iMod==5) {
      iCh1=2; // shall be 3, but in test data is an extra fake 6MHz signal
      if (EH->run==852) iCh1=1;
    }
    
    for (int iCh=iCh0; iCh<iCh1; iCh++) {
      iGain=1;
      TChannel *Channel = Setup[iMod].GetChannel(iCh);
      TSignal *Signal;
      TSignal *SignalOther;
      TSignal *SignalShort0;
      TSignal *SignalShort1;
      if (Channel->HighGainOverflowDetected()) {
        iGain=0;
        printf("[DisplayAllCounters2012] %s HG overflow detected\n", Channel->GetName());
      }
      Signal       = Channel->GetSignalLong(iGain);
      SignalOther  = Channel->GetSignalLong(1-iGain);
      SignalShort0 = Channel->GetSignalShort(2*iGain);
      SignalShort1 = Channel->GetSignalShort(2*iGain+1);
      int nSliceShort=SignalShort0->GetNslice();
      
      scintMax=Signal->GetMax0();
      scintMin=Signal->GetMin0();

      if (scintMax<1.5*errFactor*Signal->GetEmptyEventNoise()) {
        scintMax=1.5*errFactor*Signal->GetEmptyEventNoise();
      }  
      if (scintMin>-1.5*errFactor*Signal->GetEmptyEventNoise()) {
        scintMin=-1.5*errFactor*Signal->GetEmptyEventNoise();
      }  
      EventCanvas->cd(iZone);
      displayLine->SetLineWidth(1); 
      gPad->SetBottomMargin(0.2);
      gPad->SetRightMargin(0.01);
      gPad->SetTopMargin(0.1);

      double dd=scintMax-scintMin;    
      dummy->GetYaxis()->SetRangeUser(scintMin-0.05*dd, scintMax+0.05*dd);
      if (iGain==0) dummy->GetYaxis()->SetTitle("Low Gain ADC");
      if (iGain==1) dummy->GetYaxis()->SetTitle("High Gain ADC");
      dummy->SetTitle(Channel->GetName());
      dummy->DrawCopy();

      displayLine->SetLineColor(kYellow);
      displayLine->SetLineWidth(30);
      displayLine->SetLineStyle(1);
      int nPedWindows=Setup[iMod].GetNPedWindows();
      for (int iw=0; iw<nPedWindows; iw++) {
        displayLine->DrawLine(Setup[iMod].GetPedWin0(iw), 0.0,
                              Setup[iMod].GetPedWin1(iw), 0.0);
      }

      displayLine->SetLineStyle(1);
      for (int irf=0; irf<50; irf++) {
        if (!Channel->GetClusterPMTbunchSET(irf)) continue;
        double Tbunch=ProcStatus->GetEventHeader()->GetTimeGlobal()+
               irf*ProcStatus->GetEventHeader()->GetRFperiod();
        int irf1=irf-1;
        int ib0=irf+ProcStatus->GetEventHeader()->GetFirstBunch();
        int ib1=irf1+ProcStatus->GetEventHeader()->GetFirstBunch();
        while (ib0<0)  ib0+=30;
        while (ib0>29) ib0-=30;
        while (ib1<0)  ib1+=30;
        while (ib1>29) ib1-=30;
        //printf("[DisplayAllCounters2012] irf=%2d ib0=%2d(%d) ib1=%2d(%d)\n", 
        //       irf, ib0, ProcStatus->GetBunchAction(ib0), 
        //            ib1, ProcStatus->GetBunchAction(ib1));
        if (ProcStatus->GetBunchAction(ib0)!=1) continue; 
        if (ProcStatus->GetBunchAction(ib1)==1) continue; 
        if (Tbunch<Channel->GetT0pmtRec()) continue;
        
        double t0=Signal->GetTsig0(irf);
        double t1=Signal->GetTsig1(irf);
        double t0i=Signal->GetTint0(irf);
        double t1i=Signal->GetTint1(irf);
        printf("[DisplayAllCounters2012] t0=%6.1f  t1=%6.1f  t0i=%6.1f  t1i=%6.1f \n", t0, t1, t0i, t1i);

        if (t0 != -1000. && t1 !=-1000.) {
          displayLine->SetLineColor(kGreen);
          displayLine->SetLineWidth(5);
          displayLine->DrawLine(t0,  0., t1,  0.);
          displayLine->SetLineColor(kRed);
          displayLine->SetLineWidth(3);
          displayLine->DrawLine(t0i, 0., t1i, 0.);
        }
      }

      displayLine->SetLineWidth(1); /// RF cycles (bunch start)
      displayLine->SetLineStyle(2);
      displayLine->SetLineColor(kGray);
      for (double tc=EH->GetTimeGlobal(); tc<dummy->GetXaxis()->GetXmax(); tc+=EH->GetRFperiod()) {
        displayLine->DrawLine(tc, scintMin,
                              tc, scintMax);
      }
      
      
      
      displayLine->SetLineColor(kBlack);
      displayLine->SetLineWidth(2);
      displayLine->SetLineStyle(1);
      for (int is=1; is<nSlice; is++) {
        displayLine->DrawLine(Signal->GetTime(is-1), Signal->GetAmp1(is-1), 
                              Signal->GetTime(is),   Signal->GetAmp1(is));
      }

      displayLine->SetLineWidth(1);
      displayLine->SetLineStyle(2);
      displayLine->SetLineColor(kBlue);
      displayLine->DrawLine(Signal->GetTime(0),         errFactor*Signal->GetEmptyEventNoise(),
                            Signal->GetTime(nSlice-1),  errFactor*Signal->GetEmptyEventNoise());
      displayLine->DrawLine(Signal->GetTime(0),        -errFactor*Signal->GetEmptyEventNoise(),
                            Signal->GetTime(nSlice-1), -errFactor*Signal->GetEmptyEventNoise());
      



      displayLine->SetLineColor(kRed);
      displayLine->SetLineWidth(2);
      displayLine->SetLineStyle(1);
      for (int is=1; is<Signal->GetNS2013(); is++) {
//        if (Signal->GetTime2013(is-1) > Setup[iMod].GetPedWin0(0)) {
        if (Signal->GetTime2013(is-1) > Channel->GetT0pmtRec()) {
          displayLine->DrawLine(Signal->GetTime2013(is-1), Signal->GetAmp2013rec(is-1)/1.5, 
                                Signal->GetTime2013(is),   Signal->GetAmp2013rec(is)/1.5);
        } 
        if (Signal->GetTime2013(is) >  Channel->GetT1pmtRec()) {
          break;
        }
      }

      double scale=0.1;
      if (iGain==1) scale=10.;

      displayLine->SetLineColor(kGray);
      displayLine->SetLineWidth(2);
      displayLine->SetLineStyle(1);
      for (int is=1; is<Signal->GetNS2013(); is++) {
//        if (Signal->GetTime2013(is-1) > Setup[iMod].GetPedWin0(0)) {
        if (Signal->GetTime2013(is-1) > Channel->GetT0pmtRec()) {
          displayLine->DrawLine(SignalOther->GetTime2013(is-1), scale*SignalOther->GetAmp2013rec(is-1)/1.5, 
                                SignalOther->GetTime2013(is),   scale*SignalOther->GetAmp2013rec(is)/1.5);
        } 
        if (SignalOther->GetTime2013(is) >  Channel->GetT1pmtRec()) {
          break;
        }
      }


      int nClusters=Signal->GetNclusters();
      displayLine->SetLineColor(kRed);
      displayLine->SetLineWidth(2);
      displayLine->SetLineStyle(2);
      displayLatex -> SetTextColor(kBlack);
      displayLatex -> SetTextSize(0.06);
      displayLatex -> SetTextAlign(12);
      
      for (int i=0; i<nClusters; i++) {
        displayLine->DrawLine(Signal->GetTcluster(i), scintMin, Signal->GetTcluster(i), scintMax);
        if (i%2==0) {
          sprintf(NamecC, "A=%.1f", 0.1*Signal->GetAcluster(i));
          displayLatex->DrawLatex(Signal->GetTcluster(i)+10., scintMax-0.05*dd, NamecC);
          sprintf(NamecC, "W=%.1f", Signal->GetWcluster(i));
          displayLatex->DrawLatex(Signal->GetTcluster(i)+10., scintMax-0.15*dd, NamecC);
          sprintf(NamecC, "S=%.1f", Signal->GetScluster(i));
          displayLatex->DrawLatex(Signal->GetTcluster(i)+10., scintMax-0.25*dd, NamecC);
//          sprintf(NamecC, "A1=%.1f", Signal->ComputePMTcluster(Signal->GetTcluster(i)));
//          displayLatex->DrawLatex(Signal->GetTcluster(i)+10., scintMax-0.35*dd, NamecC);
        } else {
          sprintf(NamecC, "A=%.1f", 0.1*Signal->GetAcluster(i));
          displayLatex->DrawLatex(Signal->GetTcluster(i)+10., scintMin+0.35*dd, NamecC);
          sprintf(NamecC, "W=%.1f", Signal->GetWcluster(i));
          displayLatex->DrawLatex(Signal->GetTcluster(i)+10., scintMin+0.25*dd, NamecC);
          sprintf(NamecC, "S=%.1f", Signal->GetScluster(i));
          displayLatex->DrawLatex(Signal->GetTcluster(i)+10., scintMin+0.15*dd, NamecC);
//          sprintf(NamecC, "A1=%.1f", Signal->ComputePMTcluster(Signal->GetTcluster(i)));
//          displayLatex->DrawLatex(Signal->GetTcluster(i)+10., scintMin+0.05*dd, NamecC);
        }
      }
      double Tbunch=0;
      displayLatex -> SetTextColor(kBlue);
      for (int ib0=0; Tbunch<Channel->GetT1pmtRec();ib0++) {
        Tbunch=ProcStatus->GetEventHeader()->GetTimeGlobal()+
           ib0*ProcStatus->GetEventHeader()->GetRFperiod();
        int ib=ib0+ProcStatus->GetEventHeader()->GetFirstBunch(); // event offset
        int ib1=ib-1;
        while (ib<0)  ib+=30;
        while (ib>29) ib-=30;
        while (ib1<0)  ib1+=30;
        while (ib1>29) ib1-=30;
        if (!Channel->GetClusterPMTbunchSET(ib0)) {
          //printf("[DisplayAllCounters2012] bunch#%d not set\n", ib0);
          continue;
        }
        if (ProcStatus->GetBunchAction(ib)!=1) continue; 
        if (ProcStatus->GetBunchAction(ib1)==1) continue; 
        if (Tbunch<Channel->GetT0pmtRec()) continue;
        sprintf(NamecC, "%s",  Signal->GetName());
        displayLatex->DrawLatex(Tbunch+25., scintMax-0.05*dd, NamecC);
        sprintf(NamecC, "A_{high}=%.1f",  Channel->GetClusterPMTbunchAmp2(ib0, 1));
        displayLatex->DrawLatex(Tbunch+25., scintMax-0.15*dd, NamecC);
        sprintf(NamecC, " A_{low}=%.1f",  Channel->GetClusterPMTbunchAmp2(ib0, 0));
        displayLatex->DrawLatex(Tbunch+25., scintMax-0.25*dd, NamecC);
        sprintf(NamecC, "MIPS=%.1f",Channel->GetClusterPMTbunchAmpTracks(ib0));
        displayLatex->DrawLatex(Tbunch+25., scintMax-0.35*dd, NamecC);

      }

      iZone++;
    }
  }




/*
  displayLine->SetLineWidth(3);
  displayLine->SetLineStyle(1);
  displayLine->SetLineColor(kGreen);
  for (int bunch=0; bunch<Nbunches; bunch++) {
    double Tbunch=ProcStatus->GetBunchTime(bunch);
    displayLine->DrawLine(Tbunch, scintMin, Tbunch, scintMax);
  }
*/



  EventCanvas->cd(iZone);
  displayLine->SetLineWidth(1); 
  gPad->SetBottomMargin(0.2);
  gPad->SetRightMargin(0.01);
  gPad->SetTopMargin(0.1);
// show RF
  TModule *Module = &Setup[5]; // ex-monitor
  TChannel *Channel = Module->GetChannel(2); // ex-M3, now RF
  TSignal *Signal;
  int nSlice=Module->GetNslice();
  Signal=Channel->GetSignalLong(0);
  dummy->GetYaxis()->SetTitle("Low Gain ADC");

  double amax0=Signal->GetMax0();
  double amin0=Signal->GetMin0();
  double dd1=amax0-amin0;    

  dummy->GetYaxis()->SetRangeUser(amin0-0.05*dd1, amax0+0.05*dd1);
  dummy->DrawCopy();
  myLatex->SetTextAlign(22);
  displayLine->SetLineStyle(1);
  displayLine->SetLineColor(kRed);
  displayLine->SetLineWidth(2);
  displayLine->DrawLine(ProcStatus->GetEventHeader()->GetTimeGlobal(), amin0,
                          ProcStatus->GetEventHeader()->GetTimeGlobal(), amax0);
// show sin wave
  displayLine->SetLineColor(kRed);
  displayLine->SetLineWidth(1);
  double dt=3.;
  for (double t0=Signal->GetTime(0); t0<Signal->GetTime(nSlice-1); t0+=dt) {
    double t1=t0+dt;
    displayLine->DrawLine(t0, EH->GetSinWave(t0), 
                          t1, EH->GetSinWave(t1));      
  }

// show residuals sin wave

  displayLine->SetLineColor(kGray);
  displayLine->SetLineWidth(1);
  for (int is=1; is<nSlice; is++) {
    double t1=Signal->GetTime(is);
    double t0=Signal->GetTime(is-1);      
    displayLine->DrawLine(t0, 10.*(Signal->GetAmp1(is-1)-EH->GetSinWave(t0)), 
                          t1, 10.*(Signal->GetAmp1(is)-EH->GetSinWave(t1)));      
  }

// show samples
  displayLine->SetLineWidth(2);
  displayLine->SetLineColor(kBlack);
  for (int is=0; is<nSlice; is++) {
    displayLine->DrawLine(Signal->GetTime(is), Signal->GetAmp1(is)-0.03*dd1, 
                          Signal->GetTime(is), Signal->GetAmp1(is)+0.03*dd1);
    displayLine->DrawLine(Signal->GetTime(is)-12., Signal->GetAmp1(is), 
                          Signal->GetTime(is)+12., Signal->GetAmp1(is));
  }
 
 
  EventCanvas->Update();
  EventCanvas->Show();
  EventCanvas->Update();
  

  Rep=1;
  ProcStatus->SetReportMode(Rep);

  dummy->Delete();
  displayLine->Delete();
  displayLatex->Delete();

  return Rep; 
}


