// ---- 2012 online with RF event display ----//
int DisplayForMaslennikov(TModule* Setup, ProcessingStatus* ProcStatus, CherenkovBuffer* CherBuff) {

  bool Dbg=false;
  char TitleC[200];
  //char titleC[200];
  char NamecC[200];
  int Rep=0;

  EventHeader *EH = ProcStatus->GetEventHeader();
  RunHeader   *RH = ProcStatus->GetRunHeader();
  BurstHeader *BH = ProcStatus->GetBurstHeader();
  int dWidth  = ProcStatus->GetDisplayWidth();
  //int dHeight = ProcStatus->GetDisplayHeight();
  int Nbunches = ProcStatus->GetNbunches();

  TLine  * displayLine  = new TLine(); 
  TLatex * displayLatex = new TLatex();


  if (dWidth>1800) dWidth=1800;
  Dbg=false;
  //Dbg=true;


  sprintf(TitleC, "Special Run #%d, Burst#%d, Event#%d(%d), T(10MHz)=%.1fms Phase=%.1fns RFoffset=%d iMax=%d Type=%02d SignalFactor=%.2f", 
	  EH->run, EH->burst, EH->event, EH->burstEvent, EH->scaler10mhz/10000., EH->GetTimeGlobal(),
          (EH->scaler6mhz)%30, EH->GetFirstBunch(), EH->pattern[0], EH->SignalFactor); 
  sprintf(NamecC, "SpecialDisplay"); 

  TH1D    *dummy = new TH1D("dummy_display_new", "", 2500, -150., 150.+25.*EH->nSlice);
  dummy->GetXaxis()->SetTitle("Time, ns");

  EventCanvasSpecial->SetTitle(TitleC);
  EventCanvasSpecial->SetName(NamecC);
  EventCanvasSpecial->Clear();
  EventCanvasSpecial->Divide(1, 2, 0.002, 0.002);
  EventCanvasSpecial->Update();
  EventCanvasSpecial->Show();

  //return 0;
  //if (EH->isClone) EventCanvas->SetFillColor(2);

  dummy->SetTitleSize(0.04);
  dummy->SetStats(0);
  dummy->GetYaxis()->SetTitleSize(.1);
  dummy->GetYaxis()->SetTitleOffset(.4);
  dummy->GetYaxis()->SetLabelSize(.08);
  dummy->GetXaxis()->SetTitleSize(.1);
  dummy->GetXaxis()->SetTitleOffset(.75);
  dummy->GetXaxis()->SetLabelSize(.08);
  dummy->SetLineStyle(2);

  int iModX=RH->AntennaModule;
  TModule *ModuleX = &Setup[iModX];
  printf("[DisplayForMaslennikov] In beam:%d, Coherent Source = %s\n", RH->FCalInBeam, ModuleX->GetName()); 
  int iZone=1;
  for (int iMod=1;iMod<5;iMod+=3) {
    TModule *Module = &Setup[iMod];
    sprintf(NamecC, Module->GetName()); 
    EventCanvasSpecial->cd(iZone++);
    displayLine->SetLineWidth(1); 
    dummy->GetYaxis()->SetTitle("High Gain ADC");
    gPad->SetBottomMargin(0.2);
    gPad->SetRightMargin(0.01);
    gPad->SetTopMargin(0.02);

    int nSlice=Module->GetNslice();
    int myGain=1;
    int fGain=1.0;
    
    if (iMod<4) {// display modules sums //
      double amin=Module->GetSumMin();
      double amax=Module->GetSumMax();

      myGain=1;
      if (Module->CheckHGoverflow()) {
        myGain=0;
        fGain=10.5;
      }

      if (Module->GetSumFinalMin(myGain)<amin) amin=Module->GetSumFinalMin(myGain);   
      if (Module->GetSumFinalMax(myGain)>amax) amax=Module->GetSumFinalMax(myGain);   
      printf("[DisplayForMaslennikov] final signal range: %.1f...%.1f\n", Module->GetSumFinalMin(myGain), Module->GetSumFinalMax(myGain)); 
      if (1.15*amax<5.0)  amax=5.0;
      if (1.05*amin>-5.0) amin=-5.0;

      //dummy->SetTitle(Module->GetName());
      dummy->GetYaxis()->SetRangeUser(1.15*amin, 1.15*amax);
      dummy->DrawCopy();
      int nPedWindows=ProcStatus->GetPedWindows();

/*
      displayLine->SetLineWidth(16); /// preliminary pedestal windows from processing status
      displayLine->SetLineStyle(1);
      displayLine->SetLineColor(kYellow);
      for (int iw=0; iw<nPedWindows; iw++) {
        displayLine->DrawLine(ProcStatus->GetPedWindow0(iw), 0.0,
                              ProcStatus->GetPedWindow1(iw), 0.0);
      }
*/
      displayLine->SetLineWidth(32); /// pedestal windows from modules 
      displayLine->SetLineStyle(1);
      displayLine->SetLineColor(kYellow);
      nPedWindows=Setup[iMod].GetNPedWindows();
      for (int iw=0; iw<nPedWindows; iw++) {
        //if(!Setup[iMod].GetPedWinX(iw)) continue;
        displayLine->DrawLine(Setup[iMod].GetPedWin0(iw), 0.0,
                              Setup[iMod].GetPedWin1(iw), 0.0);
      }

      displayLine->SetLineWidth(1); /// RF cycles (bunch start)
      displayLine->SetLineStyle(2);
      displayLine->SetLineColor(kGray);
      for (double tc=EH->GetTimeGlobal(); tc<dummy->GetXaxis()->GetXmax(); tc+=EH->GetRFperiod()) {
        displayLine->DrawLine(tc, amin,
                              tc, amax);
      }


      displayLine->SetLineWidth(2); // redraw the "0" //
      displayLine->SetLineColor(kBlack);
      displayLine->SetLineStyle(2);
      displayLine->DrawLine(dummy->GetXaxis()->GetXmin(), 0., dummy->GetXaxis()->GetXmax(), 0.); 

      displayLine->SetLineWidth(2);
      displayLine->SetLineStyle(1);
/*
      displayLine->SetLineWidth(2); // pure high gain sum //
      displayLine->SetLineColor(kRed);
      for (int is=1; is<nSlice; is++) {
        displayLine->DrawLine(Module->GetTime(is-1), Module->GetSumSampleH(is-1), 
                              Module->GetTime(is),   Module->GetSumSampleH(is));
      }
*/      
      
      
      if (Module->CheckLGoverflow()) {
        displayLine->SetLineColor(kRed);
      } else {
        displayLine->SetLineColor(kGray);
      }
      if (Module->GetCold()) displayLine->SetLineColor(kBlue);

      TSignal* SumSignal=Module->GetChannel(4)->GetSignalLong(myGain);
      //printf("[DisplayAllPulses2012] plotting %s, LGoverflow=%d\n", 
      //        SumSignal->GetName(), Module->CheckLGoverflow());
 
      displayLine->SetLineWidth(4);
      for (int is=1; is<nSlice; is++) {
        displayLine->DrawLine(Module->GetTime(is-1), SumSignal->GetAmp1(is-1), // show with noise suppressed
                              Module->GetTime(is),   SumSignal->GetAmp1(is));
      }
   
      //continue;

      int is0=SumSignal->GetFirstSample(0);
      int is1=SumSignal->GetLastSample(nPedWindows-1);
      if (is0==-1 || is1==-1) continue; // do not show FFT correction if it is not there

      //if (iMod!=iModX && EH->notEmpty) {      
      if (iMod!=iModX) {      
/*
        displayLine->SetLineWidth(1); // baseline fits at Pedestal windows //
        displayLine->SetLineColor(kGreen);
        for (int iw=0; iw<nPedWindows; iw++) {
          if(!Module->GetPedWinX(iw)) continue;
          is0=SumSignal->GetFirstSample(iw);
          for (int i=1; is0+i<nSlice; i++) {
            displayLine->DrawLine(Module->GetTime(is0+i-1), SumSignal->GetTailAmplitude(iw, i-1), 
                                  Module->GetTime(is0+i),   SumSignal->GetTailAmplitude(iw, i));
          }
        }

        displayLine->SetLineWidth(1); // 'tailed' signal //
        displayLine->SetLineColor(kBlue);
        is0=SumSignal->GetFirstSample(0);
        is1=SumSignal->GetLastSample(nPedWindows-1);
        for (int i=1; is0+i<nSlice; i++) {
          displayLine->DrawLine(Module->GetTime(is0+i-1), SumSignal->GetTailedSignal(i-1), 
                                Module->GetTime(is0+i),   SumSignal->GetTailedSignal(i));
        }
*/
/*
        displayLine->SetLineWidth(2); // 'no baseline correction' signal //
        displayLine->SetLineColor(kRed);
        for (int i=1; is0+i<is1; i++) {
          displayLine->DrawLine(Module->GetTime(is0+i-1), SumSignal->GetFinalSignal(i-1)+SumSignal->GetPolyCorrection(i-1), 
                                Module->GetTime(is0+i),   SumSignal->GetFinalSignal(i)+SumSignal->GetPolyCorrection(i));
        }

        displayLine->SetLineWidth(1); // 'baseline' correction //
        displayLine->SetLineColor(kBlue);
        for (int i=1; is0+i<is1; i++) {
          displayLine->DrawLine(Module->GetTime(is0+i-1), SumSignal->GetPolyCorrection(i-1), 
                                Module->GetTime(is0+i),   SumSignal->GetPolyCorrection(i));
        }
*/
        displayLine->SetLineWidth(3); // 'final' signal //
        displayLine->SetLineColor(kBlack);
        for (int i=1; is0+i<is1; i++) {
          displayLine->DrawLine(Module->GetTime(is0+i-1), SumSignal->GetFinalSignal(i-1), 
                                Module->GetTime(is0+i),   SumSignal->GetFinalSignal(i));
        }
      }

/*      
      displayLine->SetLineWidth(2); // 'raw' signal //
      displayLine->SetLineColor(kGreen);
      for (int i=1; i<nSlice; i++) {
        double a0=0.;
        double a1=0.;
        for (int iCh=0; iCh<4; iCh++) {
          a0+=Module->GetChannel(iCh)->GetSignalLong(myGain)->GetAmp2(i-1)*fGain;
          a1+=Module->GetChannel(iCh)->GetSignalLong(myGain)->GetAmp2(i)*fGain;
        }
        displayLine->DrawLine(Module->GetTime(i-1), a0, 
                              Module->GetTime(i),   a1);
      }

*/
      displayLatex -> SetTextColor(kBlack);
      displayLatex -> SetTextSize(0.08);
      displayLatex -> SetTextAlign(13);

      displayLatex -> DrawLatex(0., amax, NamecC); 


      if (EH->notEmpty) {


/*
        displayLine->SetLineWidth(2);
        displayLine->SetLineColor(kRed);
 
        for (int bunch=0; bunch<Nbunches; bunch++) {
          int i0=Module->GetSampleX0(bunch);
          int i1=Module->GetSampleX1(bunch);
          for (int i=i0; i<=i1; i++) {
            displayLine->DrawLine(Module->GetTime(i-1), Module->GetSumSampleX(i-1), 
                                  Module->GetTime(i),   Module->GetSumSampleX(i));
          }
        }
*/
        for (int bunch=0; bunch<Nbunches; bunch++) {
          double Tbunch=ProcStatus->GetBunchTime(bunch);
          double RateADC = Setup[4].GetChannel(3)->GetResponseADC(bunch);
          double Rate =   RateADC*CherBuff->GetGainCorrection();
          double Sum =    Setup[iMod].GetMySumRateCal(bunch);
          double SumRAW = Setup[iMod].GetMySumRate(bunch);
          printf("[DisplayForMaslennikov] %10s bunch#%d Rate=%.2ep rawADC=%7.1f SumOld=%7.1f SumNew=%7.1f SumNewCal=%7.1f\n", 
               Setup[iMod].GetName(),       bunch, Rate, SumRAW, Sum, 
               Setup[iMod].GetAnear(bunch), Setup[iMod].GetAnearCal(bunch));
          displayLine->SetLineWidth(2); 
          displayLine->SetLineColor(kBlack);
          displayLine->SetLineStyle(2);
          displayLine->DrawLine(Tbunch, amax, Tbunch, amin);
        }    
      }

    } else if (iMod==4) { // cherenkov only

      TChannel *Channel = Module->GetChannel(3);
      TSignal *Signal;
      int iGain=0;
      int iColor = kBlack;
      if (Channel->HighGainOverflowDetected()) {
        Signal=Channel->GetSignalLong(0);
        dummy->GetYaxis()->SetTitle("Low Gain, ADC");
      } else {
        Signal=Channel->GetSignalLong(1);
        dummy->GetYaxis()->SetTitle("High Gain, ADC");
        iGain=1;
      }
      if (Channel->LowGainOverflowDetected()) {
        displayLine->SetLineColor(kRed);
        iColor = kRed;
      }
      double amax0=Signal->GetMax0();
      double amin0=Signal->GetMin0();
      double dd=amax0-amin0;    
      dummy->GetYaxis()->SetRangeUser(amin0-0.05*dd, amax0+0.05*dd);
      dummy->DrawCopy();
      
      displayLine->SetLineWidth(32); /// pedestal windows from modules 
      displayLine->SetLineStyle(1);
      displayLine->SetLineColor(kYellow);
      int nPedWindows=Setup[iMod].GetNPedWindows();
      for (int iw=0; iw<nPedWindows; iw++) {
        if(!Setup[iMod].GetPedWinX(iw)) continue;
        displayLine->DrawLine(Setup[iMod].GetPedWin0(iw), 0.0,
                              Setup[iMod].GetPedWin1(iw), 0.0);
      }

      displayLine->SetLineWidth(1); /// RF cycles (bunch start)
      displayLine->SetLineStyle(2);
      displayLine->SetLineColor(kGray);
      for (double tc=EH->GetTimeGlobal(); tc<dummy->GetXaxis()->GetXmax(); tc+=EH->GetRFperiod()) {
        displayLine->DrawLine(tc, amin0, tc, amax0);
      }

      displayLine->SetLineWidth(2); // pure high gain sum //
      displayLine->SetLineStyle(1);

      displayLatex -> SetTextColor(kBlack);
      displayLatex -> SetTextSize(0.08);
      displayLatex -> SetTextAlign(13);
      sprintf(NamecC, "Cherenkov");
      displayLatex -> DrawLatex(0., amax0, NamecC); 

      displayLine->SetLineStyle(1);

      displayLine->SetLineColor(kRed);
      displayLine->SetLineWidth(2);
      displayLine->SetLineStyle(1);
      for (int is=1; is<Signal->GetNS2013(); is++) {
//        if (Signal->GetTime2013(is-1) > Setup[iMod].GetPedWin0(0)) {
        if (Signal->GetTime2013(is-1) > Channel->GetT0pmtRec()) {
          displayLine->DrawLine(Signal->GetTime2013(is-1), Signal->GetAmp2013rec(is-1)/1.5, 
                                Signal->GetTime2013(is),   Signal->GetAmp2013rec(is)/1.5);
        } 
        if (Signal->GetTime2013(is) >  Channel->GetT1pmtRec()) {
          break;
        }
      }

      displayLine->SetLineColor(kBlack);
      displayLine->SetLineWidth(4);
      for (int is=1; is<nSlice; is++) {
        displayLine->DrawLine(Signal->GetTime(is-1), Signal->GetAmp1(is-1), 
                              Signal->GetTime(is),   Signal->GetAmp1(is));
      }
/*
      displayLine->SetLineWidth(2); // 'raw' signal //
      displayLine->SetLineColor(kGray);
      for (int is=1; is<nSlice; is++) {
        displayLine->DrawLine(Signal->GetTime(is-1), Signal->GetAmp2(is-1), 
                              Signal->GetTime(is),   Signal->GetAmp2(is));
      }
*/
/*
      double dT=5.;
      for (int iDelay=0; iDelay<2; iDelay++) {
        displayLine->SetLineColor(kBlack+iDelay+1);
        displayLine->SetLineWidth(1);
        TSignal *SignalShort;
        SignalShort = Channel->GetSignalShort(2*iGain+iDelay);
        for (double t=SignalShort->GetTime(0)-5.; t<SignalShort->GetTime(nSlice/2-1)+5.; t+=dT) {
          displayLine->DrawLine(t-dT,  SignalShort->getSinWave(t-dT),
                                t,     SignalShort->getSinWave(t));
        }
        for (int i=0; i<nSlice/2; i++) {
          double a=SignalShort->GetAmp1(i)-SignalShort->getSinWave(SignalShort->GetTime(i));
          SignalShort->SetAmp1(i, a);
          Signal->SetAmp1(i*2+iDelay, a);
        }
      }

      displayLine->SetLineColor(kBlack);
      displayLine->SetLineWidth(2);
      for (int is=1; is<nSlice; is++) {
        displayLine->DrawLine(Signal->GetTime(is-1), Signal->GetAmp1(is-1), 
                              Signal->GetTime(is),   Signal->GetAmp1(is));
      }
*/

      printf("[main->DisplayAllPulses2012] GoodCherenkov=%d\n", BH->goodGatedCherenkov);
      if (BH->goodGatedCherenkov) {
        double cherScale=1.;
        if (iGain==0) cherScale=10.;
        displayLine->SetLineWidth(2);
        displayLine->SetLineColor(kGray);
        displayLatex->SetTextColor(kRed);
        double tRF=EH->GetRFperiod();
        double t00=EH->GetTimeGlobal();
        double s0=0.;
        double s1=0.;
        double s2=0.;
        for (int i=0; i<39; i++) {
          double t0=t00+i*tRF;
          double t1=t00+i*tRF+tRF;
          int iGC=CherBuff->GetSampleIndex(i);
          int iAction=ProcStatus->GetBunchAction(iGC%30);
          sprintf(NamecC, "%d", iGC%30);
          if (iAction==1) {
            //displayLine->SetLineColor(kRed);
          } else if (iAction==2) {
            //displayLine->SetLineColor(6);
            double a=CherBuff->GetSampleAdjustD(i)/cherScale;
            s0+=1.0;
            s1+=a;
            s2+=a*a;
          } else {
            //displayLine->SetLineColor(kBlue);
          }
          //displayLatex->DrawLatex(0.5*(t0+t1), 1.05*CherBuff->GetSampleAdjustD(i)/cherScale, NamecC);
          displayLine->DrawLine(t0, CherBuff->GetSampleAdjustD(i)/cherScale, 
                                t1, CherBuff->GetSampleAdjustD(i)/cherScale);
          if (i<38) displayLine->DrawLine(t1, CherBuff->GetSampleAdjustD(i)/cherScale, 
                                          t1, CherBuff->GetSampleAdjustD(i+1)/cherScale); 
        }

        s1/=s0;
        s2/=s0;
        s2-=s1*s1;
        if (s2<0.0) s2=0.0;
        s2=sqrt(s2);
        sprintf(NamecC, "Gated noise=%.1f (ADC)", s2);
        //displayLatex -> DrawLatex(0., 0.8*amax0, NamecC); 
        sprintf(NamecC, "Gated noise/noise0=%.2f", s2/CherBuff->GetSpillNoise());
        //displayLatex -> DrawLatex(0., 0.6*amax0, NamecC); 
        

        displayLine->SetLineStyle(2);
        displayLine->SetLineColor(6);
        displayLine->SetLineWidth(1);
        displayLine->DrawLine(Signal->GetTime(0),         CherBuff->GetSpillNoise(),
                              Signal->GetTime(nSlice-1),  CherBuff->GetSpillNoise());
        displayLine->DrawLine(Signal->GetTime(0),        -CherBuff->GetSpillNoise(),
                              Signal->GetTime(nSlice-1), -CherBuff->GetSpillNoise());
      
      } // Gated Spill data is OK
      
/*
      for (int bunch=0; bunch<Nbunches; bunch++) {
        double Tbunch=ProcStatus->GetBunchTime(bunch);
        double RateADC=Setup[4].GetChannel(3)->GetResponseADC(bunch);
        if (RateADC>15.0) {
          double Rate=RateADC*CherBuff->GetGainCorrection();
          displayLatex -> SetTextColor(kBlack);
          displayLatex -> SetTextSize(0.08);
          displayLatex -> SetTextAlign(13);
          //sprintf(NamecC, "%5.2ep", Rate);
          //displayLatex -> DrawLatex(Tbunch+100., 0.8*amax0, NamecC); 
          displayLine->SetLineWidth(1); 
          displayLine->SetLineColor(kBlack);
          displayLine->SetLineStyle(2);
          displayLine->DrawLine(Tbunch, amax0, Tbunch, amin0);
        }
      }
*/    
    }
    EventCanvasSpecial->Update();
  }

  EventCanvasSpecial->Update();


  dummy->Delete();
  displayLine->Delete();
  displayLatex->Delete();

  return Rep; 
}


