double GlobalPhase(TModule* Setup, ProcessingStatus* ProcStatus) {
  // version 2009-06-26 takes into account new timing scheme (each signal has its own time[i] values)
  int NSLICE=ProcStatus->GetEventHeader()->nSlice;
  int nMax=0;
  double Tg=-1000.0;
  bool Dbg=false;


  if (Dbg || ProcStatus->GetReportMode()==10012) {
    printf("[GlobalPhase] Folding %d samples Report=%d...\n", 2*NSLICE, ProcStatus->GetReportMode());
  }
  double asum[512]={512*0.0}, wsum[512]={512*0.0};
  double GlobalMax=-1000;
  int   iGlobalMax=0;

  //for (int iMod=0;iMod<4;iMod++) { // timing by calorimeterzzzzz!!!!
  for (int iMod=0;iMod<5;iMod++) {  // Timing by A123 and Cherenkov
    TModule *Module = &Setup[iMod];
    //for (int iChan=3;iChan<4;iChan++) { // try cherenkov only!
    for (int iChan=0;iChan<4;iChan++) {
      TChannel *Channel = Module->GetChannel(iChan);
      if (Channel->GetStatus()) {
        //for (int iGain=0;iGain<1;iGain++) {
        for (int iGain=0;iGain<2;iGain++) {
          TSignal *Signal = Channel->GetSignalLong(iGain);
	  int iFold=0;
	  int i1=0;
          if (Dbg || ProcStatus->GetReportMode()==10012) printf("[GlobalPhase]  Module#%d Channel#%d Gain#%d----> \n", iMod, iChan, iGain);
          //bool OvrUnrd=false;

          double PrePed=0.0;
	  for (int i=0;i<NSLICE*2;i++) {
            double a=Signal->GetRaw(i);
            PrePed+=a;            
	  }
          PrePed/=(NSLICE*2); 
         
	  for (int i=0;i<NSLICE*2;i++) {
	    double t=Signal->GetTime(i);
            double t1=t-iFold*989.772;
            double a=-1000.;

	    if (t1>989.772) {
	      iFold++;
              t1-=989.772;
	    } else if (t1<0.0) {
              t1+=989.772;
	    }
            i1=int(t1/12.5);            
            
            if (Dbg || ProcStatus->GetReportMode()==10012) printf("[GlobalPhase]     i=%4d i1=%2d t=%6.1f t1=%6.1f ", i, i1, t, t1);
            a=Signal->GetRaw(i)-PrePed;

            if (a>0) a=sqrt(a);
	    else a=-sqrt(-a);
            
            asum[i1]+=a;
            wsum[i1]+=1.0;
            if (Dbg || ProcStatus->GetReportMode()==10012) {
              printf("[GlobalPhase]  a=%6.1f w=%2.0f <a>=%6.1f \n", a, wsum[i1], asum[i1]/wsum[i1]);
	    }
	    if (i1>nMax) nMax=i1;
	  }
        }
      }
    }
  }


  for (int i=0;i<nMax;i++) {
    if (wsum[i]>0.0) {
      asum[i]/=wsum[i];
      if (Dbg || ProcStatus->GetReportMode()==10012) printf("[GlobalPhase]  sample %5d sum=%6.1f w=%f\n", i, asum[i], wsum[i]);
    }
  }
  
  if (Dbg || ProcStatus->GetReportMode()==10012) printf("\n\n[GlobalPhase] searching for peak in %d samples\n", nMax);
  TSignal *Signal = Setup[4].GetChannel(3)->GetSignalLong(0); // signal to use for time marks
  double TimeMax=Signal->GetTime(nMax);

  double s2=0.0, s1=0.0;
  for (int isamp=0;isamp<nMax;isamp++) { // locate the peak of the Crude 'Pilot' Pulse (larges bunch)
    double s4=0.0, t4=0.0, s0=0.;

    s1+=asum[isamp];
    s2+=asum[isamp]*asum[isamp];

    int i1=isamp-3;
    int i2=isamp+3;
    if (Dbg || ProcStatus->GetReportMode()==10012) printf("[GlobalPhase]  sample#%2d range->(%2d, %2d) A=%6.1f ", isamp, i1, i2, asum[isamp]);
    for (int i=i1; i<i2; i++) {
      double a=0.0, t=0.0;
      s0+=1.;
      if (i<0) {
        a=asum[i+nMax];
        t=Signal->GetTime(i+nMax) - TimeMax;
      } else if (i>nMax-1) {
        a=asum[i-nMax];
        t=Signal->GetTime(i-nMax) + TimeMax;
      } else {
        a=asum[i];
        t=Signal->GetTime(i);
      }
      s4+=a;
      t4+=t*a;
    }
    t4/=s4;
    s4/=s0;
    if (Dbg || ProcStatus->GetReportMode()==10012) printf(" #%2d S5=%6.1f (Max=%6.1f) <t>=%6.1f", isamp, s4, GlobalMax, t4);
    if (s4>GlobalMax && t4>0.0) {
      GlobalMax=s4;
      Tg = t4;
      if (Dbg || ProcStatus->GetReportMode()==10012) printf(" ->> new tG=%6.1f Amax=%6.1f", Tg, GlobalMax);
      iGlobalMax=isamp;
    }
    if (Dbg || ProcStatus->GetReportMode()==10012) printf("\n");
  }

  s1/=nMax;
  s2/=nMax;
  s2-=s1*s1;
  s2=sqrt(s2);
  double signalFactor=0.;
  if (s2>0.01) {
    signalFactor=GlobalMax/s2;
  }
  if (Dbg || ProcStatus->GetReportMode()==10012 || ProcStatus->GetReportMode()==1) {
    printf("[GlobalPhase] Tglobal=%.1f  Max/RMS=%.2f GlobalMax=%.2f RMS=%.2f\n", Tg, GlobalMax/s2, GlobalMax, s2); 
  }
  ProcStatus->GetEventHeader()->SignalFactor=signalFactor;
  //SignalFactor1->Fill(signalFactor);
  ProcStatus->GetEventHeader()->TimeGlobal=Tg;
  
  return Tg;
  
}


