double GlobalPhase2011(TModule* Setup, ProcessingStatus* ProcStatus) {
  // version 2010-10-18 takes into account new timing scheme (each signal has its own time[i] values)
  // based on the Short signals
  // starts with Cherenkov, than adds calorimeters from FCal to EMEC and HEC until ether
  // phase found ot run out of data;
  char NameC[200];
  int NSLICE=ProcStatus->GetEventHeader()->nSlice;
  int nMax=0;
  double Tg=-1000.0;
  double TimeMax=0.0;
  double signalFactor=0.;
  int iGmax=-1;
  int i1max=-1;
  int i2max=-1;
  bool Dbg=false;
  //Dbg=true;  

  double Phases[6]={58.9, 222.7, 388.2, 553.0, 716.9, 883.2};
  int nAdded;

  

  if (Dbg) {
    printf("[GlobalPhase2011] Folding %d samples Report=%d...\n", 2*NSLICE, ProcStatus->GetReportMode());
  }
  double asum[512]={512*0.0}, asum1[512]={512*0.0}, wsum[512]={512*0.0};
  double GlobalMax=-1000;
  int   iGlobalMax=0;
  int   jMod[6]={4, 3, 2, 1, 0, 0}; // without Monitors
  //int   jMod[6]={4, 3, 2, -1, -1, -1};

  for (int i=0; i<512; i++) {asum[i]=0.0; asum1[i]=0.0; wsum[i]=0.0;}

  nAdded=0; // number of added channels //

  for (int iGain=0; iGain<2; iGain++) {

  //for (int iM=0; iM<5; iM++) {  // Timing by ALL, S123 and Cherenkov
  //for (int iM=0; iM<1; iM++) {  // Timing by S123 and Cherenkov
  for (int iM=0; iM<3; iM++) {  // Timing by Cherenkov and FCals
    GlobalMax=-1000;
    int iMod=jMod[iM];
    if (iMod==-1) continue;
    TModule *Module = &Setup[iMod];

    if (Dbg) { 
      printf("[GlobalPhase2011] %10s Empty Event Noise = (%.2f, %.2f)\n", 
             Module->GetName(), Module->GetEmptyEventNoise(0), Module->GetEmptyEventNoise(1));
    }

    int iCh0=0;
    int iCh1=4;
    if (iMod==4) iCh0=3;

    for (int iChan=iCh0; iChan<iCh1; iChan++) { // try cherenkov only!
    //for (int iChan=0;iChan<4;iChan++) {
      TChannel *Channel = Module->GetChannel(iChan);
      if (Channel->GetStatus()) {
        //for (int iGain=0;iGain<1;iGain++) {
        //for (int iGain=0; iGain<2; iGain++) {
          double eNoise=Channel->GetSignalLong(iGain)->GetEmptyEventNoise();
          if (Dbg) {
             printf("[GlobalPhase2011]       Chan#%d (%s) Gain#%d Empty Event Noise = %.2f\n", 
                    iChan, Channel->GetName(), iGain, eNoise);
          }

          for (int iDelay=0; iDelay<2; iDelay++) {
            TSignal *Signal = Channel->GetSignalShort(2*iGain+iDelay);
//            double tCorr=Setup[4].GetChannel(3)->GetTime(0)-Signal->GetTime(0); // align to Cherenkov
	    int iFold=0;
	    int i1=0;
            //if (Dbg || ProcStatus->GetReportMode()==10012) printf("[GlobalPhase]  Module#%d Channel#%d Gain#%d----> \n", iMod, iChan, iGain);


            double PrePed=0.0;
            double PrePed1=0.0;
            double PrePed0=0.0;
            double PrePed2=0.0;
            double s1=2000., s2=8000.;
            double Amax=-100000.;
            double Amin=100000.;

            if (Dbg) printf("[GlobalPhase2011] %20s --> ", Signal->GetName());

            for (int iLoop=0; iLoop<1; iLoop++) {
              PrePed0=0.0;
              PrePed1=0.0;
              PrePed2=0.0;
	      for (int i=0;i<NSLICE;i++) {
                double a=Signal->GetRaw(i);
                if (iLoop==0 && a>Amax) Amax=a;
                if (iLoop==0 && a<Amin) Amin=a;
                if (a>s1-3.0*s2 && a<s1+3.0*s2) {
                  PrePed0+=1.0;   
                  PrePed1+=a;            
                  PrePed2+=a*a;
                }
 	      }
              if (PrePed0<0.75*NSLICE) break;
              PrePed1/=PrePed0; 
              PrePed2/=PrePed0; 
              PrePed2-=PrePed1*PrePed1;
              s1=PrePed1;
              s2=sqrt(PrePed2);
              //printf("[GlobalPhase2011]  loop#%d N=%.0f ped=%.1f rms=%.1f Amp=%.1f\n", iLoop, PrePed0, s1, s2, Amax-Amin); 
            }
          
            PrePed=s1;

            if (Amax-Amin<10.) {
              if (Dbg) printf("[GlobalPhase2011] skipping the empty A=%f\n", Amax-Amin);
              continue;
            }
            if (iMod<4) {
              if (Amax-Amin>2000. || Amax>2800.) {
                if (Dbg) printf("[GlobalPhase2011] skipping the Huge A=%f\n", Amax-Amin);
                continue; // skip huge signals;
              }
            } else if (Amax-Amin>3000. || Amax>3200.) {
              if (Dbg) printf("[GlobalPhase2011] skipping the Huge A=%f\n", Amax-Amin);
              continue; // skip huge signals;
            }

            
            if (Amin==100000. || Amin<-1000.) {
              if (Dbg) printf("[GlobalPhase2011] skipping the Huge A=%f\n", Amax-Amin);
              continue; // skip huge signals;
            }
            if (Amax==-100000. || Amax>2800.) {
              if (Dbg) printf("[GlobalPhase2011] skipping the Huge A=%f\n", Amax-Amin);
              continue; // skip huge signals;
            }

            if (Dbg) {
              printf(" Amin=%f Amax=%f", Amin, Amax);
              printf("[GlobalPhase]->Used!\n");
            }
            nAdded++;
//            if (iMod==4 && iChan==3) {
//              if (Amax-Amin<6.0  && iGain==0) continue;
//              if (Amax-Amin<20.0 && iGain==1) continue;
//            } else {
//              if (Amax-Amin<3.0  && iGain==0) continue;
//              if (Amax-Amin<7.5 && iGain==11) continue;
//            }

            if (Dbg) { 
              printf("[GlobalPhase] adding to the Timing Sum -> %10s ch#%d gain#%d delay#%d -> Used!\n",
                   Module->GetName(), iChan, iGain, iDelay);
            }

            iFold=0;
	    for (int i=0;i<NSLICE;i++) {
	      double t=Signal->GetTimeUniform(i);
              double t1=t-iFold*989.772;
              double a=0.;

	      if (t1>989.772) {
	        iFold++;
                t1-=989.772;
	      } else if (t1<0.0) {
                t1+=989.772;
	      }
              i1=int(t1/12.5);            
            
              
              a=Signal->GetRaw(i)-PrePed;

              if (iMod==4 && iChan!=3) a/=10.;

              //if (a>0.) a=sqrt(a);
              //else      a=-sqrt(-a);

              if (a>50.) {
                a=50.+sqrt(a-50.);
              }
	      else if (a<-50.) {
                a=-50.-sqrt(-a-50.);
              }
            
              asum[i1]+=a;
              wsum[i1]+=1.0;
              if (Dbg || ProcStatus->GetReportMode()==10012) {
                printf("[GlobalPhase] %10s  adding sample i=%4d i1=%2d iFold=%d t=%6.1f t1=%6.1f w=%.0f\n ", 
                        Signal->GetName(), i, i1, iFold, t, t1, wsum[i1]);
              }

              if (Dbg || ProcStatus->GetReportMode()==10012) {
                 printf("[GlobalPhase]  a=%6.1f w=%2.0f <a>=%6.1f \n", a, wsum[i1], asum[i1]/wsum[i1]);
	      }
	      if (i1>nMax) {
                nMax=i1;
                TimeMax=i1*12.5;
              }
            } // samples loop
/*
              int iqq1;
              printf("Push!");
              cin >> iqq1;
              if (iqq1==0) exit(-1);
*/
	  } // delay loop 
        //} // gain loop
      } // good channel
    } // channel loop

    if (Dbg || ProcStatus->GetReportMode()==10012 || ProcStatus->GetReportMode()==1) {
      printf("[GlobalPhase] gain#%d, %d signal for timing was accumulated, searching for a peak\n", iGain, nAdded);
    }

    if (nAdded==0) continue;

  //Dbg=true;
    for (int i=0;i<nMax;i++) {
      if (wsum[i]>0.0) {
        if (Dbg || ProcStatus->GetReportMode()==10012) {
          printf("[GlobalPhase] sample %5d sum=%6.1f w=%f\n", i, asum1[i], wsum[i]);
        }
        asum1[i]=asum[i]/wsum[i];
      }
    }
  
    if (Dbg || ProcStatus->GetReportMode()==10012) {
      printf("\n\n[GlobalPhase] searching for peak in %d samples, Tmax=%.1f\n", nMax, TimeMax);
    }
    //TSignal *Signal = Setup[4].GetChannel(3)->GetSignalLong(0); // signal to use for time marks
    //TimeMax=Signal->GetTime(nMax);
    GlobalMax=0.0;
    double s2=0.0, s1=0.0;

    for (int isamp=0; isamp<nMax; isamp++) { // locate the peak of the Crude 'Pilot' Pulse (larges bunch)
      double s4=0.0, t4=0.0, s0=0.;

      s1+=asum1[isamp];
      s2+=asum1[isamp]*asum1[isamp];

      int i1=isamp-2;
      int i2=isamp+3;

      if (Dbg || ProcStatus->GetReportMode()==10012) {
        printf("[GlobalPhase]  sample#%2d range->(%2d, %2d) A=%6.1f ", isamp, i1, i2, asum1[isamp]);
      }

      for (int i=i1; i<i2; i++) {
        double a=0.0, t=0.0;
        s0+=1.;
        if (i<0) {
          a=asum1[i+nMax];
          //t=Signal->GetTime(i+nMax) - TimeMax;
        } else if (i>nMax-1) {
          a=asum1[i-nMax];
          //t=Signal->GetTime(i-nMax) + TimeMax;
        } else {
          a=asum1[i];
          //t=Signal->GetTime(i);
        }
        t=i*12.5;
        s4+=a;
        t4+=t*a;
      }
      if (Dbg || ProcStatus->GetReportMode()==10012) { 
        printf(" #%2d S5=%6.1f (Max=%6.1f) <t>=%6.1f", isamp, s4/s0, GlobalMax, t4/s4);
      }
      t4/=s4;
      s4/=s0;
      if (s4>0.0 && s4>GlobalMax && t4>0.0) {
        GlobalMax=s4;
        iGmax=isamp;
        i1max=i1;
        i2max=i2;

        Tg = t4;
        if (Dbg || ProcStatus->GetReportMode()==10012) printf(" ->> new tG=%6.1f Amax=%6.1f", Tg, GlobalMax);
        iGlobalMax=isamp;
      }
      if (Dbg || ProcStatus->GetReportMode()==10012) printf("\n");
    }

    Tg+=6.25;

    s1/=nMax;
    s2/=nMax;
    s2-=s1*s1;
    s2=sqrt(s2);
    if (s2>0.01) {
      signalFactor=GlobalMax/s2;
    }
    if (Dbg || ProcStatus->GetReportMode()==10012) {
      printf("[GlobalPhase] %10s added gain#%d, Tglobal=%.1f  Max/RMS=%.2f GlobalMax=%.2f RMS=%.2f\n", 
              Module->GetName(), iGain, Tg, signalFactor, GlobalMax, s2); 
    }

    if (signalFactor>2.5) {
      break; // signal found, leave the loop
    }

    Dbg=false;

  } // module loop

    if (signalFactor>2.5) {
      break; // signal found, leave the loop
    }
  } // gain loop
  
  ProcStatus->GetEventHeader()->SetSignalFactor(signalFactor);
  ProcStatus->GetEventHeader()->SetTimeGlobal(Tg);
  //ProcStatus->GetEventHeader()->SignalFactor=signalFactor;
  //ProcStatus->GetEventHeader()->TimeGlobal=Tg;
  //SignalFactor1->Fill(signalFactor);


  if (ProcStatus->GetReportMode()==1) { // display pilot pulse //
    EventHeader *EH = ProcStatus->GetEventHeader();
    //BurstHeader *BH = ProcStatus->GetBurstHeader();
    TLine  * displayLine  = new TLine(); 
    TLatex * displayLatex = new TLatex();
    sprintf(NameC, "Evt#%d SF=%.2f", EH->event, signalFactor);

    TH1D    *dummy = new TH1D("dummy", NameC, 2500, -50., 50.+TimeMax);
    dummy->GetXaxis()->SetTitle("Time, ns");
    dummy->GetYaxis()->SetTitle("Pilot Pulse");

    PilotCanvas->Clear();
    PilotCanvas->Show();
    PilotCanvas->cd(0);
    double amax=0.;
    double amin=0.;
    for (int isamp=0; isamp<nMax; isamp++) {
      if (asum1[isamp]>amax) amax=asum1[isamp];
      if (asum1[isamp]<amin) amin=asum1[isamp];
    }
    dummy->GetYaxis()->SetRangeUser(1.05*amin, 1.05*amax);
    dummy->DrawCopy();

    displayLatex->SetTextAlign(22);

    for (int isamp=0; isamp<nMax; isamp++) {
      double t=isamp*12.5;
      double a=asum1[isamp];
      if (isamp==iGmax) displayLatex->SetTextColor(kRed);
      if (isamp<i2max && isamp>=i1max) displayLatex->SetTextColor(kGreen);
      else              displayLatex->SetTextColor(kBlack);
      displayLatex->DrawLatex(t, a, "o");
    }
    displayLine->SetLineColor(kRed);
    displayLine->SetLineWidth(3);
    displayLine->SetLineStyle(2);
    displayLine->DrawLine(Tg-6.25, amin, Tg-6.25, amax);

    displayLine->SetLineColor(kBlue);
    displayLine->SetLineWidth(1);
    displayLine->SetLineStyle(2);
    for (int i=0; i<6; i++) {
      displayLine->DrawLine(Phases[i], amin, Phases[i], amax);
    }
    PilotCanvas->Update();
    
    dummy->Delete();
    displayLine->Delete();
    displayLatex->Delete();
          
  }
  double bestPhase=0;
  double best=10000.0;
  for (int i=0; i<6; i++) {
    if (fabs(Tg-Phases[i])<best) {
      best=fabs(Tg-Phases[i]);
      bestPhase=Phases[i];
    }
  }
/*
  Tg=bestPhase;
*/
  Dbg=false;
  return Tg;
  
}


