bool LoadChannelStatus(Int_t Run, TModule* Setup) {
  // status=0 no signal
  // status=1 normal
  // status=5 correction needed
  // status <0 signal problems
  //        -2  gain in one delay less than half of another
  //        -3  same delay, too big difference between relative signals in different GAINs
  //        -10 no statistics for pulse shape
  //        ** -100 penalty for bad shape
  //
  //
  std::ifstream statusData;
  char FileName[100];
  int Status;
  double Correction, Lhood;
  int iMod1, iChan1, iGain1, iDelay1;

  sprintf(FileName, "../ShapesQC/status20090514_Run%d.QC", Run);
  statusData.open(FileName, ios::in);
  if (!statusData.is_open()) {
    statusData.clear();
    //printf("[LoadChannelStatus] Status file %s open failure \n", FileName);
    sprintf(FileName, "../ShapesQC/status20090514_Run000.QC");
    statusData.open(FileName, ios::in);
    if (!statusData.is_open()) {
      statusData.clear();
      printf("[LoadChannelStatus] Main and Default Status file %s open failure \n", FileName);
      return false;
    } else {
      //printf("[LoadChannelStatus] Default Status file %s will be used \n", FileName);
    }
  }

  for (int iMod=0; iMod<4; iMod++) {
    for (int iChan=0; iChan<4; iChan++) {
      statusData >> iMod1 >>  iChan1 >> Status; // ChanStatus[iMod][iChan]
      if (iMod != iMod1 || iChan != iChan1) {
        printf("[LoadChannelStatus] data Error, Mod=(%d, %d) Chan=(%d, %d) \n", iMod, iMod1, iChan, iChan1);
        return false;
      }
      TChannel *Channel = Setup[iMod].GetChannel(iChan);
/* 
      if (Run>391 && Run<411) { // special case - electronic saturation in those run is 1100 ADC instead of 2500 ADC!
        Channel->SetHighLowSwitch(1100.);
      } else {
        Channel->SetHighLowSwitch(2200.);
      }
*/
      //printf("[LoadChannelStatus] %12s High-Low switch set to %.1f\n", 
      //   Channel->GetName(), Channel->GetHighLowSwitch());  


      if (Status) {
        Channel->On();
      } else {
        Channel->Off();
        //Channel->CloseDisplayCanvas();
      }

      //if (isOnline) Channel->CloseDisplayCanvas();

      for (int iGain=0; iGain<2; iGain++) {
        statusData >> iMod1 >> iChan1 >> iGain1 >> Status; // GainStatus[iMod][iChan][iGain]
        if (iMod != iMod1 || iChan != iChan1 || iGain != iGain1) {
          printf("[LoadChannelStatus] data Error, Mod=(%d, %d) Chan=(%d, %d) Gain=(%d,  %d)\n", 
                 iMod, iMod1, iChan, iChan1, iGain, iGain1);
          return false;
        }
        TSignal *SignalLong = Channel->GetSignalLong(iGain);
        SignalLong->On();

	for (int iDelay=0; iDelay<2; iDelay++) {
	  statusData >> iMod1 >> iChan1 >> iGain1 >> iDelay1 
                     >> Status        // DelayStatus[iMod][iChan][iGain][iDelay]
		     >> Lhood         // LHood[iMod][iChan][iGain][iDelay]/wHood1[iMod][iGain]
		     >> Correction;   // Correction[iMod][iChan][iGain][iDelay] << endl;
	  //if (Status==5) Status=1;
          if (iMod != iMod1 || iChan != iChan1 || iGain != iGain1 || iDelay != iDelay1) {
            printf("[LoadChannelStatus] data Error, Mod=(%d, %d) Chan=(%d, %d) Gain=(%d,  %d) Delay=(%d, %d)\n", 
                   iMod, iMod1, iChan, iChan1, iGain, iGain1, iDelay, iDelay1);
            return false;
          }
          TSignal *SignalShort = Channel->GetSignalShort(2*iGain+iDelay);
          if (Status==1) {
	    SignalShort->On();
            SignalShort->SetSignalCorrection(1.0);
          } else if (Status==-10) {
            SignalShort->SetStatus(-10);
            SignalShort->SetSignalCorrection(1.0);
	  } else if (Status<1) {
            SignalLong->Off();
            SignalShort->Off();
	  } else if (Status==5) {
            if (Lhood>0.05) {
	      SignalShort->On();
              SignalShort->SetSignalCorrection(Correction);
	    } else {
              SignalLong->Off();
              SignalShort->Off();
	    }
	  } else {
            printf("[LoadChannelStatus] Unknown Status=%d for Mod#%d Chan#%d Gain=%d Delay=%d\n",
                   Status, iMod, iChan, iGain, iDelay);
            exit(-1);
	  }
           

	}
      }
    }
  }
  statusData.close();

// load High/Low ratios from run summary files produced by HiLum20100222hilow.C
  sprintf(FileName,"../HighToLow2009/runs472_515.dat");
  if (Run>300 && Run<456) sprintf(FileName,"../HighToLow2009/runs300_455.dat");
  if (Run>455 && Run<516) sprintf(FileName,"../HighToLow2009/runs472_515.dat");
  if (Run>515 && Run<645) sprintf(FileName,"../HighToLow2009/runs516_644.dat");
  if (Run>644) sprintf(FileName,"../HighToLow2009/runs000.dat");

  printf("[main->LoadChannelStatus] Run#%d High-To-Low file -> %s\n",
         Run,  FileName);

  std::ifstream HiToLowFileAll;
  HiToLowFileAll.open(FileName, ios::in);
  for (int iMod=0; iMod<6; iMod++) {
    for (int iChan=0; iChan<4; iChan++) {
      if (iMod==5 && iChan==3) {
        break;
      } else {
        double s0x;
        double s1;
        double s2;
        int i, j;
        HiToLowFileAll >> i >> j >> s0x >> s1 >> s2;
        Setup[i].GetChannel(j)->SetHighLow(s1);
      }
    }
  }

  HiToLowFileAll.close();
  
  
  return true;
}

