
bool LoadRFcorrection(Int_t run, TModule* Setup) {
  char FileName[200];
  int xMod, xCh, xGain, xDelay;
  double dT, sdT, A, sA, P, eP;
  double checkSum=0.0;
  sprintf(FileName, "reports/RFcorrection/run%04d_RF_correction_2012.12.18.dat", run);
  RFdata.open(FileName, ios::in);
  if (!RFdata.is_open()) {
    RFdata.clear();
    return false;
  }
  for (int iMod=4; iMod<6; iMod++) {
    for (int iCh=0; iCh<4; iCh++) {
      if (iMod==5 && iCh>1) continue;
      TChannel *Channel = Setup[iMod].GetChannel(iCh);
      for (int iGain=0; iGain<2; iGain++) {
        for (int iDelay=0; iDelay<2; iDelay++) {
           RFdata >> xMod >> xCh >> xGain >> xDelay;
           if (xMod != iMod || xCh != iCh || xGain != iGain || xDelay != iDelay) {
             RFdata.close();
             printf("[LoadRFcorrection] RF correction load failed: index = (%d, %d, %d, %d)\n", iMod, iCh, iGain, iDelay);
             printf("[LoadRFcorrection]                             file = (%d, %d, %d, %d)\n", xMod, xCh, xGain, xDelay);
             printf("[LoadRFcorrection] %s\n", FileName);
             exit(-1);
           }
           RFdata >> dT >> sdT >> A >> sA >> P >> eP;
           checkSum+=A;
           TSignal *Signal = Channel->GetSignalShort(2*iGain+iDelay);
           Signal->SetRFcorrection(dT, A, P); 
        } // end delay
      } // end gain
    } // end channel
  } // end module
     
  RFdata.close();
  if (checkSum==0.0) {
    printf("[LoadRFcorrection] Problem with file %s\n", FileName);
    printf("[LoadRFcorrection] RF correction load found that run#%d has no pedestal events\n", run);
    return false;
  }
  return true;
}

