// class MIPcounter uses likelyhood track counting based on S1, S2, S3 amplitudes.
#include <TFile.h>
class MIPcounter {
  private:
bool noiseLoaded;
bool noiseLoaded1[3];
bool spectLoaded;
char fName[200];
char hName[200];
int  nTrackMAX;
int    nbMIP;
double x0MIP;
double x1MIP;
double scint[3];
double aMIP[3]; // average single track signal for S1,2,3, FFT ADC units
TH1D mipHood[303];
TH1D trackHood[303];
TH1D lHood[4]; // S1, S2, S3, Combo
//===================================
  public:
MIPcounter() {
noiseLoaded=false;
noiseLoaded1[0]=false;
noiseLoaded1[1]=false;
noiseLoaded1[2]=false;
spectLoaded=false;    
nTrackMAX=60;
nbMIP=0;
x0MIP=0.;
x1MIP=0.;

for (int iCh=0; iCh<3; iCh++) aMIP[iCh]=1.0; // set default to 1

for (int iCh=0; iCh<4; iCh++) {
  if (iCh<3) sprintf(hName, "S%d likelihood", iCh);
  else       sprintf(hName, "Combined Likelihood"); 
  lHood[iCh].SetNameTitle(hName, hName);
  lHood[iCh].SetBins(nTrackMAX+1, -0.5, nTrackMAX+0.5);
}
}
//=======================================================
MIPcounter(int Nmax) {
noiseLoaded=false;
noiseLoaded1[0]=false;
noiseLoaded1[1]=false;
noiseLoaded1[2]=false;
spectLoaded=false;    
nTrackMAX=Nmax;
if (nTrackMAX!=30 && nTrackMAX!=60 && nTrackMAX!=100) {
  printf("[MIPcounter] nTrackMAX=%d not implemented, bye\n", nTrackMAX);
  exit(-1);
}
nbMIP=0;
x0MIP=0.;
x1MIP=0.;

for (int iCh=0; iCh<3; iCh++) aMIP[iCh]=1.0; // set default to 1

for (int iCh=0; iCh<4; iCh++) {
  if (iCh<3) sprintf(hName, "S%d likelihood", iCh);
  else       sprintf(hName, "Combined Likelihood"); 
  lHood[iCh].SetNameTitle(hName, hName);
  lHood[iCh].SetBins(nTrackMAX+1, -0.5, nTrackMAX+0.5);
}
}



//=======================================================
int    GetBins() {return  nbMIP;}
double GetX0()   {return  x0MIP;}
double GetX1()   {return  x1MIP;}

void SetMIPaverage(int iCh, double MIP) {aMIP[iCh]=MIP;} 

//=======================================================
bool LoadMIPaverage(int iCh, int hvGroup) {

std::ifstream dataMIP;
dataMIP.open("reports/BeamHV/HVgroupsMIP_1014step3.dat", ios::in);
if (!dataMIP.is_open()) {
  printf("[MIPcounter2012->LoadMIPaverage] failed to open file with averages\n");
  dataMIP.clear();
  return false;
}

while(!dataMIP.eof()) {
  int    iChFile=0;
  int    iGrGile=0;
  double MIP=0.;
  int    nEvts=0;
  dataMIP >> iChFile >> iGrGile >> MIP >> nEvts;
  if (iChFile==iCh && iGrGile==hvGroup) {
    printf("[MIPcounter2012->LoadMIPaverage] S%d MIP is set to %f FFT ADC\n", iCh+1, MIP);
    aMIP[iCh]=MIP;
    return true;        
  }
  if (iChFile==-1 && iGrGile==-1) {// termination line
     printf("[MIPcounter2012->LoadMIPaverage] reached termination line for iCh=%d hvGroup=%d\n",
            iCh, hvGroup);
    return false;
  } 
}
dataMIP.close();
//dataMIP.clear();
return true;
}

//------------------------------------------------
double GetMIPaverage(int iCh) {
  if (iCh<0 || iCh>2) {
    printf("[MIPcounter2012->LoadMIPaverage] requesting wrong counter id=%d, bye\n", iCh);
    exit(-1);
  }
  return aMIP[iCh];
}
double* GetMIPaveragePointer() {return &aMIP[0];}

//-------------------------------
void PrepareLhood(double s1, double s2, double s3) {
  scint[0]=s1;
  scint[1]=s2;
  scint[2]=s3;
  for (int iCh=0; iCh<3; iCh++) scint[iCh]/=aMIP[iCh]; // calibrate to <MIP> units
  for (int nTr=0; nTr<nTrackMAX+1; nTr++) {
    lHood[3].SetBinContent(nTr+1, 1.0);
  }
  for (int iCh=0; iCh<3; iCh++) {
    lHood[iCh].Reset();
    int ib=mipHood[iCh].FindBin(scint[iCh]);
    if (ib==nbMIP+1) ib=nbMIP;
    for (int nTr=0; nTr<nTrackMAX+1; nTr++) {
      int id=nTr*3+iCh;
      double w=trackHood[id].GetBinContent(ib);
      lHood[iCh].SetBinContent(nTr+1, w);
      double w1=w*lHood[3].GetBinContent(nTr+1);
      lHood[3].SetBinContent(nTr+1, w1);
    }
  }
}

//-------------------------------
TH1D* GetLhood(int iCh) {
  return &lHood[iCh];
}

//-------------------------------
double GenerateTracks(int iCh, int nTracks) {
  if (nTracks<1 || nTracks>60) {
    printf("[GenerateTracks] bad nTracks=%d, bye\n", nTracks);
    exit(-1);
  }
  int id=iCh+3*nTracks; // for single tracks //
  bool seek=true;
  double x=0., y=0.;
  while (seek) {
    x=x0MIP+gRandom->Rndm()*(x1MIP-x0MIP);
    y=gRandom->Rndm()*mipHood[id].GetMaximum();
    seek=y>mipHood[id].GetBinContent(mipHood[id].FindBin(x));
  }
  return x;
}

//-------------------------------
double GenerateSingle(int iCh) {
  int id=iCh+3; // for single tracks //
  bool seek=true;
  double x=0., y=0.;
  while (seek) {
    x=x0MIP+gRandom->Rndm()*(x1MIP-x0MIP);
    y=gRandom->Rndm()*mipHood[id].GetMaximum();
    seek=y>mipHood[id].GetBinContent(mipHood[id].FindBin(x));
  }
  return x;
}

//-------------------------------
double GeneratePedestal(int iCh) {
  int id=iCh; // for pedestal tracks //
  bool seek=true;
  double x=0., y=0.;
  while (seek) {
    x=x0MIP+gRandom->Rndm()*(x1MIP-x0MIP);
    y=gRandom->Rndm()*mipHood[id].GetMaximum();
    seek=y>mipHood[id].GetBinContent(mipHood[id].FindBin(x));
  }
  return x;
}

//-------------------------------
bool LoadNoise(int iHV, int iChan) {
if (nTrackMAX==60) {
  sprintf(fName, "reports/BeamHV/HVgroup%02dnoiseS%d_1014step3_60.root", iHV, iChan+1);
} else if (nTrackMAX==30) {
  sprintf(fName, "reports/BeamHV/HVgroup%02dnoiseS%d_1014step3.root", iHV, iChan+1);
}  else if (nTrackMAX==100) {
  sprintf(fName, "/raid04/HiLum/analysis2012/reports/BeamHV/HVgroup%02dnoiseS%d_1118step3_100.root", iHV, iChan+1);
} else {
  printf("[MIPcounter2012->LoadNoise] nTrackMAX=%d not implemented, bye\n", nTrackMAX);
  exit(-1);
}
printf("[MIPcounter2012->LoadNoise] loading noise from file %s\n", fName);

TFile f(fName, "READ");
if (f.IsZombie()) {
  printf("[MIPcounter->LoadNoise] Error opening histogram file %s", fName);
  f.Close();
  return false;
}
sprintf(hName, "S%d Pedestal", iChan+1);
TH1D *h = (TH1D*)f.Get(hName);
printf("[MIPcounter2012->LoadNoise] noise histogram %s loaded, nBins=%d\n", hName, h->GetNbinsX());

int    nbMIP=h->GetNbinsX();
double x0MIP=h->GetXaxis()->GetXmin();
double x1MIP=h->GetXaxis()->GetXmax();
sprintf(hName, "Noise_HVgr%02d_S%d", iHV, iChan);
printf("[MIPcounter2012->LoadNoise] new name = <%s>\n",hName); 

mipHood[iChan].SetNameTitle(hName, hName);
mipHood[iChan].SetBins(nbMIP, x0MIP, x1MIP);
printf("[MIPcounter2012->LoadNoise] bins set\n");

for (int i=1; i<nbMIP+1; i++) {
  mipHood[iChan].SetBinContent(i, h->GetBinContent(i));
}
printf("[MIPcounter2012->LoadNoise] content copied\n");

f.Close();
noiseLoaded1[iChan]=true;
printf("[MIPcounter2012->LoadNoise] file closed\n");
return true;
}

//--------------------------------
bool LoadMIPs() { // loads single and multi track spectra
bool result=true;
for (int iCh=0; iCh<3; iCh++) {
  if (!noiseLoaded1[iCh]) {
    printf("[MIPcounter] noise not loaded for S%d, bye\n", iCh+1);
    exit(-1);
    return false;
  }
}

if (nTrackMAX==60) {
  sprintf(fName, "reports/MIP/MIPhood/mipHood1014step3_60.root");
} else if (nTrackMAX==30) {
  sprintf(fName, "reports/MIP/MIPhood/mipHood1014step3.root");
}  else if (nTrackMAX==100) {
//  sprintf(fName, "/raid04/HiLum/analysis2012/reports/MIP/MIPhood/mipHood1014step3_100.root");
  sprintf(fName, "/raid04/HiLum/analysis2012/reports/MIP/MIPhood/mipHood1014step3_100_cut015.root");
} else {
  printf("[MIPcounter2012->LoadNoise] nTrackMAX=%d not implemented, bye\n", nTrackMAX);
  exit(-1);
}

TFile f(fName, "READ");
if (f.IsZombie()) {
  printf("[MIPcounter->LoadMIPs] Error opening histogram file %s\n", fName);
  f.Close();
  return false;
}
for (int iCh=0; iCh<3; iCh++) {
  for (int mips=1; mips<nTrackMAX+1; mips++) {
    int id=mips*3+iCh;
    sprintf(hName, "S%d %dMIPs", iCh+1, mips);
    //printf("[MIPcounter->LoadMIPs] mips=%2d loading histogram <%s> id=%d\n", mips, hName, id);
    TH1D *h = (TH1D*)f.Get(hName);
    nbMIP=h->GetNbinsX();
    x0MIP=h->GetXaxis()->GetXmin();
    x1MIP=h->GetXaxis()->GetXmax();
    sprintf(hName, "%0dtracks_in_S%d", mips, iCh+1);
    mipHood[id].SetNameTitle(hName, hName);
    mipHood[id].SetBins(nbMIP, x0MIP, x1MIP);
    for (int i=1; i<nbMIP+1; i++) {
      mipHood[id].SetBinContent(i, h->GetBinContent(i));
    }
    sprintf(hName, "%0dweight_in_S%d", mips, iCh+1);
    //printf("[MIPcounter->LoadMIPs] setting histogram <%s>...", hName);
    trackHood[id].SetNameTitle(hName, hName);
    trackHood[id].SetBins(nbMIP, x0MIP, x1MIP);
    //printf(" done\n");
  }
  
  printf("[MIPcounter->LoadMIPs] S%d mipHood loaded\n", iCh+1);

  sprintf(hName, "%0dweight_in_S%d", 0, iCh+1);
  trackHood[iCh].SetNameTitle(hName, hName);
  trackHood[iCh].SetBins(nbMIP, x0MIP, x1MIP);

  for (int i=1; i<nbMIP+1; i++) {
    double sw=0.;
    for (int nmips=0; nmips<nTrackMAX+1; nmips++) {
      int id=nmips*3+iCh;
      sw+=mipHood[id].GetBinContent(i);
    }
    for (int nmips=0; nmips<nTrackMAX+1; nmips++) {
      int id=nmips*3+iCh;
      double w=mipHood[id].GetBinContent(i);

      if (w==0. && sw==0.) {
        if (nmips==0) w=1.;
        else w=0.;
      } else if (sw==0.) {
        printf("\n\n [LoadNoise] SW shall not be zero! bye! \n\n");
        exit(-1);
      } else {
        w/=sw;
      }
      trackHood[id].SetBinContent(i, w);      
    }
  }
  
}  
spectLoaded=true;

f.Close();
return result;
}
//--------------------------------
TH1D* GetMIPs(int iCh, int nMIPs) {
int id=nMIPs*3+iCh;
return &mipHood[id];  
}
TH1D* GetTracks(int iCh, int nMIPs) {
int id=nMIPs*3+iCh;
return &trackHood[id];  
}


//--------------------------------
void DisplayNoiseSpectra() {
int iqq=0;
TCanvas *showNoise = new TCanvas("showNoise", "Noise", 800, 1100);
showNoise->Divide(1, 3);
for (int iChan=0; iChan<3; iChan++) {
  showNoise->cd(iChan+1);
  mipHood[iChan].GetXaxis()->SetRangeUser(-1.,1.);
  mipHood[iChan].DrawCopy();
}
showNoise->Update();
printf("push to continue->");
cin >> iqq;
if (iqq==0) {
  exit(-1);
}
showNoise->Close();
}
//
};
