bool PrepareTimeSlices(Int_t Run, TModule* Setup, ProcessingStatus* ProcStatus) {
  bool Dbg=false;
  double dT=25.;
  double Toffset=0.0;
  double CherHV[17]={0.9, 1.0, 1.1, 1.2, 1.3,
                     1.4, 1.5, 1.6, 1.7, 1.8,
                     1.9, 2.0, 2.1, 2.2, 2.3,
                     2.4, 2.5};
  double CherDT[17]={19.0, 16.3, 14.4, 12.6, 11.0, 
                     10.1,  8.7,  7.6,  6.5,  5.7,  
                      4.8,  3.8,  2.7,  2.5,  1.8,  
                      1.3,  0.0};
  //Dbg=true;

  if (Dbg) printf("[main->PrepareTimeSlices] preparing slices for Run#%d\n", Run);
  
  if (Run>300) {
    //Dbg=true;
    for (int iMod=0; iMod<nMod; iMod++) {
      TModule *Module = &Setup[iMod];
      if (Dbg) printf("[main->PrepareTimeSlices] preparing slices for %s\n", Module->GetName());
      Module->ClearTime(); // clear the ModuleSum time array, prepare for computation of new time marks
      int nCH=4;
      if (iMod<4) nCH=5; // extra sum channel must be set for modules
      for (int iChan=0;iChan<nCH;iChan++) {
        TChannel *Channel = Module->GetChannel(iChan);
        Toffset=0.0;
        if (iMod==4 && iChan==3) { // Cherenkov, adjust delay for HV!
          double HV=ProcStatus->GetRunHeader()->HVbeam[3];
          if (HV<CherHV[0]) {
            Toffset=CherDT[0];
          } else if (HV>CherHV[16]) {
            Toffset=CherDT[16];
          } else {
            for (int i=0; i<16; i++) { 
              if (HV>=CherHV[i] && HV<CherHV[i+1]) {
                Toffset=CherDT[i]*(CherHV[i+1]-HV) + CherDT[i+1]*(HV-CherHV[i]);
                Toffset/=(CherHV[i+1]-CherHV[i]);
                break;
              }
            }
          }
          printf("[main->PrepareTimeSlices] HV cherenkov=%f Toffset=%f \n", HV, Toffset);
        } else if (iMod==5 && iChan==2) { // RF!
          Toffset=0.0;
        } 

        if (Run>1070) { // 2013 data
          if (iMod<4) Toffset-=25.;
        } else if (Run>850) { // 2012 data
          if (iMod<4)  Toffset+=5.;
          else Toffset+=20.;
        }

        for (int iGain=0;iGain<2;iGain++) {
          if (Dbg || ProcStatus->GetReportMode()==10013) {
            printf("[PrepareTimeSlices] %s, %s gain=%d\n", Module->GetName(), Channel->GetName(), iGain);
	  }
	  TSignal* SignalLong   = Channel->GetSignalLong(iGain);
	  TSignal* SignalShort0 = Channel->GetSignalShort(2*iGain);
	  TSignal* SignalShort1 = Channel->GetSignalShort(2*iGain+1);
          double dT0=SignalShort0->GetTriggerOffset();
          double dT1=SignalShort1->GetTriggerOffset();
          int NsliceShort=SignalShort0->GetNslice();

          //printf("[PrepareTimeSlices] %15s trigger offsets = (%f, %f)\n", Channel->GetName(), dT0, dT1);
   
          if (iMod==5 && iChan==2) { // RF!
            dT0=0.0;
            dT1=0.0;
          }

          if (Dbg || ProcStatus->GetReportMode()==10013) {
            printf("[PrepareTimeSlices] %s %d slices to be set ->\n", Channel->GetName(), NsliceShort);
	    printf("[PrepareTimeSlices] dT0=%6.2f dT1=%6.2f\n", dT0, dT1);
	  }

          for (int i=0;i<NsliceShort;i++) {
            double t0=i*dT-Toffset;
	    double t1=t0+14.5;
	    if (iMod>3) t1=t0+16.1;
//	    if (iMod>3) t1=t0+18.5;
            if (iMod==5 && iChan==2) { // RF!
              t1=t0+16.1;
            }
	    t0+=dT0;
	    t1+=dT1;
            if (iMod>3) {
              t1-=0.7;
	    }
            SignalShort0 -> SetTime(i, t0);
            SignalShort1 -> SetTime(i, t1);
            SignalLong   -> SetTime(2*i,   t0);
            SignalLong   -> SetTime(2*i+1, t1);
            SignalShort0 -> SetTimeUniform(i, t0);
            SignalShort1 -> SetTimeUniform(i, t0+12.5);
            SignalLong   -> SetTimeUniform(2*i,   t0);
            SignalLong   -> SetTimeUniform(2*i+1, t0+12.5);
            Module->AccumulateTime(2*i,   t0);
            Module->AccumulateTime(2*i+1, t1);
            
            if (Dbg || (ProcStatus->GetReportMode()==10013 && i<70)) {
              printf("[PrepareTimeSlices] %4d t0=%6.1f t1=%6.1f\n", i, t0, t1);
            }
	  }
        }
      }

      if (iMod<6) Module->ComputeTimeSlices(); // here it will set time slices for the sum signals !

    } // module loop 
    Dbg=false;
    return true;
  } else {
    printf("*** Error *** Time grid is available for runs > 300 only , check proper settings for earlier runs!\n");
    return false;
  }
}


