#include <cstdlib>
using std::strtoul;

class ProcessingStatus { // contains general information about data processing //
private:
  bool isOnline;
  int  location; // 1 - laptop, 2 - desktop
  char inString[10000];
  char RunString[2000];
  char RunStringCopy[2000];
  char bunchFileName[100];
  char dataFolder[200];
  char logbookFile[200];
  char CommentString[200]; // 2014-12-23
  int numberOfRuns;
  int runIndex;
  int SelectedRuns[10000];
  bool fillModule1[6]; // same but for current run;
  bool fillModule[10000][6];
  double RunRate[10000];
  int numberOfRateGroups;
  int RunRateGroup[10000];
  double GroupRate[10000];
  
  int nHVgroups[4]; /// number of groups per module
  double HVgroup[4][100]; // HV  for module in group
  int hvGroupRun[4][1600]; 
  int AttnIn[1600]; /// attenuator for Run# 0, 1, 2, 3 for 1., 3., 10., 30.
//  :promptfind
  double HVchozen[4];
  bool   HVchozenSet[4];

  double HVmod[6]; // selected HV on the module[iMod]
  double cryoX[3]; // position of 3 cryostats - HEC, EMEC, FCal
  float HVmonitorVoltageMAX[300][4][4]; //max HV for spill*module*channel
  float HVmonitorVoltageMIN[300][4][4]; //max HV for spill*module*channel
  float HVmonitorCurrentMAX[300][4][4]; //max HV for spill*module*channel
  float HVmonitorCurrentMIN[300][4][4]; //max HV for spill*module*channel
  int   HVmonitorVoltIndexMAX[300][4][4]; // index of the MAX reading;
  int   HVmonitorVoltIndexMIN[300][4][4];
  int   SpillStatusHV[300][4][4]; // 0 is good
                                  // -1 no spill
                                  // 1 voltage min<0.9*set  
                                  // 2 voltage max>1.1*set    
                                  // 3 multiple records
  int   SpillStatusHC[300][4][4]; // 0 is good
                                  // -1 no spill
                                  // 3 multiple records

  int   SpillStatusI[300];// spill status based on Intensity record (IonCham and SED)
                          // 0 is good 
                          // -1 no spill
                          // 3 multiple records
  int   SpillStatusS[300];// spill status based on Spill Summary record 
                          // 0 is good 
                          // -1 no spill
                          // 1 abnormal number of events
                          // 3 multiple records
                          // 4 abnormal time
                          // 5 not same spill at the beginning and the end record
                          // 6 Cherenkov HV is wrong;
                          // 7 Cherenkov Rate < 1. or spill is empty
                          // 8 Abnormal Cherenkov HV, seen only in Run#699 starting spill#40 
  int FirstSpillSumm;    // number of the first spill in the summary records //
  int FirstSpillIntens;  // number of the first spill in the intensity records //

  double ionChamb[300];
  double ionCalib[300];
  double SEDx[16][300];
  double SEDy[16][300];
  double SEDsumX[300];
  double SEDsumY[300];
  int    EvtNum[300][2];
  double CherInt[300];
  double CherHV[300];
  double CherFill[300][6]; // filled sample detection sums (to suppress empty spills)
  double SpillStart[300];
  double SpillCOG[300];
  double SpillEnd[300];
  


  int CurrentRun;
  int ReportMode;   
  int nCases;      // report mode cases to use/show
  EventHeader EH;
  BurstHeader BH;
  RunHeader   RH;
  bool firstRunEvent;
  bool firstBurstEvent;
  bool firstRunBurst;
  bool BeginningOfBurst;

  const char* pauseCase[100];
  int   pauseMode[100];
  bool  activeMode[100];

  bool useFitFiles;
  bool useCherenkovStream;

  double maxLogbookIntensity;

  int displayWidth;
  int displayHeight;

  int DumpMode; // 0 - no dump
                // -n - first n spills
                // +n - first n events
  int DumpCount;
  int Nbunches; // number of bunches in current event to process

  const char *module_name[7];

  double Tdrift[4];

  bool   bunchPatternSet;
  bool   bunchSlopesLoaded;
  double cherHVcurrent;
  double bunchPattern[30];
  double bunchPattern1[30]; // intermediate pattern
  bool   bunchIsolated[30];
  int    bunchAction0[30]; 
  int    bunchAction[30]; // 0 - nothing
                          // 1 - add to signal
                          // 2 - add to noise
  bool    bunchToFill[60]; // flag for full bunch
  bool    bunchIndex[30];  // index of bunches to fill;
  double  bunchTime[30];
  int     nFilled;         // number of filled bunches out of 30

  int     nPedWindows;    // number of commnon pedestal windows in the event
  double  pedWindow0[10]; // boundaries of pedestal window
  double  pedWindow1[10];

  double bunchSlopes[10];



public:
  ProcessingStatus() { // Constructor //
    nPedWindows=0;
    Nbunches=0;
    nFilled=0;
    DumpMode=0; 
    DumpCount=0; 
    CurrentRun = 0;
    ReportMode = 10001;
    sprintf (dataFolder, "%s", "../data");
    sprintf (logbookFile, "%s", "../doc/runbook_nov08.elog");
    useFitFiles=false;
    useCherenkovStream=false;
    SetCases();
    numberOfRuns=0;
    for (int i=0;i<10000;i++) SelectedRuns[i]=-1;
    maxLogbookIntensity=0.0;
    module_name[0]="HEC";
    module_name[1]="EMEC";
    module_name[2]="FCal025";
    module_name[3]="FCal010";
    module_name[4]="Beam";
    module_name[5]="Monitor";
    module_name[6]="Integrator";
 
    displayWidth=1280;
    displayHeight=1024;

    BeginningOfBurst=true;
  
    for (int iMod=0; iMod<4; iMod++) {
      nHVgroups[iMod]=1;
      for (int i=0; i<100; i++) HVgroup[iMod][i]=0.0;
      for (int i=0; i<1600; i++) hvGroupRun[iMod][i]=0;
    }
    for (int i=0; i<1600; i++) AttnIn[i]=0;

    bunchPatternSet=false;
    for (int i=0; i<30; i++) {
      bunchPattern[i]=-1.0e10;      
    }
    EH.SetFirstBunch(0);
  }

  ProcessingStatus(bool useFit, bool useCherStream) { // Constructor //
    CurrentRun = 0;
    DumpMode=0; 
    DumpCount=0; 
    Nbunches=0;
    nFilled=0;
    ReportMode = 10001;
    sprintf (dataFolder, "%s", "../data");
    sprintf (logbookFile, "%s", "../doc/runbook_nov08.elog");
    useFitFiles=useFit;
    useCherenkovStream=useCherStream;
    SetCases();
    numberOfRuns=0;
    runIndex=0;
    for (int i=0;i<10000;i++) SelectedRuns[i]=-1;
    maxLogbookIntensity=0.0;
    displayWidth=1280;
    displayHeight=1024;

    BeginningOfBurst=true;

    module_name[0]="HEC";
    module_name[1]="EMEC";
    module_name[2]="FCal025";
    module_name[3]="FCal010";
    module_name[4]="Beam";
    module_name[5]="Monitor";
    module_name[6]="Integrator";
    for (int iMod=0; iMod<4; iMod++) {
      nHVgroups[iMod]=1;
      for (int i=0; i<100; i++) HVgroup[iMod][i]=0.0;
      for (int i=0; i<1600; i++) hvGroupRun[iMod][i]=0;
    }
    for (int i=0; i<1600; i++) AttnIn[i]=0;
    bunchPatternSet=false;
    for (int i=0; i<30; i++) {
      bunchPattern[i]=-1.0e10;      
    }
    EH.SetFirstBunch(0);
  }

  void SetCases() {
    nCases=0;
    pauseCase[nCases]="** Stop program, keep plots";            		  pauseMode[nCases]=-1;      activeMode[nCases]=true;  nCases++;
    pauseCase[nCases]="** Quit root for new run";                                 pauseMode[nCases]= 0;      activeMode[nCases]=true;  nCases++;
    pauseCase[nCases]="** Display all event's signals together";                  pauseMode[nCases]= 1;      activeMode[nCases]=true;  nCases++;
    pauseCase[nCases]="** No interrupts";                                         pauseMode[nCases]= 2;      activeMode[nCases]=true;  nCases++;
    pauseCase[nCases]="** Special debug";                                         pauseMode[nCases]= 3;      activeMode[nCases]=true;  nCases++;
    pauseCase[nCases]="** Next Burst   ";                                         pauseMode[nCases]= 4;      activeMode[nCases]=true;  nCases++;
    pauseCase[nCases]="** Next Signal event";                                     pauseMode[nCases]= 5;      activeMode[nCases]=true;  nCases++;
    pauseCase[nCases]="** Next Burst Signal event";                               pauseMode[nCases]= 6;      activeMode[nCases]=true;  nCases++;
    pauseCase[nCases]="** Next Run first Signal event";                           pauseMode[nCases]= 7;      activeMode[nCases]=true;  nCases++;
    pauseCase[nCases]="** Display all event's signals bunch-by-bunch";            pauseMode[nCases]= 10016;  activeMode[nCases]=false;  nCases++;
    pauseCase[nCases]="Pedestal windows setting control";                         pauseMode[nCases]= 10002;  activeMode[nCases]=false;  nCases++;
    pauseCase[nCases]="Follow cleaning of pedestal windows";                      pauseMode[nCases]= 10003;  activeMode[nCases]=false;  nCases++;
    pauseCase[nCases]="Calculating bunch time based on shape fit data";           pauseMode[nCases]= 10004;  activeMode[nCases]=false;  nCases++;
    pauseCase[nCases]="Detais about response computation based on Fit results";   pauseMode[nCases]= 10005;  activeMode[nCases]=false;  nCases++;
    pauseCase[nCases]="Event display tuneup";                                     pauseMode[nCases]= 10006;  activeMode[nCases]=false;  nCases++;
    pauseCase[nCases]="Bump Filtering tuneup";                                    pauseMode[nCases]= 10007;  activeMode[nCases]=false;  nCases++;
    pauseCase[nCases]="Fitting procedure, global";                                pauseMode[nCases]= 10008;  activeMode[nCases]=false; nCases++;
    pauseCase[nCases]="Fitting procedure, local";                                 pauseMode[nCases]= 10009;  activeMode[nCases]=false; nCases++;
    pauseCase[nCases]="Finding bunch on module level";                            pauseMode[nCases]= 10010;  activeMode[nCases]=false;  nCases++;
    pauseCase[nCases]="Signals module-by-module in every event";                  pauseMode[nCases]= 10011;  activeMode[nCases]=false;  nCases++;
    pauseCase[nCases]="Calculation of the Global Phase";                          pauseMode[nCases]= 10012;  activeMode[nCases]=false;  nCases++;
    pauseCase[nCases]="Time slices preparation";                     		  pauseMode[nCases]= 10013;  activeMode[nCases]=false;  nCases++;
    pauseCase[nCases]="Cherenkov Stream extraction";                     	  pauseMode[nCases]= 10014;  activeMode[nCases]=false;  nCases++;
    pauseCase[nCases]="FillHistogram tuning";		                     	  pauseMode[nCases]= 10015;  activeMode[nCases]=false;  nCases++;
    pauseCase[nCases]="Display selected channel";                                 pauseMode[nCases]= 10021;  activeMode[nCases]=false;  nCases++;
    pauseCase[nCases]="Cherenkov sum in event and integrators";                   pauseMode[nCases]= 10022;  activeMode[nCases]=false;  nCases++;
    pauseCase[nCases]="Bunch fit done";                                           pauseMode[nCases]= 10023;  activeMode[nCases]=false;  nCases++;
    pauseCase[nCases]="Basic pulse params (A, half-width etc)";                   pauseMode[nCases]= 10024;  activeMode[nCases]=false;  nCases++;
    pauseCase[nCases]="Module Sum vs Integrator check";                           pauseMode[nCases]= 10025;  activeMode[nCases]=false;  nCases++;
    pauseCase[nCases]="Event done";                                               pauseMode[nCases]= 10100;  activeMode[nCases]=false;  nCases++;
    pauseCase[nCases]="Spill done";                                               pauseMode[nCases]= 10200;  activeMode[nCases]=false;  nCases++;
    pauseCase[nCases]="Run done";                                                 pauseMode[nCases]= 10300;  activeMode[nCases]=false;  nCases++;
    pauseCase[nCases]="FillHistogram problem!";		                     	  pauseMode[nCases]= 19000;  activeMode[nCases]=true;  nCases++;
    
  }

  double GetHVmod(int iMod) {
    return HVmod[iMod];
  }

  double GetTdrift(int iMod) {
    if (iMod<0 || iMod>3) return 0.0;
    return Tdrift[iMod];   
  }

  void SetOnline(bool b) {
    isOnline=b;
  }
  bool GetOnline() {
    return isOnline;
  }


  bool SetRun(int Run, bool myOnline) { 
// returns false if some of RUN related files not loaded properly //
// more to be moved here!                                         //
    bool result=true;
    CurrentRun = Run;
    firstRunEvent=true;
    firstBurstEvent=true;
    firstRunBurst=true;

    FirstSpillSumm=-1;    
    FirstSpillIntens=-1;    
    result = LoadLogbookRecord(Run, myOnline);

    //printf("[ProcessingStatus->SetRun] Cherenkov HV set to %.2fkV\n", RH.HVbeam[3]);

    if (Run>650 && !myOnline) {
    //  result = result && LoadHVrecord(Run);
    //  result = result && LoadSummaryDataNew();
    //  result = result && LoadIntensityDataNew();

      MakeSpillSummary();

      for (int is=0; is<300; is++) {
        if (SpillStatusS[is]==0) {
          if (CherHV[is]!=0.) RH.HVbeam[3]=CherHV[is];
          //printf("[ProcessingStatus->SetRun] Cherenkov HV set to %.2fkV\n", RH.HVbeam[3]);
          break;
        }
      }
    }

    for (int iMod=0; iMod<4; iMod++) {
      fillModule1[iMod]=true;
      if (RH.HVset[iMod]==0.0) fillModule1[iMod]=false;
      HVchozenSet[iMod]=false;
    }

    return result;
  }

  bool LoadHVrecord(int Run) {
//  loads HV sytem voltage and current for every HV channel and every spill; doing some checks also;
    for (int i=0; i<300; i++) {
      for (int iMod=0; iMod<4; iMod++) {
        for (int iChan=0; iChan<4; iChan++) {
          SpillStatusHV[i][iMod][iChan]=-1;
          SpillStatusHC[i][iMod][iChan]=-1;
        }
      }
    }

    for (int iMod=0; iMod<4; iMod++) {
      int nChan=4;
      if (iMod==1) nChan=3;

      for (int iChan=0; iChan<nChan; iChan++) {
     
        char VoltageFileName[100];
        char CurrentFileName[100];
        sprintf(VoltageFileName, "reports/spill/%s_04_2010_HV_%d.spill", module_name[iMod], iChan);
        sprintf(CurrentFileName, "reports/spill/%s_04_2010_HC_%d.spill", module_name[iMod], iChan);

        std::ifstream indataV;
        indataV.open(VoltageFileName, ios::in);
        if (!indataV.is_open()) {
          printf("[ProcessingStatus->LoadHVrecord] HV-Voltage file %s not open!\n",VoltageFileName);
          indataV.clear();
          return false; 
        }
        if (!indataV.good()) {
          printf("[ProcessingStatus->LoadHVrecord] HV-Voltage file %s is bad!\n",VoltageFileName);
          indataV.close();
          return false; 
        }

        std::ifstream indataC;
        indataC.open(CurrentFileName, ios::in);
        if (!indataC.is_open()) {
          printf("[ProcessingStatus->LoadHVrecord] HV-Current file %s not open!\n",CurrentFileName);
          indataC.clear();
          indataV.close();
          return false; 
        }
        if (!indataC.good()) {
          printf("[ProcessingStatus->LoadHVrecord] HV-Current file %s is bad!\n",CurrentFileName);
          indataC.close();
          indataV.close();
          return false; 
        }


        int RunFile=0;
        int SpillFile=0;
        int IP=0;
        int IA=0;
        int IX=0;
        double Val=0.;
        char c1[100];
        char c2[100];
        char c3[100];
        char c4[100];
        char c5[100];

        //printf("[ProcessingStatus->LoadHVrecord] loading Voltage file %s\n", VoltageFileName);
        while (RunFile<=Run && !indataV.eof()) {
          indataV >> RunFile >> SpillFile >> IP >> IA 
                  >> c1 >> c2 >> c3;
          //if (RunFile==Run) cout << "run=" << RunFile << " spill#" << SpillFile << " (" << IP << ", " << IA << ") " << c1 << c2 << c3 << endl;
          float ValMAX=-100000;
          float ValMIN=10000;
                 
          for (int i=IP; i<=IA && !indataV.eof(); i++) {
            indataV >> IX >> Val >> c4 >> c5;
            if (RunFile==Run) {
              //cout << IX << " (" << IP << "," << IA << ") hv=" << Val << " " << c4 << " " << c5 << endl;
            }
            if (Val>ValMAX) {ValMAX=Val; HVmonitorVoltIndexMAX[SpillFile][iMod][iChan]=IX;}
            if (Val<ValMIN) {ValMIN=Val; HVmonitorVoltIndexMIN[SpillFile][iMod][iChan]=IX;}
          }
          if (RunFile==Run) {
            HVmonitorVoltageMIN[SpillFile][iMod][iChan]=ValMIN;
            HVmonitorVoltageMAX[SpillFile][iMod][iChan]=ValMAX;
            if (SpillStatusHV[SpillFile][iMod][iChan]==-1) {
              SpillStatusHV[SpillFile][iMod][iChan]=0;
              if (RH.HVset[iMod]>0.0 && ValMIN<0.9*RH.HVset[iMod]) {
                //printf("    Run#%d Spill#%3d %s#%d HVmin=%5.1f HVmax=%5.1f HVlog=%5.1f  -> Low!\n", 
                //          RunFile,  SpillFile, module_name[iMod], iChan, ValMIN, ValMAX, RH.HVset[iMod]);
                SpillStatusHV[SpillFile][iMod][iChan]=1;
              } else if (RH.HVset[iMod]>0.0 && ValMAX>1.1*RH.HVset[iMod]) {
                //printf("    Run#%d Spill#%3d %s#%d HVmin=%5.1f HVmax=%5.1f HVlog=%5.1f  -> High!\n", 
                //          RunFile,  SpillFile, module_name[iMod], iChan, ValMIN, ValMAX, RH.HVset[iMod]);
                SpillStatusHV[SpillFile][iMod][iChan]=2;
              } else {
              }
            } else { // duplicate
              SpillStatusHV[SpillFile][iMod][iChan]=3;
              if (iMod==0 && iChan==0) printf("    Run#%d Spill#%3d  -> Duplicate\n", RunFile, SpillFile);
            }
            //printf("  spill#%3d iMod=%d iChan=%d  status=%d\n", SpillFile, iMod, iChan, SpillStatusHV[SpillFile][iMod][iChan]);
          }
        }

        // fix typical situation when several spills covered with 2 same measurements and LAST measurement is bad
        // in this situation only last spill must be dropped!

        for (int iSpill=0; iSpill<299; iSpill++) {
          if (SpillStatusHV[iSpill][iMod][iChan]==1 &&                                                 // voltage considered low
              HVmonitorVoltIndexMIN[iSpill][iMod][iChan]>HVmonitorVoltIndexMAX[iSpill][iMod][iChan] && // lowest is second
              HVmonitorVoltageMAX[iSpill][iMod][iChan]>0.9*RH.HVset[iMod]) { // highest is ok -> check if following spill has same HVmax and HVmin indexes
            for (int iSpill1=iSpill+1; iSpill1<300; iSpill1++) {
              if (SpillStatusHV[iSpill1][iMod][iChan]>-1) { // there is a spill after iSpill...
                if (HVmonitorVoltIndexMIN[iSpill][iMod][iChan]==HVmonitorVoltIndexMIN[iSpill1][iMod][iChan] &&
                    HVmonitorVoltIndexMAX[iSpill][iMod][iChan]==HVmonitorVoltIndexMAX[iSpill1][iMod][iChan]) { // ... and it has same indexes ...
                  SpillStatusHV[iSpill][iMod][iChan]=0; // recover it!
                  //printf(" Run#%d Spill#%3d recovered !\n", RunFile, iSpill); 
                  break;
                } // spill shall be recovered
              } // next spill is present
            } // looking for the next spill
          } // spill HV considered low
        } // iSpill loop

        RunFile=0;
         
        //printf("[ProcessingStatus->LoadHVrecord] loading Current file %s\n", CurrentFileName);
        while (RunFile<=Run && !indataC.eof()) {
          indataC >> RunFile >> SpillFile >> IP >> IA 
                  >> c1 >> c2 >> c3;
          float ValMAX=-100000;
          float ValMIN=10000;       
          for (int i=IP; i<=IA && !indataC.eof(); i++) {
            indataC >> IX >> Val >> c4 >> c5;
            if (Val>ValMAX) ValMAX=Val;
            if (Val<ValMIN) ValMIN=Val;
          }
          if (RunFile==Run) {
            //cout  << "     " << RunFile << " " << SpillFile << " min=" << ValMIN << " max=" << ValMAX<< " " << c1 << c2 << c3 << endl;
            HVmonitorCurrentMIN[SpillFile][iMod][iChan]=ValMIN;
            HVmonitorCurrentMAX[SpillFile][iMod][iChan]=ValMAX;
            if (SpillStatusHC[SpillFile][iMod][iChan]==-1) {
              SpillStatusHC[SpillFile][iMod][iChan]=0;
            } else { // duplicate
              SpillStatusHC[SpillFile][iMod][iChan]=3;
            }
          }
        }

        indataC.close();
        indataV.close();
        
      }
    }
    printf("[LoadHVrecord] run#%d HV data loaded\n", Run);
    return true;
  }

  int GetSpillStatusHV(int spill, int iMod, int iChan) { // checks if specific channel is ok
    if (spill<0 || spill>=300 || iMod<0 || iMod>3 || iChan<0) {
      printf("[GetSpillHVStatus] wrong input : spill#%d module#%d channel#%d\n", spill, iMod, iChan); 
      return -1;
    }
    if (iMod==1) {
      if (iChan>2) {
        printf("[GetSpillHVStatus] wrong input : spill#%d module#%d channel#%d\n", spill, iMod, iChan); 
        return -1;
      }
    } else {
      if (iChan>3) {
        printf("[GetSpillHVStatus] wrong input : spill#%d module#%d channel#%d\n",spill, iMod, iChan); 
        return -1;
      }
    }
    return SpillStatusHV[spill][iMod][iChan];
  }

  int GetSpillStatusHC(int Spill, int iMod, int iChan) { // checks if specific channel is ok
    if (Spill<0 || Spill>=300 || iMod<0 || iMod>3 || iChan<0) {
      printf("[GetSpillHCStatus] wrong input : spill#%d module#%d channel#%d\n", Spill, iMod, iChan); 
      return -1;
    }
    if (iMod==1) {
      if (iChan>2) {
        printf("[GetSpillHCStatus] wrong input : spill#%d module#%d channel#%d\n", Spill, iMod, iChan); 
        return -1;
      }
    } else {
      if (iChan>3) {
        printf("[GetSpillHCStatus] wrong input : spill#%d module#%d channel#%d\n",Spill, iMod, iChan); 
        return -1;
      }
    }
    return SpillStatusHC[Spill][iMod][iChan];
  }

  int GetSpillStatusHV1(int Spill, int iMod) { // checks if specific module is 100% ok
    int nChan=4;
    if (iMod==1) nChan=3;
    for (int iChan=0; iChan<nChan; iChan++) {
      //printf("[GetSpillStatusHV1] iMod=%d iChan=%d\n", iMod, iChan);
      if (GetSpillStatusHV(Spill, iMod, iChan)!=0) return -1;
    }
    return 0;
  }  

  int GetSpillStatusHV2(int Spill) { // checks if all modules are 100% ok
    for (int iMod=0; iMod<4; iMod++) {
      if (GetSpillStatusHV1(Spill, iMod)!=0) return -1;
    }
    return 0;
  }  

  int GetSpillStatusHC1(int Spill, int iMod) { // checks if specific module is 100% ok
    int nChan=4;
    if (iMod==1) nChan=3;
    for (int iChan=0; iChan<nChan; iChan++) {
      //printf("[GetSpillStatusHC1] iMod=%d iChan=%d\n", iMod, iChan);
      if (GetSpillStatusHC(Spill, iMod, iChan)!=0) return -1;
    }
    return 0;
  }  

  int GetSpillStatusHC2(int Spill) { // checks if all modules are 100% ok
    for (int iMod=0; iMod<4; iMod++) {
      if (GetSpillStatusHC1(Spill, iMod)!=0) return -1;
    }
    return 0;
  }  

  double GetSpillHVmin(int Spill, int iMod, int iChan) {
    if (iMod==1 && iChan==3) return -1.;
    //printf("[GetSpillHVmin]  spill#%d mod=%d chan=%d\n", Spill, iMod, iChan);
    if (GetSpillStatusHV(Spill, iMod, iChan)>=0) {
      return HVmonitorVoltageMIN[Spill][iMod][iChan];
    } else {
      return -1.;
    }
  }
  double GetSpillHVmax(int Spill, int iMod, int iChan) {
    if (iMod==1 && iChan==3) return -1.;
    //printf("[GetSpillHVmax]  spill#%d mod=%d chan=%d\n", Spill, iMod, iChan);
    if (GetSpillStatusHV(Spill, iMod, iChan)>=0) {
      return HVmonitorVoltageMAX[Spill][iMod][iChan];
    } else {
      return -1.;
    }
  }
  double GetSpillCHVmax(int iSpill, int iMod, int iChan) {
    if (iMod==1 && iChan==3) return -1.;
    //printf("[GetSpillCHVmax]  spill#%d mod=%d chan=%d\n", iSpill, iMod, iChan);
    if (GetSpillStatusHC(iSpill, iMod, iChan)>=0) {
      return HVmonitorCurrentMAX[iSpill][iMod][iChan];
    } else {
      return -1.;
    }
  }
//========================================
  bool LoadIntensityData() {

    FirstSpillIntens=-1;

    std::ifstream inSlow;
    inSlow.open("reports/slow2010_spill.txt");
    if (!inSlow.is_open()) {
      inSlow.clear();
      return false;
    }
    if (!inSlow.good()) {
      inSlow.clear();
      return false;
    }

    for (int i=0; i<300; i++) {
      SpillStatusI[i]=-1;
      ionChamb[i]=0.;
      SEDsumX[i]=0.;
      SEDsumY[i]=0.;
      for (int j=0;j<16;j++) {SEDx[j][i]=0.0; SEDy[j][i]=0.0;}
    }

    int RunFile=0;
    int SpillFile=0;
    while (RunFile<=CurrentRun && !inSlow.eof()) {
      double ionCh, calib;
      double sedx[17];
      double sedy[17];
      inSlow >> RunFile >> SpillFile >> ionCh >> calib;
      for (int i=0; i<17; i++) inSlow >> sedx[i];
      for (int i=0; i<17; i++) inSlow >> sedy[i];

      if (RunFile==CurrentRun) {

        if (FirstSpillSumm==-1) {
          printf("LoadSummaryData must be called prior to this, FirstSpillSumm=%d!\n", FirstSpillSumm);
          exit(-1);
        }
        if (FirstSpillIntens==-1) FirstSpillIntens=SpillFile;

         //printf("Intensity: run#%3d spill#%3d ionC=%8.1f ", RunFile, SpillFile, ionCh*calib);
        if (SpillFile>=300 || SpillFile<0) {
          printf("[ReportStatus->LoadIntensityData] Bad spill number %d\n", SpillFile);
          exit(-1);
        }

        SpillFile+=(FirstSpillSumm-FirstSpillIntens);

        ionChamb[SpillFile]=ionCh*calib;
        ionCalib[SpillFile]=calib;

        if (SpillStatusI[SpillFile]==-1) { // if first time in the Run...
          SpillStatusI[SpillFile]=0;  // good 
        } else { // multiple records !
          SpillStatusI[SpillFile]=3;  
        }
        //printf(" status=%d\n", SpillStatusI[SpillFile]);

        for (int i=0; i<16; i++) {SEDx[i][SpillFile]=sedx[i]; SEDsumX[SpillFile]=sedx[16];}
        for (int i=0; i<16; i++) {SEDy[i][SpillFile]=sedy[i]; SEDsumY[SpillFile]=sedy[16];}
      }
    }

    printf("[ReportStatus->LoadIntensityData] done with run#%d, first spill#%d\n", CurrentRun, FirstSpillIntens);

    inSlow.close();
    inSlow.clear();
    return true;
  }

//========================================
  bool LoadIntensityDataNew() {

    char nameSlow[100];
    int SpillEvent;
    FirstSpillIntens=-1;

    std::ifstream inSlow;
    sprintf(nameSlow, "reports/runs/intensity2010_spill.run%04d.txt", CurrentRun);
    inSlow.open(nameSlow, ios::in);
    if (!inSlow.is_open()) {
      printf("[LoadIntensityDataNew] file %s open failure\n", nameSlow);
      inSlow.clear();
      return false;
    }
    if (!inSlow.good()) {
      printf("[LoadIntensityDataNew] file %s is bad\n", nameSlow);
      inSlow.clear();
      return false;
    }

    for (int i=0; i<300; i++) {
      SpillStatusI[i]=-1;
      ionChamb[i]=0.;
      SEDsumX[i]=0.;
      SEDsumY[i]=0.;
      for (int j=0;j<16;j++) {SEDx[j][i]=0.0; SEDy[j][i]=0.0;}
    }

    int RunFile=0;
    int SpillFile=0;
    while (!inSlow.eof()) {
      double ionCh, calib;
      double sedx[17];
      double sedy[17];
      inSlow >> RunFile >> SpillEvent >> SpillFile >> ionCh >> calib;
      for (int i=0; i<17; i++) inSlow >> sedx[i];
      for (int i=0; i<17; i++) inSlow >> sedy[i];

      if (inSlow.eof()) break;

      if (RunFile==CurrentRun) {

        if (FirstSpillSumm==-1) {
          printf("LoadSummaryData must be called prior to this, FirstSpillSumm=%d!\n", FirstSpillSumm);
          exit(-1);
        }
        if (FirstSpillIntens==-1) FirstSpillIntens=SpillFile;

         //printf("Intensity: run#%3d spill#%3d ionC=%8.1f ", RunFile, SpillFile, ionCh*calib);
        if (SpillFile>=300 || SpillFile<0) {
          printf("[ReportStatus->LoadIntensityData] Bad spill number %d\n", SpillFile);
          exit(-1);
        }

        //printf("[LoadIntensityData] old SpillFile=%d ", SpillFile);

        SpillFile+=(FirstSpillSumm-FirstSpillIntens);

        //printf(" Corrected=%d SpillEvent=%d\n", SpillFile, SpillEvent);


        ionChamb[SpillFile]=ionCh*calib;
        ionCalib[SpillFile]=calib;

        if (SpillStatusI[SpillFile]==-1) { // if first time in the Run...
          SpillStatusI[SpillFile]=0;  // good 
        } else { // multiple records !
          SpillStatusI[SpillFile]=3;  
        }
        //printf(" status=%d\n", SpillStatusI[SpillFile]);

        for (int i=0; i<16; i++) {SEDx[i][SpillFile]=sedx[i]; SEDsumX[SpillFile]=sedx[16];}
        for (int i=0; i<16; i++) {SEDy[i][SpillFile]=sedy[i]; SEDsumY[SpillFile]=sedy[16];}
      } else {
        printf("[ReportStatus->LoadIntensityData] defective file %s, RunFile=%d\n", 
             nameSlow, RunFile);
        exit(-1); 
      }
    }

// sweep for duplicates //
    for (int i1=0; i1<299; i1++) {
      if (SpillStatusI[i1]>-1) { // spill #i1 is in record, look for closest next!
        for (int i2=i1+1; i2<300; i2++) {
          if (SpillStatusI[i2]>-1) { // spill #i2 is in record, check for twins!
            int probe=0, same=0;
            for (int i=0; i<16; i++) { // compare SED readings
              if (SEDx[i][i1]==SEDx[i][i2]) same++;
              probe++;
              if (SEDy[i][i1]==SEDy[i][i2]) same++;
              probe++;
            }
            if (ionChamb[i1]==ionChamb[i2]) same++;
            probe++;
            if (probe==same) {
              SpillStatusI[i1]=3;
              SpillStatusI[i2]=3;
            }
            break; 
          }
        }
      }
    }

    printf("[ReportStatus->LoadIntensityData] done with run#%d, first spill#%d\n", CurrentRun, FirstSpillIntens);

    inSlow.close();
    inSlow.clear();
    return true;
  }

  int GetIntensityStatus(int spill) {
    if (spill<0 || spill>299) {
      return -1;
    }
    return SpillStatusI[spill];
  }

  int GetSummaryStatus(int spill) {
    if (spill<0 || spill>299) {
      return -1;
    }
    return SpillStatusS[spill];
  }

   double GetIonChamber(int spill) {
    if (GetIntensityStatus(spill)==0) {
      return ionChamb[spill];
    }
    return 0.;
  }

  double GetSEDsliceX(int spill, int slice) {
    if (GetIntensityStatus(spill)==0) {
      if (slice>-1 && slice<16) {
        return SEDx[slice][spill];
      }
    }
    return 0.;
  }

  double GetSEDsliceY(int spill, int slice) {
    if (GetIntensityStatus(spill)==0) {
      if (slice>-1 && slice<16) {
        return SEDy[slice][spill];
      }
    }
    return 0.;
  }


  double GetSEDsumX(int spill) {
    if (GetIntensityStatus(spill)==0) {
      return SEDsumX[spill];
    }
    return 0.;
  }

  double GetSEDsumY(int spill) {
    if (GetIntensityStatus(spill)==0) {
      return SEDsumY[spill];
    }
    return 0.;
  }

  double GetSEDcogX(int spill) {
    if (GetIntensityStatus(spill)==0) {
      double s=0.;
      if (SEDsumX[spill]>0.) {
        for (int i=0; i<16; i++) {
          s+=SEDx[i][spill]*(i-7.5);
        }
        return s/SEDsumX[spill];
      }
    }
    return -10000.;
  }

  double GetSEDcogY(int spill) {
    if (GetIntensityStatus(spill)==0) {
      double s=0.;
      if (SEDsumY[spill]>0.) {
        for (int i=0; i<16; i++) {
          s+=SEDy[i][spill]*(i-7.5);
        }
        return s/SEDsumY[spill];
      }
    }
    return -10000.;
  }

  double GetSEDrmsX(int spill) {
    if (GetIntensityStatus(spill)==0) {
      double s2=0.;
      double s1=0.;
      if (SEDsumX[spill]>0.) {
        for (int i=0; i<16; i++) {
          s2+=SEDx[i][spill]*(i-7.5)*(i-7.5);
          s1+=SEDx[i][spill]*(i-7.5);
        }
        s1/=SEDsumX[spill];
        s2/=SEDsumX[spill];
        s2-=s1*s1;
        return sqrt(s2);
      }
    }
    return -10000.;
  }

  double GetSEDrmsY(int spill) {
    if (GetIntensityStatus(spill)==0) {
      double s2=0.;
      double s1=0.;
      if (SEDsumY[spill]>0.) {
        for (int i=0; i<16; i++) {
          s2+=SEDy[i][spill]*(i-7.5)*(i-7.5);
          s1+=SEDy[i][spill]*(i-7.5);
        }
        s1/=SEDsumY[spill];
        s2/=SEDsumY[spill];
        s2-=s1*s1;
        return sqrt(s2);
      }
    }
    return -10000.;
  }

  bool LoadSummaryData() {

    FirstSpillSumm=-1;
    
    std::ifstream inSumm;
    inSumm.open("reports/runstamp2010_spill.txt");
    if (!inSumm.is_open()) {
      inSumm.clear();
      return false;
    }
    if (!inSumm.good()) {
      inSumm.clear();
      return false;
    }

    for (int i=0; i<300; i++) {
      SpillStatusS[i]=-1;
      CherInt[i]=0.0;
      CherHV[i]=0.;
      EvtNum[i][0]=-1;
      EvtNum[i][1]=-1;
    }

    int RunFile=0;
    int SpillFile=0;
    int SpillFile1=0;
    time_t t0, t1;
    int evt0, evt1;
    double chInt, chHV;
    while (RunFile<=CurrentRun && !inSumm.eof()) {
      inSumm >> RunFile >> SpillFile >> evt0 >> t0 >> SpillFile1 >> evt1 >> t1 >> chInt >> chHV ;
      if (RunFile==CurrentRun) {

        if (FirstSpillSumm==-1) FirstSpillSumm=SpillFile;
        if (SpillFile>=300 || SpillFile<0) {
          printf("[ReportStatus->LoadSummaryData] Bad spill number %d\n", SpillFile);
          exit(-1);
        }
        if (SpillFile1>=300 || SpillFile1<0) {
          printf("[ReportStatus->LoadSummaryData] Bad spill number %d\n", SpillFile1);
          exit(-1);
        }
 
        EvtNum[SpillFile][0]=evt0;
        EvtNum[SpillFile][1]=evt1;

        CherInt[SpillFile]=chInt;
        CherHV[SpillFile]=chHV;

        if (SpillStatusS[SpillFile]==-1) { // if first time in the Run...
          SpillStatusS[SpillFile]=0;  // good 
        } else { // multiple records !
          SpillStatusS[SpillFile]=3;  
        }

        if (SpillFile != SpillFile1) {
          SpillStatusS[SpillFile]=5;
          SpillStatusS[SpillFile1]=5;
        }

        if (evt1-evt0<70 || evt1-evt0>110) SpillStatusS[SpillFile]=1;
        if (t1-t0<1 || t1-t0>3) SpillStatusS[SpillFile]=4;

        if (chHV>3.)  SpillStatusS[SpillFile]=8;
      }
    }

    printf("[ReportStatus->LoadSummaryData] done with run#%d, first spill#%d\n", CurrentRun, FirstSpillSumm);

    inSumm.close();
    inSumm.clear();
    return true;
  }

  bool LoadSummaryDataNew() {

    char nameSumm[100];
    FirstSpillSumm=-1;
    
    std::ifstream inSumm;
    sprintf(nameSumm, "reports/runs/bunchstamp2010_spill.run%04d.txt", CurrentRun);
    inSumm.open(nameSumm, ios::in);
    if (!inSumm.is_open()) {
      printf("[LoadSummaryDataNew] file %s open failure\n", nameSumm);
      inSumm.clear();
      return false;
    }
    if (!inSumm.good()) {
      printf("[LoadSummaryDataNew] file %s is bad\n", nameSumm);
      inSumm.clear();
      return false;
    }

    for (int i=0; i<300; i++) {
      SpillStatusS[i]=-1;
      CherInt[i]=0.0;
      CherHV[i]=0.;
      EvtNum[i][0]=-1;
      EvtNum[i][1]=-1;
      for (int j=0;j<6;j++) CherFill[i][j]=0.0;
    }

    int RunFile=0;
    int SpillFile=0;
    int SpillFile1=0;
    time_t t0, t1;
    int evt0, evt1;
    double chInt, chHV, tStart, tCOG, tEnd;
    double CherFill00[6];
    while (!inSumm.eof()) {
      inSumm >> RunFile >> SpillFile >> evt0 >> t0 >> SpillFile1 >> evt1 >> t1 >> chInt >> chHV ;
      //cout << RunFile << " " << SpillFile << " " << evt0 << " " << t0 << " " << SpillFile1 << " " 
      //     << evt1 << " " << t1 << " " <<  chInt << " " << chHV << endl;
      for (int i=0; i<6; i++) inSumm >> CherFill00[i];
      //for (int i=0; i<6; i++) cout << CherFill00[i] << " ";

      inSumm >> tStart >> tCOG >> tEnd;
      //cout << tStart << " " << tCOG << " " << tEnd << endl;
      if (inSumm.eof()) break;

      if (RunFile==CurrentRun) {
        if (FirstSpillSumm==-1) FirstSpillSumm=SpillFile;
        if (SpillFile>=300 || SpillFile<0) {
          printf("[ReportStatus->LoadSummaryData] Bad spill number %d\n", SpillFile);
          exit(-1);
        }
        if (SpillFile1>=300 || SpillFile1<0) {
          printf("[ReportStatus->LoadSummaryData] Bad spill number %d\n", SpillFile1);
          exit(-1);
        }
 
        EvtNum[SpillFile][0]=evt0;
        EvtNum[SpillFile][1]=evt1;

        CherInt[SpillFile]=chInt;
        CherHV[SpillFile]=chHV;

        for (int i=0; i<6; i++) CherFill[SpillFile][i]=CherFill00[i];

        SpillStart[SpillFile]=tStart;
        SpillCOG[SpillFile]=tCOG;
        SpillEnd[SpillFile]=tEnd;

        double SampleMax=-100000.;
        double SampleMean=0.;
        double SampleRMS=0.0;
        double ww=0.0;
        int    iMax=-1;
        int    iNext=-1;
        for (int i=0; i<6; i++) {
          if (SampleMax<CherFill00[i]) {
            SampleMax=CherFill00[i];
            iMax=i;
            iNext=i+1;
            if (iNext==6) iNext=0;
          }
        }

        for (int i=0; i<6; i++) {
          //if (i!=iMax && i!=iNext) {
            double sample=CherFill00[i];
            ww+=1.0;
            SampleMean+=sample;
            SampleRMS+=sample*sample;
          //}
        }        

        SampleMean/=ww;
        SampleRMS/=ww;
        SampleRMS-=SampleMean*SampleMean;
        SampleRMS=sqrt(SampleRMS);


        if (SpillStatusS[SpillFile]==-1) { // if first time in the Run...
          SpillStatusS[SpillFile]=0;  // good 
        } else { // multiple records !
            SpillStatusS[SpillFile]=3;  
        }

        if (SpillFile != SpillFile1) {
          SpillStatusS[SpillFile]=5;
          SpillStatusS[SpillFile1]=5;
        }

        if (evt1-evt0<70 || evt1-evt0>110) SpillStatusS[SpillFile]=1;
        if (t1-t0<1 || t1-t0>3) SpillStatusS[SpillFile]=4;
        if (chHV>3.)  SpillStatusS[SpillFile]=8;

        if (SampleMax<20. || SampleMax/SampleRMS<2.0) {
          if (CurrentRun<768 || CurrentRun>778) { // exclude HV scan on Cherenkov
            SpillStatusS[SpillFile]=7; 
          }
        }
      } else {  
        printf("[ReportStatus->LoadSummaryData] defective file %s, RunFile=%d\n", 
             nameSumm, RunFile);
        exit(-1); 
      }
    }

    printf("[ReportStatus->LoadSummaryData] done with run#%d, first spill#%d\n", CurrentRun, FirstSpillSumm);

    inSumm.close();
    inSumm.clear();
    return true;
  }

  double GetCherHV(int spill) {
    if (spill<0 || spill>=300) {
      printf("[GetCherenkovIntegral] wrong input : spill#%d\n", spill); 
      return -1;
    }
    return CherHV[spill];
  }

  double GetSpillStart(int spill) {
    if (spill<0 || spill>=300) {
      printf("[GetSpillStart] wrong input : spill#%d\n", spill); 
      return -1;
    }
    return SpillStart[spill];
  }
  double GetSpillCOG(int spill) {
    if (spill<0 || spill>=300) {
      printf("[GetSpillStart] wrong input : spill#%d\n", spill); 
      return -1;
    }
    return SpillCOG[spill];
  }
  double GetSpillEnd(int spill) {
    if (spill<0 || spill>=300) {
      printf("[GetSpillStart] wrong input : spill#%d\n", spill); 
      return -1;
    }
    return SpillEnd[spill];
  }




  double GetCherenkovIntegral(int spill) {
    if (spill<0 || spill>=300) {
      printf("[GetCherenkovIntegral] wrong input : spill#%d\n", spill); 
      return -1;
    }
    return CherInt[spill];
  }

  double GetCherFill(int spill, int isamp) {
    if (spill<0 || spill>=300 || isamp<0 || isamp>5) {
      printf("[GetCherFill] wrong input : spill#%d sample#%d\n", spill, isamp); 
      return 0.;
    }
    return CherFill[spill][isamp];
  }

  void MakeSpillSummary() {
    for (int jS=0; jS<300; jS++) {
      int StatusSum=0;
      int CountSum=0;
      for (int iMod=0; iMod<4; iMod++) {
        int nChan=4;
        if (iMod==1) nChan=3;
        for (int iChan=0; iChan<nChan; iChan++) {
          StatusSum+=SpillStatusHV[jS][iMod][iChan];
          StatusSum+=SpillStatusHC[jS][iMod][iChan];
          CountSum+=2;
        }
      }
      StatusSum+=SpillStatusI[jS];
      StatusSum+=SpillStatusS[jS];
      CountSum+=2;

      if (StatusSum != -CountSum) { // spill is present at least somewhere, not all are -1 //
        
        // set spill status for timing etc


        if (SpillStatusI[jS]!=0 || SpillStatusS[jS]!=0 || GetSpillStatusHV2(jS)!=0 || GetSpillStatusHC2(jS)!=0) {
        //printf("spill#%3d IntStat=%3d SumStat=%3d Ion=%8.1f SED=(%6.1f, %6.1f) Evts=(%4d, %4d) Cher=%7.2e\n", 
        //        jS, SpillStatusI[jS], SpillStatusS[jS], ionChamb[jS], SEDsumX[jS], SEDsumY[jS], 
       //         EvtNum[jS][0], EvtNum[jS][1], CherInt[jS]);

          for (int iMod=0; iMod<4; iMod++) {
            if (GetSpillStatusHV1(jS, iMod) != 0) { 
              printf("HV problem in %10s -> (", module_name[iMod]);
              int nChan=4;
              if (iMod==1) nChan=3;
              for (int iChan=0; iChan<nChan; iChan++) {
                printf(" %3d", GetSpillStatusHV(jS, iMod, iChan));
              }
              printf(")\n");
            }

          }

        }
      }
    }
//    printf("---------------------------------------\nPush!");
//    int iqq;
//    cin >> iqq;

    return;
  }

    

  int  GetRun()                     {return CurrentRun;}
  bool IsFirstRunEvent()            {return firstRunEvent;}
  bool IsFirstBurstEvent()          {return firstBurstEvent;}
  bool IsFirstRunBurst()            {return firstRunBurst;}
  void SetFirstRunEvent(bool set)   {firstRunEvent=set;}
  void SetFirstBurstEvent(bool set) {firstBurstEvent=set;}
  void SetFirstRunBurst(bool set)   {firstRunBurst=set;}
  
  void SetReportMode(int Mode)  {
    if (Mode==1) printf("************************************************************\n");
    printf("[ProcessingStatus->SetReportMode] Setting Mode to %d\n", Mode);
    if (Mode==1) printf("************************************************************\n");
    ReportMode=Mode;
  }
  int  GetReportMode()          {return ReportMode;}

  EventHeader *GetEventHeader() {return &((ProcessingStatus*)this)->EH;}
  BurstHeader *GetBurstHeader() {return &((ProcessingStatus*)this)->BH;}
  RunHeader   *GetRunHeader()   {return &((ProcessingStatus*)this)->RH;}

  int Get10mhz() {
    return EH.scaler10mhz;
  }
  int Get6mhz() {
    return EH.scaler6mhz;
  }
  int Get10mhzOld() {
    return EH.scaler10mhzOld;
  }
  int Get6mhzOld() {
    return EH.scaler6mhzOld;
  }

  void SetDataFolder(const char* Folder)    {sprintf (dataFolder,    "%s", Folder);}
  void SetLogbookFile(const char* FileName) {sprintf(logbookFile, "%s", FileName);}
  const char* GetDataFolder()               {return dataFolder;}
  const char* GetLogFile()                  {return logbookFile;}

  bool GetUseFit() {return useFitFiles;}

  void SetDisplay(int w, int h) {
    displayWidth=w;
    displayHeight=h;
  }

  int GetDisplayWidth()  {return displayWidth;}
  int GetDisplayHeight() {return displayHeight;}
//********************************************************************/
bool RunIsGood(int Run) {
  bool result = RH.Good>0;
  if (!result) {
    printf("[ProcessingStatus->RunIsGood] Run#%d has logbook status %d, skipping\n", Run, RH.Good);
    return false;
  }
  //result = (RH.HECvoltage[0]+RH.EMECvoltage[0]+
  //          RH.FCal025voltage[0]+RH.FCal010voltage[0]>0.0);
  if (!result) {
    printf("[rocessingStatus->RunIsGood] Run#%d has zero HV settings in all modules, skipping\n", Run);
    return false;
   }
  return result;
}
/********************************************************************/
bool LoadLocation() { //loads location of the analysis (laptop or desktop)
  std::ifstream locationFile;
  locationFile.open("reports/location.dat");
  if (!locationFile.is_open()) {
    locationFile.clear();
    printf("[ProcessingStatus->LoadLocation] location file not found;\n");
    exit(-1);
    return false;
  }
  locationFile >> location;
  locationFile.close();
  locationFile.clear();
  if (location==1) printf("[ProcessingStatus->LoadLocation] working on a laptop\n");
  else if (location==0) printf("[ProcessingStatus->LoadLocation] working on a desktop\n");
  else {
    printf("[ProcessingStatus->LoadLocation] location#%d not implemented\n", location);
    return false;
  }
  return true;
}
/********************************************************************/
int GetLocation() {return location;}

char* GetCommentString() {return &CommentString[0];}
/********************************************************************/
bool LoadLogbookRecord(int Run, bool myOnline) {

  std::ifstream logbookData;
//  char CommentString[200]; 2014-12-23
  logbookData.open(logbookFile, ios::in);
  if (!logbookData.is_open()) {
    printf("\n\n[LoadLogbookRecord] logbook file %s could not be open\n", logbookFile);
    logbookData.clear();
    return false;
  } else {
    if (ReportMode==1) printf("\n\n[LoadLogbookRecord] logbook file in use ->  %s\n\n", logbookFile);
  }

  while (! (CommentString[0] == '-' && 
            CommentString[1] == '-' && 
            CommentString[2] == '-') ) {
    logbookData >> CommentString;
  }

  int RunLog=-1; 
  int RunLogOld=-2;
  while (RunLog != Run && RunLog != 0) {

    logbookData >> RunLog;
    //printf(" [LoadLogbookRecord] loading data for run # %d, seeking run # %d\n", RunLog, Run);
    if (RunLogOld==RunLog) {
      printf("[LoadLogbookRecord] Logbook Format Error, check line for run#%d, looking for run#%d\n", RunLog, Run);
      exit(-1);
    }
    RunLogOld=RunLog;
    if (RunLog<460) {   // 2008.11 run e-logbook info

      logbookData >> RH.FCalInBeam
  		  >> RH.Good
		  >> RH.Attn
	  	  >> RH.HVset[0]
		  >> RH.HVset[1]
		  >> RH.HVset[2]
		  >> RH.HVmap[2]
		  >> RH.HVset[3]
		  >> RH.HVmap[3]
		  >> RH.BeamIntensityLog
		  >> RH.S123inBeam
		  >> RH.HVbeam[3]
		  >> RH.cherPressure
		  >> RH.HVbeam[0]
		  >> CommentString;
    } else if (RunLog<645) { // 2009.11 run e-logbook info
      logbookData >> RH.Good
		  >> RH.Attn
		  >> RH.S123inBeam
		  >> RH.HVset[2]
		  >> RH.HVset[3]
		  >> RH.HVset[0]
		  >> RH.HVset[1]
		  >> RH.HVbeam[0]
		  >> RH.HVbeam[3]
                  >> RH.cryoX[0]
                  >> RH.cryoX[1]
                  >> RH.cryoX[2]
		  >> RH.BeamIntensityCher
 		  >> RH.RateS4
		  >> RH.RateS123
                  >> RH.RateIonCh
		  >> CommentString;
                 RH.cherPressure=1.0;
                 if (RH.cryoX[2]<40.) {
                   RH.FCalInBeam=25;
                 } else if (RH.cryoX[2]>70. && RH.cryoX[2]<100.) {
                   RH.FCalInBeam=10;
                 } else {
                   RH.FCalInBeam=0;
                 }
     } else if (RunLog<848) { // 2010.04 run e-logbook info
      logbookData >> RH.Good
		  >> RH.Attn
		  >> RH.S123inBeam
		  >> RH.HVset[2]
		  >> RH.HVset[3]
		  >> RH.HVset[1]
		  >> RH.HVset[0]
		  >> RH.HVbeam[3]
                  >> RH.cryoX[0]
                  >> RH.cryoX[1]
                  >> RH.cryoX[2]
		  >> RH.BeamIntensityCher
 		  >> RH.RateS4
		  >> RH.RateS123
                  >> RH.RateIonCh
		  >> CommentString;

      for (int i=0; i<4; i++) RH.HVset[i]*=1000.; //convert to Volts!

      double gaps[4]={1.97, 2.10, 0.27, 0.12}; // detector gaps in mm
      //double mobility=0.14; // mm2/sec/V
      double mobility=0.1; // mm2/sec/V
      if (RunLog == Run) {
        for (int i=0; i<4; i++) {
          double Dspeed=mobility*RH.HVset[i]/gaps[i];
          if (Dspeed>0.) {
            Tdrift[i]=gaps[i]/Dspeed;
          } else {
            Tdrift[i]=0.;
          }
          //printf("Module#%d Tdtift=%f msec\n", i, Tdrift[i]*1000.);
          Tdrift[i]*=1.0e9; //convert to Beloved nanoseconds
        }
      }
      

      if (RunLog==780) RH.cherPressure=0.65;
      else if (RunLog==781) RH.cherPressure=0.70;
      else if (RunLog==782) RH.cherPressure=0.75;
      else if (RunLog==784) RH.cherPressure=0.80;
      else if (RunLog==785) RH.cherPressure=0.85;
      else if (RunLog==786) RH.cherPressure=0.90;
      else if (RunLog==787) RH.cherPressure=0.95;                 
      else RH.cherPressure=1.0;
      RH.HVbeam[0]=0.0;
      fillModule1[0]=false;
      fillModule1[1]=false;
      fillModule1[2]=false;
      fillModule1[3]=false;
      if (RH.HVset[0]!=0.0) fillModule1[0]=true;
      if (RH.HVset[1]!=0.0) fillModule1[1]=true;
      if (RH.HVset[2]!=0.0) fillModule1[2]=true;
      if (RH.HVset[3]!=0.0) fillModule1[3]=true;

      if (RH.cryoX[2]<40.) {
        RH.FCalInBeam=10;
        if (RH.HVset[3]!=0.0) fillModule1[3]=true;
        fillModule1[2]=false;
      } else if (RH.cryoX[2]>70. && RH.cryoX[2]<100.) {
        RH.FCalInBeam=25;
        if (RH.HVset[2]!=0.0) fillModule1[2]=true;
        fillModule1[3]=false;
      } else {
        fillModule1[3]=false;
        fillModule1[2]=false;
        RH.FCalInBeam=0;
      }

    } else if (RunLog<1068) { // 2012 data
      logbookData >> RH.Good;
      logbookData >> RH.AttnSet;
      logbookData >> RH.HVset[2];
      logbookData >> RH.HVset[3];
      logbookData >> RH.HVset[1];
      logbookData >> RH.HVset[0];
      logbookData >> RH.HVbeam[3];
      logbookData >> RH.cherPressure;
      logbookData >> RH.cryoX[0];
      logbookData >> RH.cryoX[1];
      logbookData >> RH.cryoX[2];
      logbookData >> RH.S123inBeam;
      logbookData >> RH.BeamIntensityCher;
      logbookData >> CommentString;
      if (RH.cryoX[2]<100) { // 0.25mm FCal in beam
        RH.AntennaModule=3;
        RH.FCalInBeam=2;
      } else {               // 0.10mm FCal in beam
        RH.AntennaModule=2;
        RH.FCalInBeam=3;
      }

    } else if (RunLog<1195) { // 2013 data
      logbookData >> RH.Good;
      logbookData >> RH.AttnSet;
      logbookData >> RH.HVset[2];
      logbookData >> RH.HVset[3];
      logbookData >> RH.HVset[1];
      logbookData >> RH.HVset[0];
      logbookData >> RH.HVbeam[3];
      logbookData >> RH.cherPressure;
      logbookData >> RH.cryoX[0];
      logbookData >> RH.cryoX[1];
      logbookData >> RH.cryoX[2];
      logbookData >> RH.PickupCorr;
      RH.PickupCorrVeto=(RH.PickupCorr==1);              
      logbookData >> RH.S123inBeam;
      logbookData >> RH.BeamIntensityCher;
      logbookData >> CommentString;
      if (RH.cryoX[2]>30) {  // 0.25mm FCal in beam
        RH.AntennaModule=3;
        RH.FCalInBeam=2;
      } else {               // 0.10mm FCal in beam
        RH.AntennaModule=2;
        RH.FCalInBeam=3;
      }


    } else {
      printf("*********** there is no runs with number > 1067 ******************\n");
      exit(-1);
    }
    RH.cherHVlogbook = RH.HVbeam[3];
  
    fillModule1[0]=false;
    fillModule1[1]=false;
    fillModule1[2]=false;
    fillModule1[3]=false;
    if (RH.HVset[0]!=0.0) fillModule1[0]=true;
    if (RH.HVset[1]!=0.0) fillModule1[1]=true;
    if (RH.HVset[2]!=0.0) fillModule1[2]=true;
    if (RH.HVset[3]!=0.0) fillModule1[3]=true;

    double gaps[4]={1.97, 2.10, 0.27, 0.12}; // detector gaps in mm
      //double mobility=0.14; // mm2/sec/V
    double mobility=0.1; // mm2/sec/V
    if (RunLog == Run) {
      for (int i=0; i<4; i++) {
        double Dspeed=mobility*RH.HVset[i]/gaps[i];
        if (Dspeed>0.) {
          Tdrift[i]=gaps[i]/Dspeed;
        } else {
          Tdrift[i]=0.;
        }
        if (ReportMode==1) printf("Module#%d Tdtift=%f msec\n", i, Tdrift[i]*1000.);
        Tdrift[i]*=1.0e9; //convert to Beloved nanoseconds
      }
    }


    if (Run<848) {
      AttnIn[Run]=RH.Attn;
    
      if (Run==RunLog) {
        if (RH.Attn==1) RH.AttnCorr=10.0;
        else RH.AttnCorr=1.0;
        //printf("[ProcessingStatus] Attenuator set to %f, AttnIn=%d\n", RH.AttnCorr, AttnIn[Run]);
      }
    } else {
      if (RH.AttnSet==1.0) {
        AttnIn[Run]=0;
      } else if (RH.AttnSet==3.0) {
        AttnIn[Run]=1;
      } else if (RH.AttnSet==10.0) {
        AttnIn[Run]=2;
      } else if (RH.AttnSet==30.0) {
        AttnIn[Run]=3;
      }
      if (Run==RunLog) {
        RH.AttnCorr=(double)RH.AttnSet;
        //printf("[ProcessingStatus] Attenuator set to %f, AttnIn=%d\n", RH.AttnCorr, AttnIn[Run]);
      }
    }


    if (RunLog==0) {
      printf("[LoadLogbookRecord] Run#%d missing in logbook !\n", Run);
      logbookData.close();
      if (myOnline) { // enable this for online version only! 
        RH.Good = 1;
        return true;
        printf("Enter Cherenkov HV (kV)->");
        cin >> RH.HVbeam[3];
        RH.Good = 1;
        if (RH.HVbeam[3]>0.7 && RH.HVbeam[3]<2.8) {
          return true;
        } else {
          printf("[ProcessingStatus->LoadLogbookRecord] Bad Cherenkov HV=%f entered, bye!\n",
                 RH.HVbeam[3]);
          exit(-1);
        }  
      }
      return false;          
    }
    if (Run==RunLog) {
      printf("[LoadLogbookRecord] Run#%d found in logbook ! Comment->%s\n", RunLog, CommentString);
      logbookData.close();
      return true;
    }
    //ReportLevel=WhatToDo(100, ReportLevel, 10000); 
  }

  /*   enable this for online version only! 
  if (!RH.Good) {
    printf("[LoadLogbookRecord] Run#%d not found in LogBook!\n", Run);
    printf("Enter Cherenkov HV (kV)->");
    cin >> RH.HVbeam[3];
    RH.Good = true;
  }
  */

  logbookData.close();
  //exit(-1);
  return true;
}

int GetAttenuatorStatus(int run) { 
  if (run<0 || run>1599) return 0;
  return AttnIn[run];
}
int GetAttenuatorStatus() { 
  if (RH.Attn) return 1; 
  return 0;
}

double GetIntensityCher() {return RH.BeamIntensityCher;}

//********************************************************************************/
// this function collects run numbers to preocess according to User Selection and
// sets some useful variables as maximum intensity in the list of runs
//
void SetRunList() {
  int iSelection=-1;
  char *tokenPtr;
  char *ctemp;
  char RunStringSpecial[300];
  sprintf(RunStringSpecial, "702,709,717,725,745,746,755,756,763,767,779,789,790,795,802,803,818,821,827,828,833,837");  
  char RunStringSpecial3[300];
  sprintf(RunStringSpecial3, "776,777,827,779,789-793,795-805,807-814,816-819,821-824,826-831,833,837-842");  
  char RunStringSpecial4[300];
  sprintf(RunStringSpecial4, "830-831,833,837-842");  
  char RunStringSpecial5[300];
  sprintf(RunStringSpecial5, "803-805,807-814,816-819,821-824,826-831,833,837-842");  
  char RunStringSpecial6[300];
  sprintf(RunStringSpecial6, "799-805,807-814,816-819,821-824,826-831,833,837-842"); 
  char RunStringSpecial7[300];
  sprintf(RunStringSpecial7, "793,795,802,803,809,817,818,821,827,828,833"); 

  for(int iMod=0; iMod<4; iMod++) {
    nHVgroups[iMod]=0;
    HVgroup[iMod][0]=0.0;
  }

  while (iSelection != 1 && 
         iSelection != 2 && 
         iSelection != 3 && 
         iSelection != 4 && 
         iSelection != 5 && 
         iSelection != 6 && 
         iSelection != 7 && 
         iSelection!=0) {
    printf("enter 1 for direct run selection,\n");
    printf("      2 for selection of HV settings\n");
    //printf("      3 for list (%s) \n", RunStringSpecial3);
    //printf("      4 for 10^11 list (%s) \n", RunStringSpecial4);
    //printf("      5 for 10^9 to 10^11 list (%s) \n", RunStringSpecial5);
    //printf("      6 for 10^8 to 10^11 list (%s) \n", RunStringSpecial6);
    //printf("      7 for 10^6 to 10^11, max HV list (%s) \n", RunStringSpecial7);
    printf("      0 for exit ->");
    cin >> iSelection;
    if (iSelection==0 || iSelection>10) exit(-1);
  }
//  if (iSelection==1 || iSelection==3 || iSelection==4 || iSelection==5 || iSelection==6 || iSelection==7) {
  if (iSelection==1) {
    int decRun1=0, decRun2=0;
    if (iSelection==1) {
      printf("enter list of runs separated by comma or dash [no spaces], like 345,392-395,455 ->");
      cin >> RunString;
    } else if (iSelection==3) {
      printf(" using set of runs -> %s", RunStringSpecial3);
      sprintf(RunString, "%s", RunStringSpecial3);
    } else if (iSelection==4) {
      printf(" using 10^11 set of runs -> %s", RunStringSpecial4);
      sprintf(RunString, "%s", RunStringSpecial4);
    } else if (iSelection==5) {
      printf(" using 10^9-10^11 set of runs -> %s", RunStringSpecial5);
      sprintf(RunString, "%s", RunStringSpecial5);
    } else if (iSelection==6) {
      printf(" using 10^8-10^11 set of runs -> %s", RunStringSpecial6);
      sprintf(RunString, "%s", RunStringSpecial6);
    } else if (iSelection==7) {
      printf(" using 10^6 to 10^11, max HV set of runs -> %s", RunStringSpecial7);
      sprintf(RunString, "%s", RunStringSpecial7);
    }
    //printf("input->%s\n", RunString);
    sprintf(RunStringCopy, "%s", RunString);
    tokenPtr = strtok(RunString, ",");
    while (tokenPtr != NULL) {
      decRun1 = (int)strtoul(tokenPtr, &ctemp, 0);
      //printf(" run#%d reminder=%s\n", decRun1, ctemp);
      decRun2 = (int)strtoul(ctemp, &ctemp, 0);
      //printf(" run#%d reminder=%s\n", decRun2, ctemp);
      tokenPtr = strtok(NULL, ",");
      if (decRun2==0) {
        if (LoadLogbookRecord(decRun1, false)) {
          fillModule[numberOfRuns][0]=(RH.HVset[0]>0.0);
          fillModule[numberOfRuns][1]=(RH.HVset[1]>0.0);
          fillModule[numberOfRuns][2]=(RH.FCalInBeam==25 && RH.HVset[2]>0.0);
          fillModule[numberOfRuns][3]=(RH.FCalInBeam==10 && RH.HVset[3]>0.0);
//          if (RH.Attn) AttnIn[decRun1]=1;
//          else AttnIn[decRun1]=0;
          printf("[LoadLogbookRecord]Run%d Attn=%d %d\n", decRun1, RH.Attn, AttnIn[decRun1]);
          SelectedRuns[numberOfRuns++]=decRun1;
          if (RH.BeamIntensityLog>maxLogbookIntensity) maxLogbookIntensity=RH.BeamIntensityLog;
//--> add run to module HV groups
          for (int iMod=0; iMod<4; iMod++) {
            if (RH.HVset[iMod]==0.0) continue;
            //if (!fillModule1[iMod]) continue;
            //printf("[SetRunList] current HV setting for %s -> %.3f\n",
            //       module_name[iMod], RH.HVset[iMod]);
            bool found = false;
            for (int ig=0; ig<nHVgroups[iMod]; ig++) {
              if (RH.HVset[iMod]==HVgroup[iMod][ig]) {
                hvGroupRun[iMod][decRun1]=ig;
                found=true;
                break;
              }
            }
            if (!found) { // add HV group
              //printf("[SetRunList] adding %s HVgroup#%d HV=%.2f\n", 
              //        module_name[iMod], nHVgroups[iMod], RH.HVset[iMod]);
              HVgroup[iMod][nHVgroups[iMod]]=RH.HVset[iMod];
              hvGroupRun[iMod][decRun1]=nHVgroups[iMod];
              nHVgroups[iMod]++;
            }
          }            
        }
      } else if (decRun2<0) {
        for (int run=decRun1; run<=-decRun2; run++) {
          if (LoadLogbookRecord(run, false)) {
            fillModule[numberOfRuns][0]=(RH.HVset[0]>0.0);
            fillModule[numberOfRuns][1]=(RH.HVset[1]>0.0);
            fillModule[numberOfRuns][2]=(RH.FCalInBeam==25 && RH.HVset[2]>0.0);
            fillModule[numberOfRuns][3]=(RH.FCalInBeam==10 && RH.HVset[3]>0.0);
            SelectedRuns[numberOfRuns++]=run;
//            if (RH.Attn) AttnIn[run]=1;
//            else AttnIn[run]=0;
            printf(" Run%d Attn=%d %d\n", run, RH.Attn, AttnIn[run]);
            if (RH.BeamIntensityLog>maxLogbookIntensity) maxLogbookIntensity=RH.BeamIntensityLog;
//--> add run to module HV groups
            for (int iMod=0; iMod<4; iMod++) {
              if (RH.HVset[iMod]==0.0) continue;
              //if (!fillModule1[iMod]) continue;
              //printf("[SetRunList] current HV setting for %s -> %.3f\n",
              //       module_name[iMod], RH.HVset[iMod]);
              bool found = false;
              for (int ig=0; ig<nHVgroups[iMod]; ig++) {
                if (RH.HVset[iMod]==HVgroup[iMod][ig]) {
                  hvGroupRun[iMod][run]=ig;
                  found=true;
                  break;
                }
              }
              if (!found) { // add HV group
                //printf("[SetRunList] adding %s HVgroup#%d HV=%.2f\n", 
                //       module_name[iMod], nHVgroups[iMod], RH.HVset[iMod]);
                HVgroup[iMod][nHVgroups[iMod]]=RH.HVset[iMod];
                hvGroupRun[iMod][run]=nHVgroups[iMod];
                nHVgroups[iMod]++;
              }
            }            
          }
        }
      } else {
        printf("[SetRunList] Error, bad Run list entered! %s\n", RunString);
        exit(-1);
      }
    }

   
  } else if (iSelection==2) {
    bool oksir=0;
    while (!oksir) {
      printf("Enter HV settings (kV) for \n");
      for (int i=0; i<4; i++) {
        printf("%10s -> ", module_name[i]);
        cin >> HVmod[i];
      }

       //cin >> HVmod[0] >> HVmod[1] >> HVmod[2] >> HVmod[3];
      printf("you entered : %4.2fV for HEC      \n", HVmod[0]);
      printf("you entered : %4.2fV for EMEC     \n", HVmod[1]);
      printf("you entered : %4.2fV for FCal-250 \n", HVmod[2]);
      printf("you entered : %4.2fV for FCal-100 \n", HVmod[3]);
      printf("--------- is it what you wanted ?  Enter 1 for \"yes\", 0 for \"no\" ->");
      cin >> oksir;
    }
    for (int iMod=0; iMod<4; iMod++) {
      HVchozen[iMod]=HVmod[iMod];
      HVchozenSet[iMod]=true;
    }
    // look for desired runs int the logbook //
    numberOfRuns=0;
    //for (int run=463; run<645; run++) {
    for (int run=644; run<848; run++) {
      if (LoadLogbookRecord(run, false)) {
        //printf("run#%d HVmod=(%3.1f, %3.1f, %3.1f, %3.1f) HVset=(%3.1f, %3.1f, %3.1f, %3.1f)\n", 
        //       run, HVmod[0], HVmod[1], HVmod[2], HVmod[3], RH.HVset[0], RH.HVset[1], RH.HVset[2], RH.HVset[3]);
        if ((HVmod[0]-RH.HVset[0]<0.11 && HVmod[0]-RH.HVset[0] >-0.11) || 
            (HVmod[1]-RH.HVset[1]<0.11 && HVmod[1]-RH.HVset[1] >-0.11) ||
            (HVmod[2]==RH.HVset[2] && RH.HVmap[2]==15 && RH.FCalInBeam==25) ||
            (HVmod[3]==RH.HVset[3] && RH.HVmap[3]==15 && RH.FCalInBeam==10) ) {
          if (RH.Good>0) {
            fillModule[numberOfRuns][0]=(HVmod[0]-RH.HVset[0]<0.11 && HVmod[0]-RH.HVset[0] >-0.11);
            fillModule[numberOfRuns][1]=(HVmod[1]-RH.HVset[1]<0.11 && HVmod[1]-RH.HVset[1] >-0.11);
            fillModule[numberOfRuns][2]=(HVmod[2]==RH.HVset[2] && RH.HVmap[2]==15 && RH.FCalInBeam==25);
            fillModule[numberOfRuns][3]=(HVmod[3]==RH.HVset[3] && RH.HVmap[3]==15 && RH.FCalInBeam==10);
            RunRate[numberOfRuns]=RH.BeamIntensityLog;
            SelectedRuns[numberOfRuns++]=run;
            if (RH.BeamIntensityLog>maxLogbookIntensity) maxLogbookIntensity=RH.BeamIntensityLog;
          } 
        }        
      }
    }
  } else {
    exit(-1);
  }
  printf("[SetRunList] %d runs were selected : -> \n", numberOfRuns);
  numberOfRateGroups=0;
  for (int i=0; i<numberOfRuns; i++) {
    bool found=false;
    for (int j=0; j<numberOfRateGroups; j++) {
      if (GroupRate[j]==RunRate[i]) {
        RunRateGroup[i]=j;
        found=true;
        break;
      }
    }

    if (!found) { // set new rate group
      RunRateGroup[i]=numberOfRateGroups;
      GroupRate[numberOfRateGroups]=RunRate[i];
      numberOfRateGroups++;
    }
  }

  for (int iMod=0; iMod<4; iMod++) {
    //printf("%10s HV list -> ", module_name[iMod]);
    for (int ig=0; ig<nHVgroups[iMod]; ig++) {
      //printf(" %.1f", HVgroup[iMod][ig]);
    }
    //printf("\n");
  }

  //printf("  reordering HV groups...\n");
 
  for (int iMod=0; iMod<4; iMod++) {
    //printf(" Ordering voltages for %s, %d groups ----->", module_name[iMod], nHVgroups[iMod]); 
    for (int ig0=0; ig0<nHVgroups[iMod]-1; ig0++) {
      for (int ig1=ig0+1; ig1<nHVgroups[iMod]; ig1++) {
        //printf("group#%d HV=%.0f <---> group#%d HV=%.0f\n", ig0, HVgroup[iMod][ig0], ig1, HVgroup[iMod][ig1]);
        double HV0=HVgroup[iMod][ig0];
        double HV1=HVgroup[iMod][ig1];
        if (HV1>HV0) {
          //printf("... moving, checking %d runs...!\n", numberOfRuns);
          for (int i=0; i<numberOfRuns; i++) { // switch the groups in run hv group index
            if (hvGroupRun[iMod][SelectedRuns[i]]==ig1) {
              //printf("    Run#%d has group %d, changing to %d\n", SelectedRuns[i], ig1, ig0);
              hvGroupRun[iMod][SelectedRuns[i]]=ig0;
            } else if (hvGroupRun[iMod][SelectedRuns[i]]==ig0) {
              //printf("    Run#%d has group %d, changing to %d\n", SelectedRuns[i], ig0, ig1);
              hvGroupRun[iMod][SelectedRuns[i]]=ig1;
            }
          }
          //printf("  reassigning HV=%.0f to group#%d, HV=%.0f to group#%d\n", HV1, ig0, HV0, ig1); 
          HVgroup[iMod][ig0]=HV1;
          HVgroup[iMod][ig1]=HV0;
        } else {
          //printf("\n");
        }
      }
    }
  }

 
  for (int iMod=0; iMod<4; iMod++) {
    printf("%10s HV list -> ", module_name[iMod]);
    for (int ig=0; ig<nHVgroups[iMod]; ig++) {
      printf(" %.1f", HVgroup[iMod][ig]);
    }
    printf("\n");
  }

  //int iqq=0;
  //printf("Push!");
  //cin >> iqq;  
  //if (iqq==0) exit(-1);

  for (int i=0; i<numberOfRuns; i++) {
    printf("           %d  FillModule -> (%d, %d, %d, %d) ", 
           SelectedRuns[i], fillModule[i][0], fillModule[i][1], fillModule[i][2], fillModule[i][3]);
    printf(" Attenuator = %d ", AttnIn[SelectedRuns[i]]);
    printf(" HVGroup-> (%d, %d, %d, %d) ",
           hvGroupRun[0][SelectedRuns[i]],
           hvGroupRun[1][SelectedRuns[i]],
           hvGroupRun[2][SelectedRuns[i]],
           hvGroupRun[3][SelectedRuns[i]]);
    printf(" HV -> (%.1f, %.1f, %.1f, %.1f) \n",
           HVgroup[0][hvGroupRun[0][SelectedRuns[i]]],
           HVgroup[1][hvGroupRun[1][SelectedRuns[i]]],
           HVgroup[2][hvGroupRun[2][SelectedRuns[i]]],
           HVgroup[3][hvGroupRun[3][SelectedRuns[i]]]);

  }

  //printf("Push!");
  //cin >> iqq;  
  //if (iqq==0) exit(-1);

  //printf("[SetRunList] maximum beam rate is %.1e\n", maxLogbookIntensity);
  runIndex=0;
  //exit(-1);
}

bool GetFillModule(int Module) {
  if (HVchozenSet[Module] && RH.HVset[Module]!=HVchozen[Module]) fillModule[runIndex-1][Module]=false;
  return fillModule[runIndex-1][Module];
}

int  GetAntennaModule() {return RH.AntennaModule;}

/********************************************************************************/
//  simplifyed single run analysis
//
void SetRunListBasic() {

  printf("Run number ->");
  numberOfRuns=0;
  cin >> SelectedRuns[numberOfRuns++];
  runIndex=0;
  sprintf(RunString, "%d",  SelectedRuns[numberOfRuns]);
//  printf("FCal in Beam (1 for 100, 2 for 250) ->");
//  int inBeam=0;
//  cin >> inBeam;
//  RH.AntennaModule=-1;
//  if (inBeam==1)      RH.AntennaModule=2;
//  else if (inBeam==2) RH.AntennaModule=3;
//  else {
//    printf("[SetRunListBasic] Wrong FCAL index %d selected, shall be 1 or 2\n", RH.AntennaModule);
//    exit(-1); 
//  }
//  printf("[SetRunListBasic] inBeam=%d RH.FCalInBeam=%d\n", inBeam, RH.FCalInBeam);
}

const char* GetRunString() {return RunStringCopy;}

double GetMaxLogbookIntensity() {return maxLogbookIntensity;}

int GetNumberOfRuns()     {return numberOfRuns;}

void ClearRunIndex() {runIndex=0;}


int GetNextRunToProcess() {
  if (runIndex<numberOfRuns) {
    //printf("[GetNextRunToProcess] i=%d run to process --> %d\n", runIndex, SelectedRuns[runIndex]);
    return SelectedRuns[runIndex++];
  } else {
   // printf("[GetNextRunToProcess] all runs processed, bye!\n");
    runIndex=0; 
    return 0;
  }
} 

double GetAttenuator() {return RH.AttnCorr;}

void SetDumpMode() {
  DumpCount=0;
  printf("Enter Dump Mode: 0 - no dump\n");
  printf("                +n - n events>");
  cin >> DumpMode;
  if (DumpMode>0) printf("%d events will be dumped\n", DumpMode);
  return;
}
int GetDumpMode() {return DumpMode;}
int GetDumpCount() {return DumpCount;}
void SetDumpCount(int n) {DumpCount=n;}
void CountDump() {
  DumpCount++;
  if (DumpCount>=DumpMode) {
    DumpMode=0;
  } 
  return;
}

void SetNbunches(int N) {Nbunches=N; EH.Nbunches=N;}
int  GetNbunches() {return Nbunches;}
int  GetNfilled()  {return nFilled;}
double  GetBunchTime(int ib) {
  if (ib<0 || ib>=Nbunches) {
    printf("[ProcessinStatus->GetBunchTime] wrong bunch index %d nBunches=%d\n", ib, Nbunches);
    exit(-1);
  }
  return bunchTime[ib];
}

int GetNumberOfHVgroups(int iMod) {
 if (iMod<0 || iMod>3) return 0;
 return nHVgroups[iMod];
}

int GetGroupHVrun(int iMod, int Run) {
 if (iMod<0 || iMod>3) return 0;
 if (Run<652 || Run>1600) return 0;
 return hvGroupRun[iMod][Run];
}

double GetHVgroup(int iMod, int iGroup) {
 if (iMod<0 || iMod>3) return 0.0;
 if (iGroup<0.|| iGroup>=nHVgroups[iMod]) return 0.0;
 return HVgroup[iMod][iGroup];
}

void SetHVchosen(int iMod, double HV) {
  HVchozen[iMod]=HV;
  HVchozenSet[iMod]=true;
}

double GetHVchosen(int iMod) {
  if (HVchozenSet[iMod]) return HVchozen[iMod];
  else return 0.;
}

bool GetBeginningOfBurst() {return BeginningOfBurst;}
void SetBeginningOfBurst(bool x) {BeginningOfBurst=x;}
/********************************************************************/
 int WhatToDo(const char*Location, int mode) {
    char boo[5];
    if (mode==-200) {
      printf("[WhatToDo] -> Burst#%d Done\n", EH.burst);
    } else if (mode==-300) {
      printf("[WhatToDo] -> Run Done\n");
    } else if (mode==-100) {
      printf("[WhatToDo] -> Event Done\n");
    } else if (mode==-1) {
      return mode;
    }
    if (ReportMode == mode || 
        (mode==-100 && ReportMode<10100 && ReportMode>10000) ||
        (mode==-100 && ReportMode==19000) ||
        (mode==-100 && ReportMode==1) ||
        (mode==-300 && ReportMode<10300 && ReportMode>10000) ||
        (mode==-200 && ReportMode<10200 && ReportMode>10000)) {
      bool goodMode=false;
 
      while (!goodMode) {
        for (int i=0; i<nCases; i++) {
          if (activeMode[i]) printf("%6d %s\n", pauseMode[i], pauseCase[i]);
        }
        cout << Location << " ReportMode=" << mode << " Enter New Value "; 
        cin >> boo;
        ReportMode=-100000;
        sscanf(boo, "%d", &ReportMode);

        for (int i=0; i<nCases; i++) {
          if (ReportMode==pauseMode[i]) {
            printf("[WhatToDo] matching mode#%d=%d\n", i, pauseMode[i]);
            goodMode=true;
            break;
          }
        }
        if (!goodMode) printf("[WhatToDo] ReportMode=%s, is unkown push!\n", boo);
/* old mode input, buggy
        cin >> ReportMode;
        for (int i=0; i<nCases; i++) {
          if (ReportMode==pauseMode[i]) {
            goodMode=true;
            break;
          }
        }
        printf("\n");
*/
      }
      if (ReportMode==0) exit(-1);
    }
    return ReportMode;  
  }


  bool LoadLogbook() {
    if (!CurrentRun) {
      printf("[ProcessingStatus->LoadLogbook] Run number not set\n");
      return false;
    }
    
  }

  void SetBunchPattern(int i, double a) {
    if (i<0 || i>29) {
      printf("[ProcessingStatus->SetBunchPattern] wrong value i=%d\n", i);
      exit(-1);
    }
    bunchPattern[i]=a;
  }

  bool CheckSlopesLoaded() {
    return bunchSlopesLoaded;
  }
  double GetBunchSlope(int i) {
    if (i<0 || i>9) {
      printf("[ProcessingStatus->GetBunchSlope] wrong value i=%d\n", i);
      exit(-1);
    }
    return bunchSlopes[i];
  }

  bool CheckBunchPattern() {
    return bunchPatternSet;
  }
  void SetBunchPatternSet(bool val) {
    bunchPatternSet=val;
  }

  double GetBunchPattern(int i) {
    if (i<0 || i>29) return 0.;
    return bunchPattern[i];
  }

  int GetBunchAction(int i) {
    if (i<0 || i>29) return 0;
    return bunchAction[i];
  }

  bool GetBunchIsolated(int i) {
    if (i<0 || i>29) return false;
    return bunchIsolated[i];
  }

  int* GetBunchActionPointer() {
    return &bunchAction[0];
  }



  bool LoadSpillSummary(int iyear, int imonth, int iday, int ihour, int imin, int isec) {
    bool result=false;
    double bpSum=0.0;
    double bpmax=0.;
    double bpmin=1.0e10;
    std::ifstream indataX;

    EH.SetFirstBunch(0); // set fake first filled bunch; shall be 
                              // changed later by Main->EventPreProcessing
    sprintf(bunchFileName, "GADCtails/GADCtail20151005_run%04d_spill%04d.slopes", CurrentRun, BH.burst);
    printf("[ProessingStatus->LoadSpillSummary] setting filling pattern for run#%d bunch#%d->\n", CurrentRun, BH.burst); 
    printf("[ProessingStatus->LoadSpillSummary] file->%s\n",  bunchFileName);

    indataX.open(bunchFileName, ios::in);
    if (!indataX.is_open()) {
      printf("[ProessingStatus->LoadSpillSummary] no slopes file %s found\n", bunchFileName);
      bunchSlopesLoaded=false;
      for (int i=0; i<10; i++) {
        bunchSlopes[i]=0.0;
      }
      bunchSlopes[0]=0.167;
      bunchSlopes[1]=0.056;
      bunchSlopes[2]=0.010;
      bunchSlopes[3]=0.007;
    } else {
      double s1, s2;
      for (int i=0; i<10; i++) {
        indataX >> s1 >> s2;
        bunchSlopes[i]=0.5*(s1+s2);
        printf("[ProessingStatus->LoadSpillSummary]   #%2d slope=%.4f\n", i, bunchSlopes[i]); 
      }
      indataX.close();
      for (int i=0; i<10; i++) {
        if (bunchSlopes[i]>0.5 || 
            i>0 && bunchSlopes[i]>0.1 || 
            i>1 && bunchSlopes[i]>0.05) {
          printf("*******************************************************\n");
          printf("[ProessingStatus->LoadSpillSummary]   #%2d bad slope=%.4f\n", i, bunchSlopes[i]); 
          printf("*******************************************************\n");
          bunchSlopesLoaded=false;
          indataX.clear();
          return false;
        }
      }      
      bunchSlopesLoaded=true;
    }
    indataX.clear();

    if (CurrentRun<852) { // old runs use 5x(100000) pattern
      for (int i=0; i<30; i++) {
        if (i%6==0) bunchPattern[i]=1.0;
        else        bunchPattern[i]=0.0;        
      }
      bunchPatternSet=true;
      result=true;
    } else { // shall load a file with the pattern !
      bunchPatternSet=true;

//   /home/data0/Cherenkov/bunches/bunches-YYYY-MM-DD-HH-MM-SS.dat
// HVcher and 30 bunches, integrated in protons
      int istep=1;    
      int  iz=1;

      while (!indataX.is_open() && istep<10 && istep>-10) {
        //sprintf(bunchFileName, "../data0/Cherenkov/bunches/bunches-2012-04-09-04-13-50.dat");
        sprintf(bunchFileName, "../data0/Cherenkov/bunches/bunches-%04d-%02d-%02d-%02d-%02d-%02d.dat",
                iyear, imonth, iday, ihour, imin, isec);
        printf("[ProessingStatus->LoadSpillSummary] trying file %s\n", bunchFileName);
        indataX.open(bunchFileName, ios::in);
        isec+=iz*istep;

        if (isec<0) {
          isec+=60;
          imin-=1;
          if (imin<0) {
            imin+=60;
            ihour-=1;
            if (ihour<0) {
              ihour+=24;
              iday-=1;
            }
          }
        }

        if (isec>59) {
          isec-=60;
          imin+=1;
          if (imin>59) {
            imin-=60;
            ihour+=1;
            if (ihour>24) {
              ihour-=24;
              iday+=1;
            }
          }
        }

        istep++;
        iz=-iz;
      }


      if (!indataX.is_open()) {
        sprintf(bunchFileName," ");
        indataX.clear();
        return false;
      } else {
        double cherHV;
        indataX >> cherHV;
        printf("[ProcessingStatus->LoadSpillSummary] Cherenkov HV measured=%.2fV\n", cherHV);
        RH.HVbeamRaw[3]=cherHV/1000.; //V->kV
        if (RH.run>850 && RH.run<1070) {// 2012 data
          RH.HVbeam[3]=0.973*RH.HVbeamRaw[3]+0.023; 
          printf("[CherenkovBuffer] 2012 corrected HV=%.2fkV\n", RH.HVbeam[3]);
        } else if (RH.run>1070 && RH.run<1195) {// 2013 data
          RH.HVbeam[3]=0.955*RH.HVbeamRaw[3]+0.018; 
          printf("[ProcessingStatus->LoadSpillSummary] 2013 corrected HV=%.2fkV\n", RH.HVbeam[3]);
        }
        RH.Good = 1;
        bpSum=0.0;
        int iBPmin=-1;
        for (int i=0; i<30; i++) {
          indataX >> bunchPattern[i]; 
          if (bunchPattern[i]>bpmax) bpmax=bunchPattern[i];
          if (bunchPattern[i]<bpmin) {bpmin=bunchPattern[i]; iBPmin=i;}
        }

        for (int i=0; i<30; i++) {
          bunchPattern[i]-=bpmin;
          bunchPattern1[i]=bunchPattern[i];
          bpSum+=bunchPattern[i];         
//          printf("   %2d a=%1.3f\n", i, bunchPattern[i]);
        }

        for (int ix=iBPmin; ix<iBPmin+30; ix++) {
          int i=ix;
          if (i>29) i-=30;
          for (int j=0; j<10; j++) {
            if (bunchSlopes[j]==0.) continue;
            int i1=i+j+1;
            if (i1>29) i1-=30;
            bunchPattern1[i1]-=bunchSlopes[j]*bunchPattern1[i];
          }
        }

        for (int i=0; i<30; i++) {
          bunchPattern[i]/=bpmax;
          bunchPattern1[i]/=bpmax;
          if (bunchPattern1[i]<0.0) bunchPattern1[i]=0.0;
        }

      }
      indataX.close();
      bunchPatternSet=true;
      result=true;
    }
 
    double maxminratio=(bpmax-bpmin)/(bpmin+bpmax);   
    printf("[ProcessingStatus->LoadSpillSummary] bunch pattern integral=%3.1e Max/Min factor=%.2f\n", bpSum, maxminratio);

    for (int i=0; i<30; i++) {
      bunchIsolated[i]=false;
    }

    int nnn=0;
    int nn2=0;

    for (int i=0; i<30; i++) {
      bunchAction[i]=2;
      bunchAction0[i]=2;
      int i1=i-1;
      if (i1<0) i1+=30;
      if ((bunchPattern1[i]>0.1  && bunchPattern[i]>0.3*bunchPattern[i1])  ||
          (bunchPattern[i]>0.2   && bunchPattern[i]>0.5*bunchPattern[i1])) {
        bunchAction[i]=1;
        bunchAction0[i]=1;
        nnn++;
      }
    }
    
    int nIsol=0;
    for (int i=0; i<30; i++) {
      if (bunchAction[i]==1) {//check if isolated //
        bunchIsolated[i]=true;
        for (int j1=i-4; j1<i+5; j1++) {
          int j=j1;
          if (j<0)  j+=30;
          if (j>29) j=-30;
          if (i!=j && bunchAction[j]==1) {
            bunchIsolated[i]=false;
            break;
          }
        }  
        if (bunchIsolated[i]) nIsol++;      
      }
    }

    int iAdd=1;
    while (iAdd>0) {
      iAdd=0;
      for (int i=0; i<30; i++) { // check for cases when cluster of filled bunches is soft - walk
        int i1=i-1;              // backwards from filled bunch
        if (i1<0) i1+=30;
        if (bunchAction[i1]!=1 && 
            bunchAction[i]==1  && 
            bunchPattern[i1]>0.2*bunchPattern[i]) {
          bunchAction[i1]=1;
          iAdd++;
        }
      }
    } 

    for (int i=0; i<30; i++) {
      int nAct=0;
      for (int j=-4; j<0; j++) { // no measurements of noise and signal AFTER signal
        int i1=i+j;
        if (i1<0)  i1+=30;
        if (i1>29) i1-=30;
        if (bunchAction0[i1]==1) nAct++;
      }
      //printf("%d", nAct);
      if (nAct != 0 && bunchAction[i]==2) bunchAction[i]=0;
      if (bunchAction[i]==2) nn2++;
    }
    //printf("\n");
/*
// plug cluster holes in structures likse ...1.1.......1.......1.......
    for (int i0=0; i0<30; i0++) {
      if (bunchAction[i0]==1) {
        for (int j1=i0-4; j1<i-1; j1++) {
          int i1=j1;
          if (i1<0) i1+=30;
          if (bunchAction[i1]==1) {
            int ix=i1+1;
            if (ix>29) ix-=30;
            for (int j2=i1+1; ix<i0; j2++) {
              int i2=j2;
              if (i2>30) i2-=30;
              ix=i2;
              bunchAction[i2]=1;
            }
          }
        }
      }
    }
*/
    printf("[ProcessingStatus->LoadSpillSummary] filled count = %d, noise count = %d\n", nnn, nn2);
    nFilled=nnn;

    if (nnn>14) { // empty spill! ... nothing but noise
      BH.SetGoodSpill(false); 
      bunchPatternSet=false;
      result=false;
      printf("[ProcessingStatus->LoadSpillSummary] Too many bunches appear filled: %d\n", nnn);
      //exit(-1);
    }
    if (nn2==0) { // weird spill, no chances to get a pedestal; 
      BH.SetGoodSpill(false); 
      bunchPatternSet=false;
      result=false;
      printf("[ProcessingStatus->LoadSpillSummary] Bad spill - No empty bunches: %d\n", nn2);
    }
    if (nIsol==0) { // no isolated bunches found 
      BH.SetGoodSpill(false); 
      bunchPatternSet=false;
      result=false;
      printf("[ProcessingStatus->LoadSpillSummary] Bad spill - No isolated bunches: %d\n", nIsol);
    }

    if (maxminratio<0.5) {
      BH.SetGoodSpill(false); 
      bunchPatternSet=false;
      for (int i=0; i<30; i++) { bunchAction[i]=2; }
      printf("[ProcessingStatus->LoadSpillSummary] Flat spill, likely to be empty: MaxMin factor = %.3f\n", maxminratio);
    }

    for (int i=0; i<30; i++) printf(" #%2d  ", i);                   printf("\n");
    for (int i=0; i<30; i++) printf(" %5.2f", bunchPattern[i]*10.);  printf("\n");
    for (int i=0; i<30; i++) printf(" %5.2f", bunchPattern1[i]*10.); printf("\n");
    //for (int i=0; i<30; i++) printf(" %3d ", bunchAction0[i]);     printf("\n");
    for (int i=0; i<30; i++) printf(" %3d  ", bunchAction[i]);       printf("\n");
    for (int i=0; i<30; i++) printf(" %3d  ", bunchIsolated[i]);     printf("\n");


/*
    int iqq=0;
    //if (EH.run==858 && EH.event>1010) iqq=1;
    if (!result) iqq=1;
    if (iqq==1) {
      printf("what to do? ->");
      cin >> iqq;
      if (iqq==0) exit(-1);
    }
*/
    indataX.close();
    return result;
  }
//---------------------------------------------------------------------
  void PrepareBunches() {
    bool Dbg=false;
    int firstBunch=EH.GetFirstBunch();
//  prepare pedestal windows using the pedestal bunches
    Nbunches=0;
    nPedWindows=0;
    for (int iw=0; iw<10; iw++) {pedWindow0[iw]=-1000.; pedWindow1[iw]=-1000.;}
    double Tb=-100.;
    for (int ir=0; Tb<6400.; ir++) {
      Tb=EH.GetTimeGlobal()+ir*EH.GetRFperiod();
      int ib=ir+firstBunch;
      while (ib>29) ib-=30;
      if (bunchAction[ib]==2) {
        if (pedWindow0[nPedWindows]==-1000.) { // starting new pedestal window
          pedWindow0[nPedWindows]=Tb-EH.GetRFperiod();
          //printf("[ProcessinStatus->PrepareBunches] entering pedestal window #%d T=%7.1f\n", nPedWindows, Tb);
        }
      } else {
        if (pedWindow0[nPedWindows]!=-1000.) { // we were IN the ped window!
          pedWindow1[nPedWindows]=Tb;
          //printf("[ProcessinStatus->PrepareBunches] closing pedestal window #%d T=%7.1f\n", nPedWindows, Tb);
          nPedWindows++;
        }
        if (nPedWindows==10) {
          printf("\n\n[ProcessinStatus->PrepareBunches] bad number of pedestal windows N=%d\n\n", nPedWindows);
          exit(-1);
        }
      }
    }

    if (pedWindow1[nPedWindows] == -1000. && 
        pedWindow0[nPedWindows] != -1000. && 
        pedWindow0[nPedWindows] <   6000.) {
      //printf("[ProcessinStatus->PrepareBunches] closing unfinished pedestal window #%d\n", nPedWindows-1);
      pedWindow1[nPedWindows]=6400.; // close the last pedestal window if it was not closed  
      nPedWindows++; 
    }

    if (pedWindow1[0]<400.) {
      for (int iw=0; iw<nPedWindows-1; iw++) {
        pedWindow0[iw]=pedWindow0[iw+1];
        pedWindow1[iw]=pedWindow1[iw+1];
      } 
      nPedWindows--;
    }

    for (int ib=0; ib<60; ib++) {bunchToFill[ib]=false;}

    if (firstBunch<0 || firstBunch>29) {
      printf("[ProcessinStatus->PrepareBunches] bad first bunch#%d\n", firstBunch);
      exit(-1);
    }
    
    if (ReportMode == 1) {
      for (int iw=0; iw<nPedWindows; iw++) {
        printf("[ProcessinStatus->PrepareBunches] iw=%2d %5.1f...%5.1f\n", iw, pedWindow0[iw], pedWindow1[iw]);
      }
      printf("[ProcessinStatus->PrepareBunches] looking for signals in range %.f...%.1f\n",
             pedWindow1[0]-5., pedWindow0[nPedWindows-1]+5. );
    }
//  mark individual(isolated) bunches corresponding to the signal we will analyse
    Tb=-100.;
    for (int ir=0; Tb<5900.; ir++) {
      Tb=EH.GetTimeGlobal()+ir*EH.GetRFperiod();
      int ib=ir+firstBunch;
      if (Tb<150.) continue; // too early to be a good bunch for analysis /// shall be increased?!

      int ib0=ib-1;
      while (ib0<0)  ib0+=30;
      while (ib0>29) ib0-=30;
      int ib01=ib-2;
      while (ib01<0)  ib01+=30;
      while (ib01>29) ib01-=30;
      int ib1=ib;
      while (ib1<0)  ib1+=30;
      while (ib1>29) ib1-=30;
      int ib2=ib+1;
      while (ib2<0)  ib2+=30;
      while (ib2>29) ib2-=30;
      int ib21=ib+2;
      while (ib21<0)  ib21+=30;
      while (ib21>29) ib21-=30;
//      if (ReportMode == 1) printf("      ir=%3d ib=%3d Tb=%7.1f", ir, ib, Tb);
//      if (ReportMode == 1) printf(" ib*->(%2d, %2d, %2d) Action->(%d, %d, %d) ",
//                                  ib1, ib0, ib2, bunchAction[ib0], bunchAction[ib1], bunchAction[ib2]);
      if (bunchAction[ib1]==1   && 
          bunchAction[ib01]!=1  &&
          bunchAction[ib21]!=1  && 
          bunchAction[ib0]!=1   &&
          bunchAction[ib2]!=1   &&
          Tb<pedWindow0[nPedWindows-1]+5. &&
          Tb>pedWindow1[0]-5.) {
         bunchToFill[ib]=true;
         bunchIndex[Nbunches]=ib;
         bunchTime[Nbunches]=Tb;
         Nbunches++;
//         if (ReportMode == 1) printf("   passed #%d\n", Nbunches);
      } else {
//         if (ReportMode == 1) printf("\n");
      }
    }
    if (Dbg) {
      for (int i=0; i<60; i++) printf("%d",  bunchToFill[i]);
      printf("\n");
      printf("[ProcessinStatus->PrepareBunches] Nbunches=%d ", Nbunches);
      for (int i=0; i<Nbunches; i++) printf(" %4.1f", bunchTime[i]);
      printf("\n");
    }
  }

// called from TModule to wipe out bunches that are out of the Module's windows range
void FilterBunches(double T0, double T1) {
  int nBunchNew=0;
  for (int ib=0; ib<Nbunches; ib++) {
    if (bunchTime[ib]>T0 && bunchTime[ib]<T1) {
      bunchTime[nBunchNew] =bunchTime[ib];
      bunchIndex[nBunchNew]=bunchIndex[ib];
      nBunchNew++;
    } else {
      bunchToFill[bunchIndex[ib]]=false;
    }
  }
  Nbunches=nBunchNew; 
}


int GetPedWindows() {
  return nPedWindows;
}

double GetPedWindow0(int iw) {
  if (iw<0 || iw>=nPedWindows) {
    printf("[ProcessinStatus->GetPedWindow0] wrong window index iw=%d\n", iw);
    exit(-1);
  }
  return pedWindow0[iw];
}
double GetPedWindow1(int iw) {
  if (iw<0 || iw>=nPedWindows) {
    printf("[ProcessinStatus->GetPedWindow1] wrong window index iw=%d\n", iw);
    exit(-1);
  }
  return pedWindow1[iw];
}

   
};


