//#include <classes/TModule.C>
#include <cstdlib>
using std::strtoul;

class ProcessingStatus { // contains general information about data processing //
private:

  char dataFolder[200];
  char logbookFile[200];
  int numberOfRuns;
  int runIndex;
  int SelectedRuns[500];
  bool fillModule[500][6];
  double RunRate[500];
  int numberOfRateGroups;
  int RunRateGroup[500];
  double GroupRate[500];
  double HVmod[6]; // selected HV on the module[iMod]

  int CurrentRun;
  int ReportMode;   
  int nCases;      // report mode cases to use/show
  EventHeader EH;
  BurstHeader BH;
  RunHeader   RH;
  bool firstRunEvent;
  bool firstBurstEvent;
  bool firstRunBurst;

  char* pauseCase[100];
  int   pauseMode[100];
  bool  activeMode[100];

  bool useFitFiles;
  bool useCherenkovStream;

  double maxLogbookIntensity;

  char *module_name[7];

public:
  ProcessingStatus() { // Constructor //
    CurrentRun = 0;
    ReportMode = 10001;
    sprintf (dataFolder, "%s", "../data");
    sprintf (logbookFile, "%s", "../doc/runbook_nov08.elog");
    useFitFiles=false;
    useCherenkovStream=false;
    SetCases();
    numberOfRuns=0;
    //for (int i=0;i<500;i++) SelectedRuns[i]=-1;
    maxLogbookIntensity=0.0;
    module_name[0]="HEC";
    module_name[1]="EMEC";
    module_name[2]="FCal025";
    module_name[3]="FCal010";
    module_name[4]="Beam";
    module_name[5]="Monitor";
    module_name[6]="Integrator";
  }

  ProcessingStatus(bool useFit, bool useCherStream) { // Constructor //
    CurrentRun = 0;
    ReportMode = 10001;
    sprintf (dataFolder, "%s", "../data");
    sprintf (logbookFile, "%s", "../doc/runbook_nov08.elog");
    useFitFiles=useFit;
    useCherenkovStream=useCherStream;
    SetCases();
    numberOfRuns=0;
    runIndex=0;
    //for (int i=0;i<10000;i++) SelectedRuns[i]=-1;
    maxLogbookIntensity=0.0;
    module_name[0]="HEC";
    module_name[1]="EMEC";
    module_name[2]="FCal025";
    module_name[3]="FCal010";
    module_name[4]="Beam";
    module_name[5]="Monitor";
    module_name[6]="Integrator";
   }

  void SetCases() {
    nCases=0;
    pauseCase[nCases]="** Stop program, keep plots";            		  pauseMode[nCases]=-1;    activeMode[nCases]=true;  nCases++;
    pauseCase[nCases]="** Quit root";                           		  pauseMode[nCases]= 0;    activeMode[nCases]=true;  nCases++;
    pauseCase[nCases]="** Display all event's signals together";                  pauseMode[nCases]= 1;    activeMode[nCases]=true;  nCases++;
    pauseCase[nCases]="** Display all event's signals bunch-by-bunch";            pauseMode[nCases]= 16;   activeMode[nCases]=true;  nCases++;
    pauseCase[nCases]="** No interrupts";                                         pauseMode[nCases]=10000; activeMode[nCases]=true;  nCases++;
    pauseCase[nCases]="Pedestal windows setting control";                         pauseMode[nCases]= 2;    activeMode[nCases]=true;  nCases++;
    pauseCase[nCases]="Follow cleaning of pedestal windows";                      pauseMode[nCases]= 3;    activeMode[nCases]=true;  nCases++;
    pauseCase[nCases]="Calculating bunch time based on shape fit data";           pauseMode[nCases]= 4;    activeMode[nCases]=true;  nCases++;
    pauseCase[nCases]="Detais about response computation based on Fit results";   pauseMode[nCases]= 5;    activeMode[nCases]=true;  nCases++;
    pauseCase[nCases]="Event display tuneup";                                     pauseMode[nCases]= 6;    activeMode[nCases]=true;  nCases++;
    pauseCase[nCases]="Bump Filtering tuneup";                                    pauseMode[nCases]= 7;    activeMode[nCases]=true;  nCases++;
    pauseCase[nCases]="Fitting procedure, global";                                pauseMode[nCases]= 8;    activeMode[nCases]=false; nCases++;
    pauseCase[nCases]="Fitting procedure, local";                                 pauseMode[nCases]= 9;    activeMode[nCases]=false; nCases++;
    pauseCase[nCases]="Finding bunch on module level";                            pauseMode[nCases]=10;    activeMode[nCases]=true;  nCases++;
    pauseCase[nCases]="Signals module-by-module in every event";                  pauseMode[nCases]=11;    activeMode[nCases]=true;  nCases++;
    pauseCase[nCases]="Calculation of the Global Phase";                          pauseMode[nCases]=12;    activeMode[nCases]=true;  nCases++;
    pauseCase[nCases]="Time slices preparation";                     		  pauseMode[nCases]=13;    activeMode[nCases]=true;  nCases++;
    pauseCase[nCases]="Cherenkov Stream extraction";                     	  pauseMode[nCases]=14;    activeMode[nCases]=true;  nCases++;
    pauseCase[nCases]="FillHistogram tuning";		                     	  pauseMode[nCases]=15;    activeMode[nCases]=true;  nCases++;
    pauseCase[nCases]="Display selected channel";                                 pauseMode[nCases]=21;    activeMode[nCases]=true;  nCases++;
    pauseCase[nCases]="Cherenkov sum in event and integrators";                   pauseMode[nCases]=22;    activeMode[nCases]=true;  nCases++;
    pauseCase[nCases]="Bunch fit done";                                           pauseMode[nCases]=23;    activeMode[nCases]=true;  nCases++;
    pauseCase[nCases]="Basic pulse params (A, half-width etc)";                   pauseMode[nCases]=24;    activeMode[nCases]=true;  nCases++;
    pauseCase[nCases]="Module Sum vs Integrator check";                           pauseMode[nCases]=25;    activeMode[nCases]=true;  nCases++;
    pauseCase[nCases]="Event done";                                               pauseMode[nCases]=100;   activeMode[nCases]=true;  nCases++;
    pauseCase[nCases]="Spill done";                                               pauseMode[nCases]=200;   activeMode[nCases]=true;  nCases++;
    pauseCase[nCases]="Run done";                                                 pauseMode[nCases]=300;   activeMode[nCases]=true;  nCases++;
    pauseCase[nCases]="FillHistogram problem!";		                     	  pauseMode[nCases]=9000;  activeMode[nCases]=true;  nCases++;
  }

  bool SetRun(int Run) { 
// returns false if some of RUN related files not loaded properly //
// more to be moved here!                                         //
    bool result=true;
    CurrentRun = Run;
    firstRunEvent=true;
    firstBurstEvent=true;
    firstRunBurst=true;
    result = result && LoadLogbookRecord(Run);
    return result;
  }
  int  GetRun()        {return CurrentRun;}
  bool IsFirstRunEvent()            {return firstRunEvent;}
  bool IsFirstBurstEvent()          {return firstBurstEvent;}
  bool IsFirstRunBurst()            {return firstRunBurst;}
  void SetFirstRunEvent(bool set)   {firstRunEvent=set;}
  void SetFirstBurstEvent(bool set) {firstBurstEvent=set;}
  void SetFirstRunBurst(bool set)   {firstRunBurst=set;}
  
  void SetReportMode(int Mode)  {ReportMode=Mode;}
  int  GetReportMode()          {return ReportMode;}

  EventHeader *GetEventHeader() {return &((ProcessingStatus*)this)->EH;}
  BurstHeader *GetBurstHeader() {return &((ProcessingStatus*)this)->BH;}
  RunHeader   *GetRunHeader()   {return &((ProcessingStatus*)this)->RH;}

  int Get10mhz() {
    return EH.scaler10mhz;
  }

  void SetDataFolder(const char* Folder)    {sprintf (dataFolder,    "%s", Folder);}
  void SetLogbookFile(const char* FileName) {sprintf(logbookFile, "%s", FileName);}
  const char* GetDataFolder()               {return dataFolder;}
  const char* GetLogFile()                  {return logbookFile;}

  bool GetUseFit() {return useFitFiles;}


/********************************************************************/
bool RunIsGood(int Run) {
  bool result = RH.Good>0;
  if (!result) {
    printf("[ProcessingStatus->RunIsGood] Run#%d has logbook status %d, skipping\n", Run, RH.Good);
    return false;
  }
  //result = (RH.HECvoltage[0]+RH.EMECvoltage[0]+
  //          RH.FCal025voltage[0]+RH.FCal010voltage[0]>0.0);
  if (!result) {
    printf("[rocessingStatus->RunIsGood] Run#%d has zero HV settings in all modules, skipping\n", Run);
    return false;
   }
  return result;
}
/********************************************************************/
bool LoadLogbookRecord(int Run) {
  char CommentString[500];

  std::ifstream logbookData;
  logbookData.open(logbookFile, ios::in);
  if (!logbookData.is_open()) {
    printf("[LoadLogbookRecord] logbook file %s could not be open\n", logbookFile);
    logbookData.clear();
    return false;
  }

  while (! (CommentString[0] == '-' && 
            CommentString[1] == '-' && 
            CommentString[2] == '-') ) {
    logbookData >> CommentString;
  }

  int RunLog=-1; int HVmap[4]={15, 15, 15, 15};

  while (RunLog != Run && RunLog != 0) {

    logbookData >> RunLog  
                >> RH.FCalInBeam
		>> RH.Good
		>> RH.Attn
		>> RH.HVset[0]
		>> RH.HVset[1]
		>> RH.HVset[2]
		>> RH.HVmap[2]
		>> RH.HVset[3]
		>> RH.HVmap[3]
		>> RH.BeamIntensityLog
		>> RH.S123inBeam
		>> RH.HVbeam[3]
		>> RH.cherPressure
		>> RH.HVbeam[0]
		>> CommentString;
/*
    cout << RunLog  << " " 
                << RH.FCalInBeam << " " 
		<< RH.Good << " "
		<< RH.Attn << " "
		<< RH.HVset[0] << " "
		<< RH.HVset[1] << " "
		<< RH.HVset[2] << " "
		<< RH.HVmap[2] << " "
		<< RH.HVset[3] << " "
		<< RH.HVmap[3] << " "
		<< RH.BeamIntensityLog << " "
		<< RH.S123inBeam << " "
		<< RH.HVbeam[3] << " "
		<< RH.cherPressure << " "
		<< RH.HVbeam[0] << " "
		<< CommentString << endl; 

    printf(" RunLog=%d Run=%d\n", RunLog, Run);
*/   
  }

  bool result=(Run==RunLog);

  if (result) {
    printf("[LoadLogbookRecord] Run#%d found in logbook ! Comment=%s\n", RunLog, CommentString);
  } else {
    printf("[LoadLogbookRecord] Run#%d not found in LogBook!\n", Run);
  } 


  printf("[LoadLogbookRecord] closing logbook file...\n");
  logbookData.close();
  printf("[LoadLogbookRecord] departing...\n");
  return result;
}


/********************************************************************************/
// this function collects run numbers to preocess according to User Selection and
// sets some useful variables as maximum intensity in the list of runs
//
void SetRunList() {
  int iSelection=-1;
  char inString[1000];
  char *tokenPtr;
  char *ctemp;
  while (iSelection != 1 && 
         iSelection != 2 && 
         iSelection!=0) {
    printf("for direct run selection,     enter 1\n");
    printf("for selection of HV settings, enter 2\n");
    printf("                    for exit, enter 0 ->");
    cin >> iSelection;
  }
  if (iSelection==1) {
   int decRun1=0, decRun2=0;
    printf("enter list of runs separated by comma or dash [no spaces], like 345,392-395,455 ->");
    cin >> inString;
    //printf("input->%s\n", inString);
   
    tokenPtr = strtok(inString, ",");
    while (tokenPtr != NULL) {
      decRun1 = (int)strtoul(tokenPtr, &ctemp, 0);
      //printf(" run#%d reminder=%s\n", decRun1, ctemp);
      decRun2 = (int)strtoul(ctemp, &ctemp, 0);
      //printf(" run#%d reminder=%s\n", decRun2, ctemp);
      tokenPtr = strtok(NULL, ",");
      if (decRun2==0) {
        if (LoadLogbookRecord(decRun1)) {
          fillModule[numberOfRuns][0]=1;
          fillModule[numberOfRuns][1]=1;
          fillModule[numberOfRuns][2]=(RH.FCalInBeam==25);
          fillModule[numberOfRuns][3]=(RH.FCalInBeam==10);
          SelectedRuns[numberOfRuns++]=decRun1;
          if (RH.BeamIntensityLog>maxLogbookIntensity) maxLogbookIntensity=RH.BeamIntensityLog;
        }
      } else if (decRun2<0) {
        for (int run=decRun1; run<=-decRun2; run++) {
          if (LoadLogbookRecord(run)) {
            SelectedRuns[numberOfRuns++]=run;
            if (RH.BeamIntensityLog>maxLogbookIntensity) maxLogbookIntensity=RH.BeamIntensityLog;
          }
        }
      } else {
        printf("[SetRunList] Error, bad Run list entered! %s\n", inString);
        exit(-1);
      }
    }
   
  } else if (iSelection==2) {
    bool oksir=0;
    while (!oksir) {
      printf("Enter HV settings (kV) for \n");
      for (int i=0; i<4; i++) {
        printf("%10s -> ", module_name[i]);
        cin >> HVmod[i];
      }

       //cin >> HVmod[0] >> HVmod[1] >> HVmod[2] >> HVmod[3];
      printf("you entered : %4.2fkV for HEC      \n", HVmod[0]);
      printf("you entered : %4.2fkV for EMEC     \n", HVmod[1]);
      printf("you entered : %4.2fkV for FCal-250 \n", HVmod[2]);
      printf("you entered : %4.2fkV for FCal-100 \n", HVmod[3]);
      printf("--------- is it what you wanted ?  Enter 1 for \"yes\", 0 for \"no\" ->");
      cin >> oksir;
    }
    // look for desired runs int the logbook //
    numberOfRuns=0;
    for (int run=310; run<455; run++) {
      if (LoadLogbookRecord(run)) {
        //printf("run#%d HVmod=(%3.1f, %3.1f, %3.1f, %3.1f) HVset=(%3.1f, %3.1f, %3.1f, %3.1f)\n", 
        //       run, HVmod[0], HVmod[1], HVmod[2], HVmod[3], RH.HVset[0], RH.HVset[1], RH.HVset[2], RH.HVset[3]);
        if (HVmod[0]-RH.HVset[0]<0.11 && HVmod[0]-RH.HVset[0] >-0.11 || 
            HVmod[1]-RH.HVset[1]<0.11 && HVmod[1]-RH.HVset[1] >-0.11 ||
            HVmod[2]==RH.HVset[2] && RH.HVmap[2]==15 && RH.FCalInBeam==25||
            HVmod[3]==RH.HVset[3] && RH.HVmap[3]==15 && RH.FCalInBeam==10) {
          if (RH.Good) {
            fillModule[numberOfRuns][0]=(HVmod[0]-RH.HVset[0]<0.11 && HVmod[0]-RH.HVset[0] >-0.11);
            fillModule[numberOfRuns][1]=(HVmod[1]-RH.HVset[1]<0.11 && HVmod[1]-RH.HVset[1] >-0.11);
            fillModule[numberOfRuns][2]=(HVmod[2]==RH.HVset[2] && RH.HVmap[2]==15 && RH.FCalInBeam==25);
            fillModule[numberOfRuns][3]=(HVmod[3]==RH.HVset[3] && RH.HVmap[3]==15 && RH.FCalInBeam==10);
            RunRate[numberOfRuns]=RH.BeamIntensityLog;
            SelectedRuns[numberOfRuns++]=run;
            if (RH.BeamIntensityLog>maxLogbookIntensity) maxLogbookIntensity=RH.BeamIntensityLog;
          } 
        }        
      }
    }
  } else {
    exit(-1);
  }
  printf("[SetRunList] %d runs were selected : -> \n", numberOfRuns);
  numberOfRateGroups=0;
  for (int i=0; i<numberOfRuns; i++) {
    bool found=false;
    for (int j=0; j<numberOfRateGroups; j++) {
      if (GroupRate[j]==RunRate[i]) {
        RunRateGroup[i]=j;
        found=true;
        break;
      }
    }

    if (!found) { // set new rate group
      RunRateGroup[i]=numberOfRateGroups;
      GroupRate[numberOfRateGroups]=RunRate[i];
      numberOfRateGroups++;
    }
  }

  for (int i=0; i<numberOfRuns; i++) {
    printf("           %d  FillModule -> (%d, %d, %d, %d) Rate=%.1e Group=%d\n", 
           SelectedRuns[i], fillModule[i][0], fillModule[i][1], fillModule[i][2], fillModule[i][3],
           RunRate[i], RunRateGroup[i]);
  }
  printf("[SetRunList] maximum beam rate is %.1e\n", maxLogbookIntensity);
  runIndex=0;
  exit(-1);
}

bool GetFillModule(int Module) {return fillModule[runIndex-1][Module];} 

double GetMaxLogbookIntensity() {return maxLogbookIntensity;}

int GetNumberOfRuns()     {return numberOfRuns;}
int GetNextRunToProcess() {
  if (runIndex<numberOfRuns) {
    printf("[GetNextRunToProcess] i=%d run to process --> %d\n", runIndex, SelectedRuns[runIndex]);
    return SelectedRuns[runIndex++];
  } else {
    printf("[GetNextRunToProcess] all runs processed, bye!\n");
    runIndex=0; 
    return 0;
  }
} 

/********************************************************************/
  int WhatToDo(const char*Location, int mode) {
   if (mode==-200) {
      printf("[WhatToDo] -> Burst#%d Done\n", EH.burst);
    } else if (mode==-300) {
      printf("[WhatToDo] -> Run Done\n");
    } else if (mode==-100) {
      printf("[WhatToDo] -> Event Done\n");
    } else if (mode==-1) {
      return mode;
    }
    if (ReportMode == mode || 
        (mode==-100 && ReportMode<100) ||
        (mode==-100 && ReportMode==9000) ||
        (mode==-300 && ReportMode<300) ||
        (mode==-200 && ReportMode<200)) {
      bool goodMode=false;
 
      while (!goodMode) {
        for (int i=0; i<nCases; i++) {
          if (activeMode[i]) printf("%6d %s\n", pauseMode[i], pauseCase[i]);
        }
        cout << Location << " ReportMode=" << mode << " Enter New Value "; 
        cin >> ReportMode;
        for (int i=0; i<nCases; i++) {
          if (ReportMode==pauseMode[i]) {
            goodMode=true;
            break;
          }
        }
        printf("\n");
      }
      if (ReportMode==0) exit(-1);
    }
    return ReportMode;  
  }


  bool LoadLogbook() {
    if (!CurrentRun) {
      printf("[LoadLogbook] Run number not set\n");
      return false;
    }
    
  }
   
};


