
class RingBuffer {
private:
  int StackSize;
  int StackPointer;
  int nFilled;
  double Sum;
  double Sum2;
  double *Stack;
  bool   *StackSet;
public:
  RingBuffer(int N) {
    StackSize=N;
    Stack    = (double*) calloc(StackSize, sizeof(double));
    StackSet = (bool*)   calloc(StackSize, sizeof(bool));
    for (int i=0; i<StackSize; i++) {
      Stack[i]=0.0;
      StackSet[i]=false;
    }
    Sum=0.;
    nFilled=0;
    StackPointer=0;
  }

  void Clear() {
    for (int i=0; i<StackSize;i++) {
      Stack[i]=0.0;
      StackSet[i]=false;
    }
    StackPointer=0;
    Sum=0.;
    nFilled=0;
  }

  void Push(double A) { // add element to ring buffer
    if (StackSet[StackPointer]) {
      Sum-=Stack[StackPointer];  // subtract the old value if it was set
      Sum2-=Stack[StackPointer]*Stack[StackPointer];
    }
    Sum+=A;                                                // add new value;
    Sum2+=A*A;      
    Stack[StackPointer]=A;
    StackSet[StackPointer]=true;  
    StackPointer++;
    if (StackPointer==StackSize) StackPointer=0;
    if (nFilled != StackSize) nFilled++;
  }

  bool Check(int N) { // checks if stack element N is set
    if (N<0 || N>=StackSize) return false;
    int index=StackPointer-N;
    if (index<0) index+=StackSize;
    return StackSet[index];
  }

  double Get(int N) { // get the stack element N is set
    if (N<0 || N>=StackSize) return 0.0;
    int index=StackPointer-N;
    if (index<0) index+=StackSize;
    return Stack[index];
  }

  double GetSum()  {return Sum;}
  double GetSum2() {return Sum2;}

  double GetMean() {
    if (nFilled==0) return 0.;
    return Sum/nFilled;
  }
  double GetRMS() {
    double result=0.0;
    if (nFilled==0) return result;
    result=Sum/nFilled;
    result=Sum2/nFilled-result*result;
    if (result<0.) result=0.;
    else result=sqrt(result);
    return result;
  }

  int GetFilled() {return nFilled;}
};
