struct RunHeader {

  int run;
  int eventLast;
  int FCalInBeam; // 25 or 10
  int Good;

  int PickupCorr;     // set if mysterious inverted signals found in the HV=0 out-of-beam FCal 
  bool PickupCorrVeto; //  shows only in the 2013 (2012?) data

  bool   Attn;   // if there is an attenuator installed at the FEB input//
  double AttnSet;   // attenuator value - for 2012, 2013 data//
  double AttnCorr; // 20dB attenuator //
  double HECvoltage[4];
  bool   HECon[4];
  double EMECvoltage[4];
  bool   EMECon[4];
  double FCal025voltage[4];
  bool   FCal025on[4];
  double FCal010voltage[4];
  bool   FCal010on[4];
  double HVset[4];// HV set on HEC, EMEC, FCal250, FCal100
  double BeamIntensityLog;
  double BeamIntensityCher;
  double RateS4;
  double RateS123;
  double RateIonCh;
  bool   S123inBeam;
  double HVbeam[4]; // S1 S2 S3 Cherenkov, from GADC record if any
  double HVbeamRaw[4]; // S1 S2 S3 Cherenkov, from GADC record if any
  double cherHVlogbook; // Cherenkov HV from logbook;
  double cherPressure;
  double HVmon[4];
  double CryoPosition[3];
  int    HVmap[4]; // HV map for Calorimeters
  double cryoX[3];// position of 3 cryostats: 0=HEC 1=EMEC 3=FCal 
  int    cloneCount;
  int    trainCount;
  int    spikyCount;
  int    LFcount;    // number of events with Low Frequency present
  int    AntennaModule;
  double AntennaRange[252]; // spectral density range for given run
  int    PedestalEvents; // number of empty events in the beginning of spill //
  int    notEmpty;
  int    nSpills;
  int    grayEvents;
  int    LowGainOverflow[100];

  RunHeader() {
    PickupCorrVeto=false;
    AttnCorr=10.0;
    cloneCount=0;
    trainCount=0; 
    spikyCount=0;
    grayEvents=0;
    PedestalEvents=0;
    nSpills=0;
    notEmpty=0;
    LFcount=0;
    for (int i=0; i>252; i++) AntennaRange[i]=0.0;
  }

  bool*    GetS123pointer()        {return &S123inBeam;}
  double*  GetAttenuatorPointer()  {return &AttnSet;}
  double*  GetHVbeamPointer()      {return &HVbeam[0];}
  double*  GetHVbeamRawPointer()   {return &HVbeamRaw[0];}
  double*  GetHVsetPointer()       {return &HVset[0];}
  double*  GetCryoXPointer()       {return &cryoX[0];}
  double*  GetCryoPositionPointer(){return &CryoPosition[0];}
  int*     GetAntennaPointer()     {return &AntennaModule;}


  void ClearLowGainOverflows() {
    for (int i=0; i<40; i++) LowGainOverflow[i]=0;
  }
  void addLowGainOverflow(int iMod, int iChan) {
    LowGainOverflow[5*iMod+iChan]++;
  }
  int getLowGainOverflow(int iMod, int iChan) {
    return LowGainOverflow[5*iMod+iChan];
  }

  double GetHVmodule(int iMod) {
    if (iMod<0 || iMod>3) {
      printf("[RunHeader->getHVmodule] wrong iMod=%d, ciao!\n", iMod);
      exit(-1);
    }
    return HVset[iMod];
  }
};

