#include <TGraphErrors.h>

class TReference {
private:
  char Name[20];
  int nRef;
  double t0, dt; // t0 and dt for reference signal
  double *ref;  // reference shape where available 
  double *rer;  // reference error where available
  TGraphErrors *refShape; 
public:
  TReference() {
    refShape = new TGraphErrors(2);
    nRef=-1;
    t0=0.0;
    dt=0.0;
  }
  bool SetReference(int n, double T0, double dT) {
    if (nRef==-1) {
      nRef=n;
      ref = new double[nRef];
      rer = new double[nRef];
      refShape->Set(nRef);
      t0=T0;
      dt=dT;
      return true;
    } else {
      //cout << "Error, reference is already set! nRef=" << nRef << endl;
      return false;
    }
  }
  bool LoadReference(int i, double A, double R) {
    if (i>-1 && i<nRef) {
      ref[i]=A;
      rer[i]=R;
      refShape->SetPoint(i, t0+i*dt, A);
      refShape->SetPointError(i, 0., R);
      return true;
    } else {
      return false;
    }
  }
  void SetName(const char* xName) {
    sprintf(Name, "%s", xName);
    refShape->SetTitle(Name);
    return;
  }

  double GetRefPoint(int i) {return ref[i];}
  double GetRerPoint(int i) {return rer[i];}
  double GetRexPoint(int i) {return t0+i*dt;}

  double GetRef(double t) {
    double f;
    double samp=(t - t0)/dt;

    if (samp<t0) {
      f=ref[0];
    } else if (samp>nRef) {
      f=ref[nRef-1];
    } else {
      int i1=(int)samp;
      int i2=i1+1;
      f=(samp-i1)*ref[i2]+
        (i2-samp)*ref[i1];
    }
    return f;
  }

  double GetRer(double t) {
    double f;
    double samp=(t - t0)/dt;

    if (samp<t0) {
      //f=rer[0];
      f=0.0;
    } else if (samp>nRef-5) {
      //f=rer[nRef-1];
      f=0.0;
    } else {
      int i1=(int)samp;
      int i2=i1+1;
      f=(samp-i1)*rer[i2]+
        (i2-samp)*rer[i1];
    }
    return f;
  }

  TGraphErrors* GetShape() { // returns pointer to the reference pulse shape
    return ((TReference*)this)->refShape;
  }
  const char* GetName() {return Name;}
};
