//===================== Module Signal
struct Signal {
  int iGain;
  int iChan;
  int iMod;
  int nPedWindows;
  int firstSample[25];
  int lastSample[25];
  float ampI[512]; // signal shape in module channel, raw-ped
  float ampF[512]; // FTT-reconstructed curremt, microAmps
  float time[512];
  float clusterIntegral[5]; // there can not be more than 5 clusters
  bool  maxminReadyMPI;
  bool  maxminReadyFFT;
  double maxFFT, minFFT, maxMPI, minMPI;
  double eventSum;
  double TphaseRF;
//--------------------------------
  double spillPed[3];
  double spillPedSlope[3];
  double eventPed[5][3];
  double eventPedSlope[5][3];
//--------------------------------
  double clusterIntegralFull[5];
//================================================================================
  void Clear() {
    eventSum=0.;
    maxminReadyFFT=false;
    maxminReadyMPI=false;
    for (int i=0; i<5; i++) {
      clusterIntegral[i]=0.;
      clusterIntegralFull[i]=0.;
      for (int j=0; j<3; j++) {
        eventPed[i][j]=0.;
        eventPedSlope[i][j]=0.;
      }
    }
  }

//================================================================================
  void ClearSpill() {
    for (int i=0; i<3; i++) {
      spillPed[i]=0.;
      spillPedSlope[i]=0.;
    }
  }
//================================================================================
  void SetPhaseRF(double t) {
    TphaseRF=t;
    SetTime();
  }

//================================================================================
void AdjustLG(double ag) {
  if (iGain==1) return;
  for (int i=0; i<512; i++) {
    ampI[i]*=ag;
    ampF[i]*=ag;
  }
}
//================================================================================
   void SetTime() {
    for (int i=0; i<512; i++) time[i]=TphaseRF+12.5*i;
  }


//================================================================================
  void doMaxMinMPI() {
    maxminReadyMPI=true;
    if (iGain==0) {
      maxMPI=10.;
      minMPI=-10.;
    } else {
      maxMPI=50.;
      minMPI=-50.;
    }
    for (int i=0; i<512; i++) {
      if (maxMPI<ampI[i]) maxMPI=ampI[i];
      if (minMPI>ampI[i]) minMPI=ampI[i];
    }
  }

//================================================================================
  void doMaxMinFFT() {
    maxminReadyFFT=true;
    if (iGain==0) {
      maxFFT=5.;
      minFFT=-5.;
    } else {
      maxFFT=5.;
      minFFT=-5.;
    }
    for (int i=0; i<512; i++) {
      if (maxFFT<ampF[i]) maxFFT=ampF[i];
      if (minFFT>ampF[i]) minFFT=ampF[i];
    }
  }

//================================================================================
// empty event data, statistics ove empty events only
//================================================================================
void FillEmptyPed(int i0, int i1) { // fill empty event pedestal window
  double s0=0.;
  double sx=0.;
  double sy=0.;
  double sx2=0.;
  double sy2=0.;
  double sxy=0.;
  int is0=firstSample[0];
  for (int i=i0-is0; i<i1-is0-2; i++) {
    s0+=1.0;
    sx +=i;
    sx2+=i*i;
    sy +=ampF[i];
    sy2+=ampF[i]*ampF[i];
    sxy+=i*ampF[i];

    spillPed[0]+=1.0;
    spillPed[1]+=ampF[i];          // Sy
    spillPed[2]+=ampF[i]*ampF[i];  // Sy2
  }

  sx/=s0;
  sy/=s0;
  sx2/=s0;
  sy2/=s0;
  sxy/=s0;
   
  double slope=(sxy-sx*sy)/(sx2-sx*sx);
  spillPedSlope[0]+=1.0;
  spillPedSlope[1]+=slope;
  spillPedSlope[2]+=slope*slope;  
}

//================================================================================
double GetEmptyStat() {
  return spillPed[0];
}

//================================================================================
double GetEmptyPed() {
  return spillPed[1]/spillPed[0];
}

//================================================================================
double GetEmptyPedSigma() {
  double a1=spillPed[1]/spillPed[0];
  double a2=spillPed[2]/spillPed[0];
  a2-=a1*a1;
  if (a2<0.) a2=0;
  else a2=sqrt(a2);
  return a2;
}
 
//================================================================================
double GetEmptyPedSlope() {
  return spillPedSlope[1]/spillPedSlope[0];
} 

//================================================================================
double GetEmptyPedSlopeSigma() {
  double a1=spillPedSlope[1]/spillPedSlope[0];
  double a2=spillPedSlope[2]/spillPedSlope[0];
  a2-=a1*a1;
  if (a2<0.) a2=0;
  else a2=sqrt(a2);
  return a2;
}
//================================================================================
// filled event data, ped-window by ped-window
//================================================================================
void FillPedWindow(int iw, int i0, int i1) { // fill filled  event pedestal window
  double s0=0.;
  double sx=0.;
  double sy=0.;
  double sx2=0.;
  double sy2=0.;
  double sxy=0.;
  int is0=firstSample[0];

//  if (iMod==1 & iChan==4 & iGain==1) {
//    printf("\n[mySignal]  is0=%03d evt window#%d i0=%03d t0=%7.2fns i1=%03d  t1=%7.2fns \n",
//           is0, iw, i0, time[i0], i1, time[i1]); 
//  }
  for (int i=i0-is0; i<i1-is0-2; i++) {
    s0+=1.0;
    sx +=i;
    sx2+=i*i;
    sy +=ampF[i];
    sy2+=ampF[i]*ampF[i];
    sxy+=i*ampF[i];

    eventPed[iw][0]+=1.0;
    eventPed[iw][1]+=ampF[i];
    eventPed[iw][2]+=ampF[i]*ampF[i];

    double av=eventPed[iw][1]/eventPed[iw][0];
    double rm=eventPed[iw][2]/eventPed[iw][0];
    rm=sqrt(rm-av*av);

//    if (iMod==1 & iChan==4 & iGain==1) {    
//      printf("     module#%d i=%03d t=%7.1f MPI=%8.0f curr=%8.0fuA s0=%4.0f s1=%7.0f s2=%7.0f <a>=%7.0f rms=%7.0f\n",
//               iMod, i, time[i+is0], ampI[i], ampF[i],
//               eventPed[iw][0], eventPed[iw][1], eventPed[iw][2], av, rm);
//    }
  }
  sx/=s0;
  sy/=s0;
  sx2/=s0;
  sy2/=s0;
  sxy/=s0;
   
  double slope=(sxy-sx*sy)/(sx2-sx*sx);
  eventPedSlope[iw][1]=slope;
  eventPedSlope[iw][2]=1.0/sqrt(sx2-sx*sx)/sqrt(s0);  
}
 
//================================================================================
double GetPedWindow(int iw) {
  return eventPed[iw][1]/eventPed[iw][0];
}

//================================================================================
double GetPedWindowSigma(int iw) {
  double a1=eventPed[iw][1]/eventPed[iw][0];
  double a2=eventPed[iw][2]/eventPed[iw][0];
  a2-=a1*a1;
  if (a2<0.) a2=0;
  else a2=sqrt(a2);
  return a2;
}
 
//================================================================================
double GetPedWindowSlope(int iw) {
  return eventPedSlope[iw][1];
} 

//================================================================================
double GetPedWindowSlopeSigma(int iw) {
  return eventPedSlope[iw][2];
}

//================================================================================
double ComputeClusterIntegralFull(int iCl) {
//  if (iGain==1 && iChan==4) printf("\n\n[mySignal->GetClusterIntegralFull] iMod=%d iCh=%d\n", iMod, iChan);
  clusterIntegralFull[iCl]=0.;
  if (iCl > nPedWindows-1 || iCl<0) {
    printf("\n\n[mySignal->GetClusterIntegralFull] wriong cluster#%d, bye!\n", iCl);
    exit(-1);
  }
  for (int is=lastSample[iCl]-1; is<firstSample[iCl+1]; is++) {
    clusterIntegralFull[iCl]+=ampF[is-firstSample[0]]; //sum of currents, uA
//    if (iGain==1 && iChan==4) {
//      printf("    i=%03d/%03d t=%6.1fns I=%7.2fmA Q=%7.2fpC\n", 
//              is,  is-firstSample[0], time[is], ampF[is-firstSample[0]]/1000.,
//              clusterIntegralFull[iCl]*12.5/1000.);
//    }
  }
  clusterIntegralFull[iCl]*=12.5; //fC
  return clusterIntegralFull[iCl];
}

//================================================================================
double GetClusterIntegralFull(int iCl) {
  if (iCl > nPedWindows-1 || iCl<0) {
    printf("\n\n[mySignal->GetClusterIntegralFull] wriong cluster#%d, bye!\n", iCl);
    exit(-1);
  }
  return clusterIntegralFull[iCl];
}
//=================================================================================
};


